#include "platform.h"
#include "keyboard.h"
#include "conf.h"
#include "histogram.h"
#include "font.h"
#include "camera.h"
#include "raw.h"
#include "draw_palette.h"
#include "gui_osd.h"
#include "batt_grid.h"
#include "core.h"
#include "stdlib.h"
#include "script.h"

//-------------------------------------------------------------------

#define CONF_FILE  "A/CHDK/CHDK.CFG"
#define CONF_MAGICK_VALUE (0x55014453)
#define CONF_INFO(id, param, type, def, func) { id, sizeof( param ), &param , type, {def}, func }
#define CONF_DEF_PTR    1
#define CONF_DEF_VALUE  2

//-------------------------------------------------------------------
typedef struct 
{
    unsigned short      id;
    unsigned short      size;
    void                *var;
    int                 type;
    union {
           void         *ptr;
           int             i;
           color          cl;
          };
    void           (*func)();
} ConfInfo;

//-------------------------------------------------------------------
Conf conf;

int state_shooting_progress = SHOOTING_PROGRESS_NONE;
int state_expos_recalculated;
int state_expos_under;
int state_expos_over;
int auto_started;
int shot_histogram_enabled;
int debug_propcase_show;
int debug_propcase_page;
int debug_vals_show;


//-------------------------------------------------------------------
static int def_ubasic_vars[SCRIPT_NUM_PARAMS] = {0};
static int def_batt_volts_max, def_batt_volts_min;
static OSD_pos def_histo_pos, def_dof_pos, def_batt_icon_pos, def_batt_txt_pos,def_zoom_pos,def_increment_pos, 
               def_mode_state_pos, def_values_pos, def_clock_pos, def_stereo_pos, def_synch_pos,def_overrides_pos;
#if defined(OPT_SERIALIO)
static OSD_pos def_sio_pos;
#endif

// ********************************************************
static int def_cam_spacing;
static int def_deviation_factor;
static int def_min_distance_factor;
static int def_decade_factor;
static int def_distance_setting;
static int def_manual_far;
// ********************************************************
static void conf_change_script_file();
static void conf_change_histo_mode();
static void conf_change_histo_layout();
static void conf_change_font_cp();
static void conf_change_menu_rbf_file();
static void conf_change_alt_mode_button();
static void conf_change_grid_file();

static void conf_change_video_bitrate();

void ubasic_camera_set_raw(int mode)
{
    conf.save_raw = mode;
}

void ubasic_camera_set_nr(int mode)
{
	//"Auto", "Off", "On"
    conf.raw_nr = mode;
}

void ubasic_camera_set_tv_override(int mode)
{
    conf.tv_override = mode;
}

int ubasic_camera_get_tv_override()
{
 return conf.tv_override;
}


int ubasic_camera_script_autostart()
{
 //1 = Autostarted	
 return auto_started;
}

void ubasic_camera_set_script_autostart(int state)
{
 //1 = Autostarted	
 auto_started = state;
}

int ubasic_camera_get_nr()
{
  //"Auto", "Off", "On"
    return conf.raw_nr;
}

static const ConfInfo conf_info[] = {
    CONF_INFO(140, conf.heading1,               CONF_DEF_VALUE, i:0, NULL), // Zebra and Histogram
    CONF_INFO(  4, conf.show_histo,             CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 20, conf.histo_layout,           CONF_DEF_VALUE, i:OSD_HISTO_LAYOUT_A, conf_change_histo_layout),
    CONF_INFO( 17, conf.histo_mode,             CONF_DEF_VALUE, i:HISTO_MODE_LINEAR, conf_change_histo_mode),
    CONF_INFO( 16, conf.show_overexp,           CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 19, conf.histo_ignore_boundary,  CONF_DEF_VALUE, i:4, NULL),
    CONF_INFO( 18, conf.histo_auto_ajust,       CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 49, conf.zebra_draw,             CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 50, conf.zebra_mode,             CONF_DEF_VALUE, i:ZEBRA_MODE_BLINKED_2, NULL),
    CONF_INFO( 54, conf.zebra_under,            CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 53, conf.zebra_over,             CONF_DEF_VALUE, i:31, NULL),
    CONF_INFO( 51, conf.zebra_restore_screen,   CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 52, conf.zebra_restore_osd,      CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 56, conf.zebra_draw_osd,         CONF_DEF_VALUE, i:ZEBRA_DRAW_HISTO, NULL),
    CONF_INFO(124, conf.zebra_multichannel,     CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(141, conf.heading2,               CONF_DEF_VALUE, i:0, NULL), // Video
    CONF_INFO( 97, conf.video_mode,             CONF_DEF_VALUE, i:0, NULL), 
    CONF_INFO( 96, conf.video_bitrate,          CONF_DEF_VALUE, i:3, conf_change_video_bitrate),
    CONF_INFO( 98, conf.video_quality,          CONF_DEF_VALUE, i:84,NULL), 
    CONF_INFO(122, conf.unlock_optical_zoom_for_video,  CONF_DEF_VALUE, i:0, NULL), 
    CONF_INFO(129, conf.mute_on_zoom,            CONF_DEF_VALUE, i:0, NULL), 
    CONF_INFO(142, conf.heading3,               CONF_DEF_VALUE, i:0, NULL), // Raw
    CONF_INFO(  2, conf.save_raw,               CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(188, conf.blank_jpg,              CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 70, conf.raw_nr,                 CONF_DEF_VALUE, i:NOISE_REDUCTION_AUTO_CANON, NULL),
    CONF_INFO( 60, conf.raw_save_first_only,    CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(143, conf.heading4,               CONF_DEF_VALUE, i:0, NULL),  // Bracketing and Overrides
    CONF_INFO(134, conf.tv_override_enable,     CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 94, conf.tv_override,		CONF_DEF_VALUE, i:19, NULL),
    CONF_INFO( 108, conf.av_override_value,	CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 81, conf.user_range_set,         CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(121, conf.nd_filter_state,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(139, conf.nd_constant_exposure,   CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(112, conf.dist_step_size,		CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(110, conf.subj_dist_bracket_value,CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(165, conf.save_auto,              CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(167, conf.dist_mode,              CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(109, conf.tv_bracket_value,	      CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(111, conf.bracket_type,		CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(133, conf.custom_timer_synch,     CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(156, conf.disable_overrides,      CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(144, conf.heading5,               CONF_DEF_VALUE, i:0, NULL), // Stereo Parameters Synch
    CONF_INFO( 101, conf.synch_enable,          CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 102, conf.synch_delay_enable,    CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 104, conf.synch_delay_range,     CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(120, conf.synch_user_delay,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(114, conf.synch_delay_coarse_value,CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(135, conf.add_synch_delays,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(145, conf.heading6,                CONF_DEF_VALUE, i:0, NULL), // Stereo Parameters General
    CONF_INFO(168, conf.compact,                 CONF_DEF_VALUE, i:0, NULL),
#if !defined(CAMERA_TX1)
    CONF_INFO(136, conf.remote_zoom_enable,      CONF_DEF_VALUE, i:0, NULL),
#else
    CONF_INFO(136, conf.remote_zoom_enable,      CONF_DEF_VALUE, i:1, NULL),
#endif
    CONF_INFO(137, conf.zoom_timeout,            CONF_DEF_VALUE, i:5, NULL),
    CONF_INFO(155, conf.extend_lens,             CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(125, conf.outline_mode,            CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(138, conf.disable_deviation_guideline,CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(195, conf.dc,                      CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(126, conf.stereo_mode,             CONF_DEF_VALUE, i:2, NULL),
    CONF_INFO(190, conf.edge_and_osd,            CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(128, conf.edge_overlay_thresh,      CONF_DEF_VALUE, i:40, NULL),  // name change
    CONF_INFO(174, conf.edge_overlay_color,      CONF_DEF_VALUE, cl:EDGE_OVERLAY_COLOUR, NULL),
    CONF_INFO(175, conf.edge_overlay_play,       CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(176, conf.edge_overlay_lock,       CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(177, conf.edge_overlay_zoom,       CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 87, conf.camera_position,        CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 88, conf.camera_orientation,     CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 89, conf.camera_rotation,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 194,conf.invert_playback,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 75, conf.stereo_spacing,         CONF_DEF_PTR,   ptr:&def_cam_spacing, NULL),
    CONF_INFO( 76, conf.stereo_deviation,       CONF_DEF_PTR,   ptr:&def_deviation_factor, NULL),
    CONF_INFO( 77, conf.minimum_dist_factor,    CONF_DEF_PTR,   ptr:&def_min_distance_factor, NULL),
    CONF_INFO( 82, conf.distance_setting,       CONF_DEF_PTR,   ptr:&def_distance_setting, NULL),
    CONF_INFO( 84, conf.focus_mode,             CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 79, conf.save_xml_file,          CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 83, conf.save_same_dir,          CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(146, conf.heading7,               CONF_DEF_VALUE, i:0, NULL), // Scripting
    CONF_INFO(  3, conf.script_shoot_delay,     CONF_DEF_VALUE, i:2, NULL),
    CONF_INFO( 99, conf.script_startup,         CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(189, conf.script_param_set,       CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(80,  conf.heading19,              CONF_DEF_VALUE, i:0, NULL),  // Stereo Slit Camera
    CONF_INFO(107, conf.raw_strip_mode,         CONF_DEF_VALUE, i:0, NULL),  // for slit camera
    CONF_INFO(118, conf.strip_images,           CONF_DEF_VALUE, i:1, NULL),  // for slit camera
    CONF_INFO(173, conf.strip_width,            CONF_DEF_VALUE, i:0, NULL),  // for slit camera
    CONF_INFO(78,  conf.strip_offset,           CONF_DEF_VALUE, i:50, NULL), // for slit camera
    CONF_INFO(163,  conf.rotation_distance,      CONF_DEF_VALUE, i:200, NULL),// for slit camera
    CONF_INFO(73, conf.heading18,               CONF_DEF_VALUE, i:0, NULL),  // Experimental Sunrise script
    CONF_INFO(178, conf.sunrise,                CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(179, conf.sunrise_tv96,           CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(180, conf.sunrise_repeats,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(181, conf.sunrise_shots,          CONF_DEF_VALUE, i:4,NULL),
    CONF_INFO(183, conf.sunrise_kf_1,           CONF_DEF_VALUE, i:4,NULL),
    CONF_INFO(184, conf.sunrise_kf_2,           CONF_DEF_VALUE, i:4,NULL),
    CONF_INFO(185, conf.sunrise_kf_3,           CONF_DEF_VALUE, i:4,NULL),
    CONF_INFO(186, conf.sunrise_tv96_2,         CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(187, conf.sunrise_tv96_3,         CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(182, conf.srb,                    CONF_DEF_VALUE, i:0, NULL), 

    CONF_INFO(158, conf.heading16,              CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/Digiscoping
    CONF_INFO(159, conf.digi_obj_diam,		CONF_DEF_VALUE, i:62, NULL),
    CONF_INFO(160, conf.digi_scope_fl,		CONF_DEF_VALUE, i:352, NULL),
    CONF_INFO(161, conf.digi_ep_fl,		      CONF_DEF_VALUE, i:25, NULL),
    CONF_INFO(162, conf.digi_zoom_setting,      CONF_DEF_VALUE, i:4, NULL),
    CONF_INFO(164, conf.digi_ref_dist,          CONF_DEF_VALUE, i:500, NULL),
    CONF_INFO(166, conf.digi_step,              CONF_DEF_VALUE, i:2, NULL),  // index to step-size table for data entry
    CONF_INFO(191, conf.heading17,              CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/Serial Comms
    CONF_INFO(100, conf.remote_enable,         CONF_DEF_VALUE, i:1, NULL),  // moved from Scripting
    CONF_INFO(198, conf.use_af_led,			CONF_DEF_VALUE, i:0, NULL),  // use af led for serial comms
    CONF_INFO(192, conf.bw,                     CONF_DEF_VALUE, i:10, NULL), // unit bit width 
    CONF_INFO(193, conf.md,                     CONF_DEF_VALUE, i:22, NULL), // tweak factor for creating msec delays
#ifdef OPT_SERIALIO
    CONF_INFO(35, conf.show_sio,                CONF_DEF_VALUE, i:1, NULL),  // show serial i/o errors and rate
#endif
    CONF_INFO(147, conf.heading8,               CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/OSD Parameters
    CONF_INFO( 14, conf.show_state,             CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 15, conf.show_values,            CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 58, conf.zoom_value,             CONF_DEF_VALUE, i:2, NULL),
    CONF_INFO(199, conf.zf_size,                CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 41, conf.show_clock,             CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(148, conf.heading9,               CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/OSD Parameters/Battery Parameters
    CONF_INFO(  8, conf.batt_volts_max,         CONF_DEF_PTR,   ptr:&def_batt_volts_max, NULL),
    CONF_INFO(  9, conf.batt_volts_min,         CONF_DEF_PTR,   ptr:&def_batt_volts_min, NULL),
    CONF_INFO( 10, conf.batt_step_25,           CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(149, conf.heading10,              CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/OSD Parameters/Grid Lines
    CONF_INFO( 68, conf.show_grid_lines,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 196,conf.enable_yaw_guideline,   CONF_DEF_VALUE, i:0, NULL),  // vertical or diagonal line to show angular rotation for panoramas
    CONF_INFO( 197,conf.yaw,                    CONF_DEF_VALUE, i:3, NULL),  // yaw angle shown in degrees
    CONF_INFO( 69, conf.grid_lines_file,        CONF_DEF_PTR,   ptr:"", conf_change_grid_file),
    CONF_INFO( 71, conf.grid_force_color,       CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 72, conf.grid_color,             CONF_DEF_VALUE, cl:MAKE_COLOR(COLOR_BG, COLOR_FG), NULL),
    CONF_INFO(150, conf.heading11,              CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/visual Settings
    CONF_INFO( 65, conf.font_cp,                CONF_DEF_VALUE, i:FONT_CP_WIN_1251, conf_change_font_cp),
    CONF_INFO(  7, conf.shortcut_lang,          CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 28, conf.osd_color,              CONF_DEF_VALUE, cl:MAKE_COLOR(COLOR_BG, COLOR_FG), NULL),
    CONF_INFO( 27, conf.histo_color,            CONF_DEF_VALUE, cl:MAKE_COLOR(COLOR_BG, COLOR_WHITE), NULL),
    CONF_INFO( 48, conf.histo_color2,           CONF_DEF_VALUE, cl:MAKE_COLOR(COLOR_RED, COLOR_WHITE), NULL),
    CONF_INFO( 55, conf.zebra_color,            CONF_DEF_VALUE, cl:MAKE_COLOR(COLOR_RED, COLOR_RED), NULL),
    CONF_INFO( 29, conf.batt_icon_color,        CONF_DEF_VALUE, cl:COLOR_WHITE, NULL),
    CONF_INFO( 30, conf.menu_color,             CONF_DEF_VALUE, cl:MAKE_COLOR(COLOR_BG, COLOR_FG), NULL),
    CONF_INFO( 31, conf.reader_color,           CONF_DEF_VALUE, cl:MAKE_COLOR(COLOR_GREY, COLOR_WHITE), NULL),
    CONF_INFO( 46, conf.reader_codepage,        CONF_DEF_VALUE, i:FONT_CP_WIN, NULL),
    CONF_INFO(151, conf.heading12,              CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/Miscellaneous/Text File Reader
    CONF_INFO( 61, conf.reader_wrap_by_words,   CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 43, conf.reader_autoscroll,      CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 44, conf.reader_autoscroll_delay,CONF_DEF_VALUE, i:5, NULL),
    CONF_INFO( 47, conf.splash_show,            CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 59, conf.use_zoom_mf,            CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 63, conf.alt_mode_button,        CONF_DEF_VALUE,
															#if defined (CAMERA_sx120is)
																i:KEY_FACE, conf_change_alt_mode_button),
															#else
                                                                                                i:KEY_PRINT, conf_change_alt_mode_button),
                                                                                          #endif
    CONF_INFO( 67, conf.alt_prevent_shutdown,   CONF_DEF_VALUE, i:ALT_PREVENT_SHUTDOWN_ALT,conf_update_prevent_shutdown),
    CONF_INFO(202, conf.lockout_time,           CONF_DEF_VALUE, i:0,NULL),
    CONF_INFO(119, conf.flash_dim_value,        CONF_DEF_VALUE, i:0,NULL),
    CONF_INFO(203, conf.slave_flash,            CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(207, conf.bright_screen,          CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(152, conf.heading13,              CONF_DEF_VALUE, i:0, NULL),  // Advanced Menu/Debug Parameters
    CONF_INFO( 34, conf.ns_enable_memdump,      CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(153, conf.heading14,              CONF_DEF_VALUE, i:0, NULL),  // Layout positions
    CONF_INFO( 21, conf.histo_pos,              CONF_DEF_PTR,   ptr:&def_histo_pos, NULL),
    CONF_INFO( 22, conf.dof_pos,                CONF_DEF_PTR,   ptr:&def_dof_pos, NULL),
    CONF_INFO( 23, conf.batt_icon_pos,          CONF_DEF_PTR,   ptr:&def_batt_icon_pos, NULL),
    CONF_INFO( 24, conf.batt_txt_pos,           CONF_DEF_PTR,   ptr:&def_batt_txt_pos , NULL),
    CONF_INFO( 25, conf.mode_state_pos,         CONF_DEF_PTR,   ptr:&def_mode_state_pos , NULL),
    CONF_INFO( 26, conf.values_pos,             CONF_DEF_PTR,   ptr:&def_values_pos , NULL),
    CONF_INFO( 200, conf.zoom_pos,              CONF_DEF_PTR,   ptr:&def_zoom_pos , NULL),
    CONF_INFO( 201, conf.increment_pos,         CONF_DEF_PTR,   ptr:&def_increment_pos , NULL),
    CONF_INFO( 39, conf.reader_pos,             CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(115, conf.stereo_osd_pos,         CONF_DEF_PTR,   ptr:&def_stereo_pos, NULL),
    CONF_INFO(116, conf.synch_pos,              CONF_DEF_PTR,   ptr:&def_synch_pos, NULL),
    CONF_INFO( 42, conf.clock_pos,              CONF_DEF_PTR,   ptr:&def_clock_pos , NULL),
    CONF_INFO(157, conf.overrides_pos,          CONF_DEF_PTR,   ptr:&def_overrides_pos , NULL),
#if defined(OPT_SERIALIO)
    CONF_INFO(36, conf.sio_pos,                CONF_DEF_PTR,   ptr:&def_sio_pos , NULL),
#endif
    CONF_INFO(154, conf.heading15,              CONF_DEF_VALUE, i:0, NULL), // System parameters (restricted access)
    CONF_INFO(  1, conf.show_osd,               CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(  5, conf.ubasic_vars,            CONF_DEF_PTR,   ptr:&def_ubasic_vars, NULL),
    CONF_INFO(  6, conf.script_file,            CONF_DEF_PTR,   ptr:"A/CHDK/SCRIPTS/time_lapse.bas", conf_change_script_file),
    CONF_INFO( 11, conf.batt_perc_show,         CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 12, conf.batt_volts_show,        CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO( 13, conf.batt_icon_show,         CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO( 38, conf.reader_file,            CONF_DEF_PTR,   ptr:"", conf_change_script_file),
    CONF_INFO( 45, conf.reader_rbf_file,        CONF_DEF_PTR,   ptr:"", NULL),
    CONF_INFO( 64, conf.lang_file,              CONF_DEF_PTR,   ptr:"", NULL),
    CONF_INFO( 66, conf.menu_rbf_file,          CONF_DEF_PTR,   ptr:"A/CHDK/FONTS/SDM.RBF", conf_change_menu_rbf_file),
 // CONF_INFO( 74, conf.show_stereo,            CONF_DEF_VALUE, i:1, NULL), // not needed, see gui.c
 // CONF_INFO( 85, conf.stereo_display_mode,    CONF_DEF_VALUE, i:1, NULL), // not needed, see gui.c
    CONF_INFO( 86, conf.layout_mode,            CONF_DEF_VALUE, i:0, NULL),
 // CONF_INFO( 95, conf.tv_bracket_value,		CONF_DEF_VALUE, i:0, NULL), // duplicated with 109 since introduction in SDM 1.40 !
    CONF_INFO( 103, conf.synch_delay_value,     CONF_DEF_VALUE, i:100, NULL),
 // CONF_INFO( 105, conf.rangefinder_near,      CONF_DEF_VALUE, i:2000, NULL),// not needed
 // CONF_INFO( 106, conf.rangefinder_far,       CONF_DEF_VALUE, i:65000, NULL), // not needed
 // CONF_INFO(117, conf.main_flash,             CONF_DEF_VALUE, i:1, NULL),// not needed
    CONF_INFO(127, conf.save_viewport,          CONF_DEF_VALUE, i:0, NULL),
 // CONF_INFO(131, conf.computed_distance,      CONF_DEF_VALUE, i:1000, NULL),// not needed
    CONF_INFO(132, conf.osd_page_index,         CONF_DEF_VALUE, i:2, NULL),
    CONF_INFO(123, conf.ricoh_ca1_mode,         CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(169, conf.cs,                     CONF_DEF_VALUE, i:1, NULL),
    CONF_INFO(170, conf.sdm_console,            CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(171, conf.num_lines,              CONF_DEF_VALUE, i:3, NULL),
    CONF_INFO(172, conf.line_length,            CONF_DEF_VALUE, i:25, NULL),
    CONF_INFO(204, conf.focus_pos,              CONF_DEF_VALUE, i:2000, NULL),
    CONF_INFO(205, conf.zoom_point,             CONF_DEF_VALUE, i:0, NULL),
    CONF_INFO(206, conf.fpd,                    CONF_DEF_VALUE, i:50, NULL),
    CONF_INFO(208, conf.script_set,             CONF_DEF_VALUE, i:0, NULL), // ******* SDM NG uses new values from that number
};


#define CONF_NUM (sizeof(conf_info)/sizeof(conf_info[0]))

//-------------------------------------------------------------------
static void conf_change_histo_mode() 
{
    histogram_set_mode(conf.histo_mode);
}

static void conf_change_histo_layout() 
{
    if (conf.histo_layout==OSD_HISTO_LAYOUT_Y || conf.histo_layout==OSD_HISTO_LAYOUT_Y_argb) 
    {
        histogram_set_main(HISTO_Y);
    } 
    else 
    {
        histogram_set_main(HISTO_RGB);
    }
}

static void conf_change_font_cp() 
{
    font_init();
    font_set(conf.font_cp);
}

static void conf_change_script_file() 
{
    script_load(conf.script_file,2);
}

static void conf_change_menu_rbf_file() 
{
    if (!rbf_load(conf.menu_rbf_file))
        rbf_load_from_8x16(current_font);
    rbf_set_codepage(FONT_CP_WIN);
}

static void conf_change_alt_mode_button() 
{
#if CAM_ADJUSTABLE_ALT_BUTTON
    kbd_set_alt_mode_key_mask(conf.alt_mode_button);
#elif defined (CAMERA_sx120is)
	conf.alt_mode_button = KEY_FACE;
#else
    conf.alt_mode_button = KEY_PRINT;
#endif
}

static void conf_change_grid_file() 
{
    grid_lines_load(conf.grid_lines_file);
}

static void conf_change_video_bitrate() 
{
    shooting_video_bitrate_change(conf.video_bitrate);
}

 /* 
 CHDK CS 986 Reyalp	
 Update the prevent display off/prevent shutdown based on current state 
 doesn't really belong in conf but not clear where else it should go 
 */ 
 void conf_update_prevent_shutdown(void) 
{ 
 if(conf.alt_prevent_shutdown == ALT_PREVENT_SHUTDOWN_ALWAYS  
 || (conf.alt_prevent_shutdown == ALT_PREVENT_SHUTDOWN_ALT && gui_get_mode() != GUI_MODE_NONE) 
 || (conf.alt_prevent_shutdown == ALT_PREVENT_SHUTDOWN_ALT_SCRIPT && state_kbd_script_run)) 
 { 
  disable_shutdown(); 
 } 
 else 
 { 
  enable_shutdown(); 
 } 
} 
//-------------------------------------------------------------------
static void conf_init_defaults() 
{
    // init some defaults values
    def_batt_volts_max = get_vbatt_max();
    def_batt_volts_min = get_vbatt_min();
#if defined(CAMERA_tx1)
    def_histo_pos.y = 240-HISTO_HEIGHT-40; 
    def_mode_state_pos.x=124;
    def_values_pos.x=0;
    def_clock_pos.x=360-5*FONT_WIDTH-2;
#elif defined(CAMERA_s95)
    def_histo_pos.y =145;
    def_mode_state_pos.x=294;
    def_values_pos.x=256;
    def_clock_pos.x=278;
#else
    def_histo_pos.y =145;
    def_mode_state_pos.x=334;
    def_values_pos.x=294;
    def_clock_pos.x=288;
#endif
    def_histo_pos.x=155;
    def_dof_pos.x = 40;
    def_dof_pos.y = 70;
    def_mode_state_pos.y=30;
    def_values_pos.y=160;
#if defined (CAMERA_g11) || defined (CAMERA_sx200is)|| defined(CAMERA_s90)|| defined(CAMERA_s95)|| defined(CAMERA_ixus95)|| defined(CAMERA_ixus105)
    def_batt_icon_pos.x = 10;
    def_batt_icon_pos.y = 1;
    def_batt_txt_pos.x=4;
    def_batt_txt_pos.y=20;
    def_synch_pos.x = 100;
    def_synch_pos.y = 0;
#else
    def_batt_icon_pos.x = 40;
    def_batt_icon_pos.y = 1;
    def_batt_txt_pos.x=74;
    def_batt_txt_pos.y=0;
    def_synch_pos.x = 160;
    def_synch_pos.y = 0;
#endif
    def_zoom_pos.x=120;
    def_zoom_pos.y=80;
    def_increment_pos.x=0;
    def_increment_pos.y=0;
    def_clock_pos.y=30;
#if defined(OPT_SERIALIO)
    def_sio_pos.x=288;
    def_sio_pos.y=50;
#endif
// *****************************************************
    def_cam_spacing = 70;
    def_deviation_factor = 25;
    def_min_distance_factor = 15;
    def_distance_setting = 3000;
    def_stereo_pos.x = 25;
    def_stereo_pos.y = 20;
    def_overrides_pos.x = 0;
    def_overrides_pos.y = 0;
    def_manual_far = 65000;
// *****************************************************
}

//-------------------------------------------------------------------
void conf_load_defaults() 
{
    register int i;

    for (i=0; i<CONF_NUM; ++i) 
    {
        switch (conf_info[i].type) 
        {
            case CONF_DEF_VALUE:
                memcpy(conf_info[i].var, &(conf_info[i].i), conf_info[i].size);
                break;
            case CONF_DEF_PTR:
                memcpy(conf_info[i].var, conf_info[i].ptr, conf_info[i].size);
                break;
        }

        if (conf_info[i].func) 
        {
            conf_info[i].func();
        }

    }
}

//-------------------------------------------------------------------
void conf_save() 
{
    static const long t=CONF_MAGICK_VALUE;
    register int i;
    int fd;
    static char buf[sizeof(t)+CONF_NUM*(sizeof(conf_info[0].id)+sizeof(conf_info[0].size))+sizeof(conf)];
    char *p=buf;

    static char fn[17];
    if(((int)CAM_BITMAP_PALETTE)!=1)
    sprintf(fn,"A/CHDK/CHDK.CFG%1d",(int)CAM_BITMAP_PALETTE);
    else sprintf(fn,"A/CHDK/CHDK.CFG");
    fd = open(fn, O_WRONLY|O_CREAT, 0777);
 
    if (fd>=0)
    {
        memcpy(p, &t, sizeof(t));
        p+=sizeof(t);
        for (i=0; i<CONF_NUM; ++i) 
        {
            memcpy(p, &(conf_info[i].id), sizeof(conf_info[i].id));
            p+=sizeof(conf_info[i].id);
            memcpy(p, &(conf_info[i].size), sizeof(conf_info[i].size));
            p+=sizeof(conf_info[i].size);
            memcpy(p, conf_info[i].var, conf_info[i].size);
            p+=conf_info[i].size;
        }

        write(fd, buf, p-buf);
        close(fd);
    }
}

//-------------------------------------------------------------------
void conf_restore() 
{
    int fd, rcnt, i;
    long t;
    unsigned short id, size;
    void *ptr;
    conf_init_defaults();
    conf_load_defaults();

    static char fn[17];
    if(((int)CAM_BITMAP_PALETTE)!=1)
    sprintf(fn,"A/CHDK/CHDK.CFG%1d",(int)CAM_BITMAP_PALETTE);
    else sprintf(fn,"A/CHDK/CHDK.CFG");

    fd = open(fn, O_RDONLY, 0777);
    if (fd>=0)
     {
        // read magick value
        rcnt = read(fd, &t, sizeof(t));
     
          if (rcnt==sizeof(t))
            {
            while (1) 
               {
                rcnt = read(fd, &id, sizeof(id));
                if (rcnt!=sizeof(id)) break;

                rcnt = read(fd, &size, sizeof(size));
                if (rcnt!=sizeof(size)) break;

                for (i=0; i<CONF_NUM; ++i) 
                    {
                    if (conf_info[i].id==id && conf_info[i].size==size) 
                       {
                        ptr=umalloc(size);
                        if (ptr) 
                         {
                            rcnt = read(fd, ptr, size);
                            if (rcnt == size) 
                              {
                               memcpy(conf_info[i].var, ptr, size);

                               if (conf_info[i].func) 
                                {
                                   conf_info[i].func();
                                }
                              }
                            ufree(ptr);
                         }
                        break;
                       }
                    }
                if (i == CONF_NUM) 
                 { //unknown id, just skip data
                  lseek(fd, size, SEEK_CUR);
                 }

              }// _while
           } // _rcnt
	close(fd);
    }// fd>=0

}

//-------------------------------------------------------------------
