#include "stdlib.h"
#include "platform.h"
#include "core.h"
#include "keyboard.h"
#include "conf.h"
#include "font.h"
#include "lang.h"
#include "ubasic.h"
#include "gui.h"
#include "draw_palette.h"
#include "gui_lang.h"
#include "kbd.h"

// -------------------------------------------------------------------
static volatile int full_palette;
static color cl;
static volatile char counter;
static int palette_mode;
static void (*palette_on_select)(color clr);
static int gui_palette_redraw;
#define COUNTER_N 100

// -------------------------------------------------------------------
void gui_palette_init(int mode, color st_color, void (*on_select)(color clr)) {
    full_palette = (mode!=PALETTE_MODE_SELECT);
    cl = st_color;
    palette_mode = mode;
    palette_on_select = on_select;
    counter = COUNTER_N;
    gui_palette_redraw = 1;
}

// -------------------------------------------------------------------
void gui_palette_kbd_process() 
{
    switch (get_vkey(kbd_get_autoclicked_key())) {
    case KEY_DOWN:
    	if (!full_palette) {
            cl = ((((cl>>4)+1)<<4)|(cl&0x0f))&0xFF;
            gui_palette_redraw = 1;
        }
        break;
    case KEY_UP:
    	if (!full_palette) {
            cl = ((((cl>>4)-1)<<4)|(cl&0x0f))&0xFF;
            gui_palette_redraw = 1;
        }
        break;
    case KEY_LEFT:
    	if (!full_palette) {
            cl = ((((cl&0x0f)-1)&0x0f)|(cl&0xf0))&0xFF;
            gui_palette_redraw = 1;
        }
        break;
    case KEY_RIGHT:
    	if (!full_palette) {
            cl = ((((cl&0x0f)+1)&0x0f)|(cl&0xf0))&0xFF;
            gui_palette_redraw = 1;
        }
        break;
    case KEY_SET:
        if (palette_mode!=PALETTE_MODE_SELECT) {
            full_palette = !full_palette;
            counter = 0;
            gui_palette_redraw = 1;
        } else {
            if (palette_on_select) 
                palette_on_select(cl);
           gui_set_mode(GUI_MODE_MENU);
        }
        break;
    }
}

// -------------------------------------------------------------------
void gui_palette_draw() {
    unsigned int x, y;
    char f=0;
    color c;
    static char buf[64];

#if defined(CAMERA_ixus980)|| defined(CAMERA_ixus100)
 int  sw = vid_get_bitmap_width()-360;
#elif defined(CAMERA_tx1)
 int  sw = vid_get_bitmap_width()-120;
#else
 int  sw = vid_get_bitmap_width();
#endif

if(conf.camera_orientation==1)sw-=120;                  // width is really screen height
    switch (palette_mode) 
     {
        case PALETTE_MODE_DEFAULT:
            if (full_palette) 
               {
                if (!counter || counter == COUNTER_N) 
                   {
                    for (y=0; y<screen_height; ++y) 
                       {
                        for (x=0; x<sw; ++x) 
                          {
                            c = ((y*16/screen_height)<<4)|(x*16/sw);
                            draw_pixel(x, y, c);
                          }
                        }
                      if (counter) 
                       {
                        draw_txt_string(6, 7, lang_str(LANG_PALETTE_TEXT_1), MAKE_COLOR(COLOR_BLACK, COLOR_WHITE));
                        draw_txt_string(6, 8, lang_str(LANG_PALETTE_TEXT_2), MAKE_COLOR(COLOR_BLACK, COLOR_WHITE));
                       }
                     }
                    if (counter)
                    --counter;
                   f=1;
               }//_full_palette
            
            if (!full_palette) 
              {
//  ----------------   Grey header with hex value   ---------------------------------------------------
                sprintf(buf, " %s:0x%02X   ", lang_str(LANG_PALETTE_TEXT_COLOR), cl);
                draw_txt_string(0, 0, buf, MAKE_COLOR(COLOR_BLACK, COLOR_WHITE));
                draw_filled_rect(20, 20, sw-20, screen_height-20, MAKE_COLOR(cl, COLOR_WHITE));
// -------------------------------------------------------------------------------------------------------------
              }
            break;
        case PALETTE_MODE_SELECT:
            if (gui_palette_redraw) 
              {
                #define CELL_SIZE   13
                #define BORDER_SIZE 8
                #define CELL_ZOOM   5
                draw_string(8*FONT_WIDTH, 0, "Arrow keys to change  ", MAKE_COLOR(COLOR_BLACK, COLOR_WHITE),1);
                sprintf(buf, " %s:0x%02X ", lang_str(LANG_PALETTE_TEXT_COLOR), cl);
                draw_string(0, 0, buf, MAKE_COLOR(COLOR_BLACK, COLOR_WHITE),1);
// ---------------------------------- Draw main palette ----------------------------------------
              for (y=0; y<16; ++y) 
                    {
                     for (x=0; x<16; ++x) 
                      {
                       c = (y<<4)|x;
                       draw_filled_rect(BORDER_SIZE+x*CELL_SIZE, BORDER_SIZE+y*CELL_SIZE+FONT_HEIGHT, BORDER_SIZE+(x+1)*CELL_SIZE, BORDER_SIZE+(y+1)*CELL_SIZE+FONT_HEIGHT, MAKE_COLOR(c, COLOR_BLACK));
                      }
                    }
// ---------------------------------------------------------------------------------------------
                draw_filled_rect(0, FONT_HEIGHT, sw-1, FONT_HEIGHT+BORDER_SIZE-1, MAKE_COLOR(COLOR_GREY, COLOR_GREY)); //top
                draw_filled_rect(0, FONT_HEIGHT+BORDER_SIZE+16*CELL_SIZE+1, sw-1, FONT_HEIGHT+BORDER_SIZE+16*CELL_SIZE+BORDER_SIZE, MAKE_COLOR(COLOR_GREY, COLOR_GREY)); //bottom
                draw_filled_rect(0, FONT_HEIGHT+BORDER_SIZE, BORDER_SIZE-1, FONT_HEIGHT+BORDER_SIZE+16*CELL_SIZE, MAKE_COLOR(COLOR_GREY, COLOR_GREY)); //left
                draw_filled_rect(BORDER_SIZE+16*CELL_SIZE+1, FONT_HEIGHT+BORDER_SIZE, BORDER_SIZE+16*CELL_SIZE+BORDER_SIZE, FONT_HEIGHT+BORDER_SIZE+16*CELL_SIZE, MAKE_COLOR(COLOR_GREY, COLOR_GREY)); //middle
                draw_filled_rect(sw-BORDER_SIZE, FONT_HEIGHT+BORDER_SIZE, sw-1, FONT_HEIGHT+BORDER_SIZE+16*CELL_SIZE, MAKE_COLOR(COLOR_GREY, COLOR_GREY)); //right

                y=(cl>>4)&0x0F; x=cl&0x0F;
                draw_filled_rect(BORDER_SIZE+x*CELL_SIZE-CELL_ZOOM, FONT_HEIGHT+BORDER_SIZE+y*CELL_SIZE-CELL_ZOOM, BORDER_SIZE+(x+1)*CELL_SIZE+CELL_ZOOM, FONT_HEIGHT+BORDER_SIZE+(y+1)*CELL_SIZE+CELL_ZOOM, MAKE_COLOR(cl, COLOR_RED)); //selected
                draw_rect(BORDER_SIZE+x*CELL_SIZE-CELL_ZOOM-1, FONT_HEIGHT+BORDER_SIZE+y*CELL_SIZE-CELL_ZOOM-1, BORDER_SIZE+(x+1)*CELL_SIZE+CELL_ZOOM+1, FONT_HEIGHT+BORDER_SIZE+(y+1)*CELL_SIZE+CELL_ZOOM+1, COLOR_RED); //selected
// large rectangle to display chosen colour
                draw_filled_rect(BORDER_SIZE+16*CELL_SIZE+BORDER_SIZE, FONT_HEIGHT+BORDER_SIZE,sw-BORDER_SIZE-1, FONT_HEIGHT+BORDER_SIZE+16*CELL_SIZE, MAKE_COLOR(cl, COLOR_WHITE));

                gui_palette_redraw = 0;
            }
            break;
    }
}

// -------------------------------------------------------------------
static char*    frame_buffer[2];
unsigned int    screen_width=0, screen_height=0, screen_size=0;
unsigned int    screen_buffer_width=0, screen_buffer_height=0, screen_buffer_size=0;
void            (*draw_pixel_proc)(coord x,coord y,color cl);

// ---------------------------------------------------------------------
 static void draw_pixel_std(coord x,coord y,color cl)
{
 unsigned int offsetx;
#if defined (CAMERA_ixus980) || defined(CAMERA_ixus100) || defined(CAMERA_sx120is)                      
    frame_buffer[0][y * screen_buffer_width + (x<<1)] = frame_buffer[1][y * screen_buffer_width + (x<<1)] = cl & 0xff;     // draw two pixels side-by-side
    frame_buffer[0][y * screen_buffer_width + (x<<1)+1] = frame_buffer[1][y * screen_buffer_width + (x<<1)+1] = cl & 0xff;

#elif defined (CAMERA_sx200is)|| defined(CAMERA_g11)|| defined(CAMERA_s90)|| defined(CAMERA_s95)|| defined(CAMERA_ixus95)|| defined(CAMERA_ixus105)
     for(offsetx=(((x<<3)+x)>>2);offsetx<=((((x+1)<<3)+x)>>2);offsetx++)                                                         // draw two or three pixels
     {
      frame_buffer[0][y*screen_buffer_width+offsetx]=frame_buffer[1][y*screen_buffer_width+offsetx]=cl & 0xff;
     }
#else  
       frame_buffer[0][y*screen_buffer_width+x] = frame_buffer[1][y*screen_buffer_width+x] = cl & 0xff;                                                                                                                   // draw one pixel
 //   frame_buffer[0][y*vid_get_bitmap_width()+x] = frame_buffer[1][y*vid_get_bitmap_width()+x] = cl & 0xff;
#endif
}
// --------------------------------------------------------------------- 
 void draw_pixel_inverted(coord x,coord y,color cl)
{
 #if defined (CAMERA_ixus980) || defined(CAMERA_ixus100)
 draw_pixel_std(screen_width/2-x,screen_height-y,cl);
#else
 draw_pixel_std(screen_width-x,screen_height-y,cl);
#endif
}
// --------------------------------------------------------------------- 
 void draw_pixel_double(coord x,coord y,color cl)
{
 if(conf.camera_orientation==2)x=360-x-1;                                                                                 // inverted
 else if ((conf.camera_orientation==1)&& !conf.camera_rotation)x=360-y-1;                                               // vertical left
 else if ((conf.camera_orientation==1)&& conf.camera_rotation)y=screen_height-x-1;                                      // vertical right

 frame_buffer[0][y*screen_buffer_width+(x<<1)]=frame_buffer[1][y*screen_buffer_width+(x<<1)]=cl & 0xff;
 frame_buffer[0][y*screen_buffer_width+(x<<1)+1]=frame_buffer[1][y*screen_buffer_width+(x<<1)+1]=cl & 0xff;
}
// --------------------------------------------------------------------- 
 void draw_pixel_rotated_left(coord x,coord y,color cl)
{
 #if defined (CAMERA_ixus980) || defined(CAMERA_ixus100)
 draw_pixel_std(screen_width/2-y-1,x,cl);
#else
 draw_pixel_std(screen_width-y-1,x,cl);
#endif
}
// ---------------------------------------------------------------------
 void draw_pixel_rotated_right(coord x,coord y,color cl)
{
 draw_pixel_std(y,screen_height-x-1,cl);
}
// -------------------------------------------------------------------
void draw_pixel(coord x, coord y, color cl) 
{
#if !defined(CAMERA_sx1)
    if (x >= 360 || y >= 240) return;
#else
    if (x >= 480 || y >= 240) return;
#endif
   else draw_pixel_proc(x,y,cl);
}
// ------------------------------------------------------------------
color draw_get_pixel(coord x, coord y) 
{
 if (x >= screen_width || y >= screen_height) return 0;
#if defined (CAMERA_sx200is)|| defined(CAMERA_g11)|| defined(CAMERA_s90)|| defined(CAMERA_s95)|| defined(CAMERA_ixus95)|| defined(CAMERA_ixus105)
 return frame_buffer[0][y * screen_buffer_width + (((x<<3)+x) >>2)];
#else
 return frame_buffer[0][y * screen_width + x];
#endif
}
// -------------------------------------------------------------------
 void draw_set_draw_proc(void (*pixel_proc)(coord x,coord y,color cl)) 
{
    draw_pixel_proc = (pixel_proc)?pixel_proc:draw_pixel_std;
}
// --------------------------------------------------------------------

void draw_init() {
    register int i;

    screen_width = vid_get_bitmap_width();
    screen_height  = vid_get_bitmap_height();
    screen_size = screen_width * screen_height;
    screen_buffer_width = vid_get_bitmap_buffer_width();
    screen_buffer_height = vid_get_bitmap_buffer_height();
    screen_buffer_size = screen_buffer_width * screen_buffer_height;
    frame_buffer[0] = vid_get_bitmap_fb();
    frame_buffer[1] = frame_buffer[0] + screen_buffer_size;
    draw_set_draw_proc(NULL);                            // draw_pixel_std
}

// -------------------------------------------------------------------
static inline int abs(int v) {
    return ((v<0)?-v:v);
}
#define swap(v1, v2)   {v1^=v2; v2^=v1; v1^=v2;}
// -------------------------------------------------------------------
void draw_line(coord x1, coord y1, coord x2, coord y2, color cl) {
     unsigned char steep = abs(y2 - y1) > abs(x2 - x1);
     if (steep) {
         swap(x1, y1);
         swap(x2, y2);
     }
     if (x1 > x2) {
         swap(x1, x2);
         swap(y1, y2);
     }
     int deltax = x2 - x1;
     int deltay = abs(y2 - y1);
     int error = 0;
     int y = y1;
     int ystep = (y1 < y2)?1:-1;
     int x;
     for (x=x1; x<=x2; ++x) {
         if (steep) draw_pixel(y, x, cl);
         else draw_pixel(x, y, cl);
         error += deltay;
         if ((error<<1) >= deltax) {
             y += ystep;
             error -= deltax;
         }
     }
}
// -------------------------------------------------------------------
void draw_rect(coord x1, coord y1, coord x2, coord y2, color cl) 
{
    unsigned int xMin, yMin, xMax, yMax, x, y;
    if (x1>x2) 
    {
    	xMax=x1; xMin=x2;
    } else 
    {
    	xMin=x1; xMax=x2;
    }
    if (y1>y2) 
    {
    	yMax=y1; yMin=y2;
    } else 
    {
    	yMin=y1; yMax=y2;
    }
#if !defined(CAMERA_sx200is)&& !defined(CAMERA_g11)&& !defined(CAMERA_s90)&& !defined(CAMERA_s95)&& !defined(CAMERA_ixus95)&& !defined(CAMERA_ixus105)
    if (xMax>=360) xMax=360-1;
    if (xMin>=360) xMin=360-1;
#else
    if (xMax>=320) xMax=320-1;
    if (xMin>=320) xMin=320-1;
#endif
    if (yMax>=screen_height) yMax=screen_height-1;
    if (yMin>=screen_height) yMin=screen_height-1;

    for (y=yMin; y<=yMax; ++y) 
    {
	draw_pixel(xMin, y, cl & 0xff);
	draw_pixel(xMax, y, cl & 0xff);
    }

    for (x=xMin+1; x<=xMax-1; ++x) 
    {
        draw_pixel(x, yMin, cl & 0xff);
        draw_pixel(x, yMax, cl & 0xff);
    }
}

// -------------------------------------------------------------------
void draw_filled_rect(coord x1, coord y1, coord x2, coord y2, color cl) 
{
    unsigned int xMin, yMin, xMax, yMax, x, y;
    if (x1>x2) {
    	xMax=x1; xMin=x2;
    } else {
    	xMin=x1; xMax=x2;
    }
    if (y1>y2) {
    	yMax=y1; yMin=y2;
    } else {
    	yMin=y1; yMax=y2;
    }
#if !defined(CAMERA_sx200is)&& !defined(CAMERA_g11)&& !defined(CAMERA_s90)&& !defined(CAMERA_s95)&& !defined(CAMERA_ixus95)&& !defined(CAMERA_ixus105)
    if (xMax>=360) xMax=360-1;
    if (xMin>=360) xMin=360-1;
#else
    if (xMax>=320) xMax=320-1;
    if (xMin>=320) xMin=320-1;
#endif

    if (yMax>=screen_height) yMax=screen_height-1;
    if (yMin>=screen_height) yMin=screen_height-1;

    draw_rect(x1, y1, x2, y2, cl);
    for (y=yMin+1; y<=yMax-1; ++y) {
	for (x=xMin+1; x<=xMax-1; ++x) {
	    draw_pixel(x, y, cl>>8);
	}
    }
}


// -------------------------------------------------------------------
void draw_string(coord x, coord y, const char *s, color cl,int m) 
{
    while(*s)
     {
	draw_char(x, y, *s, cl,m);
	s++;
	x+=m*FONT_WIDTH;
	if ((x>=screen_width) && (*s)){
	    draw_char(x-m*FONT_WIDTH,y, '>', cl,m);
	    break;
	}
    }
}

// -------------------------------------------------------------------


void draw_char(coord x, coord y, const char ch, color cl, int m) 
{
    const unsigned char *sym = (unsigned char*)current_font +
	    ((const unsigned char)ch)*FONT_HEIGHT;  
                  
    int i, ii,j,jj;
  if(m<0)m=0;
  if(m>16)m=16;

    for (i=0; i<FONT_HEIGHT; i++)                                                         // source font row                                  
    {
	for (ii=0; ii<FONT_WIDTH; ii++)                                                     // source font column                             
      {
        for(j=0;j<m;j++)                                                                  // repeat pixel vertically
         {
           for(jj=0;jj<m;jj++)                                                            // repeat pixel horizontally
           {
            draw_pixel(x+jj+(m*ii),y+j+(m*i), (sym[i] & (0x80>>ii))? cl&0xff : cl>>8);
           }  
         }
      } // _next source column
    }   // _next source row
}

// -------------------------------------------------------------------
void draw_big_string(coord x, coord y, const char *s, color cl) 
{
    while(*s)
    {
	draw_char(x, y, *s, cl,2);
	s++;
	x+=2*FONT_WIDTH;
	if ((x>=screen_width) && (*s))
        {
	    draw_char(x-2*FONT_WIDTH,y, '>', cl,2);
	    break;
	  }
    }
}
// -------------------------------------------------------------------

void draw_big_string_centre_len(int x, int y, int len, const char *str, color cl) 
{
    int l=0,i,j, yy;
    const char *s=str;

     j = (len-(strlen(s)*BIG_FONT_WIDTH))>>1;   // leading space

  if(j>1)
     {      
       for (i=0; i<j;i++)                       // for each column
         {
         for (yy=y; yy<y+BIG_FONT_HEIGHT; ++yy) // for each row of column
           {
            draw_pixel(x+i, yy, cl>>8);         // draw bg colour
           }
         }
      }

        draw_big_string(j+x,y,s,cl);            // text
        l = j+x+(strlen(s)*BIG_FONT_WIDTH);     // end of text                            

          for (; l<len+x; ++l)                      // starting at 'l', draw bg color until end
           {
            for (yy=y; yy<y+BIG_FONT_HEIGHT; ++yy) // for each row of column
             {
              draw_pixel(l, yy, cl>>8);          // draw bg color
             }
           }
}
// -------------------------------------------------------------------
void draw_txt_rect(coord col, coord row, unsigned int length, unsigned int height, color cl) 
{
    draw_rect(col*FONT_WIDTH, row*FONT_HEIGHT, (col+length)*FONT_WIDTH-1, (row+height)*FONT_HEIGHT-1, cl);
}

// -------------------------------------------------------------------
void draw_txt_rect_exp(coord col, coord row, unsigned int length, unsigned int height, unsigned int exp, color cl) {
    draw_rect(col*FONT_WIDTH-exp, row*FONT_HEIGHT-exp, (col+length)*FONT_WIDTH-1+exp, (row+height)*FONT_HEIGHT-1+exp, cl);
}

// -------------------------------------------------------------------
void draw_txt_filled_rect(coord col, coord row, unsigned int length, unsigned int height, color cl) 
{
    draw_filled_rect(col*FONT_WIDTH, row*FONT_HEIGHT, (col+length)*FONT_WIDTH-1, (row+height)*FONT_HEIGHT-1, cl);
}

// -------------------------------------------------------------------
void draw_txt_filled_rect_exp(coord col, coord row, unsigned int length, unsigned int height, unsigned int exp, color cl) 
{
    draw_filled_rect(col*FONT_WIDTH-exp, row*FONT_HEIGHT-exp, (col+length)*FONT_WIDTH-1+exp, (row+height)*FONT_HEIGHT-1+exp, cl);
}

// -------------------------------------------------------------------
void draw_txt_string(coord col, coord row, const char *str, color cl) 
{
    if((*str == '!') || (*str=='#') || (*str==' ')) str++;     // ignore colour format character
    draw_string(col*FONT_WIDTH, row*FONT_HEIGHT, str, cl,1);
}

// -------------------------------------------------------------------
void draw_txt_char(coord col, coord row, const char ch, color cl) 
{
    draw_char(col*FONT_WIDTH, row*FONT_HEIGHT, ch, cl,1);
}

// -------------------------------------------------------------------
void draw_clear() 
{
    register unsigned int i;
    for (i=0; i<screen_buffer_size; ++i)
        frame_buffer[0][i] = frame_buffer[1][i] = COLOR_TRANSPARENT;
}

// -------------------------------------------------------------------
void draw_restore() 
{
    vid_bitmap_refresh();;
}

// -------------------------------------------------------------------
void draw_fill(coord x, coord y, color cl_fill, color cl_bound) 
{
   if (draw_get_pixel(x, y) != cl_bound && draw_get_pixel(x, y) != cl_fill) {
       draw_pixel(x, y, cl_fill);

       draw_fill((x+1), y, cl_fill,cl_bound);
       draw_fill((x-1), y, cl_fill,cl_bound);
       draw_fill(x, (y+1), cl_fill,cl_bound);
       draw_fill(x, (y-1), cl_fill,cl_bound);
   }
}

// -------------------------------------------------------------------
void draw_circle(coord x, coord y, const unsigned int r, color cl) 
{
    int dx = 0;
    int dy = r;
    int p=(3-(r<<1));

    do {
        draw_pixel((x+dx),(y+dy),cl);
        draw_pixel((x+dy),(y+dx),cl);
        draw_pixel((x+dy),(y-dx),cl);
        draw_pixel((x+dx),(y-dy),cl);
        draw_pixel((x-dx),(y-dy),cl);
        draw_pixel((x-dy),(y-dx),cl);
        draw_pixel((x-dy),(y+dx),cl);
        draw_pixel((x-dx),(y+dy),cl);

        ++dx;

        if (p<0) 
            p += ((dx<<2)+6);
        else {
            --dy;
            p += (((dx-dy)<<2)+10);
        }
    } while (dx<=dy);
}

// -------------------------------------------------------------------
void draw_ellipse(coord xc, coord yc, unsigned int a, unsigned int b, color cl) 
{
    int x = 0, y = b;
    long a2 = (long)a*a, b2 = (long)b*b;
    long crit1 = -((a2>>2) + (a&1) + b2);
    long crit2 = -((b2>>2) + (b&1) + a2);
    long crit3 = -((b2>>2) + (b&1));
    long t = -a2*y;
    long dxt = b2*x*2, dyt = -2*a2*y;
    long d2xt = b2*2, d2yt = a2*2;

    while (y>=0 && x<=a) {
        draw_pixel(xc+x, yc+y, cl);
        if (x!=0 || y!=0)
            draw_pixel(xc-x, yc-y, cl);
        if (x!=0 && y!=0) {
            draw_pixel(xc+x, yc-y, cl);
            draw_pixel(xc-x, yc+y, cl);
        }
        if (t + b2*x <= crit1 || t + a2*y <= crit3)
            ++x, dxt += d2xt, t += dxt;
        else if (t - a2*y > crit2)
            --y, dyt += d2yt, t += dyt;
        else {
            ++x, dxt += d2xt, t += dxt;
            --y, dyt += d2yt, t += dyt;
        }
   }
}

// -------------------------------------------------------------------
void draw_filled_ellipse(coord xc, coord yc, unsigned int a, unsigned int b, color cl) 
{
    int x = 0, y = b;
    int rx = x, ry = y;
    unsigned int width = 1;
    unsigned int height = 1;
    long a2 = (long)a*a, b2 = (long)b*b;
    long crit1 = -((a2>>2) + (a&1) + b2);
    long crit2 = -((b2>>2) + (b&1) + a2);
    long crit3 = -((b2>>2) + (b&1));
    long t = -a2*y;
    long dxt = 2*b2*x, dyt = -2*a2*y;
    long d2xt = 2*b2, d2yt = 2*a2;
    
    color cl_fill = ((cl >> 8) & 0xff) | (cl & 0xff00);

    if (b == 0) {
        draw_filled_rect(xc-a, yc, (a<<1)+1, 1, cl_fill);
    } else {
        while (y>=0 && x<=a) {
            if (t + b2*x <= crit1 || t + a2*y <= crit3) {
                if (height == 1)
                    ; //draw nothing 
                else if (ry*2+1 > (height-1)*2) {
                    draw_filled_rect(xc-rx, yc-ry, xc-rx+width-1, yc-ry+(height-1)-1, cl_fill);
                    draw_filled_rect(xc-rx, yc+ry+1, xc-rx+width-1, yc+ry+1+(1-height)-1, cl_fill);
                    ry -= height-1;
                    height = 1;
                }
                else {
                    draw_filled_rect(xc-rx, yc-ry, xc-rx+width-1, yc-ry+(ry*2+1)-1, cl_fill);
                    ry -= ry;
                    height = 1;
                }
                ++x, dxt += d2xt, t += dxt;
                rx++;
                width += 2;
            }
            else if (t - a2*y > crit2) {
                --y, dyt += d2yt, t += dyt;
                height++;
            }
            else {
                if (ry*2+1 > height*2) {
                    draw_filled_rect(xc-rx, yc-ry, xc-rx+width-1, yc-ry+height-1, cl_fill);
                    draw_filled_rect(xc-rx, yc+ry+1, xc-rx+width-1, yc+ry+1-height-1, cl_fill);
                }
                else {
                    draw_filled_rect(xc-rx, yc-ry, xc-rx+width-1, yc-ry+(ry*2+1)-1, cl_fill);
                }
                ++x, dxt += d2xt, t += dxt;
                --y, dyt += d2yt, t += dyt;
                rx++;
                width += 2;
                ry -= height;
                height = 1;
            }
        }

        if (ry > height) {
            draw_filled_rect(xc-rx, yc-ry, xc-rx+width-1, yc-ry+height-1, cl_fill);
            draw_filled_rect(xc-rx, yc+ry+1, xc-rx+width-1, yc+ry+1-height-1, cl_fill);
        }
        else {
            draw_filled_rect(xc-rx, yc-ry, xc-rx+width-1, yc-ry+(ry*2+1)-1, cl_fill);
        }
    }
}

