#include "stdlib.h"
#include "platform.h"
#include "core.h"
#include "keyboard.h"
#include "conf.h"
#include "camera.h"
#include "ubasic.h"
#include "font.h"
#include "lang.h"
#include "gui.h"
#include "gui_lang.h"
#include "draw_palette.h"
#include "gui_menu.h"
#include "mbox_popup.h"
#include "gui_read.h"
#include "gui_fselect.h"
#include "batt_grid.h"
#include "gui_osd.h"
#include "edgeoverlay.h"
#include "histogram.h"
#include "script.h"
#include "motion_detector.h"
#include "gui_sdm_config.h"
#include "raw.h"
#include "math.h"

extern const int ovr[66];
#ifdef OPT_SERIALIO
extern int gShotsRemaining;
extern int gSerialIORemoteShootingOn,kbd_blocked;
extern int ttyMasterSlave;
#endif
extern char cfg_name[100];
extern char cfg_set_name[100];
char* MX3 = "57";
char* FINGALO = "128";
char* ALLBEST = "16";
char *CS = "788";
extern int sw_delay;
extern int current_focus,pw,edge_count;
extern int rf_focus;              // optimum focus for near far distances
static int digi_mode;
int fpd;
int sfo=0;                      // set focus-override when exiting ALT mode after manual focus with certain cameras
int subject_near;               // subject nearpoint set by half-press at max telephoto
int subject_far;                // subject farpoint set by half-press at max telephoto
int  current_rotation = 0;
int  current_orientation = 0;
int playback = 0; // In playback determines if images browsed or movie control panel .
int gDebug=0;            	
#if CAM_MULTIPART
int cp = 1;       // current partition, '1' or '2'
#endif

#define SPLASH_TIME               20
 
//forward declarations  --------------------------------------------

static int osd_pages[]={1,0,2,0,3,0,4,0,5,0,6,0,7,0,8,0,9,0,10,0}; // added panorama data
#define OSD_PAGES (sizeof(osd_pages)/sizeof(osd_pages[0]))
int afl = 0;                                      // auto-focus lock

const char* tl_modes[]={
                        "Single shot mode",
                        "Tv Continuous alternate",
                        "Tv timer ",
                        "Cont equal-step  ",
                        "Timer equal step ",
                        "Cont auto stack  ",
                        "Timer auto stack ",
                        "Continuous digi  ",
                        "Timer digiscope  ",
                        "Cont equal Tv    ",
                        "Cont auto Tv     ",
                        "Cont digi Tv     ",
                        "Timer equal Tv   ",
                        "Timer auto Tv    ",
                        "Timer digi Tv    ",
                       };


long myav;
static void gui_draw_osd();
extern int myhyp,previous_focus,inf_focus;
static char* my_version_number = "1.85";
static void gui_draw_spm_splash();


//Menu procs
//-------------------------------------------------------------------
static void gui_draw_fselect(int arg);
static void gui_show_build_info(int arg);
static void gui_draw_osd_le(int arg);
static void gui_load_script(int arg);
static void gui_draw_load_menu_rbf(int arg);
static void gui_draw_load_rbf(int arg);
static void gui_draw_sdm_configuration(int arg);
static void gui_draw_shortcuts(int arg);
static void gui_draw_load_lang(int arg);
static void gui_menuproc_edge_save(int arg);
static void gui_menuproc_edge_load(int arg);
static void gui_menuproc_edge_free(int arg);
static void gui_menuproc_reset(int arg);
static void gui_grid_lines_load(int arg);
static const char* gui_histo_mode_enum(int change, int arg);
static const char* gui_histo_layout_enum(int change, int arg);
static const char* gui_zebra_mode_enum(int change, int arg);
static const char* gui_zebra_draw_osd_enum(int change, int arg);
static const char* gui_font_enum(int change, int arg);
static const char* gui_raw_nr_enum(int change, int arg);
void gui_draw_read(int arg);
static void gui_draw_read_selected(const char *fn);
static void gui_draw_read_last(int arg);
static const char* gui_reader_codepage_enum(int change, int arg);
static const char* gui_shortcut_lang_enum(int change, int arg);
static const char* gui_zoom_value_enum(int change, int arg);
#if CAM_ADJUSTABLE_ALT_BUTTON
static const char* gui_alt_mode_button_enum(int change, int arg);
#endif
static const char* gui_alt_power_enum(int change, int arg);
static const char* gui_video_mode_enum(int change, int arg);
static const char* gui_video_bitrate_enum(int change, int arg);
static const char* gui_tv_enum(int change, int arg);
static const char* gui_nd_filter_state_enum(int change, int arg);
static const char* gui_av_enum(int change, int arg);
static const char* gui_bracket_type_enum(int change, int arg);
static const char* gui_tv_bracket_values_enum(int change, int arg);
static const char* gui_script_param_set_enum(int change, int arg);
#if !(CAM_HAS_ZOOM_LEVER)
static const char* gui_range_enum(int change, int arg);
static const char* gui_dist_step_enum(int change, int arg);
static const char* gui_synch_step_enum(int change, int arg);
static const char* gui_digi_step_enum(int change, int arg);
static void cb_step_25();
#endif
static const char* gui_synch_zoom_enum(int change, int arg);
static const char* gui_focus_enum(int change, int arg);
static const char* gui_camera_position_enum(int change, int arg);
static const char* gui_camera_orientation_enum(int change, int arg);
static const char* gui_camera_rotation_enum(int change, int arg);
static const char* gui_stereo_playback_enum(int change, int arg);
static const char* gui_dist_mode_enum(int change, int arg);
static const char* gui_header_enum(int change, int arg); 
static const char* gui_yaw_enum(int change, int arg); 
#if CAM_MULTIPART
static void gui_menuproc_swap_partitions(int arg);
#endif
// **************************************************************

//Menu callbacks
//-------------------------------------------------------------------

// static void cb_perc();
#if defined(OPT_OMNI) && defined(DNG_SUPPORT)
int g360Images,gViewCircle;
static void cb_slitcam_menu_change();
#endif
static void cb_volts();
static void cb_battery_menu_change(unsigned int item);
static void cb_zebra_restore_screen();
static void cb_zebra_restore_osd();
static void cb_edge_overlay();
static void cb_stereo_menu_change();
static void cb_synch_menu_change();
static void cb_grid();
static void cb_bracketing_menu_change();
static void cb_digi_menu_change();
static void cb_experimental_menu_change();
void rinit();
#ifdef OPT_SERIALIO
static CMenu *all_menus[32];             // array of pointers to menu
#endif

// ************************************************
//Menu definition
//-------------------------------------------------------------------

#if !(CAM_HAS_ZOOM_LEVER)
int digi_data_step;
static int delay_step = 10;
#endif

static CMenuItem misc_submenu_items[] = {
    {0,LANG_MENU_MISC_SHOW_SPLASH,        MENUITEM_BOOL,    &conf.splash_show },
#if CAM_USE_ZOOM_FOR_MF
    {0,LANG_MENU_MISC_ZOOM_FOR_MF,        MENUITEM_BOOL,    &conf.use_zoom_mf },
#endif
#if CAM_ADJUSTABLE_ALT_BUTTON
    {0,LANG_MENU_MISC_ALT_BUTTON,         MENUITEM_ENUM,    (int*)gui_alt_mode_button_enum },
#endif
    {0,LANG_MENU_MISC_DISABLE_LCD_OFF,    MENUITEM_ENUM,    (int*)gui_alt_power_enum },
    {0,LANG_MENU_MISC_LOCKOUT_TIME,       MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.lockout_time, MENU_MINMAX(0, 255)},
    {0,LANG_MENU_MISC_RIGHT_FLASH,        MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.flash_dim_value, MENU_MINMAX(0,12)},
    {0,LANG_MENU_MISC_SLAVE_FLASH,        MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.slave_flash, MENU_MINMAX(0,3)},
    {0,LANG_MENU_BRIGHT_SCREEN,           MENUITEM_BOOL,          &conf.bright_screen },
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu misc_submenu = {0,0, LANG_MENU_MISC_TITLE, NULL, misc_submenu_items };


static CMenuItem debug_submenu_items[] = {
    {0,LANG_MENU_DEBUG_SHOW_PROPCASES,    MENUITEM_BOOL,          &debug_propcase_show },
    {0,LANG_MENU_DEBUG_PROPCASE_PAGE,     MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,   &debug_propcase_page, MENU_MINMAX(0, 128) },    
    {0,LANG_MENU_DEBUG_SHOW_MISC_VALS,    MENUITEM_BOOL,          &debug_vals_show },
    #if CAM_MULTIPART
    {0,LANG_MENU_DEBUG_SWAP_PART,         MENUITEM_PROC, 	    	(int*)gui_menuproc_swap_partitions },
    #endif
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu debug_submenu = {1,0, LANG_MENU_DEBUG_TITLE, NULL, debug_submenu_items };


static int voltage_step;


static CMenuItem battery_submenu_items[] = {
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_BATT_VOLT_MAX,           MENUITEM_INT|MENUITEM_ARG_ADDR_INC,     &conf.batt_volts_max,   (int)&voltage_step },
    {0,LANG_MENU_BATT_VOLT_MIN,           MENUITEM_INT|MENUITEM_ARG_ADDR_INC,     &conf.batt_volts_min,   (int)&voltage_step },
    {0,LANG_MENU_BATT_STEP_25,            MENUITEM_BOOL|MENUITEM_ARG_CALLBACK,    &conf.batt_step_25,     (int)cb_step_25 },
#else
    {0,LANG_MENU_BATT_VOLT_MAX,           MENUITEM_INT,     &conf.batt_volts_max },
    {0,LANG_MENU_BATT_VOLT_MIN,           MENUITEM_INT,     &conf.batt_volts_min },
#endif	
    {0,(int)"",                           MENUITEM_SEPARATOR },	
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu battery_submenu = {2,0, LANG_MENU_BATT_TITLE, cb_battery_menu_change, battery_submenu_items };

static CMenuItem edge_overlay_submenu_items[] = {
    {0,LANG_MENU_EDGE_OVERLAY_ENABLE,     MENUITEM_BOOL,          &conf.outline_mode },
    {0,LANG_MENU_EDGE_OVERLAY_OSD,        MENUITEM_BOOL,          &conf.edge_and_osd },
    {0,LANG_MENU_EDGE_SAVE,			MENUITEM_PROC,		(int*)gui_menuproc_edge_save },
    {0,LANG_MENU_EDGE_ZOOM,               MENUITEM_BOOL,          &conf.edge_overlay_zoom },
    {0,LANG_MENU_EDGE_LOCK,               MENUITEM_BOOL,          &conf.edge_overlay_lock },
    {0,LANG_MENU_EDGE_OVERLAY_TRESH,      MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.edge_overlay_thresh, MENU_MINMAX(0, 255)},
    {0,LANG_MENU_EDGE_OVERLAY_COLOR,      MENUITEM_COLOR_FG,      (int*)&conf.edge_overlay_color},
#if !defined(CAMERA_ixus100)
    {0,LANG_MENU_EDGE_PLAY,			MENUITEM_BOOL,		&conf.edge_overlay_play }, //does not work on cams like s-series, which dont have a real "hardware" play/rec switch, need a workaround, probably another button
#endif
    {0,LANG_MENU_EDGE_FREE,			MENUITEM_PROC,		(int*)gui_menuproc_edge_free },
#if !defined(CAMERA_ixus100)
    {0,LANG_MENU_EDGE_LOAD,			MENUITEM_PROC,		(int*)gui_menuproc_edge_load },
#endif
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};

static CMenu edge_overlay_submenu = {3,0,LANG_MENU_EDGE_OVERLAY_TITLE,cb_edge_overlay, edge_overlay_submenu_items };


static CMenuItem grid_submenu_items[] = {
    {0,LANG_MENU_SHOW_GRID,               MENUITEM_BOOL,		&conf.show_grid_lines },
    {0,LANG_MENU_GRID_LOAD,               MENUITEM_PROC,		(int*)gui_grid_lines_load },
    {0,LANG_MENU_ENABLE_YAW_GUIDELINE,    MENUITEM_ENUM,          (int*)gui_yaw_enum},
    {0,LANG_MENU_YAW_DEGREES,             MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.yaw, MENU_MINMAX(1,30)},
    {0,LANG_MENU_GRID_CURRENT,            MENUITEM_SEPARATOR },
    {0,(int)grid_title,                   MENUITEM_TEXT },
    {0,(int)"",                           MENUITEM_SEPARATOR },
    {0,LANG_MENU_GRID_FORCE_COLOR,        MENUITEM_BOOL,          &conf.grid_force_color },
    {0,LANG_MENU_GRID_COLOR_LINE,         MENUITEM_COLOR_FG,      (int*)&conf.grid_color },
    {0,LANG_MENU_GRID_COLOR_FILL,         MENUITEM_COLOR_BG,      (int*)&conf.grid_color },
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu grid_submenu = {4,0, LANG_MENU_GRID_TITLE,cb_grid, grid_submenu_items };

// ******************************  Stereo submenu    ******************************************
static int distance_step = 1;
// ******************************  Stereo Synch submenu    ******************************************

static CMenuItem synch_submenu_items[] = {
    {0,LANG_MENU_SYNCH_STATUS,              MENUITEM_BOOL, &conf.synch_enable},
    {0,LANG_MENU_SYNCH_DELAY,               MENUITEM_BOOL, &conf.synch_delay_enable},
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_DIST_STEP_SIZE ,           MENUITEM_ENUM,(int*)gui_synch_step_enum },
    {0,LANG_MENU_SYNCH_DELAY_VALUE,         MENUITEM_INT|MENUITEM_ARG_ADDR_INC,  &conf.synch_user_delay,(int)&delay_step},
    {0,LANG_MENU_SYNCH_DELAY_COARSE_VALUE,  MENUITEM_INT|MENUITEM_ARG_ADDR_INC,  &conf.synch_delay_coarse_value,(int)&delay_step},
#else
    {0,LANG_MENU_SYNCH_DELAY_VALUE,         MENUITEM_INT,  &conf.synch_user_delay},
    {0,LANG_MENU_SYNCH_DELAY_COARSE_VALUE,  MENUITEM_INT,  &conf.synch_delay_coarse_value},
#endif
    {0,LANG_MENU_ADD_SYNCH_DELAYS,          MENUITEM_BOOL, &conf.add_synch_delays},
    {0,LANG_MENU_BACK,                      MENUITEM_UP },
    
    {0}
};
static CMenu synch_submenu = {5,0,LANG_MENU_SYNCH_TITLE,cb_synch_menu_change , synch_submenu_items };

// ************************************************************************************************
static CMenuItem visual_submenu_items[] = {
    {0,LANG_MENU_VIS_LANG,                MENUITEM_PROC,      (int*)gui_draw_load_lang },
    {0,LANG_MENU_READ_CODEPAGE,           MENUITEM_ENUM,      (int*)gui_reader_codepage_enum },
    {0,LANG_MENU_SHORTCUT_LANG,           MENUITEM_ENUM,      (int*)gui_shortcut_lang_enum },
    {0,LANG_MENU_VIS_OSD_FONT,            MENUITEM_ENUM,      (int*)gui_font_enum },
    {0,LANG_MENU_VIS_MENU_FONT,           MENUITEM_PROC,      (int*)gui_draw_load_menu_rbf },
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu visual_submenu = {6,0, LANG_MENU_VIS_TITLE, NULL, visual_submenu_items };

static CMenuItem digiscope_submenu_items[] = {
    {0,LANG_MENU_DIGI_OD,                 MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.digi_obj_diam, MENU_MINMAX(40,150)},
    {0,LANG_MENU_DIGI_FL,                 MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.digi_scope_fl, MENU_MINMAX(100,800)},
    {0,LANG_MENU_DIGI_EP_FL ,             MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.digi_ep_fl, MENU_MINMAX(3,100)},
    {0,LANG_MENU_DIGI_ZOOM_SETTING,       MENUITEM_INT|MENUITEM_F_UNSIGNED,&conf.digi_zoom_setting},
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_DIST_STEP_SIZE,          MENUITEM_ENUM,      (int*)gui_digi_step_enum },
    {0,LANG_MENU_DIGI_REF_DIST,           MENUITEM_INT|MENUITEM_ARG_ADDR_INC,  &conf.digi_ref_dist,(int)&digi_data_step},
#else
    {0,LANG_MENU_DIGI_REF_DIST,           MENUITEM_INT,  &conf.digi_ref_dist},
#endif
    {0,LANG_MENU_BACK,                    MENUITEM_UP },    
    {0}
};
static CMenu digiscope_submenu = {7,0, LANG_MENU_DIGI_TITLE,cb_digi_menu_change, digiscope_submenu_items };

static CMenuItem experimental_submenu_items[] = {
    {0,LANG_MENU_SUNRISE,                 MENUITEM_BOOL, &conf.sunrise},
    {0,LANG_MENU_SUNRISE_REPEATS,         MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,&conf.sunrise_repeats,MENU_MINMAX(0,9999)},
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_DIST_STEP_SIZE,          MENUITEM_ENUM,(int*)gui_digi_step_enum },
    {0,LANG_MENU_SUNRISE_SHOTS,           MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_ARG_ADDR_INC,&conf.sunrise_shots,(int)&digi_data_step},
#else
    {0,LANG_MENU_SUNRISE_SHOTS,           MENUITEM_INT|MENUITEM_F_UNSIGNED,&conf.sunrise_shots},
#endif
    {0,LANG_MENU_SUNRISE_TV96,            MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,&conf.sunrise_tv96,MENU_MINMAX(0,96)},
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_SUNRISE_KF_1,            MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_ARG_ADDR_INC,&conf.sunrise_kf_1,(int)&digi_data_step},
#else
    {0,LANG_MENU_SUNRISE_KF_1,            MENUITEM_INT|MENUITEM_F_UNSIGNED,&conf.sunrise_kf_1},
#endif
    {0,LANG_MENU_SUNRISE_TV96_2,          MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,&conf.sunrise_tv96_2,MENU_MINMAX(0,96)},
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_SUNRISE_KF_2,            MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_ARG_ADDR_INC,&conf.sunrise_kf_2,(int)&digi_data_step},
#else
    {0,LANG_MENU_SUNRISE_KF_2,            MENUITEM_INT|MENUITEM_F_UNSIGNED,&conf.sunrise_kf_2},
#endif
    {0,LANG_MENU_SUNRISE_TV96_3,          MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,&conf.sunrise_tv96_3,MENU_MINMAX(0,96)},
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_SUNRISE_KF_3,            MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_ARG_ADDR_INC,&conf.sunrise_kf_3,(int)&digi_data_step},
#else
    {0,LANG_MENU_SUNRISE_KF_3,            MENUITEM_INT|MENUITEM_F_UNSIGNED,&conf.sunrise_kf_3},
#endif
    {0,LANG_MENU_SRB,                     MENUITEM_BOOL, &conf.srb},
    {0,LANG_MENU_BACK,                    MENUITEM_UP },    
    {0}
};
static CMenu experimental_submenu = {8,0, LANG_MENU_SUNRISE_TITLE,cb_experimental_menu_change,experimental_submenu_items };

static CMenuItem osd_submenu_items[] = {

    {0,LANG_MENU_OSD_SHOW_STATES,         MENUITEM_BOOL,        &conf.show_state },
    {0,LANG_MENU_OSD_SHOW_MISC_VALUES,    MENUITEM_BOOL,      &conf.show_values },
    {0,LANG_MENU_OSD_ZOOM_VALUE,          MENUITEM_ENUM,      (int*)gui_zoom_value_enum },
    {0,LANG_MENU_OSD_ZOOM_FONT_SIZE,      MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,&conf.zf_size,MENU_MINMAX(0,16)},
    {0,LANG_MENU_OSD_SHOW_CLOCK,          MENUITEM_BOOL,      &conf.show_clock },
    {0,LANG_MENU_OSD_LAYOUT_EDITOR,       MENUITEM_PROC,      (int*)gui_draw_osd_le },
    {0,LANG_MENU_OSD_BATT_PARAMS,         MENUITEM_SUBMENU,   (int*)&battery_submenu },
    {0,LANG_MENU_OSD_GRID_PARAMS,         MENUITEM_SUBMENU,   (int*)&grid_submenu },
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu osd_submenu = {9,0, LANG_MENU_OSD_TITLE,NULL, osd_submenu_items };

#if !(CAM_HAS_ZOOM_LEVER)
static int dist_bracket_step = 10;
#endif
// --------------------------------------------------------------------------------------------------

static CMenuItem zebra_submenu_items[] = {
    {0,LANG_MENU_HISTO_SHOW,              MENUITEM_BOOL,      &conf.show_histo },
    {0,LANG_MENU_HISTO_LAYOUT,            MENUITEM_ENUM,      (int*)gui_histo_layout_enum },
    {0,LANG_MENU_HISTO_MODE,              MENUITEM_ENUM,      (int*)gui_histo_mode_enum },
    {0,LANG_MENU_HISTO_EXP,               MENUITEM_BOOL,    &conf.show_overexp },
    {0,LANG_MENU_HISTO_IGNORE_PEAKS,      MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.histo_ignore_boundary,   MENU_MINMAX(0, 32)},
    {0,LANG_MENU_HISTO_MAGNIFY,           MENUITEM_BOOL,      &conf.histo_auto_ajust },
    {0,LANG_MENU_ZEBRA_DRAW,              MENUITEM_BOOL,      &conf.zebra_draw },
    {0,LANG_MENU_ZEBRA_MODE,              MENUITEM_ENUM,                            (int*)gui_zebra_mode_enum },
    {0,LANG_MENU_ZEBRA_UNDER,             MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.zebra_under,   MENU_MINMAX(0, 32)},
    {0,LANG_MENU_ZEBRA_OVER,              MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.zebra_over,    MENU_MINMAX(0, 32)},
    {0,LANG_MENU_VIS_ZEBRA_UNDER,         MENUITEM_COLOR_BG,  (int*)&conf.zebra_color },
    {0,LANG_MENU_VIS_ZEBRA_OVER,          MENUITEM_COLOR_FG,  (int*)&conf.zebra_color },
    {0,LANG_MENU_ZEBRA_RESTORE_SCREEN,    MENUITEM_BOOL|MENUITEM_ARG_CALLBACK,      &conf.zebra_restore_screen,     (int)cb_zebra_restore_screen },
    {0,LANG_MENU_ZEBRA_RESTORE_OSD,       MENUITEM_BOOL|MENUITEM_ARG_CALLBACK,      &conf.zebra_restore_osd,        (int)cb_zebra_restore_osd },
    {0,LANG_MENU_ZEBRA_DRAW_OVER,         MENUITEM_ENUM,                            (int*)gui_zebra_draw_osd_enum },
    {0,LANG_MENU_ZEBRA_MULTICHANNEL,      MENUITEM_BOOL,                            &conf.zebra_multichannel}, 
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu zebra_submenu = {10,0, LANG_MENU_ZEBRA_TITLE, NULL, zebra_submenu_items };

// --------------------------------------------------------------------------------------------------

static CMenuItem bracket_submenu_items[] = {
    {0,LANG_MENU_RAW_SHUTTER_OVERRIDE ,   MENUITEM_BOOL,     &conf.tv_override_enable}, 
    {0,LANG_MENU_SHUTTER_VALUE,           MENUITEM_ENUM,    (int*)gui_tv_enum},
#if CAM_HAS_IRIS_DIAPHRAGM
    {0,LANG_MENU_RAW_AV_OVERRIDE,         MENUITEM_ENUM,    (int*)gui_av_enum },
#endif 
#if CAM_CAN_SD_OVERRIDE
    {0,LANG_MENU_OSD_MANUAL_SET_DIST,     MENUITEM_BOOL,    &conf.user_range_set},
    {0,LANG_MENU_OSD_SAVED_FOCUS,         MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.focus_pos, MENU_MINMAX(10,65000)},
    {0,LANG_MENU_OSD_FOCUS_MODE,          MENUITEM_ENUM,(int*)gui_focus_enum },
    {0,LANG_SDM_RF_NEAR,                  MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &subject_near, MENU_MINMAX(10,65000)},
    {0,LANG_SDM_RF_FAR,                   MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &subject_far, MENU_MINMAX(10,65000)},
#endif
#if CAM_HAS_ND_FILTER
    {0,LANG_MENU_OVERRIDE_ND_FILTER,    MENUITEM_ENUM,    (int*)gui_nd_filter_state_enum },
    {0,LANG_MENU_ND_CONSTANT_EXPOSURE,    MENUITEM_BOOL,    &conf.nd_constant_exposure},
#endif 
    {0,LANG_MENU_BKT,                    MENUITEM_SEPARATOR },
#if CAM_CAN_SD_OVERRIDE 
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_DIST_STEP_SIZE,          MENUITEM_ENUM,    (int*)gui_dist_step_enum },
    {0,LANG_MENU_DIST_BRACKET_VALUE,      MENUITEM_INT|MENUITEM_ARG_ADDR_INC,&conf.subj_dist_bracket_value,(int)&dist_bracket_step},
#else
    {0,LANG_MENU_DIST_BRACKET_VALUE,      MENUITEM_INT|MENUITEM_F_MINMAX,&conf.subj_dist_bracket_value,MENU_MINMAX(1,10000)},
#endif
    {0,LANG_MENU_DIST_MODE,               MENUITEM_ENUM,    (int*)gui_dist_mode_enum },
    {0,LANG_MENU_DIGI_SAVE,               MENUITEM_BOOL,    &conf.save_auto},
#endif
    {0,LANG_MENU_TV_BRACKET_VALUE,        MENUITEM_ENUM,    (int*)gui_tv_bracket_values_enum },
    {0,LANG_MENU_BRACKET_TYPE,            MENUITEM_ENUM,    (int*)gui_bracket_type_enum },
    {0,LANG_MENU_TIMER_SYNCH ,            MENUITEM_BOOL,     &conf.custom_timer_synch},
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu bracket_submenu = {11,0, LANG_MENU_OSD_BRACKET_OVERRIDE,cb_bracketing_menu_change, bracket_submenu_items };

// --------------------------------------------------------------------------------------------------

static CMenuItem video_submenu_items[] = {
#if CAM_CHDK_HAS_EXT_VIDEO_MENU
    {0,LANG_MENU_RAW_VIDEO_MODE,          MENUITEM_ENUM,    (int*)gui_video_mode_enum}, 
    {0,LANG_MENU_RAW_VIDEO_BITRATE,       MENUITEM_ENUM,    (int*)gui_video_bitrate_enum}, 
    {0,LANG_MENU_RAW_VIDEO_QUALITY,       MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.video_quality, MENU_MINMAX(1, 99)},
#endif
#if CAM_CAN_UNLOCK_OPTICAL_ZOOM_IN_VIDEO 
    {0,LANG_MENU_OPTICAL_ZOOM_IN_VIDEO,   MENUITEM_BOOL,      &conf.unlock_optical_zoom_for_video},                                                  
#endif 
#if CAM_CAN_MUTE_MICROPHONE 
    {0,LANG_MENU_MUTE_ON_ZOOM,            MENUITEM_BOOL,  &conf.mute_on_zoom}, 	
#endif 
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu video_submenu = {12,0, LANG_MENU_VIDEO, NULL, video_submenu_items };

// --------------------------------------------------------------------------------------------
static CMenuItem raw_submenu_items[] = {
    {0,LANG_MENU_RAW_SAVE,                MENUITEM_BOOL,      &conf.save_raw },
    {0,LANG_MENU_RAW_BLANK_JPG,           MENUITEM_BOOL,      &conf.blank_jpg },
    {0,LANG_MENU_RAW_NOISE_REDUCTION,     MENUITEM_ENUM,      (int*)gui_raw_nr_enum },
//    {0,LANG_MENU_RAW_FIRST_ONLY,          MENUITEM_BOOL,      &conf.raw_save_first_only },
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
#if defined(OPT_OMNI) && defined(DNG_SUPPORT)
static CMenu raw_submenu = {13,0, LANG_MENU_RAW, NULL, raw_submenu_items };
#else
static CMenu raw_submenu = {13,0, LANG_MENU_RAW, NULL, raw_submenu_items };
#endif

// --------------------------------------------------------------------------------------------

static CMenuItem image_video_submenu_items[] = {
    {1,LANG_MENU_IMAGE,                    MENUITEM_SUBMENU,   (int*)&zebra_submenu },
    {1,LANG_MENU_VIDEO,                    MENUITEM_SUBMENU,   (int*)&video_submenu },
    {1,LANG_MENU_RAW,                      MENUITEM_SUBMENU,   (int*)&raw_submenu },
    {1,LANG_MENU_BACK,                     MENUITEM_UP },
    {0}
};
static CMenu image_video_submenu = {14,1, LANG_MENU_IMAGE_VIDEO, NULL, image_video_submenu_items };

// --------------------------------------------------------------------------------------------

static CMenuItem stereo_submenu_items[] = {
    {0,LANG_MENU_COMPACT,                  MENUITEM_ENUM,(int*)gui_header_enum },
    {0,LANG_MENU_SDM_CONFIG,               MENUITEM_PROC, (int*)gui_draw_sdm_configuration},
    {0,LANG_MENU_SHORTCUTS,                MENUITEM_PROC, (int*)gui_draw_shortcuts},
    {0,LANG_MENU_ZOOM_ENABLE,              MENUITEM_ENUM,(int*)gui_synch_zoom_enum },
    {0,LANG_MENU_ZOOM_TIMEOUT,             MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.zoom_timeout,MENU_MINMAX(2,10)},
   #if CAM_NEED_HP 
    {0,LANG_MENU_EXTEND_LENS,              MENUITEM_BOOL, &conf.extend_lens},
#endif
    {0,LANG_MENU_DOUBLE_CLICK,             MENUITEM_BOOL, &conf.dc},
#if CAM_SYNCH
    {0,LANG_MENU_OSD_SYNCH,                MENUITEM_SUBMENU,(int*)&synch_submenu },
#endif
    {0,LANG_MENU_OSD_THRESH,               MENUITEM_SUBMENU,   (int*)&edge_overlay_submenu},
    {0,LANG_MENU_DISABLE_DEVIATION_GUIDELINE,MENUITEM_BOOL,&conf.disable_deviation_guideline},
#if !defined(CAMERA_ixus100)
    {0,LANG_MENU_OSD_STEREO_PLAYBACK,      MENUITEM_ENUM,(int*)gui_stereo_playback_enum },
#endif
    {0,LANG_MENU_OSD_CAM_POSITION,         MENUITEM_ENUM,(int*)gui_camera_position_enum },
#if !defined(CAMERA_tx1)
    {0,LANG_MENU_OSD_CAM_ORIENTATION,      MENUITEM_ENUM,(int*)gui_camera_orientation_enum },
#endif
    {0,LANG_MENU_OSD_CAM_ROTATION,         MENUITEM_ENUM,(int*)gui_camera_rotation_enum },
#if !defined(CAMERA_ixus100)
    {0,LANG_MENU_OSD_INVERT_PLAY,          MENUITEM_BOOL,&conf.invert_playback},
#endif
    {0,LANG_MENU_OSD_CAM_SPACING,          MENUITEM_INT,  &conf.stereo_spacing},
    {0,LANG_MENU_OSD_DEV_FACTOR,           MENUITEM_INT|MENUITEM_F_MINMAX,  &conf.stereo_deviation,MENU_MINMAX(10,60)},
    {0,LANG_MENU_OSD_NEAR_FACTOR,          MENUITEM_INT|MENUITEM_F_MINMAX,  &conf.minimum_dist_factor,MENU_MINMAX(5,50)},
#if !defined (CAMERA_ixus700) 
#if !(CAM_HAS_ZOOM_LEVER)
    {0,LANG_MENU_OSD_DIST_STEP_SIZE,       MENUITEM_ENUM,(int*)gui_range_enum },
    {0,LANG_MENU_OSD_DISTANCE,             MENUITEM_INT|MENUITEM_ARG_ADDR_INC,  &conf.distance_setting,(int)&distance_step},
#endif
#endif
    {0,LANG_MENU_OSD_SAVE_XML,             MENUITEM_BOOL,&conf.save_xml_file},
    {0,LANG_MENU_OSD_CSV_WITH_JPEG,      MENUITEM_BOOL,&conf.save_same_dir},
    {0,LANG_MENU_OSD_SPM_URL,              MENUITEM_TEXT },	
    {0,LANG_MENU_BACK,                     MENUITEM_UP },
    {0}
};

static CMenu stereo_submenu = {15,0,LANG_MENU_STEREO_TITLE, cb_stereo_menu_change, stereo_submenu_items };

// --------------------------------------------------------------------------------------------------

static CMenuItem script_submenu_items_top[] = {
    {0,LANG_MENU_SCRIPT_LOAD,            MENUITEM_PROC,                      (int*)gui_load_script },
    {0,LANG_MENU_SCRIPT_DELAY,           MENUITEM_INT|MENUITEM_F_UNSIGNED,   &conf.script_shoot_delay },
    {0,LANG_MENU_SCRIPT_AUTOSTART,	     MENUITEM_BOOL,                       &conf.script_startup },
    {0,(int)script_title,                MENUITEM_SEPARATOR },
    {0,(int)paramdesc[0],                MENUITEM_SEPARATOR },
    {0,(int)paramdesc[1],                MENUITEM_SEPARATOR },
    {0,(int)paramdesc[2],                MENUITEM_SEPARATOR },
    {0,(int)paramdesc[3],                MENUITEM_SEPARATOR },
    {0,LANG_MENU_SCRIPT_PARAM_SET,       MENUITEM_ENUM,                      (int*)gui_script_param_set_enum},
};

static CMenuItem script_submenu_items_bottom[] = {
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};

static CMenuItem script_submenu_items[sizeof(script_submenu_items_top)/sizeof(script_submenu_items_top[0])+SCRIPT_NUM_PARAMS+
                               sizeof(script_submenu_items_bottom)/sizeof(script_submenu_items_bottom[0])];
static CMenu script_submenu = {16,0, LANG_MENU_SCRIPT_TITLE, NULL, script_submenu_items };

// --------------------------------------------------------------------------------------------------
static CMenuItem reader_submenu_items[] = {
    {0,LANG_MENU_READ_OPEN_NEW,           MENUITEM_PROC,    (int*)gui_draw_read },
    {0,LANG_MENU_READ_OPEN_LAST,          MENUITEM_PROC,    (int*)gui_draw_read_last },
    {0,LANG_MENU_READ_SELECT_FONT,        MENUITEM_PROC,    (int*)gui_draw_load_rbf },
    {0,LANG_MENU_READ_CODEPAGE,           MENUITEM_ENUM,    (int*)gui_reader_codepage_enum },
    {0,LANG_MENU_READ_WORD_WRAP,          MENUITEM_BOOL,    &conf.reader_wrap_by_words },
    {0,LANG_MENU_READ_AUTOSCROLL,         MENUITEM_BOOL,    &conf.reader_autoscroll },
    {0,LANG_MENU_READ_AUTOSCROLL_DELAY,   MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.reader_autoscroll_delay, MENU_MINMAX(0, 60) },
    {0,LANG_MENU_BACK,                    MENUITEM_UP },    
    {0}
};
static CMenu reader_submenu = {17,0, LANG_MENU_READ_TITLE,NULL, reader_submenu_items };


static CMenuItem sc_submenu_items[] = 
{
#if CAM_REMOTE
    {0,LANG_MENU_SCRIPT_REMOTE_ENABLE,   MENUITEM_BOOL, &conf.remote_enable},
#endif
    {0,LANG_MENU_USE_AF_LED,              MENUITEM_BOOL, &conf.use_af_led},
    {0,LANG_MENU_BIT_WIDTH,               MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.bw, MENU_MINMAX(2, 300) },
    {0,LANG_MENU_MSEC,                    MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.md, MENU_MINMAX(7,100) },
#ifdef OPT_SERIALIO
    {0,LANG_MENU_DISABLE_SIO,             MENUITEM_BOOL, &conf.ricoh_ca1_mode},
    {0,LANG_MENU_SIO,                     MENUITEM_BOOL, &conf.show_sio},  // show serial i/o errors and rate
#endif
    {0,LANG_MENU_BACK,                    MENUITEM_UP },    
    {0}
};
static CMenu sc_submenu = {18,0, LANG_MENU_SC_TITLE,NULL, sc_submenu_items };

#if defined(OPT_OMNI) && defined(DNG_SUPPORT)&& !defined(CAMERA_ixus105)
static CMenuItem slitcam_submenu_items[] = 
{
    {0,LANG_MENU_RAW_STRIP_MODE,          MENUITEM_BOOL,      &conf.raw_strip_mode },
    {0,LANG_MENU_RAW_STRIP_WIDTH,         MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.strip_width, MENU_MINMAX(32,(CAM_ACTIVE_AREA_Y2-CAM_ACTIVE_AREA_Y1)/2)},
    {0,LANG_MENU_RAW_STRIP_OFFSET,        MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.strip_offset, MENU_MINMAX(0,100)},
    {0,LANG_MENU_RAW_STRIP_360_IMAGES,    MENUITEM_INT|MENUITEM_ARG_ADDR_INC,&g360Images,0},
    {0,LANG_MENU_RAW_STRIP_IMAGES,        MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.strip_images, MENU_MINMAX(1,65000)},
    {0,LANG_MENU_RAW_STRIP_ROTATION,      MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX,  &conf.rotation_distance, MENU_MINMAX(0,1000)},
    {0,LANG_MENU_RAW_STRIP_VIEWING,       MENUITEM_INT|MENUITEM_ARG_ADDR_INC,&gViewCircle,0},
    {0,LANG_MENU_BACK,                    MENUITEM_UP },    
    {0}
};
static CMenu slitcam_submenu = {19,0, LANG_MENU_SLITCAM_TITLE,cb_slitcam_menu_change, slitcam_submenu_items };
#endif

static CMenuItem root_menu_items[] = {
#if defined(OPT_OMNI) && defined(DNG_SUPPORT)&& !defined(CAMERA_ixus105)
    {0,LANG_MENU_MAIN_SLITCAM,            MENUITEM_SUBMENU,   (int*)&slitcam_submenu },
#endif
    {0,LANG_MENU_MAIN_DIGISCOPE,          MENUITEM_SUBMENU,   (int*)&digiscope_submenu },
    {0,LANG_MENU_MAIN_EXPERIMENTAL,       MENUITEM_SUBMENU,   (int*)&experimental_submenu },
    {0,LANG_MENU_MAIN_SERIAL_COM,         MENUITEM_SUBMENU,   (int*)&sc_submenu },
    {0,LANG_MENU_MISC_TEXT_READER,        MENUITEM_SUBMENU,   (int*)&reader_submenu },
    {0,LANG_MENU_MISC_FILE_BROWSER,       MENUITEM_PROC,      (int*)gui_draw_fselect },
    {0,LANG_MENU_MAIN_OSD_PARAM,          MENUITEM_SUBMENU,   (int*)&osd_submenu },
    {0,LANG_MENU_MAIN_VISUAL_PARAM,       MENUITEM_SUBMENU,   (int*)&visual_submenu },
    {0,LANG_MENU_MAIN_MISC,               MENUITEM_SUBMENU,   (int*)&misc_submenu },
    {0,LANG_MENU_MAIN_DEBUG,              MENUITEM_SUBMENU,   (int*)&debug_submenu },
    {0,LANG_MENU_MAIN_RESET_OPTIONS,      MENUITEM_PROC,      (int*)gui_menuproc_reset },
    {0,LANG_MENU_BACK,                    MENUITEM_UP },
    {0}
};
static CMenu root_menu = {19,0, LANG_MENU_MAIN_TITLE, NULL, root_menu_items };

// --------------------------------------------------------------------------------------------------


static CMenuItem user_submenu_items[] = 
{
  {1,LANG_MENU_IMAGE_VIDEO ,              MENUITEM_SUBMENU,   (int*)&image_video_submenu},
  {1,LANG_MENU_BRACKET,                   MENUITEM_SUBMENU,   (int*)&bracket_submenu}, 
  {1,LANG_MENU_OSD_STEREO_PARAM,          MENUITEM_SUBMENU,   (int*)&stereo_submenu}, 
  {1,LANG_MENU_MAIN_SCRIPT_PARAM,         MENUITEM_SUBMENU,   (int*)&script_submenu },
  {1,LANG_MENU_MAIN_TITLE,                MENUITEM_SUBMENU,      (int*)&root_menu},
  {0}
};

static CMenu user_submenu = {20,1, LANG_MENU_BASIC_TITLE, NULL, user_submenu_items };

#if defined(OPT_OMNI) && defined(DNG_SUPPORT)
//-------------------------------------------------------------------
void cb_slitcam_menu_change()
{
 unsigned long Den1,Den2;
 if(conf.raw_strip_mode)conf.save_raw=1;
 g360Images=(640*CAM_JPEG_HEIGHT)/((get_fov(lens_get_zoom_point())*conf.strip_width));
 Den1=get_sensor_width()*get_sensor_width()/160000;  // * 10^2
 Den2=get_focal_length(lens_get_zoom_point())*get_focal_length(lens_get_zoom_point())/10000; // *10^2
 int offset = get_sensor_width()*conf.strip_offset/10000; // 
 gViewCircle=(offset*conf.rotation_distance)/sqrt(Den1+Den2); // num * 100
}
//-------------------------------------------------------------------
#endif

void cb_grid()
{
 if(conf.enable_yaw_guideline)conf.disable_deviation_guideline=1; 
}
// -------------------------------------------------------------------
void cb_edge_overlay()
{
 if(conf.camera_orientation)conf.outline_mode=0;     // edge outline only enabled in horizontal mode
 if(!conf.outline_mode)edge_count=0; 
}
// -------------------------------------------------------------------
#if !(CAM_HAS_ZOOM_LEVER)
void cb_step_25() {
    voltage_step = (conf.batt_step_25)?25:1;
}
#endif

void cb_battery_menu_change(unsigned int item) {
    switch (item) {
        case 0: //Voltage MAX
            if (conf.batt_volts_max<conf.batt_volts_min+25) {
                conf.batt_volts_min = conf.batt_volts_max-25;
            }
            break;
        case 1: //Voltage MIN
            if (conf.batt_volts_min>conf.batt_volts_max-25) {
                conf.batt_volts_max = conf.batt_volts_min+25;
            }
            break;
        default:
            break;
    }
}

// ***********************************************************
//                 Range limit the user-settable values
// ***********************************************************
void cb_bracketing_menu_change()
{
if(conf.subj_dist_bracket_value<1)conf.subj_dist_bracket_value=1;
if(conf.dist_mode==2)conf.bracket_type=3; // for Masuji code
if(conf.dist_mode==3)                    // digiscope
  {
   conf.synch_enable=1;
   conf.user_range_set=0;                                   // for Masuji code
  }

  if(conf.dist_mode)
   {
    conf.sunrise=0;
    conf.tv_bracket_value=0;
   }
  else if(conf.tv_bracket_value)
   {
    conf.sunrise=0;
    conf.dist_mode=0;
   }
}
// -------------------------------------------------------------------
void cb_experimental_menu_change()
{
  if(conf.sunrise)
  {
    conf.dist_mode=0;
    conf.tv_bracket_value=0;
  }
  if(conf.sunrise_shots<10) conf.sunrise_shots=10;
  if(conf.sunrise_kf_1>conf.sunrise_shots) conf.sunrise_kf_1 = conf.sunrise_shots;
  if(conf.sunrise_kf_2<conf.sunrise_kf_1) conf.sunrise_kf_2 = conf.sunrise_kf_1;
  if(conf.sunrise_kf_3<conf.sunrise_kf_2) conf.sunrise_kf_3 = conf.sunrise_kf_2;
}

// -------------------------------------------------------------------
void cb_digi_menu_change()
{
 if(conf.digi_ref_dist<450) conf.digi_ref_dist=450;
 if(conf.digi_zoom_setting<0) conf.digi_zoom_setting = 0;
 if(conf.digi_zoom_setting>=zoom_points) conf.digi_zoom_setting = zoom_points-1;
}
// -------------------------------------------------------------------
void cb_synch_menu_change()
{
if(conf.synch_enable < 0 || conf.synch_enable > 1) conf.synch_enable = 0;
if(conf.synch_enable) conf.remote_enable = 1;
if(conf.synch_delay_enable < 0 || conf.synch_delay_enable > 1) conf.synch_delay_enable = 0;
if(conf.synch_user_delay < 0) conf.synch_user_delay = 0;
if(conf.synch_user_delay> 100000) conf.synch_user_delay = 100000;
}
// -------------------------------------------------------------------
void cb_stereo_menu_change()
{
if(!conf.disable_deviation_guideline)conf.enable_yaw_guideline=0; 
if(conf.stereo_spacing < 1) conf.stereo_spacing = 1;
if(conf.stereo_deviation <10) conf.stereo_deviation = 10;
if(conf.stereo_deviation >100) conf.stereo_deviation = 100;
if(conf.minimum_dist_factor <15) conf.minimum_dist_factor = 15;
if(conf.distance_setting < 10) conf.distance_setting = 10;
if(conf.distance_setting > 65535) conf.distance_setting = 65535;
if(conf.invert_playback) conf.stereo_mode=0;
if(conf.stereo_mode) conf.invert_playback=0;
#if defined(CAMERA_ixus870)
if(conf.remote_zoom_enable==2)conf.remote_zoom_enable=1;  // cannot exit zoom=continuous mode
#endif
}
// -------------------------------------------------------------------
//     Reyalp's menu enumeration CS988
// -------------------------------------------------------------------

static void gui_enum_value_change(int *value, int change, unsigned num_items) 
{ 
 *value+=change; 
 if (*value<0) 
  *value = num_items-1; 
 else if (*value>=num_items) 
  *value = 0; 
}
 
static const char* gui_change_simple_enum(int* value, int change, const char** items, unsigned num_items) 
{ 
 gui_enum_value_change(value, change, num_items); 
 return items[*value]; 
} 

// -------------------------------------------------------------------

const char* gui_script_param_set_enum(int change, int arg) 
{
    static const char* modes[]={ "0", "1", "2", "3", "4", "5", "6", "7", "8", "9" };
    int i;

    if (change != 0)                                                      // if there has been a menu change
     {
      for(i=0;i<4;i++)paramdesc[i][0]='\0';                                  // clear description
        conf.script_param_set += change;                                  // update script-set number
        if (conf.script_param_set < 0)                                      conf.script_param_set = (sizeof(modes)/sizeof(modes[0]))-1;
        else if (conf.script_param_set >= (sizeof(modes)/sizeof(modes[0]))) conf.script_param_set=0;
        if (!load_params_values(conf.script_file, 1, 0))                    script_load(conf.script_file, 0); // load default script and values if a script-set cannot be loaded
        gui_update_script_submenu();
     }
    return modes[conf.script_param_set];
}
// -------------------------------------------------------------------
const char* histo_mode_enum()
{
  return gui_histo_mode_enum(0,0);
}
// -------------------------------------------------------------------

const char* histo_layout_enum()
{
  return gui_histo_layout_enum(0,0);
}
// -------------------------------------------------------------------

const char* zebra_mode_enum()
{
  return gui_zebra_mode_enum(0,0);
}
// -------------------------------------------------------------------
const char* video_bitrate_enum()
{
  return gui_video_bitrate_enum(0,0);
}
// -------------------------------------------------------------------
const char* tv_enum()
{
  return gui_tv_enum(0,0);
}

// -------------------------------------------------------------------
const char* tv_bracket_values_enum()
{
  return gui_tv_bracket_values_enum(0,0);
}

// -------------------------------------------------------------------
const char* focus_enum()
{
 return gui_focus_enum(0,0);
}
// -------------------------------------------------------------------
const char* gui_yaw_enum(int change, int arg) 
{
 static const char* decades[]={"Off","Vertical","Diagonal"};
 return gui_change_simple_enum(&conf.enable_yaw_guideline,change,decades,sizeof(decades)/sizeof(decades[0]));
}
// -------------------------------------------------------------------
const char* gui_header_enum(int change, int arg) 
{
 static const char* decades[]={"Standard","Compact","None"};
 return gui_change_simple_enum(&conf.compact,change,decades,sizeof(decades)/sizeof(decades[0]));
}
// -------------------------------------------------------------------
#if !(CAM_HAS_ZOOM_LEVER)
const char* gui_digi_step_enum(int change, int arg) 
{
    static const char* decades[]={"1","10","100","1000"};

    conf.digi_step+=change;
    if (conf.digi_step<0)
        conf.digi_step=(sizeof(decades)/sizeof(decades[0]))-1;
    else if (conf.digi_step>=(sizeof(decades)/sizeof(decades[0])))
        conf.digi_step=0;
    digi_data_step = atoi(decades[conf.digi_step]); 
    return decades[conf.digi_step];
}
#endif
// -------------------------------------------------------------------
const char* gui_synch_zoom_enum(int change, int arg) 
{
 static const char* decades[]={"Off","Cont"};
 return gui_change_simple_enum(&conf.remote_zoom_enable,change,decades,sizeof(decades)/sizeof(decades[0]));
}
// -------------------------------------------------------------------
const char* gui_stereo_playback_enum(int change, int arg) 
{
 static const char* decades[]={"Off","Pair","Mono","Colour"};
 return gui_change_simple_enum(&conf.stereo_mode,change,decades,sizeof(decades)/sizeof(decades[0]));
}
// -------------------------------------------------------------------
#if !(CAM_HAS_ZOOM_LEVER)
const char* gui_synch_step_enum(int change, int arg) 
{
    static const char* decades[]={"1","10","100","1000","10000"};
    conf.synch_delay_range+=change;
    if (conf.synch_delay_range<0)
        conf.synch_delay_range=(sizeof(decades)/sizeof(decades[0]))-1;
    else if (conf.synch_delay_range>=(sizeof(decades)/sizeof(decades[0])))
        conf.synch_delay_range=0;
    delay_step = atoi(decades[conf.synch_delay_range]); 
    return decades[conf.synch_delay_range];
}
#endif
// -------------------------------------------------------------------

const char* gui_camera_orientation_enum(int change, int arg) 
{
 static const char* extensions[]={ "H","V", "I"};
 return gui_change_simple_enum(&conf.camera_orientation,change,extensions,sizeof(extensions)/sizeof(extensions[0]));
}
// -------------------------------------------------------------------
const char* gui_camera_rotation_enum(int change, int arg) 
{
 static const char* extensions[]={ "L","R"};
 return gui_change_simple_enum(&conf.camera_rotation,change,extensions,sizeof(extensions)/sizeof(extensions[0]));
}
// -------------------------------------------------------------------
const char* gui_camera_position_enum (int change, int arg) 
{
 static const char* extensions[]={ "L", "R"};
 return gui_change_simple_enum(&conf.camera_position,change,extensions,sizeof(extensions)/sizeof(extensions[0]));
}

// -------------------------------------------------------------------

const char* gui_focus_enum(int change, int arg) 
{  
 static const char* extensions[]={ "INF", "HYP", "MAN","RF "};
 return gui_change_simple_enum(&conf.focus_mode,change,extensions,sizeof(extensions)/sizeof(extensions[0]));
}

// -------------------------------------------------------------------
#if !(CAM_HAS_ZOOM_LEVER)
const char* gui_range_enum(int change, int arg) 
{
    static const char* decades[]={"1","10","100","1000","10000"};

    conf.user_range+=change;
    if (conf.user_range<0)
        conf.user_range=(sizeof(decades)/sizeof(decades[0]))-1;
    else if (conf.user_range>=(sizeof(decades)/sizeof(decades[0])))
        conf.user_range=0;
    distance_step = atoi(decades[conf.user_range]);
    return decades[conf.user_range];
}

// -------------------------------------------------------------------

const char* gui_dist_step_enum(int change, int arg) 
{
    static const char* decades[]={"1","10","100","1000","10000"};

    conf.dist_step_size+=change;
    if (conf.dist_step_size<0)
        conf.dist_step_size=(sizeof(decades)/sizeof(decades[0]))-1;
    else if (conf.dist_step_size>=(sizeof(decades)/sizeof(decades[0])))
        conf.dist_step_size=0;
    dist_bracket_step = atoi(decades[conf.dist_step_size]);
    return decades[conf.dist_step_size];
}
#endif
// -------------------------------------------------------------------

const char* gui_dist_mode_enum(int change, int arg) 
{
 static const char* decades[]={"Off","Manual","Auto","Digiscope"};
 return gui_change_simple_enum(&conf.dist_mode,change,decades,sizeof(decades)/sizeof(decades[0]));
}

// -------------------------------------------------------------------

void cb_zebra_restore_screen() 
{
    if (!conf.zebra_restore_screen)
        conf.zebra_restore_osd = 0;
}

void cb_zebra_restore_osd() 
{
    if (conf.zebra_restore_osd)
        conf.zebra_restore_screen = 1;
}

// -------------------------------------------------------------------
const char* gui_histo_mode_enum(int change, int arg) 
{
    static const char* modes[]={ "Linear", "Log" };

    conf.histo_mode+=change;
    if (conf.histo_mode<0)
        conf.histo_mode=(sizeof(modes)/sizeof(modes[0]))-1;
    else if (conf.histo_mode>=(sizeof(modes)/sizeof(modes[0])))
        conf.histo_mode=0;

    histogram_set_mode(conf.histo_mode);
    return modes[conf.histo_mode];
}

// -------------------------------------------------------------------
const char* gui_histo_layout_enum(int change, int arg) 
{
    static const char* modes[]={ "RGB", "Y", "RGB Y",  "R G B", "RGB all", "Y all", "Blend", "Blend Y"};

    conf.histo_layout+=change;
    if (conf.histo_layout<0)
        conf.histo_layout=(sizeof(modes)/sizeof(modes[0]))-1;
    else if (conf.histo_layout>=(sizeof(modes)/sizeof(modes[0])))
        conf.histo_layout=0;

    if (conf.histo_layout==OSD_HISTO_LAYOUT_Y || conf.histo_layout==OSD_HISTO_LAYOUT_Y_argb) {
        histogram_set_main(HISTO_Y);
    } else {
        histogram_set_main(HISTO_RGB);
    }
    return modes[conf.histo_layout];
}

// -------------------------------------------------------------------
const char* gui_font_enum(int change, int arg) 
{
    static const char* fonts[]={ "Win1250", "Win1251", "Win1252", "Win1254", "Win1257"};

    conf.font_cp+=change;
    if (conf.font_cp<0)
        conf.font_cp=(sizeof(fonts)/sizeof(fonts[0]))-1;
    else if (conf.font_cp>=(sizeof(fonts)/sizeof(fonts[0])))
        conf.font_cp=0;

    if (change != 0) {
        font_set(conf.font_cp);
        if (!rbf_load(conf.menu_rbf_file))
            rbf_load_from_8x16(current_font);
        rbf_set_codepage(FONT_CP_WIN);
        gui_menu_init(NULL);
    }
    return fonts[conf.font_cp];
}

// -------------------------------------------------------------------
const char* gui_raw_nr_enum(int change, int arg) 
{
 static const char* modes[]={ "Auto", "Off", "On"};
 return gui_change_simple_enum(&conf.raw_nr,change,modes,sizeof(modes)/sizeof(modes[0]));
}
// -------------------------------------------------------------------
const char* gui_reader_codepage_enum(int change, int arg) 
{
 static const char* cps[]={ "Win1251", "DOS"};
 return gui_change_simple_enum(&conf.reader_codepage,change,cps,sizeof(cps)/sizeof(cps[0]));
}

// -------------------------------------------------------------------
const char* gui_shortcut_lang_enum(int change, int arg) 
{
 static const char* cps[]={"English", "Deutsch", "Francais","Espanol","Magyar"};
 return gui_change_simple_enum(&conf.shortcut_lang,change,cps,sizeof(cps)/sizeof(cps[0]));
}

// -------------------------------------------------------------------
const char* gui_zebra_mode_enum(int change, int arg) 
{
 static const char* modes[]={ "Blink 1", "Blink 2", "Blink 3", "Solid", "Zebra 1", "Zebra 2" };
 return gui_change_simple_enum(&conf.zebra_mode,change,modes,sizeof(modes)/sizeof(modes[0]));
}

// -------------------------------------------------------------------
const char* gui_zebra_draw_osd_enum(int change, int arg) 
{
 static const char* modes[]={ "Nothing", "Histo", "OSD" };
 return gui_change_simple_enum(&conf.zebra_draw_osd,change,modes,sizeof(modes)/sizeof(modes[0]));
}

// -------------------------------------------------------------------
const char* gui_zoom_value_enum(int change, int arg) 
{
 static const char* modes[]={ "X", "FL", "EFL", "FOV" };
 return gui_change_simple_enum(&conf.zoom_value,change,modes,sizeof(modes)/sizeof(modes[0]));
}

// -------------------------------------------------------------------
#if CAM_ADJUSTABLE_ALT_BUTTON
const char* gui_alt_mode_button_enum(int change, int arg) 
{

#if defined(CAMERA_s2is) || defined(CAMERA_s3is) || defined(CAMERA_s5is)
    static const char* names[]={ "Shrtcut", "Flash", "Timer", "ISO", "Video" };
    static const int keys[]={ KEY_PRINT, KEY_FLASH, KEY_TIMER, KEY_ISO, KEY_VIDEO };
#elif defined(CAMERA_g7)
    static const char* names[]={ "Print", "FE"};
    static const int keys[]={ KEY_PRINT, KEY_MICROPHONE };
#elif defined(CAMERA_g9) 
    static const char* names[]={ "Print", "FE"}; 
    static const int keys[]={ KEY_PRINT,  KEY_MICROPHONE };
#elif defined(CAMERA_a650)
    static const char* names[]={ "Print", "ISO"};
    static const int keys[]={ KEY_PRINT, KEY_ISO };
#elif defined(CAMERA_sx100is)|| defined(CAMERA_sx110is)
    static const char* names[]={ "Print", "Face"};
    static const int keys[]={ KEY_PRINT, KEY_FACE };
#elif defined(CAMERA_sx10) || defined(CAMERA_sx1)
    static const char* names[]={ "Shrtcut", "Flash", "Video"}; 
    static const int keys[]={ KEY_PRINT, KEY_FLASH, KEY_VIDEO }; 
#else
    #error camera alt-buttons not defined
#endif

    int i;

    for (i=0; i<sizeof(names)/sizeof(names[0]); ++i) {
        if (conf.alt_mode_button==keys[i]) {
            break;
        }
    }

    i+=change;
    if (i<0)
        i=(sizeof(names)/sizeof(names[0]))-1;
    else if (i>=(sizeof(names)/sizeof(names[0])))
        i=0;

    conf.alt_mode_button = keys[i];
    kbd_set_alt_mode_key_mask(conf.alt_mode_button);
    return names[i];
}
#endif

// -------------------------------------------------------------------
const char* gui_alt_power_enum(int change, int arg) 
{
    static const char* modes[]={ "Never", "Alt", "Script","Always" };

    conf.alt_prevent_shutdown+=change;
    if (conf.alt_prevent_shutdown<0)
        conf.alt_prevent_shutdown=(sizeof(modes)/sizeof(modes[0]))-1;
    else if (conf.alt_prevent_shutdown>=(sizeof(modes)/sizeof(modes[0])))
        conf.alt_prevent_shutdown=0;
    conf_update_prevent_shutdown(); 
    return modes[conf.alt_prevent_shutdown];
}


const char* gui_video_mode_enum(int change, int arg) 
{
 static const char* modes[]={ "Bitrate", "Quality"};
 return gui_change_simple_enum(&conf.video_mode,change,modes,sizeof(modes)/sizeof(modes[0]));
}

// -------------------------------------------------------------------
const char* gui_video_bitrate_enum(int change, int arg) 
{
    static const char* modes[]={ "0.25x", "0.5x","0.75x", "1x", "1.25x", "1.5x", "1.75x", "2x", "2.5x", "3x"};

    conf.video_bitrate+=change;
    if (conf.video_bitrate<0)
        conf.video_bitrate=0;
    else if (conf.video_bitrate>=(sizeof(modes)/sizeof(modes[0])))
        conf.video_bitrate=sizeof(modes)/sizeof(modes[0])-1;

    shooting_video_bitrate_change(conf.video_bitrate);
    return modes[conf.video_bitrate];
}

// -------------------------------------------------------------------
const char* gui_nd_filter_state_enum(int change, int arg) 
{
 static const char* modes[]={ "Off", "In", "Out" };
 return gui_change_simple_enum(&conf.nd_filter_state,change,modes,sizeof(modes)/sizeof(modes[0]));
}

// -------------------------------------------------------------------
const char* gui_tv_enum(int change, int arg) 
{
 static const char* modes[]={ "65\"","50\"", "40\"", "30\"", "25\"","20\"","15\"","13\"","10\"","8\"","6\"","5\"","4\"","3.2\"","2.5\"","2\"","1.6\"","1.3\"","1\"","Null","0.8","0.6","0.5","0.4","0.3","1/4","1/5","1/6","1/8","1/10","1/13","1/15","1/20","1/25","1/30","1/40","1/50","1/60","1/80", "1/100","1/125","1/160","1/200","1/250","1/320","1/400","1/500","1/640","1/800", "1/1000","1/1250","1/1600","1/2000", "1/2500","1/3200","1/4000", "1/5000", "1/6400", "1/8000", "1/10000", "1/12800", "1/16000", "1/20000", "1/26000","1/32000","1/40000"};
 return gui_change_simple_enum(&conf.tv_override,change,modes,sizeof(modes)/sizeof(modes[0]));
}
// -------------------------------------------------------------------
const char* gui_av_enum(int change, int arg) 
{

    static char buf[8];
    short prop_id;
    conf.av_override_value+=change;

    if (conf.av_override_value<0) conf.av_override_value=shooting_get_aperture_sizes_table_size()+6;
    else if (conf.av_override_value>shooting_get_aperture_sizes_table_size()+6) conf.av_override_value=0;
    if (conf.av_override_value == 0)  return "Off";
    else
    {
      prop_id=shooting_get_aperture_from_av96(shooting_get_av96_override_value());		
	sprintf(buf, "%d.%02d", (int)prop_id/100, (int)prop_id%100 );
	return buf; 
    }

}
// -------------------------------------------------------------------
const char* gui_tv_bracket_values_enum(int change, int arg) 
{
  static const char* modes[]={ "Off", "1/3 EV","2/3 EV", "1 EV", "1 1/3Ev", "1 2/3Ev", "2 Ev", "2 1/3Ev", "2 2/3Ev", "3 Ev", "3 1/3Ev", "3 2/3Ev", "4 Ev "};
  return gui_change_simple_enum(&conf.tv_bracket_value,change,modes,sizeof(modes)/sizeof(modes[0]));
}
// -------------------------------------------------------------------
const char* gui_bracket_type_enum(int change, int arg) 
{
  static const char* modes[]={"lighter","darker","alternate","3 times","5 times","7 times","9 times","11 times","13 times","15 times","17 times","19 times"};
  return gui_change_simple_enum(&conf.bracket_type,change,modes,sizeof(modes)/sizeof(modes[0]));
}


// -------------------------------------------------------------------

void gui_update_script_submenu() 
{
    register int p=0, i;

    for (i=0; i<sizeof(script_submenu_items_top)/sizeof(script_submenu_items_top[0]); ++p, ++i) 
    {
        script_submenu_items[p]=script_submenu_items_top[i];
    }
    for (i=0; i<SCRIPT_NUM_PARAMS; ++i) 
    {
        if (script_params[i][0]) {
            script_submenu_items[p].text=(int)script_params[i];
            script_submenu_items[p].type=MENUITEM_INT;
            script_submenu_items[p].value=&conf.ubasic_vars[i];
            ++p;
        }
    }
    for (i=0; i<sizeof(script_submenu_items_bottom)/sizeof(script_submenu_items_bottom[0]); ++p, ++i) 
    {
        script_submenu_items[p]=script_submenu_items_bottom[i];
    }
}

// -------------------------------------------------------------------

#if CAM_MULTIPART
static void gui_menuproc_swap_partitions(int arg)
{
 if (get_part_count()<2) gui_mbox_init(LANG_ERROR, LANG_ONLY_ONE_PARTITION, MBOX_BTN_OK|MBOX_TEXT_CENTER, NULL);
 else swap_partitions();
}
#endif

// -------------------------------------------------------------------
static volatile enum Gui_Mode gui_mode;
static volatile int gui_restore;
static volatile int gui_in_redraw;
static int gui_splash, gui_splash_mode;
static char osd_buf[32];

static Conf old_conf;

#ifdef OPT_SERIALIO
static void get_all_menus()
{
 all_menus[0]  = &misc_submenu;
 all_menus[1]  = &debug_submenu;
 all_menus[2]  = &battery_submenu;
 all_menus[3]  = &edge_overlay_submenu;
 all_menus[4]  = &grid_submenu;
 all_menus[5]  = &synch_submenu;
 all_menus[6]  = &visual_submenu;
 all_menus[7]  = &digiscope_submenu;
 all_menus[8]  = &experimental_submenu;
 all_menus[9]  = &osd_submenu;
 all_menus[10] = &zebra_submenu;
 all_menus[11] = &bracket_submenu;
 all_menus[12] = &video_submenu;
 all_menus[13] = &raw_submenu;
 all_menus[14] = &image_video_submenu;
 all_menus[15] = &stereo_submenu;
 all_menus[16] = &script_submenu;
 all_menus[17] = &reader_submenu;
 all_menus[18] = &sc_submenu;
 all_menus[19] = &root_menu;
 all_menus[20] = &user_submenu;
}
#endif

// -------------------------------------------------------------------
void gui_init()
{
    current_orientation = 0; // horizontal
    current_rotation = 0; // rotated left
    gui_mode = GUI_MODE_NONE;
    gui_restore = 0;
    gui_in_redraw = 0;
    gui_splash = (conf.splash_show)?SPLASH_TIME:0;
    gui_lang_init();
    draw_init();
#ifdef OPT_SERIALIO
    if(!conf.ricoh_ca1_mode)conf.remote_enable=0;    // for serial I/O disable USB switch, checks mmio directly for switch press
#endif
    exposition_thresh = screen_size/500;
    voltage_step = (conf.batt_step_25)?25:1;
    if(conf.batt_volts_max<=1500) conf.batt_volts_max = get_vbatt_max();
    if(conf.batt_volts_min<=1500) conf.batt_volts_min = get_vbatt_min();
    digi_mode = 0;
    if(!auto_started) conf.outline_mode=0;
    conf.invert_playback=0;
    conf.stereo_mode=0;
    disable_shutdown();
    myav=shooting_get_min_real_aperture();
   // load_bad_pixels_list("A/CHDK/badpixel");
#ifdef OPT_SERIALIO
    get_all_menus();   // pointers to all menus
#endif
	
}

// -------------------------------------------------------------------
static void gui_load_edge_selected( const char* fn ) 
{
    if( fn )
	load_edge_overlay(fn);
}
// -------------------------------------------------------------------

enum Gui_Mode gui_get_mode() 
{
    return gui_mode;
}

// -------------------------------------------------------------------
void gui_set_mode(enum Gui_Mode mode) 
{
    gui_mode = mode;
}

// -------------------------------------------------------------------
void gui_force_restore() 
{
    gui_restore = gui_in_redraw;
}

// -------------------------------------------------------------------

void gui_redraw()
{
    enum Gui_Mode gui_mode_old;
    static int show_script_console=0;
   int rec = ((mode_get()&MODE_MASK) == MODE_REC);
    if (gui_splash)
     {
   
        if (gui_splash>(SPLASH_TIME-4)) 
        {
            gui_draw_spm_splash();
        } 

        else if (gui_splash==1 && (mode_get()&MODE_MASK) == gui_splash_mode && (gui_mode==GUI_MODE_NONE || gui_mode==GUI_MODE_ALT)) 
        {
            draw_restore();
        }

        --gui_splash;
     }

  if (conf.camera_orientation == 2) draw_set_draw_proc(draw_pixel_inverted);
  else if (conf.camera_orientation == 1 && conf.camera_rotation == 0) draw_set_draw_proc(draw_pixel_rotated_left);
  else if (conf.camera_orientation == 1 && conf.camera_rotation == 1) draw_set_draw_proc(draw_pixel_rotated_right);
  else draw_set_draw_proc(NULL);

    if(current_orientation != conf.camera_orientation || current_rotation != conf.camera_rotation)  // orientation changed
     {
      current_orientation = conf.camera_orientation;
      current_rotation = conf.camera_rotation;
      draw_restore();
     }
    gui_in_redraw = 1;
    gui_mode_old = gui_mode;

    switch (gui_mode) {
        case GUI_MODE_MENU:
            gui_menu_draw();
           if((conf.camera_rotation == 1)&& (conf.camera_orientation == 1))         // vertical, rotated right
            draw_txt_string(35, 14, "<CNF>", MAKE_COLOR((rec)?COLOR_GREY:COLOR_GREY_P,(rec)? COLOR_WHITE:COLOR_WHITE_P));
            else
            draw_txt_string(20, 14, "<CNF>", MAKE_COLOR((rec)?COLOR_GREY:COLOR_GREY_P,(rec)? COLOR_WHITE:COLOR_WHITE_P));
            break;

        case GUI_MODE_ALT:

            gui_draw_osd();

           if((conf.camera_rotation == 1)&& (conf.camera_orientation == 1))        // vertical, rotated right
            draw_txt_string(35, 14, "<ALT>", MAKE_COLOR((rec)?COLOR_GREY:COLOR_GREY_P,(rec)? COLOR_WHITE:COLOR_WHITE_P));
            else
            draw_txt_string(20, 14, "<ALT>", MAKE_COLOR((rec)?COLOR_GREY:COLOR_GREY_P,(rec)? COLOR_WHITE:COLOR_WHITE_P));
            if ((mode_get()&MODE_MASK) == MODE_REC) 
               {
                if((conf.camera_rotation == 1)&& (conf.camera_orientation == 1))   // vertical, rotated right
                draw_txt_string(15, 14, script_title, MAKE_COLOR(COLOR_GREY,COLOR_WHITE));
                else
                draw_txt_string(0, 14, script_title, MAKE_COLOR(COLOR_GREY,COLOR_WHITE));
                if (state_kbd_script_run) show_script_console=5;

                if (show_script_console) 
                  {
                    --show_script_console;
			  md_draw_grid();
                    script_console_draw();
                   }

               }

              if ((mode_get()&MODE_MASK) == MODE_PLAY)
               {
                 draw_txt_string(27, 14,(!playback && conf.remote_enable)? "Image   ":(!playback && !conf.remote_enable)?"Image UL":(playback && conf.remote_enable)?"Movie   ":"Movie UL", MAKE_COLOR(COLOR_GREY_P,COLOR_WHITE_P));
               }

            break;

        case GUI_MODE_NONE:
            gui_draw_osd();
            break;

        case GUI_MODE_PALETTE:
            gui_palette_draw();
            break;

        case GUI_MODE_MBOX:
            gui_mbox_draw();
            break;

        case GUI_MODE_FSELECT:
            gui_fselect_draw();
            break;

        case GUI_MODE_READ:
            gui_read_draw();
            break;

        case GUI_MODE_OSD:
            gui_osd_draw();
            break;

        case GUI_MODE_MPOPUP:
            gui_mpopup_draw();
            break;

        case GUI_MODE_SDM_CONFIGURATION:
            gui_sdm_config_draw();
            break;

        default:
            break;
    }
    
    gui_in_redraw = 0;
   if ((gui_mode_old != gui_mode && (gui_mode_old != GUI_MODE_NONE && gui_mode_old != GUI_MODE_ALT) && (gui_mode != GUI_MODE_MBOX && gui_mode != GUI_MODE_MPOPUP)) || gui_restore)

       {
        gui_restore = 0;
       }   

}


// -------------------------------------------------------------------
static inline void conf_store_old_settings() 
{
    old_conf=conf;
}

// -------------------------------------------------------------------
static inline int conf_save_new_settings_if_changed() 
{
    if (memcmp(&old_conf, &conf, sizeof(Conf)) != 0) 
    {
        conf_save();
        conf_store_old_settings();
        return 1;
    }
    return 0;
}


// -------------------------------------------------------------------
void gui_kbd_process()
{
 int clicked_key;
static  long focus_inc;
static  long focus_pos;
static int pressed = 0,cfar,cnear,prev;
static float coc;
static long num,denom;
static float realav;
static float realfl;
coc=(float)(get_sensor_width())/1200000;
realfl=(float)(get_focal_length(lens_get_zoom_point()))/1000;
realav=(float)(myav)/100;

prev=lens_get_focus_pos_from_lense();

    if (kbd_is_key_clicked(KEY_MENU))
   {
        switch (gui_mode)
       {
            case GUI_MODE_ALT:
                gui_menu_init(&user_submenu);
                gui_mode = GUI_MODE_MENU;
                draw_restore();
                break;
            case GUI_MODE_MENU:
   
                conf_save_new_settings_if_changed();
   
                gui_mode = GUI_MODE_ALT;
                draw_restore();
                break;
            case GUI_MODE_PALETTE:
            case GUI_MODE_OSD:
            case GUI_MODE_SDM_CONFIGURATION:
                draw_restore();
                gui_mode = GUI_MODE_MENU;
                break;
            case GUI_MODE_FSELECT:
                gui_fselect_kbd_process();
                break;
            case GUI_MODE_READ:
                gui_read_kbd_process();
                draw_restore();
                gui_mode = GUI_MODE_MENU;
                break;
            default:
                break;
        }
        return;
    } // _key_menu
    
   switch (gui_mode) 
   {
// -----------------------------------------------------------------------------------------------------
//                                                      ALT mode
// -----------------------------------------------------------------------------------------------------
     case GUI_MODE_ALT:
        if ((!conf.user_range_set || !shooting_can_focus()) && !shooting_get_drive_mode())                  // not manual-focus mode (for some cameras)
          {
            if( (kbd_is_key_clicked(KEY_RIGHT) && conf.camera_orientation == 0) || (kbd_is_key_clicked(KEY_DOWN) && conf.camera_orientation == 1 && conf.camera_rotation == 0) || (kbd_is_key_clicked(KEY_UP) && conf.camera_orientation == 1 && conf.camera_rotation == 1)|| (kbd_is_key_clicked(KEY_LEFT) && conf.camera_orientation == 2))
             {
               if(conf.osd_page_index >= OSD_PAGES-1) conf.osd_page_index = 0;
               else conf.osd_page_index++;
               conf.layout_mode = osd_pages[conf.osd_page_index];
               draw_restore();
               gui_osd_draw_stereo();
           
             }   // key_right

            if( (kbd_is_key_clicked(KEY_LEFT) && conf.camera_orientation == 0) || (kbd_is_key_clicked(KEY_UP) && conf.camera_orientation == 1 && conf.camera_rotation == 0) || (kbd_is_key_clicked(KEY_DOWN) && conf.camera_orientation == 1 && conf.camera_rotation == 1)|| (kbd_is_key_clicked(KEY_RIGHT) && conf.camera_orientation == 2))
             {
               if(conf.osd_page_index == 0) conf.osd_page_index = OSD_PAGES-1;
               else conf.osd_page_index--;
               conf.layout_mode = osd_pages[conf.osd_page_index];
               draw_restore();
               gui_osd_draw_stereo();
 
             }   // key_left 
           }

       else if(conf.user_range_set && shooting_can_focus())                                                 // focus override and can focus
         {
           if(kbd_is_key_clicked(KEY_RIGHT))                                                                // focus further
            {
             if(!kbd_is_key_pressed(KEY_SHOOT_HALF))
             {
             denom = (long)(coc*(lens_get_focus_pos_from_lense()-realfl)-((realfl/realav)*realfl));         // denom for farpoint equation
             cfar = (int)(-((realfl/realav)*realfl*lens_get_focus_pos_from_lense())/denom);                 // current farpoint
             denom = (long)(coc*cfar-((realfl*realfl)/realav));                                                
             num = (long)(coc*cfar*realfl-((cfar*realfl*realfl)/realav));                                   // current farpoint will be nearpoint of next focus
             if((short)(num/denom)<0)
              {
               lens_set_focus_pos(65000);
               conf.distance_setting=65000;                                                                 // focus override value
               prev=65535;
              }
             else if((short)(num/denom)>prev) 
              {
               lens_set_focus_pos((short)(num/denom)+fpd);
               conf.distance_setting=(short)(num/denom);                                                   // focus override value
               prev=(short)(num/denom);
              }
             }
            else
             {
              conf.distance_setting+=conf.subj_dist_bracket_value;                                          // fine adjust with half-press
             }
             sfo = 1;                                                                                       // set focus override when exiting ALT mode
             conf.focus_mode=2;                                                                             // focus override MANual
            }

           else if(kbd_is_key_clicked(KEY_LEFT))                                                            // focus nearer
            {
             if(!kbd_is_key_pressed(KEY_SHOOT_HALF))
             {
             denom=(long)(realfl*realfl + (coc*realav*(lens_get_focus_pos_from_lense()-realfl)));           // denom for current nearpoint
             cnear = (int)((realfl*realfl*lens_get_focus_pos_from_lense())/denom);                          // current nearpoint
             denom=(long)((lens_get_focus_pos_from_lense()*coc*realav)+(realfl*realfl));
             num = (long)(lens_get_focus_pos_from_lense()*(realfl*realfl-(coc*realav*realfl)));

               if(((short)(num/denom)>0)&&((short)(num/denom)<prev)) 
               {
                lens_set_focus_pos((short)(num/denom)+fpd);
                conf.distance_setting=(short)(num/denom);
                prev=(short)(num/denom);
               }
             }
            else
             {
              conf.distance_setting-=conf.subj_dist_bracket_value;
             }
             sfo = 1;                                                                                     // set focus override when exiting ALT mode
             conf.focus_mode=2;
            }

            else if(kbd_is_key_clicked(KEY_UP)&& !shooting_get_drive_mode())                             // range-finder set in non-bracketing modes
            {
              lens_set_focus_pos(rf_focus);
              conf.distance_setting=rf_focus;
              conf.focus_mode=3;
             sfo = 1;                                                                                       // set focus override when exiting ALT mode
            }

            else if(kbd_is_key_clicked(KEY_DOWN)&& !shooting_get_drive_mode())                             // hyperfocal set in non-bracketing modes
            {
             myhyp=shooting_get_hyperfocal_distance();
             lens_set_focus_pos(myhyp);
             conf.distance_setting=myhyp;
             conf.focus_mode=1;
             sfo = 1;                                                                                       // set focus override when exiting ALT mode
            }
         }

           static int dd = 0;                                                                                // duration DISPLAY key pressed
           if(kbd_is_key_pressed(KEY_DISPLAY)&& conf.outline_mode) dd++;
           if(kbd_is_key_pressed(KEY_DISPLAY)&&(dd==50)) play_sound(4);
           if(!kbd_is_key_pressed(KEY_DISPLAY)&& dd)                                                         // key was pressed but is now released
            {
             conf.edge_overlay_lock=!conf.edge_overlay_lock;                                                 // toggle overlay lock
            if(!conf.edge_overlay_lock)unlock_overlay();
             dd=0;                                                                                           // reset key duration counter
            }

           static int sd = 0;                                                                                // duration SET key pressed

           if(kbd_is_key_pressed(KEY_SET)) sd++;
           if(kbd_is_key_pressed(KEY_SET)&&(sd==50)) play_sound(4);
          if(!kbd_is_key_pressed(KEY_SET)&& sd &&(!(shooting_get_drive_mode() && (conf.dist_mode || conf.tv_bracket_value))))      // key was pressed but is now released
          {
              if(!(conf.dist_mode && shooting_get_drive_mode())&&(sd<=50)&& ((mode_get()&MODE_MASK) == MODE_REC))          // SET pressed for <= 0.5 seconds
              {
                conf.show_osd = !conf.show_osd;
                draw_restore();
              }      
            else 

              if(!(conf.dist_mode && shooting_get_drive_mode())&&(sd>50))                                  // SET pressed for > 0.5 seconds
              {

               if(((mode_get()&MODE_MASK) == MODE_REC)&& !conf.camera_orientation)                          // only works in horizontal mode
                {
                 conf.outline_mode=!conf.outline_mode;
                 if(!conf.outline_mode) edge_count=0;
                }

                else
                if((mode_get()&MODE_MASK) == MODE_PLAY)conf.remote_enable=!conf.remote_enable;              // toggle USB download

              }

              sd=0;
            
          }

            if(kbd_is_key_clicked(KEY_SET) && shooting_get_drive_mode() && (conf.dist_mode || conf.tv_bracket_value))    // in focus-bracketing modes, display stack text file

              {
                 gui_draw_read(1);
              } 

          break;
// -------------------------------------------------------------------------------------------------------

    	case GUI_MODE_MENU:
           if(kbd_is_key_clicked(KEY_SHOOT_HALF))
            {
             play_sound(4);
             if(conf.flash_dim_value>=17)conf.flash_dim_value-=16;  // enable right-flash dimming, range 1 to 12 1/3EV's or 0 = no dimming
             else if(conf.flash_dim_value)conf.flash_dim_value+=16;                         // disable right-flash dimming, range 17 to 28 
            }
            gui_menu_kbd_process();
            break;
      case GUI_MODE_PALETTE:
            gui_palette_kbd_process();
            break;

    	case GUI_MODE_MBOX:
            gui_mbox_kbd_process();
            break;

    	case GUI_MODE_FSELECT:
            gui_fselect_kbd_process();
            break;

    	case GUI_MODE_READ:
            gui_read_kbd_process();
            break;

    	case GUI_MODE_OSD:
            gui_osd_kbd_process();
            break;

        case GUI_MODE_MPOPUP:
            gui_mpopup_kbd_process();
             break;
        default:
            break;
    }
}

// -------------------------------------------------------------------
void gui_kbd_enter()
{
    conf_store_old_settings();
    playback = !playback;
    gui_mode = GUI_MODE_ALT;
    conf_update_prevent_shutdown();
}

// -------------------------------------------------------------------
void gui_kbd_leave()
{
    // XXX restore palette

    conf_save_new_settings_if_changed();

    ubasic_error = 0;
    draw_restore();
    if (gui_mode == GUI_MODE_READ && !rbf_load(conf.menu_rbf_file))
        rbf_load_from_8x16(current_font);
    rbf_set_codepage(FONT_CP_WIN);
   if(get_sd_status() && ((conf.dist_mode!=3)&& !tl.running)) enable_shutdown(); // if shutdown disabled but not in digiscope mode or time-lapse, enable it.
    if(sfo)                                                                      // set focus override when exiting ALT mode
    {
     conf.user_range_set=1;
     sfo=0;
    }
    gui_mode = GUI_MODE_NONE;
    conf_update_prevent_shutdown();
}

// -------------------------------------------------------------------
extern long physw_status[3];
extern long GetPropertyCase(long opt_id, void *buf, long bufsize);

// -------------------------------------------------------------------------------------------------------
//                                            gui_draw_osd()
// -------------------------------------------------------------------------------------------------------

void gui_draw_osd() 
{
    unsigned int m, n = 0, mode_photo, mode_video;
    coord x;
    static int flashlight = 0, zebra = 0, zebra_init = 0, pressed = 0,usb_on = 0,header_on = 0,nd_on=0; 
    static int slave_on=0;
    static int afl_on=0, sun_on=0;
    static int far_flag = 0;                   // when using half-press at max telephoto indicates whether near or far recorded
    static int afc = 0;                       // time down button pressed for auto-focus lock 
    static int fm = 0;                        // time half-press for focus measure
    static unsigned char cnt = 0;             // to flash 'F' if right cam flash-dim active
   int shoot_mode=0;
    m = mode_get();
    mode_photo = (m&MODE_MASK) == MODE_PLAY || 
                 !((m&MODE_SHOOTING_MASK)==MODE_VIDEO_STD || (m&MODE_SHOOTING_MASK)==MODE_VIDEO_SPEED || (m&MODE_SHOOTING_MASK)==MODE_VIDEO_COMPACT ||
                   (m&MODE_SHOOTING_MASK)==MODE_VIDEO_MY_COLORS || (m&MODE_SHOOTING_MASK)==MODE_VIDEO_COLOR_ACCENT || (m&MODE_SHOOTING_MASK)==MODE_STITCH);


    fpd = shooting_get_lens_to_focal_plane_width();
int flashStatus = shooting_get_prop(PROPCASE_FLASH_MODE);  

// once only on power-up 


if((conf.dist_mode) && shooting_can_focus() && !digi_mode &&((((shooting_get_drive_mode()==1)||(shooting_get_drive_mode()==CAM_PROPSET+1))&&(conf.bracket_type>2))|| (shooting_get_drive_mode()== CAM_PROPSET+1)))
    {
     digi_mode = 1;
     conf.synch_enable=1;
     previous_focus = conf.digi_ref_dist;
     inf_focus = 0;
     conf.user_range_set=0;                                      // for Masuji code
    }


#if !defined(CAMERA_ixus40)
  if (!((canon_menu_active==(int)&canon_menu_active-4) && (canon_shoot_menu_active==0)))  return; 
#endif

  if((shooting_get_drive_mode()==CAM_PROPSET+1)&& !tl.running && conf.bracket_type>2)conf.bracket_type = 2;  // in custom-timer bracketing ensure valid setting in non time-lapse mode
// ...................................................................................
 if ((kbd_is_key_pressed(KEY_SHOOT_HALF)||get_soft_half_press())&&(gui_mode!=GUI_MODE_ALT)&& (mode_photo || (m&MODE_SHOOTING_MASK)==MODE_STITCH)) 
   {
      myav=shooting_get_real_av();                                                                         // aperture number * 100 available on key half-press
      myhyp=shooting_get_hyperfocal_distance();

     if (lens_get_zoom_point()==(zoom_points-1))                                                           // at maximum zoom
     { 
       ++fm;                                                                                               // time how long key pressed or focus achieved
      if(!far_flag && (fm>=20))
      {
       subject_near=lens_get_focus_pos_from_lense();                                                       // at maximum zoom ?
       far_flag=1;
       fm=0;
       if(shooting_get_prop(PROPCASE_FOCUS_OK))                                                                                               // next half-press records far
       play_sound(4); 
       }
     else if(far_flag && (fm>=20))
       {
        subject_far=lens_get_focus_pos_from_lense();                                                        // at maximum zoom ?
        far_flag=0; 
        fm=0;  
        if(shooting_get_prop(PROPCASE_FOCUS_OK))                                                                                          // next half-press records near
        play_sound(4);                                                                                           
        conf.focus_mode=3;                                                                                  // set RF focus mode
       }
      }

      if(!shooting_get_focus_mode() && kbd_is_key_pressed(KEY_RIGHT)) 
      {
       ++afc;                                                                                              // time the right key
       if(afc>=3)
       {
        afl = !afl;
        if(afl)DoAFLock();
        else UnlockAF();
        afc=0;
       }
      }

              if(kbd_is_key_pressed(KEY_SET))

              {
                if (!pressed) 
                 {
                  conf.disable_overrides = !conf.disable_overrides;
                  if(!conf.disable_overrides && conf.user_range_set && (conf.focus_mode==2))conf.distance_setting = current_focus;
                  if (!conf.show_osd) 
                   {
                    draw_restore();
                   }
                   pressed = 1;
                 }
               } 

            else 
               {
                pressed = 0;
               }
      

    } // key half-pressed
// ...................................................................................


      if(((m&MODE_MASK)==MODE_REC) && ((conf.show_osd && ((conf.edge_and_osd && conf.outline_mode)|| !conf.outline_mode))||(!conf.show_osd && conf.outline_mode && !conf.edge_and_osd)))
 {

// -------------------------------------------- Zoom,Synch & Direct/Script --------


// int flashStatus = shooting_get_prop(PROPCASE_FLASH_MODE);           

    int myfl=get_focal_length(lens_get_zoom_point());                            // returned as fl * 1000 as INT
    int zp=lens_get_zoom_point();
    int synch_x = conf.synch_pos.x;
    if(conf.camera_orientation==1)synch_x -= 120;                                   // vertical

 if(tl.running && (tl.shoot_mode!=3))                                            // time-lapse but not burst mode
 {
  if(tl_mod!=2) // not custom timer
   {
    sprintf(osd_buf,"%-26s",tl_modes[tl_mod]);
    draw_string(synch_x-6*FONT_WIDTH, conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
    if(tl.shutdown)draw_string(synch_x-6*FONT_WIDTH+strlen(tl_modes[tl_mod])*FONT_WIDTH, conf.synch_pos.y," shutdown",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
   }
 else                                         // custom TVtimer 
   {
     sprintf(osd_buf,"%-26s",tl_modes[tl_mod]);
     draw_string(synch_x-6*FONT_WIDTH, conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
     if(!conf.dist_mode) // not focus mode so assume Tv mode
     {
      draw_string(synch_x+3*FONT_WIDTH, conf.synch_pos.y,(!conf.bracket_type)?"lighter":(conf.bracket_type==1)?"darker":"alternate",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
     } 
   }
    
 }

   if(tl.running && tl.shoot_mode==3)                  // time-lapse burst mode
   {
    sprintf(osd_buf,"%-26s","Burst mode");
    draw_string(synch_x-6*FONT_WIDTH, conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
   }

// ---------------------------------------------------------------------------------------------------------------------------
//                                                          HEADER
// ---------------------------------------------------------------------------------------------------------------------------
 if((conf.compact!=2)&&(recreview_hold==0)) // not invisible
 {
  header_on = 1;
  if(!conf.compact) // full header
  {
   if(!tl.running) // time-lapse script not running
    {
     if((conf.dist_mode ||conf.tv_bracket_value) &&(shooting_get_drive_mode()==1))shoot_mode=1; // C_FAST  continuous non-synch
     else if((shooting_get_drive_mode()== CAM_PROPSET+1)&& !conf.custom_timer_synch)shoot_mode=2;// T_FAST timer non-synch
     else if(!shooting_get_drive_mode()&&conf.dist_mode)shoot_mode=3; // S_FAST  single non-synch
     else if(!conf.tv_bracket_value && !conf.dist_mode && (shooting_get_drive_mode()==1)&& (conf.bracket_type>2))shoot_mode=4;//BURST non synch, stop/started with switch
     else if(!conf.tv_bracket_value && !conf.dist_mode && (shooting_get_drive_mode()==1)&& (conf.bracket_type<=2))shoot_mode=5;// SPORTS, synch
     else if((shooting_get_drive_mode()== CAM_PROPSET+1)&& conf.custom_timer_synch) shoot_mode=6;  // T_SYNC synchronised timer bracketing or no bracketing
     else if(!shooting_get_drive_mode()&&!conf.dist_mode&&conf.synch_enable)shoot_mode=7; // STEREO
     else shoot_mode=0;// direct
     draw_string((conf.camera_orientation ==1)?synch_x-5*FONT_WIDTH:synch_x-6*FONT_WIDTH,conf.synch_pos.y,(shoot_mode==0)?"DIRECT ":(shoot_mode==1)?"C_FAST ":(shoot_mode==2)?"T_FAST ":(shoot_mode==3)?"S_FAST ":(shoot_mode==4)?"BURST  ":(shoot_mode==5)?"SPORTS ":(shoot_mode==6)?"T_SYNC ":"STEREO ",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1); 
     draw_string((conf.camera_orientation ==1)?synch_x+2*FONT_WIDTH:synch_x+FONT_WIDTH, conf.synch_pos.y, "Zoom:",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
     if(conf.zoom_value==1)sprintf(osd_buf,"%2d.%d",myfl/1000,(myfl%1000)/100);
     else if (!conf.zoom_value)sprintf(osd_buf,"%4d",zp);
     else if(conf.zoom_value==3)sprintf(osd_buf," %2d'",(conf.camera_orientation ==1)?(int)(((30*get_fov(zp)/4)+5)/10):get_fov(zp));
     else sprintf(osd_buf,"%4d",get_effective_focal_length(zp)/1000);
     draw_string(synch_x+6*FONT_WIDTH, conf.synch_pos.y, osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
 
   }

    if(!(conf.dist_mode && shooting_can_focus())&& !tl.running) // not in active focus-bracketing mode and time-lapse not running
    {
      if(conf.synch_enable && conf.remote_enable)
       {

        if (kbd_is_key_pressed(KEY_SHOOT_HALF) || get_soft_half_press())  
        n = conf.synch_delay_coarse_value * 1000 + conf.synch_delay_value;
        else n = conf.synch_user_delay;
        if(conf.synch_delay_enable)
         {

         sprintf(osd_buf,"Synch:%d.%04d",n/10000,n%10000);
         draw_string(synch_x+13*FONT_WIDTH,conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
         }
        else
         {
          sprintf(osd_buf,"Synch:%6s","");
          draw_string(synch_x+13*FONT_WIDTH,conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
 
         }
         if(shoot_mode>=5)  // flash synch modes single,SPORTS,timer-bracketing and STEREO (not FAST)
           {
// ------------------------------------------------------ Right flash dim status (on/off) ----------------------------------------
#if CAM_CURTAIN
              if((shooting_will_use_flash()|| (flashStatus==1))&& !shooting_get_prop(PROPCASE_CURTAIN) && ((m&MODE_SHOOTING_MASK) != MODE_M)&& (!conf.flash_dim_value ||(conf.flash_dim_value>16)||(conf.flash_dim_value && (conf.flash_dim_value<=12)&&(((cnt++)&7)==0)))) draw_string(synch_x+10*FONT_WIDTH,conf.synch_pos.y," F ",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
#else
              if((shooting_will_use_flash()|| (flashStatus==1))&& ((m&MODE_SHOOTING_MASK) != MODE_M)&& (!conf.flash_dim_value ||(conf.flash_dim_value>16)||(conf.flash_dim_value && (conf.flash_dim_value<=12)&&(((cnt++)&7)==0)))  ) draw_string(synch_x+10*FONT_WIDTH,conf.synch_pos.y," F ",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
 #endif
// ------------------------------------------------------  Bright screen status (on/off) -----------------------------------------
            else if(conf.bright_screen && (flashStatus==1) && ((m&MODE_SHOOTING_MASK) == MODE_M)&& (((cnt++)&7)==0))draw_string(synch_x+10*FONT_WIDTH,conf.synch_pos.y," B ",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
          
            else draw_string(synch_x+10*FONT_WIDTH,conf.synch_pos.y,"   ",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
          }
            else draw_string(synch_x+10*FONT_WIDTH,conf.synch_pos.y,"   ",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
       } // synch and remote enabled
 
#if OPT_SERIALIO      
        else if(!conf.remote_enable && !conf.ricoh_ca1_mode) // Serial I/O mode active
        {
          sprintf(osd_buf,"%12s",(ttyMasterSlave==2)?"SIO Slave":"SIO Master");
          draw_string(synch_x+13*FONT_WIDTH,conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
        }
#endif

       else                             // synch not enabled os USB download enabled, clear existing Synch text
        {
          sprintf(osd_buf,"%12s","");
          draw_string(synch_x+13*FONT_WIDTH,conf.synch_pos.y,osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
        }

      } // _not focus bracketing mode and time-lapse not running
// .........

       else                                                                       // focus-bracketing modes (including time-lapse)
       {
        if(!tl.running && conf.dist_mode && shooting_can_focus())                 // time-lapse not running
         {
          sprintf(osd_buf," Focus %8d",lens_get_focus_pos_from_lense());
          draw_string(synch_x+10*FONT_WIDTH,conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
         }
        else if(tl.running && conf.dist_mode && shooting_can_focus())              // time-lapse running
         {
          sprintf(osd_buf,"%5d",lens_get_focus_pos_from_lense());
          draw_string(synch_x+20*FONT_WIDTH,conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
         }
       }
// .........
    } // not compact mode

  else if(!tl.running) // compact mode and time-lapse not running
    {
      synch_x+=14*FONT_WIDTH;
     if(conf.zoom_value==1)sprintf(osd_buf,"%2d.%d",myfl/1000,(myfl%1000)/100);
     else if (!conf.zoom_value)sprintf(osd_buf,"%4d",zp);
     else if(conf.zoom_value==3)sprintf(osd_buf," %2d'",get_fov(zp));
     else sprintf(osd_buf,"%4d",get_effective_focal_length(zp)/1000);
      draw_string(synch_x, conf.synch_pos.y, osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
      if(conf.synch_enable && conf.remote_enable)
       {
        if (kbd_is_key_pressed(KEY_SHOOT_HALF) || get_soft_half_press())  
        n = conf.synch_delay_coarse_value * 1000 + conf.synch_delay_value;
        else n = conf.synch_user_delay;
        if(conf.synch_delay_enable)
         {
         sprintf(osd_buf,"%d.%04d",n/10000,n%10000);
         draw_string(synch_x+5*FONT_WIDTH,conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
         }
        else
         {
          sprintf(osd_buf,"%6s","");
          draw_string(synch_x+5*FONT_WIDTH,conf.synch_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
         }
         if(shoot_mode>=5)  // flash synch modes single,SPORTS,timer-bracketing and STEREO (not FAST
           {
#if CAM_CURTAIN
            if((shooting_will_use_flash()|| (flashStatus==1))&& !shooting_get_prop(PROPCASE_CURTAIN) && ((m&MODE_SHOOTING_MASK) != MODE_M)&& (!conf.flash_dim_value ||(conf.flash_dim_value>16)||(conf.flash_dim_value && (conf.flash_dim_value<=12)&&(((cnt++)&7)==0))) ) draw_string(synch_x+4*FONT_WIDTH,conf.synch_pos.y,"F",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
#else
            if((shooting_will_use_flash()|| (flashStatus==1))&& ((m&MODE_SHOOTING_MASK) != MODE_M)&& (!conf.flash_dim_value ||(conf.flash_dim_value>16)||(conf.flash_dim_value && (conf.flash_dim_value<=12)&&(((cnt++)&7)==0))) ) draw_string(synch_x+4*FONT_WIDTH,conf.synch_pos.y,"F",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
#endif     
            else draw_string(synch_x+4*FONT_WIDTH,conf.synch_pos.y," ",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
           }
       } // synch and remote enabled
        
       else                             // synch not enabled os USB download enabled, clear existing Synch text
        {
          sprintf(osd_buf,"%6s","");
          draw_string(synch_x+5*FONT_WIDTH,conf.synch_pos.y,osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
        }

    } // _compact mode and time-lapse not running
  } // not invisible mode 

 else  // do not display header

  {
   if(header_on ) // clear header area
     {
      sprintf(osd_buf,"%31s","");
      draw_string(conf.synch_pos.x-6*FONT_WIDTH,conf.synch_pos.y,osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
      header_on = 0;
     }
   }
              
      int ovr_y = conf.overrides_pos.y;
      int ovr_x = conf.overrides_pos.x;
      if(get_usb_power(1))
       {
        usb_on = 1;
        draw_string(ovr_x,ovr_y,"USB",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
       }

       else if(!get_usb_power(1) && usb_on)
       {
        usb_on = 0;
        draw_string(ovr_x,ovr_y,"   ",MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
       }

       ovr_y += FONT_HEIGHT
;
      if(conf.slave_flash && (flashStatus==2))                // slave mode and normal flash Off
       {
        slave_on=1;
        draw_string(ovr_x,ovr_y,"SLAVE",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
        ovr_y += FONT_HEIGHT;
       }
       else if ((!conf.slave_flash ||(flashStatus!=2)) && slave_on)   // delete SLAVE if no longer On or main flash enabled
       {
        slave_on=0;
        draw_string(ovr_x,ovr_y,"     ",MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
       }
      
       ovr_y += FONT_HEIGHT;

      if(afl)
       {
        afl_on=1;
        draw_string(ovr_x,ovr_y,"AFL",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
       }
      else if(!afl && afl_on)
       {
        afl_on=0;
        draw_string(ovr_x,ovr_y,"   ",MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
       }
      ovr_y += FONT_HEIGHT;

      if(conf.sunrise && (shooting_get_drive_mode()==1))
       {
        sun_on=1;
        draw_string(ovr_x,ovr_y,"SUN",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
       }
      else if(!(conf.sunrise && (shooting_get_drive_mode()==1)) && sun_on)
       {
        sun_on=0;
        draw_string(ovr_x,ovr_y,"   ",MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
       }
      ovr_y += FONT_HEIGHT;

      if(conf.disable_overrides)
      {
        draw_string(ovr_x,ovr_y,"OFF",(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
        ovr_y += FONT_HEIGHT;
      }
// ..........................................................................
//                      display override/bracketing legends
// ..........................................................................

      else if(!(tl.running && (tl.shoot_mode==3)))                                // do not show legends in time-lapse burst mode
      { 
        if(conf.tv_override_enable)
        {
         draw_string(ovr_x,ovr_y,"Tv ",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
         ovr_y += FONT_HEIGHT;
        }
       
        if(conf.av_override_value)
        {
         draw_string(ovr_x,ovr_y,"Av ",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
         ovr_y += FONT_HEIGHT;
        }

        if(shooting_can_focus() && conf.user_range_set)                          // focus override and can focus
        {    
          if(conf.focus_mode == 0)
           {
            draw_string(ovr_x,ovr_y,"INF",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            draw_string(ovr_x,ovr_y,"65000",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
           }
          if(conf.focus_mode == 1)
           {
            draw_string(ovr_x,ovr_y,"HYP",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            sprintf(osd_buf,"%5d",myhyp);
            draw_string(ovr_x,ovr_y,osd_buf,(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;

           }
          if(conf.focus_mode == 2)
           {
            draw_string(ovr_x,ovr_y,"MAN",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            sprintf(osd_buf,"%5d",conf.distance_setting);
            draw_string(ovr_x,ovr_y,osd_buf,(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;

           }
          if(conf.focus_mode == 3)
           {
            draw_string(ovr_x,ovr_y,"RF ",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            sprintf(osd_buf,"%5d",rf_focus);
            draw_string(ovr_x,ovr_y,osd_buf,(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
           }
       }
        #if CAM_HAS_ND_FILTER
      
        if(conf.nd_filter_state)                                                                             // nd override enable
          {
           nd_on = 1;
           draw_string(ovr_x,ovr_y,(conf.nd_filter_state == 1)?"In ":"Out",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1); // display in/out status
          }

       else if(!conf.nd_filter_state && nd_on)                                                              // nd override disabled but previously on
         {
          nd_on = 0;                                                                                          // so only cleared once
          draw_string(ovr_x,ovr_y,"   ",MAKE_COLOR(COLOR_TRANSPARENT,COLOR_TRANSPARENT),1);
          ovr_y += FONT_HEIGHT;
         }

         #endif
      } // _else overrides enabled and time-lapse burst mode not running

// .............................................................................

          if(shooting_get_drive_mode()&& !(tl.running && (tl.shoot_mode==3)))   // continuous-shooting or custom-timer (but not time-lapse burst mode)
        {
           if(conf.tv_bracket_value && !conf.dist_mode)                         // Tv mode, distance bracketing off.
           {
            draw_string(ovr_x,ovr_y,"TvB",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            draw_string(ovr_x,ovr_y,(shooting_get_drive_mode()==1)?"CONT":"TIME",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
           }
           if((conf.dist_mode==1) &&(conf.subj_dist_bracket_value > 0) && shooting_can_focus()) // equal-step manual bracketing
           {
            draw_string(ovr_x,ovr_y,"DB ",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            draw_string(ovr_x,ovr_y,(shooting_get_drive_mode()==1)?"CONT":"TIME",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
           }

          if((conf.dist_mode==2) && (conf.bracket_type>2 || tl.running) && shooting_can_focus())
           {
            draw_string(ovr_x,ovr_y,"DBA",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            draw_string(ovr_x,ovr_y,(shooting_get_drive_mode()==1)?"CONT":"TIME",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
           }

           if((conf.dist_mode==3)&& shooting_can_focus())
           {
            draw_string(ovr_x,ovr_y,"DBD",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            ovr_y += FONT_HEIGHT;
            draw_string(ovr_x,ovr_y,(shooting_get_drive_mode()==1)?"CONT":"TIME",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
           }
         }
// ............................................................................
            if(!shooting_get_drive_mode()&& (conf.dist_mode==3) )
            {
             draw_string(ovr_x,ovr_y,"DIG",(conf.camera_position)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),1);
            }
      if ((m&MODE_MASK) == MODE_REC &&(recreview_hold==0)) gui_batt_draw_osd();
    }


// ---------------------------------------------ZEBRA -------------------  
    mode_video = ((m&MODE_SHOOTING_MASK)==MODE_VIDEO_STD || (m&MODE_SHOOTING_MASK)==MODE_VIDEO_SPEED || (m&MODE_SHOOTING_MASK)==MODE_VIDEO_COMPACT ||(m&MODE_SHOOTING_MASK)==MODE_VIDEO_MY_COLORS || (m&MODE_SHOOTING_MASK)==MODE_VIDEO_COLOR_ACCENT);
    if (conf.zebra_draw && gui_mode==GUI_MODE_NONE && (kbd_is_key_pressed(KEY_SHOOT_HALF)|| get_soft_half_press()) && mode_photo) 
      {
        if (!zebra_init) 
        {
            zebra_init = 1;
            gui_osd_zebra_init();
        }
        zebra = gui_osd_draw_zebra();
      }

    if (zebra_init && !kbd_is_key_pressed(KEY_SHOOT_HALF) && !get_soft_half_press()) 
      {
        zebra_init = 0;
      }
    if (zebra) 
      {
        if (!kbd_is_key_pressed(KEY_SHOOT_HALF) && !get_soft_half_press()) 
           {
            zebra = 0;
            draw_restore();
           }
            return;
      }
// -------------------------------------------- HISTOGRAM --------------

    if (conf.show_histo && (gui_mode==GUI_MODE_NONE || gui_mode==GUI_MODE_ALT) && (kbd_is_key_pressed(KEY_SHOOT_HALF)|| get_soft_half_press()) && (mode_photo || (m&MODE_SHOOTING_MASK)==MODE_STITCH)) 
      {
        gui_osd_draw_histo();
      }

// -------------------------------  grid lines ------------------------
    if (conf.show_grid_lines || conf.enable_yaw_guideline) 
      {
       gui_grid_draw_osd(1);
      }

// -------------------------------------------------------------------------------------
//                         Master OSD must be enabled for following 
// -------------------------------------------------------------------------------------


    if (debug_vals_show && ((m&MODE_MASK)!=MODE_REC))     
     {
	sprintf(osd_buf, "0:%8x ",physw_status[0]);
	draw_txt_string(28, 10, osd_buf, conf.osd_color);

      sprintf(osd_buf, "1:%8x ",physw_status[1]);
	draw_txt_string(28, 11, osd_buf, conf.osd_color);

      sprintf(osd_buf, "2:%8x ",physw_status[2]);
	draw_txt_string(28, 12, osd_buf, conf.osd_color);
     }

 if(((m&MODE_MASK)==MODE_REC) && ((conf.show_osd && ((conf.edge_and_osd && conf.outline_mode)|| !conf.outline_mode))||(!conf.show_osd && conf.outline_mode && !conf.edge_and_osd)))
  {
      if (((m&MODE_MASK) == MODE_REC) &&(recreview_hold==0)) 
       {


// -------------------------------- Stereo static\non-static ---------

            if (conf.stereo_display_mode && (gui_mode==GUI_MODE_NONE) && (mode_photo || mode_video || (m&MODE_SHOOTING_MASK)==MODE_STITCH)) 
            {
             gui_osd_draw_stereo();
            }    
//-------------------------------- show state ------------------------
        if (conf.show_state) 
           {
            gui_osd_draw_state();
           }
//------------------------------- show misc values -------------------

        if (conf.show_values) 
           {
            gui_osd_draw_values();
           }
//------------------------------- show serial I/O values -------------------
#ifdef OPT_SERIALIO
        if (conf.show_sio) 
           {
            gui_osd_draw_sio();
           }
#endif
//---------------------------------   clock --------------------------    
        if (conf.show_clock) 
           {
            gui_osd_draw_clock();
           }

        if(conf.zf_size)
          {
           gui_osd_draw_big_zoom();
          }

       } // _mode record
//--------------------------------- Debug values ----------------------

    if (debug_vals_show)
     {
	sprintf(osd_buf, "0:%8x ",physw_status[0]);
	draw_txt_string(28, 10, osd_buf, conf.osd_color);

      sprintf(osd_buf, "1:%8x ",physw_status[1]);
	draw_txt_string(28, 11, osd_buf, conf.osd_color);

      sprintf(osd_buf, "2:%8x ",physw_status[2]);
	draw_txt_string(28, 12, osd_buf, conf.osd_color);
     }

//---------------------------------- Debug propcase -------------------
    if (debug_propcase_show)
     {
	static char sbuf[100];
	int r,i, p;

	  for (i=0;i<10;i++)
         {
	    r = 0;
	    p = debug_propcase_page*10+i;
	    get_property_case(p, &r, 4);
	    sprintf(sbuf, "%3d: %d              ", p, r);sbuf[20]=0;
	    draw_string(64,16+16*i,sbuf, conf.osd_color,1);
	   }
      }
   } //_conf.show_osd
//---------------------------------- uBasic error ---------------------

    if (ubasic_error)
     {
	const char *msg;
        if (ubasic_error >= UBASIC_E_ENDMARK) 
          {
            msg = ubasic_errstrings[UBASIC_E_UNKNOWN_ERROR];
          } 
        else 
         {
	    msg = ubasic_errstrings[ubasic_error];
	   }
	   sprintf(osd_buf, "uBASIC:%d %s ", ubasic_linenumber(), msg);
	   draw_txt_string(0, 0, osd_buf, MAKE_COLOR(COLOR_RED, COLOR_YELLOW));
      }
   
}
// -------------------------------------------------------------------------------------------------------------
//                                        end of gui_draw_osd()
// -------------------------------------------------------------------------------------------------------------
static void gui_menuproc_reset_selected(unsigned int btn) 
{
    if (btn==MBOX_BTN_YES)
        conf_load_defaults();
}

void gui_menuproc_reset(int arg)
{
    gui_mbox_init(LANG_MSG_RESET_OPTIONS_TITLE, 
                  LANG_MSG_RESET_OPTIONS_TEXT,
                  MBOX_FUNC_RESTORE|MBOX_TEXT_CENTER|MBOX_BTN_YES_NO|MBOX_DEF_BTN2, gui_menuproc_reset_selected);
}

// ********************************************************************************************************

void gui_draw_spm_splash() 
{
   static char mybuf[5];   // for my version text string
   int rec = ((mode_get()&MODE_MASK) == MODE_REC);
   sprintf(mybuf,"%s",my_version_number);
if(conf.camera_orientation!=1)                                   // horizontal or inverted
   draw_string((screen_width-256)>>1,60,mybuf,(conf.camera_position == 1)?(rec)?MAKE_COLOR(COLOR_TRANSPARENT,COLOR_BLUE):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_BLUE_P):(rec)?MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED_P),8);
else                                                           // vertical
   draw_string(0,(screen_width-224)>>1,mybuf,(conf.camera_position == 1)?(rec)?MAKE_COLOR(COLOR_TRANSPARENT,COLOR_BLUE):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_BLUE_P):(rec)?MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED_P),7);
}

// -------------------------------------------------------------------
void gui_draw_fselect(int arg) 
{
 gui_fselect_init(LANG_STR_FILE_BROWSER, "A", NULL);
}
// -------------------------------------------------------------------
static void gui_load_script_selected(const char *fn) 
{
 if (fn)
 script_load(fn,1);
}
// -------------------------------------------------------------------
void gui_load_script(int arg) 
{

    DIR   *d;
    char  *path="A/CHDK/SCRIPTS";

    d=opendir(path);
    if (d) {

        closedir(d);
    } else {

        path="A";
    }
    gui_fselect_init(LANG_STR_SELECT_SCRIPT_FILE, path, gui_load_script_selected);
}

// -------------------------------------------------------------------
static void gui_grid_lines_load_selected(const char *fn) {
    if (fn)
        grid_lines_load(fn);
}
void gui_grid_lines_load(int arg) {
    DIR   *d;
    char  *path="A/CHDK/GRIDS";

    // if exists "A/CHDK/GRIDS" go into
    d=opendir(path);
    if (d) {
        closedir(d);
    } else {
        path="A";
    }

    gui_fselect_init(LANG_STR_SELECT_GRID_FILE, path, gui_grid_lines_load_selected);
}

// -------------------------------------------------------------------
void gui_draw_osd_le(int arg) {
    gui_mode = GUI_MODE_OSD;
    gui_osd_init();
}

// -------------------------------------------------------------------
void gui_draw_sdm_configuration(int arg) 
{
    gui_mode = GUI_MODE_SDM_CONFIGURATION; 
    gui_sdm_config_init();  
    gui_sdm_config_draw();  
}

// -------------------------------------------------------------------
static void gui_draw_rbf_selected(const char *fn) 
{
    if (fn) 
    {
        strcpy(conf.reader_rbf_file, fn);
    }
}

// -------------------------------------------------------------------
void gui_draw_load_rbf(int arg) 
{
    DIR   *d;
    char  *path="A/CHDK/FONTS";

    // if exists "A/CHDK/FONTS" go into
    d=opendir(path);
    if (d) 
    {
        closedir(d);
    } else 
    {
        path="A";
    }

    gui_fselect_init(LANG_STR_SELECT_FONT_FILE, path, gui_draw_rbf_selected);
}

// -------------------------------------------------------------------
static void gui_draw_menu_rbf_selected(const char *fn) 
{
    if (fn) {
        strcpy(conf.menu_rbf_file, fn);
        if (!rbf_load(conf.menu_rbf_file))
            rbf_load_from_8x16(current_font);
        rbf_set_codepage(FONT_CP_WIN);
        gui_menu_init(NULL);
    }
}

// -------------------------------------------------------------------
void gui_draw_load_menu_rbf(int arg) 
{
    DIR   *d;
    char  *path="A/CHDK/FONTS";

    // if exists "A/CHDK/FONTS" go into
    d=opendir(path);
    if (d) 
    {
        closedir(d);
    } else 
    {
        path="A";
    }

    gui_fselect_init(LANG_STR_SELECT_FONT_FILE, path, gui_draw_menu_rbf_selected);
}

// -------------------------------------------------------------------
static void gui_draw_lang_selected(const char *fn) 
{
    if (fn) 
    {
        strcpy(conf.lang_file, fn);
        lang_load_from_file(conf.lang_file);
        gui_menu_init(NULL);
    }
}

// -------------------------------------------------------------------
void gui_draw_load_lang(int arg) 
{
    DIR   *d;
    char  *path="A/CHDK/LANG";

    // if exists "A/CHDK/LANG" go into
    d=opendir(path);
    if (d) 
    {
        closedir(d);
    } else 
    {
        path="A";
    }

    gui_fselect_init(LANG_STR_SELECT_LANG_FILE, path, gui_draw_lang_selected);
}

// -------------------------------------------------------------------
static void gui_draw_read_selected(const char *fn) 
{
    if (fn) 
    {
        if (!rbf_load(conf.reader_rbf_file))
            rbf_load_from_8x16(current_font);
        rbf_set_codepage(conf.reader_codepage);
        gui_mode = GUI_MODE_READ;
        gui_read_init(fn);
    }
}
// -------------------------------------------------------------------

void gui_draw_shortcuts(int arg)
{
  DIR   *d;
  static char fn[64];
  int fd;
  char  *path="A/CHDK/TEXTS";
  static const char* lang[]={"shortcut.eng","shortcut.deu","shortcut.fra","shortcut.esp","shortcut.mag"};
  d=opendir(path);
  if (d) 
   {
    closedir(d);
   } 
  else 
   {
    path="A";
   }
  sprintf(fn, "%s/%s",path,lang[conf.shortcut_lang]);
  fd = open(fn, O_RDONLY, 0777);                      // open file in chosen language
  if(fd>=0) gui_draw_read_selected(fn);               // and display if found
  else                                                // if cannot find
  {
   sprintf(fn, "%s/%s",path,lang[0]);                 // try English version
   fd = open(fn, O_RDONLY, 0777);
   if(fd>=0) gui_draw_read_selected(fn);              // and display if found
  } 
}

// -------------------------------------------------------------------
void gui_draw_read(int arg) {
    DIR   *d;
    char  *path="A/CHDK/STACKS";
    d=opendir(path);
    if (d) 
    {
        closedir(d);
    } else 
    {
        path="A";
    }
    gui_fselect_init(LANG_STR_SELECT_TEXT_FILE, path, gui_draw_read_selected);
}

// -------------------------------------------------------------------
static void gui_draw_read_last(int arg) 
{
    int fd;
    
    fd = open(conf.reader_file, O_RDONLY, 0777);
    if (fd >= 0) 
    {
        close(fd);
        gui_draw_read_selected(conf.reader_file);
    } else 
    {
        gui_draw_read(arg);
    }
}

// -------------------------------------------------------------------

void gui_menuproc_edge_free(int arg) 
{
    free_memory_edge_overlay();
}

// -------------------------------------------------------------------

void gui_menuproc_edge_save(int arg) 
{
    save_edge_overlay();
}

// -------------------------------------------------------------------

void gui_menuproc_edge_load(int arg) 
{
    DIR   *d;
    char  *path = EDGE_SAVE_DIR;
    const char* fn;

    // if exists, go into
    d=opendir(path);
    if (d) 
    {
        closedir(d);
    } else 
    {
        path="A";
    }

    gui_fselect_init(LANG_MENU_EDGE_LOAD, path, gui_load_edge_selected);
}

// -------------------------------------------------------------------
