#include "stdlib.h"
#include "platform.h"
#include "core.h"
#include "keyboard.h"
#include "conf.h"
#include "ubasic.h"
#include "font.h"
#include "lang.h"
#include "gui.h"
#include "draw_palette.h"
#include "gui_menu.h"
#include "kbd.h"
#include "gui_lang.h"

// -------------------------------------------------------------------
#define MENUSTACK_MAXDEPTH  3

// -------------------------------------------------------------------
typedef struct {
    CMenu       *menu;
    int         curpos;
    int         toppos;
} CMenuStacked;

// -------------------------------------------------------------------

static CMenu        *curr_menu;
static CMenuStacked gui_menu_stack[MENUSTACK_MAXDEPTH];
static unsigned int gui_menu_stack_ptr;
static int          gui_menu_curr_item;
static int          gui_menu_top_item;
static int          gui_menu_redraw;

static int          count;
static coord        x, y, w, num_lines;
static int          len_bool, len_int, len_enum, len_space, len_br1, len_br2, cl_rect;
static int          int_incr,incr_modified;
static unsigned char *item_color;
#if CAM_MULTIPART
extern int cp;         // current partition number
#endif
long t;                // epoch time
static struct tm *ttm; // local time structure
char buf[64];          // for printing
static struct 
{
int x;
int yy;
int w;
}title;
// -------------------------------------------------------------------
static void gui_menu_set_curr_menu(CMenu *menu_ptr, int top_item, int curr_item) {
    curr_menu = menu_ptr;
    gui_menu_top_item = top_item;
    gui_menu_curr_item = curr_item;
}

// -------------------------------------------------------------------


void gui_menu_init(CMenu *menu_ptr) 
{
   int width  = vid_get_bitmap_screen_width();
#if defined (CAMERA_ixus980) || defined(CAMERA_ixus100)
   width=width/2;
#endif
   int height = vid_get_bitmap_screen_height();

    if (menu_ptr) 
    {
       gui_menu_set_curr_menu(menu_ptr, 0, 0);
       gui_menu_stack_ptr = 0;
    }
if(conf.camera_orientation==1)        // camera vertical
    {
      w = height;
      x = 0;
    }

   else
 
    { 
#if defined(CAMERA_g11)|| defined (CAMERA_s90)|| defined (CAMERA_s95)
    w = screen_width-10-10;
#else
      w = width-30-30; 
#endif   
      x = 30; 
    }

    num_lines = (height-26-16)/rbf_font_height()-2;
    y = ((height-num_lines*rbf_font_height())>>1);

    len_bool = rbf_str_width("\x95");
    len_int = rbf_str_width("[99999");
    len_enum = rbf_str_width("WUBfS3a");
    len_space = rbf_char_width(' ');
    len_br1 = rbf_char_width('[');
    len_br2 = rbf_char_width(']');
    cl_rect = rbf_font_height() - 4;
    int_incr = 1;
    
    gui_menu_redraw=2;
}

// -------------------------------------------------------------------
static void gui_menu_color_selected(color clr)  
{
    *item_color = (unsigned char)(clr&0xFF);
    gui_menu_redraw=2;
    draw_restore();
    gui_force_restore();
}

// -------------------------------------------------------------------
static void gui_menu_back() 
{
    if (gui_menu_stack_ptr > 0)
    {
        gui_menu_stack_ptr--;
        gui_menu_set_curr_menu(gui_menu_stack[gui_menu_stack_ptr].menu, gui_menu_stack[gui_menu_stack_ptr].toppos, gui_menu_stack[gui_menu_stack_ptr].curpos);
        gui_menu_redraw=2;
        draw_restore();
        gui_force_restore();
    }
}
// -------------------------------------------------------------------
void menu_title(x,y,w)
{
 int rec=((mode_get()&MODE_MASK) == MODE_REC);
 if(!curr_menu->title_char_size)                                    // small characters
   {
      if(strcmp(lang_str(curr_menu->title),"Script")==0)             // show clock in Script menu only
     {
      title.x=x;
      title.yy=y;
      title.w=w;
      t=time(NULL);                                                  // epoch time
      ttm = localtime(&t);                                           // converted to local time structure ttm
      sprintf(buf,"%s   %02d:%02d:%02d",lang_str(curr_menu->title),ttm->tm_hour,ttm->tm_min,ttm->tm_sec);
      rbf_draw_string_centre_len(x,y,w,buf,MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P));
      }

     else                                                             // not Script menu
     rbf_draw_string_centre_len(x,y,w,lang_str(curr_menu->title),MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P));
    }

   else                                                               // large characters
    {
      if(conf.camera_orientation==1)x=0;                              // vertical, so align left
      else x = (w-15*BIG_FONT_WIDTH)>>1;                
      draw_big_string_centre_len(x,y,15*BIG_FONT_WIDTH,lang_str(curr_menu->title),MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P)); 
    }
}
// -------------------------------------------------------------------
void gui_menu_kbd_process() 
{
 static char sbuf[7];
 int rec=((mode_get()&MODE_MASK) == MODE_REC);
 int zi;
 if((conf.camera_orientation == 1)&& !conf.camera_rotation) zi=1;  // vertical rotated left
 else zi=0;
 switch(get_vkey(kbd_get_autoclicked_key()))
 {
// --------------------------------------------------------------------------------------
//                                        KEY UP
// --------------------------------------------------------------------------------------



        case KEY_UP:
            do 
             {
                if (gui_menu_curr_item>0) 
                {
                    if (gui_menu_curr_item-1==gui_menu_top_item && gui_menu_top_item>0) 
                        --gui_menu_top_item;
                    --gui_menu_curr_item;
                } 
               else 
                {
                    int i;
                    while (curr_menu->menu[gui_menu_curr_item+1].text)
                        ++gui_menu_curr_item;
                    gui_menu_top_item = gui_menu_curr_item - num_lines +1;
                    if (gui_menu_top_item<0) gui_menu_top_item=0;
                }
             } while ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || 
                     (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR);
#if CAM_HAS_ZOOM_LEVER
              if(gui_menu_stack_ptr)
               {
		    sprintf(sbuf, "%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_SELECTED_BG,(rec)?COLOR_SELECTED_FG:COLOR_SELECTED_FG_P),1);
               }
#endif
            gui_menu_redraw=1;
            break;


// --------------------------------------------------------------------------------------
//                                        KEY DOWN
// --------------------------------------------------------------------------------------

        case KEY_DOWN:
            do 
             {
                if (curr_menu->menu[gui_menu_curr_item+1].text) 
                 {
                    int i;
                    for (i=0; i<num_lines-1 && curr_menu->menu[gui_menu_top_item+i].text; ++i);
                    if (i==num_lines-1 && curr_menu->menu[gui_menu_top_item+i].text 
                        && gui_menu_top_item+i-1==gui_menu_curr_item && curr_menu->menu[gui_menu_top_item+i+1].text)
                        ++gui_menu_top_item;
                    ++gui_menu_curr_item;
                 } 
                else 
                 {
                    gui_menu_curr_item = gui_menu_top_item = 0;
                 }
             } while ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || 
                     (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR);
#if CAM_HAS_ZOOM_LEVER
              if(gui_menu_stack_ptr)
               {
		    sprintf(sbuf, "%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_SELECTED_BG,(rec)? COLOR_SELECTED_FG:COLOR_SELECTED_FG_P),1);
               }
#endif
            gui_menu_redraw=1;
            break;
 // --------------------------------------------------------------------------------------
//                                        KEY LEFT
// --------------------------------------------------------------------------------------     

        case KEY_LEFT:
            if (gui_menu_curr_item>=0) 
              {
                switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK) 
                  {
                    case MENUITEM_INT:
                        switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) 
                           {
                            case MENUITEM_ARG_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) -= curr_menu->menu[gui_menu_curr_item].arg;
                                break;
                            case MENUITEM_ARG_ADDR_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) -= *(int *)(curr_menu->menu[gui_menu_curr_item].arg);
                                break;
                            default:
                                *(curr_menu->menu[gui_menu_curr_item].value) -= int_incr;
                                break;

                           }
                        if (*(curr_menu->menu[gui_menu_curr_item].value) < -9999) *(curr_menu->menu[gui_menu_curr_item].value) = -9999;
                        if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_UNSIGNED) 
                           {
                            if (*(curr_menu->menu[gui_menu_curr_item].value) < 0) *(curr_menu->menu[gui_menu_curr_item].value) = 0;
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MIN) 
                              {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) < (unsigned short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (unsigned short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF);
                              }
                            } 
                            else 
                            {
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MIN) 
                              {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) < (short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF);
                              }
                             }
                        if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) == MENUITEM_ARG_CALLBACK && curr_menu->menu[gui_menu_curr_item].arg) 
                           {
                            ((void (*)())(curr_menu->menu[gui_menu_curr_item].arg))();
                           }
                        if (curr_menu->on_change) 
                           {
                            curr_menu->on_change(gui_menu_curr_item);
                           }
#if CAM_HAS_ZOOM_LEVER
                        if (incr_modified) 
                           {
				    incr_modified=0;
                            draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
				   }
#endif
                        gui_menu_redraw=1;
                        break;




                    case MENUITEM_ENUM:
                        if (curr_menu->menu[gui_menu_curr_item].value) {
                            ((const char* (*)(int change, int arg))(curr_menu->menu[gui_menu_curr_item].value))(-1, curr_menu->menu[gui_menu_curr_item].arg);
                        }
                        if (curr_menu->on_change) 
                        {
                         curr_menu->on_change(gui_menu_curr_item);
                        }
                        gui_menu_redraw=1;
                        break;
                    case MENUITEM_UP:
                        gui_menu_back();
                        break;
                }
            } else {
                gui_menu_back();
            }
            break;

// --------------------------------------------------------------------------------------
//                                        KEY RIGHT
// --------------------------------------------------------------------------------------

        case KEY_RIGHT:
            if (gui_menu_curr_item>=0) {
                switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK){
                    case MENUITEM_INT:

                        switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) {
                            case MENUITEM_ARG_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) += curr_menu->menu[gui_menu_curr_item].arg;
                                break;
                            case MENUITEM_ARG_ADDR_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) += *(int *)(curr_menu->menu[gui_menu_curr_item].arg);
                                break;
                            default:
                                *(curr_menu->menu[gui_menu_curr_item].value) += int_incr;
                                break;
                        }
                        if (*(curr_menu->menu[gui_menu_curr_item].value) > 99999) 
                            *(curr_menu->menu[gui_menu_curr_item].value) = 99999;
                        if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_UNSIGNED) {
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MAX) {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) > (unsigned short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (unsigned short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF);
                            }
                        } else {
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MAX) {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) > (short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF);
                            }
                        }
                        if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) == MENUITEM_ARG_CALLBACK && curr_menu->menu[gui_menu_curr_item].arg) {
                            ((void (*)())(curr_menu->menu[gui_menu_curr_item].arg))();
                        }
                        if (curr_menu->on_change) {
                            curr_menu->on_change(gui_menu_curr_item);
                        }

#if CAM_HAS_ZOOM_LEVER
                        if (incr_modified) 
                         {
				  incr_modified=0;
                          draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
				 }
#endif

                        gui_menu_redraw=1;
                        break;


                    case MENUITEM_ENUM:
                        if (curr_menu->menu[gui_menu_curr_item].value) {
                            ((const char* (*)(int change, int arg))(curr_menu->menu[gui_menu_curr_item].value))(+1, curr_menu->menu[gui_menu_curr_item].arg);
                        }
                        if (curr_menu->on_change)                     // if there is callback
                        {
                         curr_menu->on_change(gui_menu_curr_item);    // call callback function
                        }
                        gui_menu_redraw=1;
                        break;

                    case MENUITEM_SUBMENU:
                        gui_menu_stack[gui_menu_stack_ptr].menu = curr_menu;
                        gui_menu_stack[gui_menu_stack_ptr].curpos = gui_menu_curr_item;
                        gui_menu_stack[gui_menu_stack_ptr].toppos = gui_menu_top_item;
                        gui_menu_set_curr_menu((CMenu*)(curr_menu->menu[gui_menu_curr_item].value), 0, 0);
                            if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR) 
                            {
                              ++gui_menu_curr_item;
                            }

                        gui_menu_stack_ptr++;
                        // FIXME check on stack overrun;
                        if (gui_menu_stack_ptr > MENUSTACK_MAXDEPTH)
                         {
                            draw_txt_string(0, 0, "E1", MAKE_COLOR(COLOR_RED, COLOR_YELLOW));
                            gui_menu_stack_ptr = 0;
                         }

                        gui_menu_redraw=2;
                        draw_restore();
                        gui_force_restore();
                        break;
                }
            }
            break;

// --------------------------------------------------------------------------------------
//                                        KEY SET
// --------------------------------------------------------------------------------------
        case KEY_SET:
            if (gui_menu_curr_item>=0) {
                switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK){

                    case MENUITEM_BOOL:
                        *(curr_menu->menu[gui_menu_curr_item].value) =
                                !(*(curr_menu->menu[gui_menu_curr_item].value));
                        if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) == MENUITEM_ARG_CALLBACK && curr_menu->menu[gui_menu_curr_item].arg) {
                            ((void (*)())(curr_menu->menu[gui_menu_curr_item].arg))();
                        }
                        if (curr_menu->on_change) {
                            curr_menu->on_change(gui_menu_curr_item);
                        }
                        gui_menu_redraw=1;
                        break;

                    case MENUITEM_PROC:
                    	if (curr_menu->menu[gui_menu_curr_item].value) {
                            ((void (*)(int arg))(curr_menu->menu[gui_menu_curr_item].value))(curr_menu->menu[gui_menu_curr_item].arg);
                            if (curr_menu->on_change) {
                                curr_menu->on_change(gui_menu_curr_item);
                            }
                            gui_menu_set_curr_menu(curr_menu, 0, -1);
                            gui_menu_redraw=2;
                        }
                        break;

                    case MENUITEM_SUBMENU:
                        gui_menu_stack[gui_menu_stack_ptr].menu = curr_menu;
                        gui_menu_stack[gui_menu_stack_ptr].curpos = gui_menu_curr_item;
                        gui_menu_stack[gui_menu_stack_ptr].toppos = gui_menu_top_item;
                        gui_menu_set_curr_menu((CMenu*)(curr_menu->menu[gui_menu_curr_item].value), 0, 0);
                            if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR) 
                             {
                               ++gui_menu_curr_item;
                             }

                        gui_menu_stack_ptr++;
                        // FIXME check on stack overrun;
                        if (gui_menu_stack_ptr > MENUSTACK_MAXDEPTH)
                        {
                            draw_txt_string(0, 0, "E1", MAKE_COLOR((rec)?COLOR_RED:COLOR_RED_P,(rec)? COLOR_YELLOW:COLOR_YELLOW_P));
                            gui_menu_stack_ptr = 0;
                        }

                        gui_menu_redraw=2;
                        draw_restore();
                        gui_force_restore();
                        break;

                    case MENUITEM_UP:
                        gui_menu_back();
                        break;

                    case MENUITEM_COLOR_FG:

                    case MENUITEM_COLOR_BG:
                        draw_restore();
                        item_color=((unsigned char*)(curr_menu->menu[gui_menu_curr_item].value)) + (((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_COLOR_BG)?1:0);
                        gui_palette_init(PALETTE_MODE_SELECT, (*item_color)&0xFF, gui_menu_color_selected);
                        gui_set_mode(GUI_MODE_PALETTE);
                        gui_menu_redraw=2;
                        break;

                    case MENUITEM_ENUM:
                        if (curr_menu->menu[gui_menu_curr_item].value) {
                            ((const char* (*)(int change, int arg))(curr_menu->menu[gui_menu_curr_item].value))(+1, curr_menu->menu[gui_menu_curr_item].arg);
                        }
                        gui_menu_redraw=1;
                        break;
                }
            }
            break;
// --------------------------------------------------------------------------------------
//                                        KEY ZOOM OUT
// --------------------------------------------------------------------------------------
case KEY_ZOOM_OUT:
#if CAM_HAS_ZOOM_LEVER
               if(!zi && (int_incr >= 10)) int_incr /= 10;        // inverted or normal orientation or rotated right
               else if (zi && (int_incr <= 1000))                 // inverted rotated left
                {
                 int_incr *= 10;
                }
		    sprintf(sbuf, "%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_SELECTED_BG, (rec)?COLOR_SELECTED_FG:COLOR_SELECTED_FG_P),1);
#endif
            break;
// --------------------------------------------------------------------------------------
//                                        KEY ZOOM IN
// --------------------------------------------------------------------------------------
case KEY_ZOOM_IN:
#if CAM_HAS_ZOOM_LEVER
                if (!zi &&(int_incr <= 1000)) int_incr *= 10;     // inverted or normal orientation or rotated right   
                else if (zi && (int_incr >= 10)) int_incr /= 10;     // inverted rotated left
		    sprintf(sbuf, "%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_SELECTED_BG,(rec)? COLOR_SELECTED_FG:COLOR_SELECTED_FG_P),1);
 #endif

            break;


        case KEY_DISPLAY:
            if (gui_menu_stack_ptr > 0){
                gui_menu_stack_ptr--;
                gui_menu_set_curr_menu(gui_menu_stack[gui_menu_stack_ptr].menu, gui_menu_stack[gui_menu_stack_ptr].toppos, gui_menu_stack[gui_menu_stack_ptr].curpos);
                gui_menu_redraw=2;
                draw_restore();
                gui_force_restore();
            }

            break;
    }
}

// -------------------------------------------------------------------
void gui_menu_draw_initial() {
    static const char *f=" *** ";
    int l, xx, yy;

   int width  = vid_get_bitmap_screen_width();
#if defined (CAMERA_ixus980)|| defined(CAMERA_ixus100)
   width=width/2;
#endif
   int height = vid_get_bitmap_screen_height();
int rec=((mode_get()&MODE_MASK) == MODE_REC);

// ---------------------------------- small text size ----------------------
if(!curr_menu->title_char_size)                                       
 {

  if(conf.camera_orientation==1)
  {
   w=height;
   x=0;
  }

  else
 
  { 
   w = width-30-30;    
   x = 30; 
  }

    num_lines = (height-26-16)/rbf_font_height()-2;
    y = ((height-num_lines*rbf_font_height())>>1);    
    l = rbf_str_width(lang_str(curr_menu->title));
    yy = y-rbf_font_height()-rbf_font_height()/2;

    menu_title(x,yy,w);

    for (count=0; curr_menu->menu[count].text; ++count);
    if((strcmp(lang_str(curr_menu->title),lang_str(LANG_MENU_BASIC_TITLE))!=0)&& (count>num_lines)&& (conf.camera_orientation != 1))    // scrollbar background
      {
        draw_filled_rect((x+w), y, (x+w)+8, y+num_lines*rbf_font_height()-1, MAKE_COLOR((conf.menu_color>>8)&0xFF, (conf.menu_color>>8)&0xFF));
      }
 } // small text size

 else                                                                                                                     // large text size
 {
     yy = 8;
     if(conf.camera_orientation==1)xx=0;
     else xx = (width-15*BIG_FONT_WIDTH)>>1;
     menu_title(xx,yy,width); 
 }

}

// -------------------------------------------------------------------
void gui_menu_draw() {
    static char tbuf[64];
    int imenu, i, j, yy, xx;
   int width  = vid_get_bitmap_screen_width();
#if defined (CAMERA_ixus980) || defined(CAMERA_ixus100)
   width=width/2;
#endif
   int height = vid_get_bitmap_screen_height();
    color cl;
    const char *ch = "";
int rec=((mode_get()&MODE_MASK) == MODE_REC);

   if(conf.camera_orientation == 1 )       // camera vertical 
    {
     num_lines = (height-26-16)/rbf_font_height()-2;
      w = height;
      y = ((height-num_lines*rbf_font_height())>>1);
      x = 0;
    }

   else                                                        // camera horizontal
 
    { 
      num_lines = (height-26-16)/rbf_font_height()-2;

      w = width-30-30;
      x = 30;
     }

      if(!curr_menu->title_char_size) y = ((height-num_lines*rbf_font_height())>>1);   // small text size
      else y = 8+BIG_FONT_HEIGHT;

     if((strcmp(lang_str(curr_menu->title),"Script")==0)&& !conf.camera_orientation) // Script menu and normal orientation
     {
      t=time(NULL);                                  // epoch time
      ttm = localtime(&t);                           // converted to local time structure ttm
      sprintf(buf,"%s   %02d:%02d:%02d",lang_str(curr_menu->title),ttm->tm_hour,ttm->tm_min,ttm->tm_sec);
      rbf_draw_string_centre_len(title.x,title.yy,title.w,buf,MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P));
      }


    if (gui_menu_redraw) 
   {
        if (gui_menu_redraw==2)
            gui_menu_draw_initial();

        gui_menu_redraw=0;
       int font_height;

        for (imenu=gui_menu_top_item, i=0, yy=y; curr_menu->menu[imenu].text && i<num_lines; ++imenu, ++i)
        {
            cl = (gui_menu_curr_item==imenu)?MAKE_COLOR((rec)?COLOR_SELECTED_BG:COLOR_SELECTED_BG_P,(rec)? COLOR_SELECTED_FG:COLOR_SELECTED_FG_P):MAKE_COLOR((rec)?COLOR_BG:COLOR_BG_P,(rec)? COLOR_WHITE:COLOR_WHITE_P);
          if(!curr_menu->title_char_size)                               // small text size
           {
               if(conf.camera_orientation==1) xx = 0;
             else xx = x;
            }
           else
            {
               if(conf.camera_orientation==1) xx = 0;            
              else xx = (width-15*BIG_FONT_WIDTH)>>1;
            }

            switch (curr_menu->menu[imenu].type & MENUITEM_MASK) 
          {
// --------------------------------------------------------------------------------------
//                                        BOOLEAN
// --------------------------------------------------------------------------------------
            case MENUITEM_BOOL:
               if(conf.camera_orientation != 1)                // only a leading space in horizontal orientations
                {
                xx+=rbf_draw_char(xx, yy, ' ', cl);
                xx+=rbf_draw_string_len(xx, yy, w-len_space-len_space-len_br1-len_bool-len_br2-len_space, lang_str(curr_menu->menu[imenu].text), cl);
                xx+=rbf_draw_string(xx, yy, " [", cl);
                xx+=rbf_draw_string_len(xx, yy, len_bool, (*(curr_menu->menu[imenu].value))?"\x95":"", cl);
                rbf_draw_string(xx, yy, "] ", cl);
               }

               else
                {
                 xx+=rbf_draw_string_len(xx, yy, w-len_space-len_br1-len_bool-len_br2, lang_str(curr_menu->menu[imenu].text), cl);
                 xx+=rbf_draw_string(xx, yy, " [", cl);
                 xx+=rbf_draw_string_len(xx, yy, len_bool, (*(curr_menu->menu[imenu].value))?"\x95":"", cl);
                 rbf_draw_string(xx, yy, "]", cl);
                }

                break;
// --------------------------------------------------------------------------------------
//                                        INT
// --------------------------------------------------------------------------------------
            case MENUITEM_INT:

                if(conf.camera_orientation != 1)                // only a leading space in horizontal orientations
               {
                xx+=rbf_draw_char(xx, yy, ' ', cl);
                xx+=rbf_draw_string_len(xx, yy, w-len_space-len_space-len_br1-len_int-len_br2-len_space, lang_str(curr_menu->menu[imenu].text), cl);
                xx+=rbf_draw_string(xx, yy, " [", cl);
                sprintf(tbuf, "%d", *(curr_menu->menu[imenu].value));
                xx+=rbf_draw_string_right_len(xx, yy, len_int, tbuf, cl);
                rbf_draw_string(xx, yy, "] ", cl);
                }

                else
               {           
                xx+=rbf_draw_string_len(xx, yy, w-len_space-len_br1-len_int-len_br2, lang_str(curr_menu->menu[imenu].text), cl);
                xx+=rbf_draw_string(xx, yy, " [", cl);
                sprintf(tbuf, "%d", *(curr_menu->menu[imenu].value));
                xx+=rbf_draw_string_right_len(xx, yy, len_int, tbuf, cl);
                rbf_draw_string(xx, yy, "]", cl);
                }
                break;
// --------------------------------------------------------------------------------------
//                                        UP, SUBMENU, PROC, TEXT
// --------------------------------------------------------------------------------------
            case MENUITEM_UP:
            case MENUITEM_SUBMENU:
            case MENUITEM_PROC:
            case MENUITEM_TEXT:

             if(!curr_menu->menu[imenu].char_size)
              {
                if(conf.camera_orientation != 1)                // only a leading space in horizontal orientations
                {
#if CAM_MULTIPART
                if(curr_menu->menu[imenu].text==308)           // Swap partitions
                 {
                  xx+=rbf_draw_char(xx, yy, ' ', (cp==1)?MAKE_COLOR((rec)?COLOR_GREEN:COLOR_GREEN_P,(rec)?COLOR_WHITE:COLOR_WHITE_P):MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P));
                  sprintf(buf,(cp==1)?"%s 1":"%s 2",lang_str(curr_menu->menu[imenu].text));
                  xx+=rbf_draw_string_len(xx, yy, w-len_space-len_space,buf,(cp==1)?MAKE_COLOR((rec)?COLOR_GREEN:COLOR_GREEN_P,(rec)?COLOR_WHITE:COLOR_WHITE_P):MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P));
                  rbf_draw_char(xx, yy, ' ', (cp==1)?MAKE_COLOR((rec)?COLOR_GREEN:COLOR_GREEN_P,(rec)?COLOR_WHITE:COLOR_WHITE_P):MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P));
                 }          
                else
                 {
#endif
                  xx+=rbf_draw_char(xx, yy, ' ', cl);
                  xx+=rbf_draw_string_len(xx, yy, w-len_space-len_space, lang_str(curr_menu->menu[imenu].text), cl);
                  rbf_draw_char(xx, yy, ' ', cl);
#if CAM_MULTIPART
                  }
#endif
                 }

                else                                         // vertical orientation
                {
                 xx+=rbf_draw_string_len(xx, yy, w, lang_str(curr_menu->menu[imenu].text), cl);
                }
               }
               else
               {
                  sprintf(tbuf,"%-15.15s",lang_str(curr_menu->menu[imenu].text));
                  draw_big_string(xx,yy,tbuf,cl);
               }

                break;

// --------------------------------------------------------------------------------------
//                                        SEPARATOR
// --------------------------------------------------------------------------------------
            case MENUITEM_SEPARATOR:
                if (lang_str(curr_menu->menu[imenu].text)[0]) 
                {
                    j = rbf_str_width(lang_str(curr_menu->menu[imenu].text));                  // pixel-width of text string
// ERR99 fix http://chdk.setepontos.com/index.php/topic,4647.msg46080.html#msg46080
                #if defined (CAMERA_g11)|| defined (CAMERA_s90)|| defined (CAMERA_s95)
		       xx+=((int)w-j-len_space*2)>>1;
                #else
                    xx+=(w-j-len_space*2)>>1;                                                  // starting position to centre the string
                #endif
                    if(lang_str(curr_menu->menu[imenu].text)==script_title)                                                             // title is centred
                    {
                     draw_filled_rect(x, yy,x+((w-j)/2), yy+rbf_font_height(),MAKE_COLOR(COLOR_BLACK,COLOR_BLACK));
                     rbf_draw_string(x+((w-j)/2), yy, lang_str(curr_menu->menu[imenu].text),MAKE_COLOR(COLOR_BLACK,COLOR_WHITE));
                     draw_filled_rect(x+j+((w-j)/2), yy,x+w, yy+rbf_font_height(),MAKE_COLOR(COLOR_BLACK,COLOR_BLACK));          
                     }
                    else  if((lang_str(curr_menu->menu[imenu].text)==paramdesc[0])||(lang_str(curr_menu->menu[imenu].text)==paramdesc[1])||(lang_str(curr_menu->menu[imenu].text)==paramdesc[2])||(lang_str(curr_menu->menu[imenu].text)==paramdesc[3]))  // descriptions                                                                    // descriptions left-justified
                     {
                      rbf_draw_string(x, yy, lang_str(curr_menu->menu[imenu].text),MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
                      draw_filled_rect(x+j, yy,x+w, yy+rbf_font_height(),MAKE_COLOR(COLOR_BLUE,COLOR_BLUE));
                     }
                    else                                                                                                                 // normal CHDK
                     {
                      rbf_draw_char(x, yy, ' ', cl);
                      draw_filled_rect(x+len_space, yy, xx-1, yy+rbf_font_height()/2-1, MAKE_COLOR(cl>>8, cl>>8));
                      draw_line(x+len_space, yy+rbf_font_height()/2, xx-1, yy+rbf_font_height()/2, cl);
                      draw_filled_rect(x+len_space, yy+rbf_font_height()/2+1, xx-1, yy+rbf_font_height()-1, MAKE_COLOR(cl>>8, cl>>8));
                      xx+=rbf_draw_char(xx, yy, ' ', cl);
                      xx+=rbf_draw_string(xx, yy, lang_str(curr_menu->menu[imenu].text), cl);
                      xx+=rbf_draw_char(xx, yy, ' ', cl);            
                      draw_filled_rect(xx, yy, x+w-len_space-1, yy+rbf_font_height()/2-1, MAKE_COLOR(cl>>8, cl>>8));
                      draw_line(xx, yy+rbf_font_height()/2, x+w-1-len_space, yy+rbf_font_height()/2, cl);
                      draw_filled_rect(xx, yy+rbf_font_height()/2+1, x+w-len_space-1, yy+rbf_font_height()-1, MAKE_COLOR(cl>>8, cl>>8));
                      rbf_draw_char(x+w-len_space, yy, ' ', cl);
                     }
                }
 
              else  if((lang_str(curr_menu->menu[imenu].text)==paramdesc[0])||(lang_str(curr_menu->menu[imenu].text)==paramdesc[1])||(lang_str(curr_menu->menu[imenu].text)==paramdesc[2])||(lang_str(curr_menu->menu[imenu].text)==paramdesc[3]))  // descriptions  
                {
                  draw_filled_rect(x, yy,x+w, yy+rbf_font_height(),MAKE_COLOR(COLOR_BLUE,COLOR_BLUE));
                }

                else                                                                                                                    // normal CHDK
                {
                    rbf_draw_char(x, yy, ' ', cl);
                    draw_filled_rect(x+len_space, yy, x+w-len_space-1, yy+rbf_font_height()/2-1, MAKE_COLOR(cl>>8, cl>>8));
                    draw_line(x+len_space, yy+rbf_font_height()/2, x+w-1-len_space, yy+rbf_font_height()/2, cl);
                    draw_filled_rect(x+len_space, yy+rbf_font_height()/2+1, x+w-len_space-1, yy+rbf_font_height()-1, MAKE_COLOR(cl>>8, cl>>8));
                    rbf_draw_char(x+w-len_space, yy, ' ', cl);
                }

                break;

// --------------------------------------------------------------------------------------
//                                        COLOR_FG, COLOR_BG
// --------------------------------------------------------------------------------------
            case MENUITEM_COLOR_FG:
            case MENUITEM_COLOR_BG:
                if(conf.camera_orientation != 1)                // only a leading space in horizontal orientations
                 {
                  xx+=rbf_draw_char(xx, yy, ' ', cl);
                  xx+=rbf_draw_string_len(xx, yy, w-len_space, lang_str(curr_menu->menu[imenu].text), cl);
                  draw_filled_rect(x+w-1-cl_rect-2-len_space, yy+2, x+w-1-2-len_space, yy+rbf_font_height()-1-2, 
                                 MAKE_COLOR(((*(curr_menu->menu[imenu].value))>>(((curr_menu->menu[imenu].type & MENUITEM_MASK)==MENUITEM_COLOR_BG)?8:0))&0xFF, (cl>>8)&0xFF));
                 }

                  else                                           // vertical
                   {
                    xx+=rbf_draw_string_len(xx, yy, w, lang_str(curr_menu->menu[imenu].text), cl);
                    draw_filled_rect(x+w-1-cl_rect-2, yy+2, x+w-1-2, yy+rbf_font_height()-1-2, 
                                 MAKE_COLOR(((*(curr_menu->menu[imenu].value))>>(((curr_menu->menu[imenu].type & MENUITEM_MASK)==MENUITEM_COLOR_BG)?8:0))&0xFF, (cl>>8)&0xFF));
                    }
                break;
// --------------------------------------------------------------------------------------
//                                                   ENUM
// --------------------------------------------------------------------------------------
            case MENUITEM_ENUM:
                if (curr_menu->menu[imenu].value) {
                    ch=((const char* (*)(int change, int arg))(curr_menu->menu[imenu].value))(0, curr_menu->menu[imenu].arg);
                }
               if(conf.camera_orientation != 1)                // not vertical
               {
                xx+=rbf_draw_char(xx, yy, ' ', cl);
                xx+=rbf_draw_string_len(xx, yy, w-len_space-len_space-len_br1-len_enum-len_br2-len_space, lang_str(curr_menu->menu[imenu].text), cl);          
                xx+=rbf_draw_string(xx, yy, " [", cl);
                xx+=rbf_draw_string_right_len(xx, yy, len_enum, ch, cl);
                rbf_draw_string(xx, yy, "] ", cl);
                }
                else                                             // camera vertical
                {
                 xx+=rbf_draw_string_len(xx, yy, w-len_br1-rbf_str_width(ch)-len_br2, lang_str(curr_menu->menu[imenu].text),cl);
                 xx+=rbf_draw_string(xx, yy, "[", cl);
                 xx+=rbf_draw_string(xx, yy, ch, cl);
                 rbf_draw_string(xx, yy, "]", cl);
                }

                break;
            } // _switch

                      if(!curr_menu->menu[imenu].char_size) font_height = rbf_font_height();
                       else font_height = BIG_FONT_HEIGHT;
                       yy+=font_height;
        }   // _for
        
        //scrollbar
      if((strcmp(lang_str(curr_menu->title),lang_str(LANG_MENU_BASIC_TITLE))!=0)&& (count>num_lines)&& (conf.camera_orientation != 1))
         {
            i=num_lines*rbf_font_height()-1 -1;           //full height
            j=i*num_lines/count;                    //bar height
            if (j<20) j=20;
            i=(i-j)*((gui_menu_curr_item<0)?0:gui_menu_curr_item)/(count-1);   //top pos
            draw_filled_rect((x+w)+2, y+1, 
                             (x+w)+6, y+1+i, MAKE_COLOR((rec)?COLOR_BLACK:COLOR_BLACK_P,(rec)? COLOR_BLACK:COLOR_BLACK_P));
            draw_filled_rect((x+w)+2, y+i+j, 
                             (x+w)+6, y+num_lines*rbf_font_height()-1-1, MAKE_COLOR((rec)?COLOR_BLACK:COLOR_BLACK_P,(rec)? COLOR_BLACK:COLOR_BLACK_P));
            draw_filled_rect((x+w)+2, y+1+i, 
                             (x+w)+6, y+i+j, MAKE_COLOR((rec)?COLOR_WHITE:COLOR_WHITE_P, (rec)?COLOR_WHITE:COLOR_WHITE_P));
          }
    } 
}

