#include "stdlib.h"
#include "keyboard.h"
#include "platform.h"
#include "histogram.h"
#include "core.h"
#include "lang.h"
#include "conf.h"
#include "gui.h"
#include "draw_palette.h"
#include "gui_lang.h"
#include "math.h"
#include "gui_osd.h"
#include "batt_grid.h"
#include "kbd.h"
#ifdef OPT_SERIALIO   //spinorkit
#include "serialio.h"
#endif
#define abs(x) ((x) >= 0 ? (x) : -(x))
extern char *imgbuf;  // buffer shared by all image-processing functions
extern int imgbuf_size;
extern int gDngFinish,gDngStart;
// ******************************************************************************************************************************************
float realfl,realav,required_av = 0.0f, v, deviation,coc, max_distance, percent_deviation,fov,inf_np,my_max_deviation;
float entrance,mag,exitp;
int shift,myfl,myhyp,rfshift,rf_focus;
long fp,sensor_width;
extern long myav;
extern int subject_near,subject_far;
// *******************************************************************************************************************************************

void gui_bg(int posx,int posy,int value,color bg,color fg,int wide,int blocks);

typedef struct 
{
    int     title;
    OSD_pos *pos;
    OSD_pos size;
} OSD_elem;

typedef ShutterSpeed *SsPointer;
SsPointer ss = NULL;

typedef ISOTable *isoPointer;
isoPointer ip = NULL;

unsigned char clip8(signed short x){ if (x<0) x=0; else if (x>255) x=255; return x; } // CS841

static OSD_elem osd[]={
    {LANG_OSD_LAYOUT_EDITOR_HISTO,      &conf.histo_pos,        {HISTO_WIDTH+2, HISTO_HEIGHT}   },
    {LANG_OSD_LAYOUT_EDITOR_STATES,     &conf.mode_state_pos,   {3*FONT_WIDTH, 3*FONT_HEIGHT}   },
    {LANG_OSD_LAYOUT_EDITOR_MISC,       &conf.values_pos,       {8*FONT_WIDTH, 5*FONT_HEIGHT}   },
    {LANG_OSD_LAYOUT_EDITOR_ZOOM,       &conf.zoom_pos,         {2*FONT_WIDTH, 2*FONT_HEIGHT}   },
    {LANG_OSD_LAYOUT_EDITOR_INCREMENT,  &conf.increment_pos,    {5*FONT_WIDTH, FONT_HEIGHT}     },
    {LANG_OSD_LAYOUT_EDITOR_BAT_ICON,   &conf.batt_icon_pos,    {28, 12}                        },
    {LANG_OSD_LAYOUT_EDITOR_BAT_TEXT,   &conf.batt_txt_pos,     {5*FONT_WIDTH, FONT_HEIGHT}     },
    {LANG_OSD_LAYOUT_EDITOR_CLOCK,      &conf.clock_pos,        {5*FONT_WIDTH, FONT_HEIGHT}     },
    {LANG_OSD_LAYOUT_EDITOR_STEREO,     &conf.stereo_osd_pos,   {17*FONT_WIDTH, 2*FONT_HEIGHT}  },
    {LANG_OSD_LAYOUT_EDITOR_SYNCH,      &conf.synch_pos,        {25*FONT_WIDTH, FONT_HEIGHT}     },
    {LANG_MENU_OVERRIDE,                &conf.overrides_pos,    {3*FONT_WIDTH, 3*FONT_HEIGHT}   },
#ifdef OPT_SERIALIO
    {LANG_OSD_LAYOUT_EDITOR_SIO,        &conf.sio_pos,          {5*FONT_WIDTH, FONT_HEIGHT}   },
#endif
    {0}
};
static int osd_to_draw;
static int curr_item;
static char osd_buf[64];
static int step;
static unsigned char *img_buf, *scr_buf;
static int timer = 0;

// -------------------------------------------------------------------
void gui_osd_init() {
    osd_to_draw = 1;
    curr_item = 0;
    step = 10;
    draw_restore();
}

// -------------------------------------------------------------------
void gui_osd_draw() 

{ 
    if (osd_to_draw) 
     {
        int i;
        draw_restore();
        gui_osd_draw_histo();
        gui_batt_draw_osd();
        gui_osd_draw_state();
        gui_osd_draw_values();
#ifdef OPT_SERIALIO
        gui_osd_draw_sio();
#endif
        gui_osd_draw_clock();
        gui_osd_draw_stereo();

        for (i=1; i<=2; ++i) {
            draw_rect((osd[curr_item].pos->x>=i)?osd[curr_item].pos->x-i:0, (osd[curr_item].pos->y>=i)?osd[curr_item].pos->y-i:0, 
                      osd[curr_item].pos->x+osd[curr_item].size.x+i-1, osd[curr_item].pos->y+osd[curr_item].size.y+i-1,
                      COLOR_GREEN);
        }
        sprintf(osd_buf, " %s:  x:%d y:%d s:%d ", lang_str(osd[curr_item].title), osd[curr_item].pos->x, osd[curr_item].pos->y, step);
        draw_string(0, (osd[curr_item].pos->x<strlen(osd_buf)*FONT_WIDTH+4 && osd[curr_item].pos->y<FONT_HEIGHT+4)?screen_height-FONT_HEIGHT:0,
                    osd_buf, MAKE_COLOR(COLOR_RED, COLOR_WHITE),1);
        osd_to_draw = 0;
     }
}

// -------------------------------------------------------------------
void gui_osd_kbd_process() 
{
switch(get_vkey(kbd_get_autoclicked_key()))
 { 
  case KEY_LEFT:
       if (osd[curr_item].pos->x > 0) 
         {
          osd[curr_item].pos->x-=(osd[curr_item].pos->x>=step)?step:osd[curr_item].pos->x;
          osd_to_draw = 1;
         }
  break;
// .....................................................................................
  case KEY_RIGHT:
#if !defined(CAMERA_tx1)
       if (osd[curr_item].pos->x < screen_width-osd[curr_item].size.x) 
        {
         osd[curr_item].pos->x+=(screen_width-osd[curr_item].size.x-osd[curr_item].pos->x>step)?step:screen_width-osd[curr_item].size.x-osd[curr_item].pos->x;
 #else
       if (osd[curr_item].pos->x < screen_width-120-osd[curr_item].size.x) 
        {
         osd[curr_item].pos->x+=(screen_width-120-osd[curr_item].size.x-osd[curr_item].pos->x>step)?step:screen_width-120-osd[curr_item].size.x-osd[curr_item].pos->x;
#endif
         osd_to_draw = 1;
        }
  break;
// .......................................................................................
  case KEY_UP:
       if (osd[curr_item].pos->y > 0) // not above top of screen     
        {
         osd[curr_item].pos->y-=(osd[curr_item].pos->y>=step)?step:osd[curr_item].pos->y; // if at least one step from top of screen
         osd_to_draw = 1;
        }
  break;
// .......................................................................................
  case KEY_DOWN:
       if (osd[curr_item].pos->y < screen_height-osd[curr_item].size.y) 
        {
         osd[curr_item].pos->y+=(screen_height-osd[curr_item].size.y-osd[curr_item].pos->y>step)?step:screen_height-osd[curr_item].size.y-osd[curr_item].pos->y;
         osd_to_draw = 1;
        }
  break;
// ......................................................................................
  case KEY_SET:
     ++curr_item;
     if (!osd[curr_item].pos) 
     curr_item = 0;
     osd_to_draw = 1;
  break;
// ......................................................................................
  case KEY_DISPLAY:
     step=(step==1)?10:1;
     osd_to_draw = 1;
  break;
 }

}

// -------------------------------------------------------------------
static void gui_osd_draw_single_histo(int hist, coord x, coord y, int small) {
    register unsigned int i, v, threshold;
    register color cl, cl_over, cl_bg=conf.histo_color>>8;
    coord w=HISTO_WIDTH, h=HISTO_HEIGHT;

    switch (hist) {
        case HISTO_R:
            cl=((mode_get()&MODE_MASK) == MODE_REC)?COLOR_RED:COLOR_RED_P; 
            break;
        case HISTO_G: 
            cl=((mode_get()&MODE_MASK) == MODE_REC)?COLOR_GREEN:COLOR_GREEN_P; 
            break;
        case HISTO_B:
            cl=((mode_get()&MODE_MASK) == MODE_REC)?COLOR_BLUE:COLOR_BLUE_P; 
            break;
        case HISTO_RGB:
        case HISTO_Y:
        default:
            cl=conf.histo_color; 
            break;
    }

    if (small) {
        h>>=1; w>>=1;
        for (i=0; i<w; ++i) {
            threshold = (histogram[hist][i<<1]+histogram[hist][(i<<1)+1])>>2;

            for (v=1; v<h-1; ++v)
                draw_pixel(x+1+i, y+h-v, (v<=threshold)?cl:cl_bg);
            cl_over = (threshold==h && conf.show_overexp)?conf.histo_color2>>8:cl;
            for (; v<h; ++v)
                draw_pixel(x+1+i, y+h-v, (v<=threshold)?cl_over:cl_bg);
        }
    } else {
        for (i=0; i<w; ++i) {
            threshold = histogram[hist][i];

            for (v=1; v<h-3; ++v)
                draw_pixel(x+1+i, y+h-v, (v<=threshold)?cl:cl_bg);
            cl_over = (threshold==h && conf.show_overexp)?conf.histo_color2>>8:cl;
            for (; v<h; ++v)
                draw_pixel(x+1+i, y+h-v, (v<=threshold)?cl_over:cl_bg);
        }
    }
      
    draw_rect(x, y, x+1+w, y+h, conf.histo_color2&0xFF);

}

// -------------------------------------------------------------------
void gui_osd_zebra_init() 
{
    timer = 0;
}


// -------------------------------------------------------------------
 static void draw_pixel_buffered(coord x,coord y,color cl)
{
 unsigned int offseti,offsetf,offsetx,offsety;
#if defined (CAMERA_ixus980)  || defined(CAMERA_ixus100)|| defined(CAMERA_sx120is)
    offseti = y * screen_buffer_width + (x<<1);                         // draw two pixels side-by-side
    imgbuf[offseti] = imgbuf[offseti+1]= cl & 0xff;

#elif defined (CAMERA_sx200is)|| defined(CAMERA_g11)|| defined(CAMERA_s90)|| defined(CAMERA_s95) || defined (CAMERA_ixus95)|| defined(CAMERA_ixus105)
     offsety=y*screen_buffer_width;
     offseti = (((x<<3)+x) >>2);        // 'x' position scaled by 9/4
     x++;
     offsetf = (((x<<3)+x) >>2);        // 'x' position scaled by 9/4
     offsetf=(x >= screen_width) ? screen_buffer_width - 1 : offsetf;
     for(offsetx=offseti;offsetx<=offsetf-1;offsetx++)                  // draw two or three pixels
     {
      imgbuf[offsety+offsetx] = cl & 0xff;
     }

#else
    offseti = y * vid_get_bitmap_width() + x;                           // single pixel
    imgbuf[offseti] = cl & 0xff;
#endif
}

// -------------------------------------------------------------------
static void gui_osd_draw_zebra_osd() 
{
    switch (conf.zebra_draw_osd) 
    {
        case ZEBRA_DRAW_NONE:
            break;
        case ZEBRA_DRAW_OSD:
            if (conf.show_osd) 
             {
                draw_set_draw_proc(draw_pixel_buffered);
                if ((mode_get()&MODE_MASK) == MODE_REC) 
                  {
                    if (conf.show_grid_lines || conf.enable_yaw_guideline) 
                    {
                        gui_grid_draw_osd(1);
                    }
     
                    if (conf.show_state) 
                    {
                        gui_osd_draw_state();
                    }

                    if (conf.show_values) 
                    {
                        gui_osd_draw_values();
                    }

                    if(conf.zf_size)
                    {
                     gui_osd_draw_big_zoom();
                    }

                    gui_osd_draw_stereo();                     

                  }

                gui_batt_draw_osd();
                if (conf.show_clock) 
                {
                    gui_osd_draw_clock();
                }
                draw_set_draw_proc(NULL);
            }
            /* no break here */
        case ZEBRA_DRAW_HISTO:
        default:
            if (conf.show_histo) 
            {
                draw_set_draw_proc(draw_pixel_buffered);
                gui_osd_draw_histo();
                draw_set_draw_proc(NULL);
            }
            break;
    }
}

// -----------------------------------------------------------------------------------------------
//                                                 SX200IS and others
// -----------------------------------------------------------------------------------------------
#if defined(CAMERA_sx200is)|| defined(CAMERA_g11)|| defined(CAMERA_s90)|| defined(CAMERA_s95)|| defined (CAMERA_ixus95)|| defined (CAMERA_ixus105)

int gui_osd_draw_zebra() {
    unsigned int v, s, x, y, f, over;
    color cl_under=conf.zebra_color>>8, cl_over=conf.zebra_color&0xFF;
    static int need_restore=0;
    int viewport_height;
int viewport_width; 
 int m = ((mode_get()&MODE_MASK) == MODE_REC);
    color cls[] = {
        COLOR_TRANSPARENT,
        (m)?COLOR_BLUE:COLOR_BLUE_P,
        COLOR_GREEN,
        (m)?COLOR_BLUE_LT:COLOR_BLUE_LT_P,
        COLOR_RED,
        (m)?COLOR_PINK:COLOR_PINK_P,   // light red or pink
        (m)?COLOR_YELLOW:COLOR_YELLOW_P,
        COLOR_BLACK
    };

int ss = screen_buffer_size; // bmp buffer can be 360x240, 720x240 or 960x270
int sw=vid_get_viewport_width(); // we scan live-image width, not screen width

// -----------------------------------  allocate buffer memory  --------------------------------

    scr_buf = vid_get_bitmap_fb();                                   // address of bmp buffer

    if (!imgbuf) allocate_imgbuf();
// ----------------------------------  if memory allocated OK  ---------------------------------
    if (imgbuf) 
    {
        ++timer;

    //Try to get the best viewport buffer. In playmode its the _d one, in 
    //record mode we try to get the fast live one first 
 	if( (mode_get() & MODE_MASK) == MODE_PLAY ) 
       { 
 	  img_buf = vid_get_viewport_fb_d(); 
 	 } 
 	else 
       { 
 	  img_buf = vid_get_viewport_live_fb(); 
 	  if( !img_buf ) 
          { 
 	     img_buf = vid_get_viewport_fb(); 
 	    } 
 	 } 
// --------------------------------------------------------------------------------------------
        viewport_height = vid_get_viewport_height();
        switch (conf.zebra_mode) 
       {
            case ZEBRA_MODE_ZEBRA_1:
                f = 4;
                break;
            case ZEBRA_MODE_ZEBRA_2:
                f = 8;
                break;
            case ZEBRA_MODE_SOLID:
                f = 1; 
                break;
            case ZEBRA_MODE_BLINKED_1:
                f = timer&2; 
                break;
            case ZEBRA_MODE_BLINKED_3:
                f = timer&8; 
                break;
            case ZEBRA_MODE_BLINKED_2:
            default:
                f = timer&4; 
                break;
        }
// -------------------------------------------------------------------------------------------
        if (!f)                                                                             // no-zebra phase of blink mode zebra
       {
            if (need_restore) 
             {
                if (conf.zebra_restore_screen || conf.zebra_restore_osd) 
                 {
                    draw_restore();
                 } 
               else 
                 {

                    memset(imgbuf, COLOR_TRANSPARENT,ss);  // clear sdm buffer
                    gui_osd_draw_zebra_osd();             // draw other OSD stuff
                    memcpy(scr_buf, imgbuf,ss);           // clear first buffer
                    memcpy(scr_buf+ss, imgbuf,ss);        // clear second buffer
                }
                need_restore=0;
            }
            return !(conf.zebra_restore_screen && conf.zebra_restore_osd);
        } 
// ------------------------------------------------------------------------------------------
   else 
        {
            int step_x, step_v,sy, sx;
            over = 255-conf.zebra_over;

                if (conf.zebra_multichannel) {step_x=2; step_v=6;} else {step_x=1; step_v=3;}// for each line
                for (y=0, v=0; y<=viewport_height; ++y)                                      // for every line from 1 to live-buffer height
                {
                 sy=y*screen_buffer_width;                                                    // current line start position in screen buffer
                 sx=0;                                                                        // x position in current line as byte offset from line start
                  for (x=0; x<sw; x+=step_x, sx+=step_x, v+=step_v)                           // step along the viewport width for 360 pixels (normally)
                   {
                     register int yy, uu, vv;
                     int sel;
                     yy = img_buf[v+1];                                                       // luminance Y0 or Y2 (offset 1 or 4)
                     s=sy+sx*2;                                                               // update bmp buffer pointer for each pixel position
//----------------------------------  RGB zebra (overexp. only) --------------------------------
                     if (conf.zebra_multichannel) 
                      {
                        uu = (signed char)img_buf[v];                                        
                        vv = (signed char)img_buf[v+2];                                       
                        sel=0;
                        if (!((conf.zebra_mode == ZEBRA_MODE_ZEBRA_1 || conf.zebra_mode == ZEBRA_MODE_ZEBRA_2) && (y-x-timer)&f)) 
                         {
                           if (clip8((yy<<12) +           vv*5743 + 2048)>>12>over) sel  = 4;     // R
                           if (clip8((yy<<12) - uu*1411 - vv*2925 + 2048)>>12>over) sel |= 2;     // G
                           if (clip8((yy<<12) + uu*7258           + 2048)>>12>over) sel |= 1;     // B
                         }
                          imgbuf[s]=imgbuf[s+1]=cls[sel];                                    // draw two pixels
                          imgbuf[s+2]=imgbuf[s+3]=cls[sel];                                  // and two more               
                      }

// -----------------------------------  Zebra mode 1 or 2 ---------------------------------------
                     else if (((conf.zebra_mode == ZEBRA_MODE_ZEBRA_1 || conf.zebra_mode == ZEBRA_MODE_ZEBRA_2) && (y-x-timer)&f)) imgbuf[s]=imgbuf[s+1]=COLOR_TRANSPARENT;
// ---------------------------------- colour is under, over or transparent ----------------------
                        else imgbuf[s]=imgbuf[s+1]=(yy>over)?cl_over:(yy<conf.zebra_under)?cl_under:COLOR_TRANSPARENT;

                  } // next x 

               } // _next line y
                    
            gui_osd_draw_zebra_osd();

            memcpy(scr_buf, imgbuf, ss);
            memcpy(scr_buf+ss, imgbuf,ss);
            need_restore=1;
            return 1;
        } // _else
    }    // _if(imgbuf)

    return 0; // buffer not allocated

}
// -------------------------------------------------------------------------------------------------------------------------
//                                                       IXUS100 or SX120IS
// -------------------------------------------------------------------------------------------------------------------------
#elif defined(CAMERA_ixus100)|| defined(CAMERA_sx120is)
int gui_osd_draw_zebra() 
{
  unsigned int v, s, x, y, f, over;
  color cl_under=conf.zebra_color>>8, cl_over=conf.zebra_color&0xFF;
  static int need_restore=0;
  int viewport_height;
  int viewport_width;                                                 
  int m = ((mode_get()&MODE_MASK) == MODE_REC);
  color cls[] = 
     {
        COLOR_TRANSPARENT,
        (m)?COLOR_BLUE:COLOR_BLUE_P,
        COLOR_GREEN,
        (m)?COLOR_BLUE_LT:COLOR_BLUE_LT_P,
        COLOR_RED,
        (m)?COLOR_PINK:COLOR_PINK_P,   // light red or pink
        (m)?COLOR_YELLOW:COLOR_YELLOW_P,
        COLOR_BLACK
     };

int ss = screen_buffer_size;                                         // 960x270                          
int sw=360;

// -----------------------------------  allocate buffer memory  --------------------------------

    scr_buf = vid_get_bitmap_fb();                                   // address of bmp buffer

    if (!imgbuf) allocate_imgbuf();                                  // more than enough for a 960 x 160 central area

// ----------------------------------  if memory allocated OK  ---------------------------------
    if (imgbuf) 
    {
        ++timer;

    //Try to get the best viewport buffer. In playmode its the _d one, in 
    //record mode we try to get the fast live one first 
 	if( (mode_get() & MODE_MASK) == MODE_PLAY ) 
       { 
 	  img_buf = vid_get_viewport_fb_d(); 
 	 } 
 	else 
       { 
 	  img_buf = vid_get_viewport_live_fb(); 
 	  if( !img_buf ) 
          { 
 	     img_buf = vid_get_viewport_fb(); 
 	    } 
 	 } 
     img_buf+=360*40*3;                                              // start of 720 x 160 central area
// --------------------------------------------------------------------------------------------
        viewport_height = vid_get_viewport_height();                 // 240 pixels high
        switch (conf.zebra_mode) 
       {
            case ZEBRA_MODE_ZEBRA_1:
                f = 4;
                break;
            case ZEBRA_MODE_ZEBRA_2:
                f = 8;
                break;
            case ZEBRA_MODE_SOLID:
                f = 1; 
                break;
            case ZEBRA_MODE_BLINKED_1:
                f = timer&2; 
                break;
            case ZEBRA_MODE_BLINKED_3:
                f = timer&8; 
                break;
            case ZEBRA_MODE_BLINKED_2:
            default:
                f = timer&4; 
                break;
        }

// -------------------------------------------------------------------------------------------
        if (!f)                                                                                 // no-zebra phase of blink mode zebra
       {
            if (need_restore) 
             {
                if (conf.zebra_restore_screen || conf.zebra_restore_osd) 
                 {
                    draw_restore();
                 } 
               else 
                 {                             
                    memset(imgbuf, COLOR_TRANSPARENT,screen_buffer_width*160);                   // clear sdm buffer
                    gui_osd_draw_zebra_osd();                                                    // draw other OSD stuff
                    memcpy(scr_buf+screen_buffer_width*40, imgbuf,screen_buffer_width*160);      
                    memcpy(scr_buf+screen_buffer_width*40+ss, imgbuf,screen_buffer_width*160);  
                }
                need_restore=0;
            }
            return !(conf.zebra_restore_screen && conf.zebra_restore_osd);
        } 
// ------------------------------------------------------------------------------------------

   else 
        {
            int step_x, step_v,sy, sx;
            over = 255-conf.zebra_over;

                if (conf.zebra_multichannel) {step_x=2; step_v=6;} else {step_x=1; step_v=3;}// for each line
                for (y=0, v=0; y<=160; ++y)                                                  // for every line of live-buffer central region
                {
                 sy=y*screen_buffer_width;                                                   // bitmap buffer pointer for start of row 'y' of central area
                 sx=0;                                                                       // 'x' position along row                                                                
                  for (x=0; x<sw; x+=step_x, sx+=step_x, v+=step_v)                           // step along the viewport width for 720 pixels (normally)
                   {
                     register int yy, uu, vv;
                     int sel;
                     yy = img_buf[v+1];                                                       // luminance Y0 or Y2 (offset 1 or 4)
                     s=sy+sx*2;                                                               // update bmp buffer pointer for each pixel position
//----------------------------------  RGB zebra (overexp. only) --------------------------------
                     if (conf.zebra_multichannel) 
                      {
                        uu = (signed char)img_buf[v];                                         
                        vv = (signed char)img_buf[v+2];                                       
                        sel=0;
                        if (!((conf.zebra_mode == ZEBRA_MODE_ZEBRA_1 || conf.zebra_mode == ZEBRA_MODE_ZEBRA_2) && (y-x-timer)&f)) 
                         {
                           if (clip8((yy<<12) +           vv*5743 + 2048)>>12>over) sel  = 4;     // R
                           if (clip8((yy<<12) - uu*1411 - vv*2925 + 2048)>>12>over) sel |= 2;     // G
                           if (clip8((yy<<12) + uu*7258           + 2048)>>12>over) sel |= 1;     // B
                         }
                          imgbuf[s]=imgbuf[s+1]=cls[sel];                                         // draw two pixels
                          imgbuf[s+2]=imgbuf[s+3]=cls[sel];                                       // and two more               
                      }

// -----------------------------------  Zebra mode 1 or 2 ---------------------------------------
                     else if (((conf.zebra_mode == ZEBRA_MODE_ZEBRA_1 || conf.zebra_mode == ZEBRA_MODE_ZEBRA_2) && (y-x-timer)&f)) imgbuf[s]=imgbuf[s+1]=COLOR_TRANSPARENT;
// ---------------------------------- colour is under, over or transparent ----------------------
                        else imgbuf[s]=imgbuf[s+1]=(yy>over)?cl_over:(yy<conf.zebra_under)?cl_under:COLOR_TRANSPARENT;
                  } // next x 

               } // _next line y
                    
            gui_osd_draw_zebra_osd();

            memcpy(scr_buf+40*screen_buffer_width,imgbuf,screen_buffer_width*160);
            memcpy(scr_buf+40*screen_buffer_width+ss,imgbuf,screen_buffer_width*160);
            need_restore=1;
            return 1;
        } // _else


    }    // _if(imgbuf)

    return 0; // buffer not allocated

}

// -------------------------------------------------------------------------------------------------------------------------
#else

int gui_osd_draw_zebra() {
    unsigned int v, s, x, y, f, over;
    color cl_under=conf.zebra_color>>8, cl_over=conf.zebra_color&0xFF;
    static int need_restore=0;
    int viewport_height;
 int m = ((mode_get()&MODE_MASK) == MODE_REC);
    color cls[] = {
        COLOR_TRANSPARENT,
        (m)?COLOR_BLUE:COLOR_BLUE_P,
        COLOR_GREEN,
        (m)?COLOR_BLUE_LT:COLOR_BLUE_LT_P,
        COLOR_RED,
        (m)?COLOR_PINK:COLOR_PINK_P,   // light red or pink
        (m)?COLOR_YELLOW:COLOR_YELLOW_P,
        COLOR_BLACK
    };


#if defined(CAMERA_tx1)
int screen_buffer_size = 480 * 271;
int screen_buffer_width = 480;
int screen_buffer_height = 271;
int screen_height = 240;
int screen_width = 360;
#endif
#if defined(CAMERA_ixus980)
int ss = screen_size/2;  // screen_width * screen_height
int sw = screen_width/2; // vid_get_bitmap_width()
#else
//int ss = screen_size;  // screen_width * screen_height
int ss = screen_buffer_size; // bmp buffer can be 360x240, 720x240 or 960x270
//int sw = screen_width; // vid_get_bitmap_width()
int sw=vid_get_viewport_width(); // we scan live-image width, not screen width
#endif

#if defined (TESTS)
// -------------------------------------------------------------------
//                 Test writing rectangle to BMP buffer
//--------------------------------------------------------------------

int offset=80*screen_buffer_width;
scr_buf = vid_get_bitmap_fb();
if (!imgbuf) allocate_imgbuf();
if(imgbuf)
{
 	//  img_buf = vid_get_viewport_live_fb(); 
 	//  if( !img_buf ) 
     //     { 
 	//     img_buf = vid_get_viewport_fb(); 
 	//    } 
 memset(imgbuf,COLOR_TRANSPARENT,ss);                                  // clear screen
 memcpy(scr_buf,imgbuf,ss);
 memcpy(scr_buf+ss, imgbuf, ss);                                       // update second buffer
 for(x=0;x<120;x++)
 {
  for(y=0;y<80;y++)                                                    // for each row
  {
   imgbuf[(120*2)+2*x+(y*screen_buffer_width)+offset]=COLOR_RED;     //  from x=120 to x=240
   imgbuf[(120*2)+2*x+(y*screen_buffer_width)+offset+1]=COLOR_RED;
  }
 }
memcpy(scr_buf,imgbuf,ss);
memcpy(scr_buf+ss,imgbuf,ss);// update second buffer
 return 1;
 }
return 0;

#endif


// -----------------------------------  allocate buffer memory  --------------------------------

    scr_buf = vid_get_bitmap_fb();                                   // address of bmp buffer

    if (!imgbuf) allocate_imgbuf();
// ----------------------------------  if memory allocated OK  ---------------------------------
    if (imgbuf) 
    {
        ++timer;

    //Try to get the best viewport buffer. In playmode its the _d one, in 
    //record mode we try to get the fast live one first 
 	if( (mode_get() & MODE_MASK) == MODE_PLAY ) 
       { 
 	  img_buf = vid_get_viewport_fb_d(); 
 	 } 
 	else 
       { 
 	  img_buf = vid_get_viewport_live_fb(); 
 	  if( !img_buf ) 
          { 
 	     img_buf = vid_get_viewport_fb(); 
 	    } 
 	 } 
// --------------------------------------------------------------------------------------------
        viewport_height = vid_get_viewport_height();
        switch (conf.zebra_mode) 
       {
            case ZEBRA_MODE_ZEBRA_1:
                f = 4;
                break;
            case ZEBRA_MODE_ZEBRA_2:
                f = 8;
                break;
            case ZEBRA_MODE_SOLID:
                f = 1; 
                break;
            case ZEBRA_MODE_BLINKED_1:
                f = timer&2; 
                break;
            case ZEBRA_MODE_BLINKED_3:
                f = timer&8; 
                break;
            case ZEBRA_MODE_BLINKED_2:
            default:
                f = timer&4; 
                break;
        }
// -------------------------------------------------------------------------------------------
        if (!f) 
       {
            if (need_restore) 
             {
                if (conf.zebra_restore_screen || conf.zebra_restore_osd) 
                 {
                    draw_restore();
                 } 
               else 
                 {
#if defined(CAMERA_tx1)
                    memset(imgbuf, COLOR_TRANSPARENT, screen_buffer_size);
                    gui_osd_draw_zebra_osd();
                    memcpy(scr_buf, imgbuf, screen_buffer_size);
                    memcpy(scr_buf+screen_buffer_size, imgbuf, screen_buffer_size);
#else
                    memset(imgbuf, COLOR_TRANSPARENT,ss);  // clear sdm buffer
                    gui_osd_draw_zebra_osd();             // draw other OSD stuff
                    memcpy(scr_buf, imgbuf,ss);           // clear first buffer
                    memcpy(scr_buf+ss, imgbuf,ss);        // clear second buffer
#endif
                }
                need_restore=0;
            }
            return !(conf.zebra_restore_screen && conf.zebra_restore_osd);
        } 
// ------------------------------------------------------------------------------------------
   else {
            int step_x, step_v;
            over = 255-conf.zebra_over;

                if (conf.zebra_multichannel) {step_x=2; step_v=6;} else {step_x=1; step_v=3;}// for each line
                for (s=0, y=1, v=0; y<=viewport_height; ++y)                                 // for every line from 1 to live-buffer height
                {
                  for (x=0; x<sw; x+=step_x, s+=step_x, v+=step_v)                           // step along the viewport width for 360 pixels (normally)
                   {
                     register int yy, uu, vv;
                     int sel;
                     yy = img_buf[v+1];                                                       // luminance Y0 or Y2 (offset 1 or 4)

//----------------------------------  RGB zebra (overexp. only) --------------------------------
                     if (conf.zebra_multichannel) 
                      {
                        uu = (signed char)img_buf[v];                                         
                        vv = (signed char)img_buf[v+2];                                       
                        sel=0;
                        if (!((conf.zebra_mode == ZEBRA_MODE_ZEBRA_1 || conf.zebra_mode == ZEBRA_MODE_ZEBRA_2) && (y-x-timer)&f)) 
                         {
                           if (clip8((yy<<12) +           vv*5743 + 2048)>>12>over) sel  = 4;     // R
                           if (clip8((yy<<12) - uu*1411 - vv*2925 + 2048)>>12>over) sel |= 2;     // G
                           if (clip8((yy<<12) + uu*7258           + 2048)>>12>over) sel |= 1;     // B
                         }
#if defined(CAMERA_ixus980)                                      // bmp buffers are 720 wide, double pixels
                          imgbuf[2*s]=imgbuf[2*s+1]=imgbuf[2*s+2]=imgbuf[2*s+3]=cls[sel];
                      }

// --------------------------------- not RGB zebra ---------------------------------------------
                        else if (((conf.zebra_mode == ZEBRA_MODE_ZEBRA_1 || conf.zebra_mode == ZEBRA_MODE_ZEBRA_2) && (y-x-timer)&f)) imgbuf[2*s]=imgbuf[2*s+1]=COLOR_TRANSPARENT;
                        else imgbuf[2*s]=imgbuf[2*s+1]=(yy>over)?cl_over:(yy<conf.zebra_under)?cl_under:COLOR_TRANSPARENT;
#else
                          imgbuf[s]=imgbuf[s+1]=cls[sel];                         
                      }
// ----------------------------------------------------------------------------------------------
// -----------------------------------  Zebra mode 1 or 2 ---------------------------------------
                        else if (((conf.zebra_mode == ZEBRA_MODE_ZEBRA_1 || conf.zebra_mode == ZEBRA_MODE_ZEBRA_2) && (y-x-timer)&f)) imgbuf[s]=COLOR_TRANSPARENT;
// ---------------------------------- colour is under, over or transparent ----------------------
                        else imgbuf[s]=(yy>over)?cl_over:(yy<conf.zebra_under)?cl_under:COLOR_TRANSPARENT;
#endif
                  } // next x 


#if !defined(CAMERA_tx1)
         if(1)
                 if (y*screen_height/viewport_height == (s+sw)/sw)                    
                  {
#if defined(CAMERA_ixus980) 
                   memcpy(imgbuf+2*s, imgbuf+2*s-2*sw,2*sw);
                    s+=2*sw;
#else
                    s+=sw;                                                           
#endif
                  }
#else     
                                                                                 // TX1 camera
                 if (y*screen_height/viewport_height == (s+screen_buffer_width)/screen_buffer_width) 
                  {
                   memcpy(imgbuf+s, imgbuf+s-screen_buffer_width, screen_buffer_width);
                   s+=screen_buffer_width;
                  }
                    s+=screen_buffer_width-screen_width; 
#endif
               } // _next line y
                    
            gui_osd_draw_zebra_osd();
#if !defined(CAMERA_tx1)
#if defined(CAMERA_ixus980)
            memcpy(scr_buf, imgbuf, 2*ss);
            memcpy(scr_buf+2*ss, imgbuf,2*ss);
#else
            memcpy(scr_buf, imgbuf, ss);
            memcpy(scr_buf+ss, imgbuf,ss);
#endif
#else
            memcpy(scr_buf, imgbuf, screen_buffer_size);
            memcpy(scr_buf+screen_buffer_size, imgbuf, screen_buffer_size);
#endif

            need_restore=1;
            return 1;
        } // _else
    }    // _if(imgbuf)

    return 0; // buffer not allocated


}
// ---------------------------------------------------------------------------------------------------------------------------------
#endif
static void gui_osd_draw_blended_histo(coord x, coord y) {
    register unsigned int i, v, red, grn, blu, sel;
    int m = ((mode_get()&MODE_MASK) == MODE_REC);


    color cls[] = {
        conf.histo_color>>8,
        (m)?0xDF:0xCC,
        COLOR_GREEN,
        (m)?COLOR_BLUE_LT:0x99,
        COLOR_RED,
        (m)?0x66:0xE2,
        (m)?COLOR_YELLOW:0x66,
        COLOR_WHITE
    };

    for (i=0; i<HISTO_WIDTH; ++i) {
        red = histogram[HISTO_R][i];
        grn = histogram[HISTO_G][i];
        blu = histogram[HISTO_B][i];

        for (v=1; v<HISTO_HEIGHT; ++v) {
            sel = 0;

            if (v < red) sel = 4;
            if (v < grn) sel |= 2;
            if (v < blu) sel |= 1;

            draw_pixel(x+1+i, y+HISTO_HEIGHT-v, cls[sel]);
        }
    }

    draw_rect(x, y, x+1+HISTO_WIDTH, y+HISTO_HEIGHT, conf.histo_color2&0xFF);
}

// -------------------------------------------------------------------
void gui_osd_draw_histo() {
    switch (conf.histo_layout) {
        case OSD_HISTO_LAYOUT_Y:
                gui_osd_draw_single_histo(HISTO_Y, conf.histo_pos.x, conf.histo_pos.y, 0);
                break;
        case OSD_HISTO_LAYOUT_A_Y:
                gui_osd_draw_single_histo(HISTO_RGB, conf.histo_pos.x, conf.histo_pos.y, 0);
                gui_osd_draw_single_histo(HISTO_Y, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT, 0);
                break;
        case OSD_HISTO_LAYOUT_R_G_B:
                gui_osd_draw_single_histo(HISTO_R, conf.histo_pos.x, conf.histo_pos.y, 0);
                gui_osd_draw_single_histo(HISTO_G, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT, 0);
                gui_osd_draw_single_histo(HISTO_B, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT*2, 0);
                break;
        case OSD_HISTO_LAYOUT_A_yrgb:
                gui_osd_draw_single_histo(HISTO_RGB, conf.histo_pos.x, conf.histo_pos.y, 0);
                gui_osd_draw_single_histo(HISTO_Y, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT, 1);
                gui_osd_draw_single_histo(HISTO_R, conf.histo_pos.x+HISTO_WIDTH/2+1, conf.histo_pos.y+HISTO_HEIGHT, 1);
                gui_osd_draw_single_histo(HISTO_G, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT+HISTO_HEIGHT/2, 1);
                gui_osd_draw_single_histo(HISTO_B, conf.histo_pos.x+HISTO_WIDTH/2+1, conf.histo_pos.y+HISTO_HEIGHT+HISTO_HEIGHT/2, 1);
                break;
        case OSD_HISTO_LAYOUT_Y_argb:
                gui_osd_draw_single_histo(HISTO_Y, conf.histo_pos.x, conf.histo_pos.y, 0);
                gui_osd_draw_single_histo(HISTO_RGB, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT, 1);
                gui_osd_draw_single_histo(HISTO_R, conf.histo_pos.x+HISTO_WIDTH/2+1, conf.histo_pos.y+HISTO_HEIGHT, 1);
                gui_osd_draw_single_histo(HISTO_G, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT+HISTO_HEIGHT/2, 1);
                gui_osd_draw_single_histo(HISTO_B, conf.histo_pos.x+HISTO_WIDTH/2+1, conf.histo_pos.y+HISTO_HEIGHT+HISTO_HEIGHT/2, 1);
                break;
        case OSD_HISTO_LAYOUT_BLEND:
                gui_osd_draw_blended_histo(conf.histo_pos.x, conf.histo_pos.y);
                break;
        case OSD_HISTO_LAYOUT_BLEND_Y:
                gui_osd_draw_blended_histo(conf.histo_pos.x, conf.histo_pos.y);
                gui_osd_draw_single_histo(HISTO_Y, conf.histo_pos.x, conf.histo_pos.y+HISTO_HEIGHT, 0);
                break;
        case OSD_HISTO_LAYOUT_A:
        default:
                gui_osd_draw_single_histo(HISTO_RGB, conf.histo_pos.x, conf.histo_pos.y, 0);
                break;
    }

    if (conf.histo_layout != OSD_HISTO_LAYOUT_R_G_B) {
        if (under_exposed && conf.show_overexp) {
            draw_filled_ellipse(conf.histo_pos.x+5, conf.histo_pos.y+5, 3, 3, MAKE_COLOR(conf.histo_color2>>8, conf.histo_color2>>8));
        }

        if (over_exposed && conf.show_overexp) {
            draw_filled_ellipse(conf.histo_pos.x+HISTO_WIDTH-5, conf.histo_pos.y+5, 3, 3, MAKE_COLOR(conf.histo_color2>>8, conf.histo_color2>>8));
        }
    }

    if (histo_magnification) {
        sprintf(osd_buf, " %d.%02dx ", histo_magnification/1000, histo_magnification/10%100);
        draw_string(conf.histo_pos.x, conf.histo_pos.y-FONT_HEIGHT, osd_buf, conf.histo_color,1);
    } else {
        draw_filled_rect(conf.histo_pos.x, conf.histo_pos.y-FONT_HEIGHT, conf.histo_pos.x+8*FONT_WIDTH, conf.histo_pos.y-1, MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT));
    }
}

// -------------------------------------------------------------------
static void sprintf_dist(char *buf, float dist) {
// length of printed string is always 4
    if (dist<=0 || dist>(int) INFINITY) 
    {
        sprintf(buf, " INF");
    } else if (dist<10000) 
    {
        sprintf(buf, "%d.%03d", (int)(dist/1000), (int)(dist)%1000);
    } else if (dist<100000) 
    {
        sprintf(buf, "%02d.%d", (int)(dist/1000), (int)(dist/100)%10);
    } else 
    {
        sprintf(buf, "%4d", (int)(dist/1000));
    }
}



void load_bitmap(char *img, int viewport_size, char *fn)
{
   int fd;
   fd = open(fn, O_RDONLY, 0777);
    if (fd>=0) 
      {
      int rcnt = read(fd, img, viewport_size *3);
      close(fd);
      }
   return;
}

int get_memory_info()
{
    int size, l_size, d;
    char* ptr;
    size = 16;
    while (1) 
       {
        ptr= malloc(size);
        if (ptr) 
          {
            free(ptr);
            size <<= 1;
          } 
          else break;
        }

    l_size = size;
    size >>= 1;
    d=1024;
    while (d) 
      {
        ptr = malloc(size);
        if (ptr) 
         {
            free(ptr);
            d = l_size-size;
            if (d<0) d=-d;
            l_size = size;
            size += d>>1;
          } 
         else 
          {
            d = size-l_size;
            if (d<0) d=-d;
            l_size = size;
            size -= d>>1;
           }       
        }
     return size-1;
}


// **************************************************************************************************

void gui_osd_draw_stereo() {

    float denom,pc,img_dist,sw;
    int x,y;
    color cl;
    deviation=0.0f;
    sensor_width = get_sensor_width();              
    sw = ((float)sensor_width)/1000;                // sensor width as real value
    coc = sw/1200;                                  // circle-of-confusion (0.006 mm for 1/1.8" sensor)
    my_max_deviation = sw/conf.stereo_deviation;
    myhyp = shooting_get_hyperfocal_distance();
    realfl = ((float)get_focal_length(lens_get_zoom_point()))/1000;
    int pos_x=(conf.camera_orientation==1)?0:conf.stereo_osd_pos.x;
    int pos_y=(conf.camera_orientation==1)?conf.stereo_osd_pos.y+16:conf.stereo_osd_pos.y;
    if(conf.camera_orientation==1)pos_x=0;
// -------------------------------------------------------------------------------------------------
     if(conf.layout_mode == 0)         
  {
    
  } 

// -------------------------------------------------------------------------------------------------

     if(conf.layout_mode == 1)          
  {
   unsigned long t;
   static struct tm *ttm;
   t = time(NULL);
   ttm = localtime(&t);
   sprintf(osd_buf, "%3s%2u:%02u:%02u%4s","", ttm->tm_hour, ttm->tm_min,ttm->tm_sec,"");
   draw_big_string(pos_x,pos_y, osd_buf,MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
   pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, " SDM Ver: %s ",SDM_VERSION);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    int perc = (int)(GetFreeCardSpaceKb()*100/GetTotalCardSpaceKb());
    sprintf(osd_buf, "%2d%%",perc);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    color bg = COLOR_RED;
    color fg = COLOR_GREEN;
    gui_bg(pos_x+48,(conf.camera_orientation==1)?83+16:83,perc,bg,fg,1,12);
    sprintf(osd_buf, " %4dM",GetFreeCardSpaceKb()/1024 );
    draw_big_string(pos_x+98+3*BIG_FONT_WIDTH,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "Memory %8d",get_memory_info());
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    unsigned long v = get_batt_average();
    sprintf(osd_buf, "%-9s%ld.%03ldV","Battery ", v/1000, v%1000);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
  } 

// -------------------------------------------------------------------------------------------------

    if(conf.layout_mode == 2)          
  {
 
   int zp=lens_get_zoom_point(), av=shooting_get_real_av(), fl;

    draw_big_string(pos_x,pos_y,"  ZOOM VALUES  ",MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    pos_y+= BIG_FONT_HEIGHT;

    fl=get_focal_length(zp);
    sprintf(osd_buf, "FL :%7d.%dmm", fl/1000, fl%1000/100);
    draw_big_string(pos_x,pos_y, osd_buf, conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    fl=get_effective_focal_length(zp);
    sprintf(osd_buf, "EFL:%9dmm", fl/1000);
    draw_big_string(pos_x,pos_y, osd_buf, conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    fl=get_zoom_x(zp);
    sprintf(osd_buf, "ZP :%6d/%d.%dx", zp, fl/10, fl%10);
    draw_big_string(pos_x,pos_y, osd_buf, conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

  } 

// -------------------------------------------------------------------------------------------------
     if(conf.layout_mode == 3)         
  {
   char* ss;
   int m = mode_get();
   int flashStatus = shooting_get_prop(PROPCASE_FLASH_MODE);
 
    int t = shooting_get_prop(PROPCASE_SHOOTING_MODE);  
    ss = (char*)shooting_get_camera_tv_string();
    ip = (isoPointer)shooting_get_iso_line();
    sprintf(osd_buf, "  SYNCH FLASH  ");
    draw_big_string(pos_x,pos_y, osd_buf,MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    pos_y+= BIG_FONT_HEIGHT;

#if CAM_CURTAIN
    if(conf.synch_enable && (flashStatus==1) && !shooting_get_prop(PROPCASE_CURTAIN) && ((m&MODE_SHOOTING_MASK) != MODE_M))
#else
     if(conf.synch_enable && (flashStatus==1)&& ((m&MODE_SHOOTING_MASK) != MODE_M))
#endif

      {

        sprintf(osd_buf,"%s",conf.camera_position?"Flash        ON":"Flash dim  -4EV");
        draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
        pos_y+= BIG_FONT_HEIGHT;

        
        sprintf(osd_buf,"Tv%13s",(conf.tv_override_enable)?tv_enum():ss);
        draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
        pos_y+= BIG_FONT_HEIGHT;

        sprintf(osd_buf,"Mode%11d",t);
        draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
        pos_y+= BIG_FONT_HEIGHT;

        sprintf(osd_buf,"Delay(0.1ms)%3d",conf.synch_delay_value);
        draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
        pos_y+= BIG_FONT_HEIGHT;

        sprintf(osd_buf,"ISO%12s",ip->name);
        draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
        pos_y+= BIG_FONT_HEIGHT;   
     
      }
    else
      { 
        sprintf(osd_buf,"Invalid state  ");
        draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
        pos_y+= BIG_FONT_HEIGHT;

        if(!conf.synch_enable)
         {
          draw_big_string(pos_x,pos_y,"Synch disabled ",conf.osd_color);
          pos_y+= BIG_FONT_HEIGHT;
         }

        if(flashStatus==0)
         {
          draw_big_string(pos_x,pos_y,"Turn Flash ON  ",conf.osd_color);
          pos_y+= BIG_FONT_HEIGHT;
         }


        if((m&MODE_SHOOTING_MASK) == MODE_M)
         {
          draw_big_string(pos_x,pos_y,"MANUAL mode    ",conf.osd_color);
          pos_y+= BIG_FONT_HEIGHT;
         }
#if CAM_CURTAIN
        if(shooting_get_prop(PROPCASE_CURTAIN))
         {
          draw_big_string(pos_x,pos_y,"Second curtain ",conf.osd_color);
          pos_y+= BIG_FONT_HEIGHT;
         }
#endif
      }

  } 
// -------------------------------------------------------------------------------------------------
     if(conf.layout_mode == 4)         
  {
    sprintf(osd_buf, "STEREO SETTINGS");
    draw_big_string(pos_x,pos_y, osd_buf,MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "DIRECT mode%4s",conf.ricoh_ca1_mode?"Y":"N");
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "Cam spacing%4d",conf.stereo_spacing);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "DEV factor%5d",conf.stereo_deviation);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "NEAR factor%4d",conf.minimum_dist_factor);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;
  } 


// -------------------------------------------------------------------------------------------------
  if(conf.layout_mode == 5)         // Rangefinder mode Manual
  {
   float denom = (float)((subject_near*subject_far)/(subject_far-subject_near));
   deviation = (float)((conf.stereo_spacing*realfl)/denom);      
   percent_deviation = deviation*100/(my_max_deviation);
   draw_big_string(pos_x,pos_y,"   TWIN CAMS   ",MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
  
   //    Colour is green up to 100%, red if calculated value > 100%
   //-------------------------------------------------------------------

    x = pos_x;
    draw_big_string(pos_x,pos_y+BIG_FONT_HEIGHT,lang_str(LANG_SDM_RF_FAR),conf.osd_color);
    if (subject_far > 64000) sprintf(osd_buf,"%6s", "INF");
    else sprintf(osd_buf,"%6d", subject_far);
    draw_big_string(pos_x+9*BIG_FONT_WIDTH,pos_y+BIG_FONT_HEIGHT, osd_buf,conf.osd_color);

    cl = (subject_near>conf.stereo_spacing*conf.minimum_dist_factor)?conf.osd_color:MAKE_COLOR(COLOR_BG,COLOR_RED);
    draw_big_string(pos_x,pos_y+2*BIG_FONT_HEIGHT,lang_str(LANG_SDM_RF_NEAR),conf.osd_color);
    sprintf(osd_buf,"%6d",subject_near);
    draw_big_string(pos_x+9*BIG_FONT_WIDTH,pos_y+2*BIG_FONT_HEIGHT, osd_buf,conf.osd_color);

    rf_focus = (2*subject_far*subject_near)/(subject_near+subject_far);
    sprintf(osd_buf,"Focus at %6d",rf_focus);
    draw_big_string(pos_x,pos_y+3*BIG_FONT_HEIGHT,osd_buf,conf.osd_color);
    int required_av =  (realfl*realfl*(subject_far-subject_near))/ (coc*((subject_far*(subject_near-realfl))+(subject_near*(subject_far-realfl))));
    sprintf(osd_buf,"Av min %8d",required_av);
    draw_big_string(pos_x,pos_y+4*BIG_FONT_HEIGHT,osd_buf,conf.osd_color);
    cl = (percent_deviation<=100)?COLOR_GREEN:COLOR_RED;
    y = pos_y+5*BIG_FONT_HEIGHT;
    gui_bg(x,y,percent_deviation,COLOR_WHITE,cl,1,30);
  } 

// -------------------------------------------------------------------------------------------------
  if(conf.layout_mode == 6)         // Shift mode Manual
  {
   draw_big_string(pos_x,pos_y," SINGLE CAMERA ",MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    x = pos_x;

    draw_string(pos_x,pos_y+BIG_FONT_HEIGHT,lang_str(LANG_SDM_RF_FAR),MAKE_COLOR(COLOR_TRANSPARENT,conf.osd_color),1);
    if (subject_far > 64000) sprintf(osd_buf,"%6s", "INF");
    else sprintf(osd_buf,"%6d", subject_far);
    draw_string(pos_x+12*BIG_FONT_WIDTH,pos_y+BIG_FONT_HEIGHT,osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,conf.osd_color),1);

     rfshift = (int)(((float)get_sensor_width()/1000/conf.stereo_deviation/realfl)*subject_near*subject_far)/(subject_far-subject_near);
    if(rfshift>(subject_near/conf.minimum_dist_factor))rfshift=subject_near/conf.minimum_dist_factor;

    draw_string(pos_x,pos_y+BIG_FONT_HEIGHT+FONT_HEIGHT,lang_str(LANG_SDM_RF_NEAR),MAKE_COLOR(COLOR_TRANSPARENT,conf.osd_color),1);
    sprintf(osd_buf,"%6d",subject_near);
    draw_string(pos_x+12*BIG_FONT_WIDTH,pos_y+BIG_FONT_HEIGHT+FONT_HEIGHT, osd_buf,(rfshift*conf.minimum_dist_factor<subject_near)?MAKE_COLOR(COLOR_TRANSPARENT,conf.osd_color):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED),1);

    rf_focus = (2*subject_far*subject_near)/(subject_near+subject_far);
    sprintf(osd_buf,"Focus at %21d",rf_focus);
    draw_string(pos_x,pos_y+BIG_FONT_HEIGHT+2*FONT_HEIGHT,osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,conf.osd_color),1);

    int required_av =  (realfl*realfl*(subject_far-subject_near))/ (coc*((subject_far*(subject_near-realfl))+(subject_near*(subject_far-realfl))));
    sprintf(osd_buf,"Av min %8d",required_av);
    draw_big_string(pos_x,pos_y+BIG_FONT_HEIGHT+3*FONT_HEIGHT,osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,conf.osd_color));
 
    sprintf(osd_buf,"Shift max. %4d",rfshift);
    draw_big_string(pos_x,pos_y+3*FONT_HEIGHT+2*BIG_FONT_HEIGHT,osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,conf.osd_color));

    denom = (float)((subject_near*subject_far)/(subject_far-subject_near));
    deviation = (float)((rfshift*realfl)/denom);      
    percent_deviation = deviation*100/(my_max_deviation);

    cl = (percent_deviation<100)?COLOR_GREEN:COLOR_RED;
    y = pos_y+3*FONT_HEIGHT+3*BIG_FONT_HEIGHT;
    gui_bg(x,y,percent_deviation,COLOR_WHITE,cl,1,30);
  } 

// -------------------------------------------------------------------------------------------------
  if(conf.layout_mode == 7)         // Focused distance testing
  {
    draw_big_string(pos_x,pos_y,lang_str(LANG_SDM_FOCUS_TEST_TITLE),MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    draw_big_string(pos_x,pos_y+BIG_FONT_HEIGHT,lang_str(LANG_SDM_FOCUS_TEST_FOCUSED),conf.osd_color);
    if(lens_get_focus_pos_from_lense()>65000) sprintf(osd_buf,"  INF");
    else sprintf(osd_buf,"%6d",lens_get_focus_pos_from_lense()+fpd);
    draw_big_string(pos_x+9*BIG_FONT_WIDTH,pos_y+BIG_FONT_HEIGHT, osd_buf,conf.osd_color);

   
    draw_big_string(pos_x,pos_y+2*BIG_FONT_HEIGHT,lang_str(LANG_SDM_FOCUS_TEST_TO_SUBJECT), conf.osd_color);
    if(lens_get_focus_pos_from_lense()>65000) sprintf(osd_buf,"  INF");
    else sprintf(osd_buf,"%6d",lens_get_focus_pos_from_lense());
    draw_big_string(pos_x+9*BIG_FONT_WIDTH,pos_y+2*BIG_FONT_HEIGHT, osd_buf, conf.osd_color);

    draw_big_string(pos_x,pos_y+3*BIG_FONT_HEIGHT,lang_str(LANG_SDM_FOCUS_TEST_TO_SENSOR), conf.osd_color);
    sprintf(osd_buf,"%6d",fpd);
    draw_big_string(pos_x+9*BIG_FONT_WIDTH,pos_y+3*BIG_FONT_HEIGHT, osd_buf, conf.osd_color);
  } 

// -------------------------------------------------------------------------------------------------
     if(conf.layout_mode == 8)         
  {
   char* tvs;

    sprintf(osd_buf, "    Tv Av ISO   ");
    draw_big_string(pos_x,pos_y, osd_buf,MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    pos_y+= BIG_FONT_HEIGHT;

    tvs = shooting_get_camera_tv_string();

    sprintf(osd_buf, "Cam Tv%10s",tvs);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

#if(CAM_HAS_USER_TV_MODES)
    ss = (SsPointer)shooting_get_tv_line();
    sprintf(osd_buf, "User Tv%9s",ss->name);
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;
#endif

    cl = (1.342*realav > coc*1000)?MAKE_COLOR(COLOR_RED,conf.osd_color & 0xFF):conf.osd_color;
      sprintf(osd_buf,"Real Av%6d.%02d",(int)(myav/100),(int)(myav%100));
    draw_big_string(pos_x, pos_y, osd_buf,conf.osd_color);
     pos_y+= BIG_FONT_HEIGHT;
     sprintf(osd_buf,"ISO-M%11d",shooting_get_iso_market());
     draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
      pos_y+= BIG_FONT_HEIGHT;
     sprintf(osd_buf,"Luminance %6d",shooting_get_luminance()/100);
     draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
  } 

// -------------------------------------------------------------------------------------------------
     if(conf.layout_mode == 9)         
  {

   entrance = ((float)get_focal_length(lens_get_zoom_point()))/(10*myav);
   mag = ((float)conf.digi_scope_fl)/conf.digi_ep_fl;
   exitp = ((float)conf.digi_obj_diam)/mag;

    sprintf(osd_buf, "Digiscope Data ");
    draw_big_string(pos_x,pos_y, osd_buf,MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "Scope mag%4d.%1d",(int)mag,((int)(mag*10))%10);                                             
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "Scope EP%5d.%1d",(int)exitp,((int)(exitp*10))%10);                                             
    draw_big_string(pos_x,pos_y, osd_buf,(exitp<entrance)?MAKE_COLOR(conf.osd_color>>8,COLOR_RED):conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf, "Camera EP%4d.%1d",(int)entrance,((int)(entrance*10))%10);                                              
    draw_big_string(pos_x,pos_y, osd_buf,(exitp<=entrance)?conf.osd_color:MAKE_COLOR(conf.osd_color>>8,COLOR_RED));
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf,"EFL%8s%4d","",(int)(mag*get_focal_length(lens_get_zoom_point())/1000)); // digiscoping fl
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf,"EFL (35mm)%5d",(int)(mag*get_effective_focal_length(lens_get_zoom_point())/1000));  // 35mm equiv of EFL
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
  } 
// -------------------------------------------------------------------------------------------------

     if(conf.layout_mode == 10)  
   {
    int fov = get_fov(lens_get_zoom_point());

    sprintf(osd_buf, "Panorama Data");
    draw_big_string(pos_x,pos_y, osd_buf,MAKE_COLOR(COLOR_BLUE,COLOR_WHITE));
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf,"%-11s%2d","FOV(H)",fov);                                             
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf,"%-11s%2d","FOV(V)",(int)(fov*3/4));                                             
    draw_big_string(pos_x,pos_y, osd_buf,(exitp<entrance)?MAKE_COLOR(conf.osd_color>>8,COLOR_RED):conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf,"%-11s%2d","Images(H)",(int)((480/fov)+0.5));                                             
    draw_big_string(pos_x,pos_y, osd_buf,conf.osd_color);
    pos_y+= BIG_FONT_HEIGHT;

    sprintf(osd_buf,"%-11s%2d","Images(V)",(int)((320/fov)+0.5));                                           
    draw_big_string(pos_x,pos_y, osd_buf,(exitp<entrance)?MAKE_COLOR(conf.osd_color>>8,COLOR_RED):conf.osd_color);

   }

}

// -----------------------------------------------------------------------------------------------
void gui_osd_draw_state() 
{
    int n=0, gui_mode=gui_get_mode();

    if ((conf.save_raw || gui_mode==GUI_MODE_OSD) && !conf.outline_mode )
    {
#if defined(DNG_SUPPORT)

      if(!gDngFinish)
        draw_string(conf.mode_state_pos.x-FONT_WIDTH, conf.mode_state_pos.y," DNG ",(conf.strip_offset && conf.raw_strip_mode && shooting_get_drive_mode())? MAKE_COLOR(conf.osd_color>>8,COLOR_RED):(!conf.strip_offset && conf.raw_strip_mode && shooting_get_drive_mode())?MAKE_COLOR(conf.osd_color>>8,COLOR_BLUE):conf.osd_color,1);
      else if((shooting_get_tick_count()-gDngFinish)<2000)                    // display image-saving time for two seconds
           {
            sprintf(osd_buf,"%4d",gDngFinish-gDngStart);
            draw_string(conf.mode_state_pos.x-FONT_WIDTH, conf.mode_state_pos.y,osd_buf,MAKE_COLOR(conf.osd_color>>8,COLOR_RED),1);
           }
      else if(gDngFinish) gDngFinish=0;
#endif
        n+=FONT_HEIGHT;
    }
    if (((gui_mode==GUI_MODE_NONE || gui_mode==GUI_MODE_ALT) &&!conf.outline_mode && conf.show_histo && conf.show_overexp && kbd_is_key_pressed(KEY_SHOOT_HALF)) || gui_mode==GUI_MODE_OSD) 
    {
        draw_string(conf.mode_state_pos.x, conf.mode_state_pos.y+n, (under_exposed || over_exposed)?"EXP":"   ", conf.osd_color,1);
    }

// *****************************************************************************************************************


    if ((conf.save_xml_file || gui_mode==GUI_MODE_OSD) && !conf.outline_mode)
      {
        draw_string(conf.mode_state_pos.x, conf.mode_state_pos.y+n, "XML", conf.osd_color,1);
        n+=FONT_HEIGHT;
      }

        if (conf.save_auto)
      {
        draw_string(conf.mode_state_pos.x, conf.mode_state_pos.y+n,"LOG", conf.osd_color,1);
        n+=FONT_HEIGHT;
      }
}


// -------------------------------------------------------------------

void gui_osd_draw_big_zoom()
{
    int zp=lens_get_zoom_point(),fl;

    switch (conf.zoom_value) 
    {
        case ZOOM_SHOW_FL:
            fl=get_focal_length(zp);
            sprintf(osd_buf, "%d.%d", fl/1000, fl%1000/100);
            break;
        case ZOOM_SHOW_EFL:
            fl=get_effective_focal_length(zp);
            sprintf(osd_buf, "%3d", fl/1000);
            break;
        case ZOOM_SHOW_X:
        default:
            fl=get_zoom_x(zp);
            sprintf(osd_buf, "%ld/%d.%dx", zp, fl/10, fl%10);
            break;
    }
   draw_string(conf.zoom_pos.x,conf.zoom_pos.y,osd_buf,(conf.camera_position == 1)?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):MAKE_COLOR(COLOR_RED,COLOR_WHITE),conf.zf_size);
}

// -------------------------------------------------------------------

void gui_osd_draw_values() {
    int zp=lens_get_zoom_point(), av=shooting_get_real_av(), fl, lfp;

    switch (conf.zoom_value) {
        case ZOOM_SHOW_FL:
            fl=get_focal_length(zp);
            sprintf(osd_buf, "Z:%d.%dmm%8s", fl/1000, fl%1000/100, "");
            break;
        case ZOOM_SHOW_EFL:
            fl=get_effective_focal_length(zp);
            sprintf(osd_buf, "Z:%3dmm%8s", fl/1000, "");
            break;
        case ZOOM_SHOW_X:
        default:
            fl=get_zoom_x(zp);
            sprintf(osd_buf, "Z:%ld/%d.%dx%8s", zp, fl/10, fl%10, "");
            break;
    }
    osd_buf[8]=0;
    draw_string(conf.values_pos.x, conf.values_pos.y, osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,COLOR_WHITE),1);

#if !defined(CAMERA_ixus40)
    lfp=lens_get_focus_pos_from_lense();
#else
    lfp = shooting_get_prop(PROPCASE_SUBJECT_DIST1);
#endif
    strcpy(osd_buf, "D:");
    sprintf_dist(osd_buf+2, lfp);
    sprintf(osd_buf+strlen(osd_buf), "%8s", "");
    osd_buf[8]=0;
    draw_string(conf.values_pos.x, conf.values_pos.y+FONT_HEIGHT, osd_buf,MAKE_COLOR(COLOR_TRANSPARENT,COLOR_WHITE),1);

    if(subject_far<64000)sprintf(osd_buf,"F:%5d",subject_far);
    else  sprintf(osd_buf,"  INF");
    draw_string(conf.values_pos.x, conf.values_pos.y+2*FONT_HEIGHT, osd_buf,(deviation<my_max_deviation)?MAKE_COLOR(COLOR_TRANSPARENT,COLOR_WHITE):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED),1);

    sprintf(osd_buf,"N:%5d",subject_near,"");
    draw_string(conf.values_pos.x, conf.values_pos.y+3*FONT_HEIGHT, osd_buf,(subject_near>conf.stereo_spacing*conf.minimum_dist_factor)?MAKE_COLOR(COLOR_TRANSPARENT,COLOR_WHITE):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED),1);


   color cl;
    cl = (1.342*realav > coc*1000)?MAKE_COLOR(COLOR_TRANSPARENT,COLOR_RED):MAKE_COLOR(COLOR_TRANSPARENT,COLOR_WHITE);
#if !defined(CAMERA_ixus40)
    sprintf(osd_buf, "Av:%d.%02d ", myav/100, myav%100);
#else
    sprintf(osd_buf, "Av:%d.%02d ",PROPCASE_ALLBEST_AV/100,PROPCASE_ALLBEST_AV %100);
#endif
    osd_buf[8]=0;
    draw_string(conf.values_pos.x, conf.values_pos.y+FONT_HEIGHT*4, osd_buf,cl,1);



}

// -------------------------------------------------------------------
void gui_osd_draw_clock() {
    unsigned long t;
    static struct tm *ttm;

    t = time(NULL);
    ttm = localtime(&t);
    sprintf(osd_buf, "%2u:%02u", ttm->tm_hour, ttm->tm_min);
    draw_string(conf.clock_pos.x, conf.clock_pos.y, osd_buf, conf.osd_color,1);
}

// -------------------------------------------------------------------
#ifdef OPT_SERIALIO
void gui_osd_draw_sio() 
 {
 		if ((conf.show_sio == 1)&&gSerialIORemoteShootingOn)
               {
               sprintf(osd_buf,"Errs: %3i",getSerialIOErrs());
               draw_string(conf.sio_pos.x, conf.sio_pos.y+FONT_HEIGHT, osd_buf, conf.osd_color,1);
               sprintf(osd_buf,"Rate: %3i",getSerialLowLevelErrRatePercent());
               draw_string(conf.sio_pos.x, conf.sio_pos.y+2*FONT_HEIGHT, osd_buf, conf.osd_color,1);
               }
}
#endif
// -------------------------------------------------------------------

void gui_bg(int posx,int posy,int value,color bg,color fg,int wide,int blocks)
{
   int x;

   if(value<0)value = 0;
   if(value>100) value = 100;

    for(x=0;x<blocks;x++)
    {
     osd_buf[x]=32;
    }

    osd_buf[blocks] = 0;

    value= (int)(value*blocks*0.08);
    
    for(x=0;x<value/8;x++)
    {
     osd_buf[x]=31;
    }

    if(!value%8) osd_buf[x] = 23+value%8;
    osd_buf[blocks] = 0;

    draw_string(posx,posy, osd_buf, MAKE_COLOR(bg,fg),1);
    if(wide) 
     {
      draw_string(posx,posy+FONT_HEIGHT, osd_buf, MAKE_COLOR(bg,fg),1);
      draw_rect(posx,posy,posx+blocks*8,posy+32,COLOR_BLACK);
     }

    else
    {
    draw_string(posx,posy, osd_buf, MAKE_COLOR(bg,fg),1);
    draw_rect(posx,posy,posx+blocks*8,posy+16,COLOR_BLACK);
    }
}

