/**
	
Motion detection module
 	
	
	Author: mx3 (Max Sagaydachny) . win1251 (        )
	Email: win.drivers@gmail.com
 	Skype: max_dtc
 	ICQ#: 125-985-663
 	Country: Ukraine
 	Sity: Kharkiv
 	
 	
 	20070912 mx3: first version
	
 	20070918 mx3: speed optimization,	
                /--/-COLUMNS, ROWS to split picture into
                 |  |
                 |  |  MEASURE MODE (Y,U,V R,G,B) - U=0, Y=1, V=2, R=3, G=4, B=5
                 |  |  |
                 |  |  |  TIMEOUT
                 |  |  |  |
                 |  |  |  |  COMPARISON INTERVAL (msec)
                 |  |  |  |  |
                 |  |  |  |  |  THRESHOLD ( difference in cell to trigger detection)
                 |  |  |  |  |  |
                 |  |  |  |  |  |  DRAW GRID (0-no, 1-yes)
                 |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  RETURN VARIABLE number of cells with motion detected
                 |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |           OPTIONAL PARAMETERS:
                 |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  REGION (masking) mode: 0-no regions, 1-include,
                 |  |  |  |  |  |  |  |  |      2-exclude
                 |  |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  |  REGION FIRST COLUMN
                 |  |  |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  |  |  REGION FIRST ROW
                 |  |  |  |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  |  |  |  REGION LAST COLUM
                 |  |  |  |  |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  |  |  |  |  REGION LAST ROW
                 |  |  |  |  |  |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  |  |  |  |  |  PARAMETERS: 1-make immediate shoot,
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |   2-log debug information into file (* see note below!),
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |   4-dump liveview image from RAM to a file,
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |   8-on immediate shoot, don't release shutter.
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |   16- shoot when still
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |   32- display sample points (even if draw_grid = 0)
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |   OR-ed values are accepted, e.g. use 9 for
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |   immediate shoot & don't release shutter
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  PIXELS STEP - Speed vs. Accuracy
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  |    adjustments (1-use every pixel,
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  |    2-use every second pixel, etc)
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  MILLISECONDS DELAY to begin
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |   triggering - can be useful
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |   for calibration with DRAW-
                 |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |   GRID option.
md_detect_motion a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p


nobody tried to use MD to get overall luminosity to automatically
adjust shutter speed override?

MD setup:
set delay interval to 2-3 secs
timeout=delay_interval+1
threshold=255 (so it will not trigger ) 
cols=1
rows=1

md_get_cell_diff 1, 1, overall_luminocity

shutter_override_time = some_formulae ( overall_luminocity )

I don't have camera nearby to test it.

I have thought about timelapse movie script which would automatically
override shutter speed at night. I'm planning to make 2 days timelapse
movie ( it seems 8gb SD card and power adapter will help also :-) )


NOTE: when MD stops working on timeout, cells containg absolute values
instead of difference.

*/

void dump_memory();

#define MD_REC_CALLS_CNT 2048
#define MD_INFO_BUF_SIZE 4096

#include "motion_detector.h"

#include "gui.h"
#include "draw_palette.h"
#include "kbd.h"
#include "keyboard.h"
extern int get_luminance;
extern char* imgbuf;
int gDone_once =0;

#define MD_XY2IDX(x,y) ((y)*motion_detector.columns+x)

void kbd_sched_shoot();
void md_kbd_sched_immediate_shoot(int no_release);

enum {

	MD_MEASURE_MODE_U=0,
	MD_MEASURE_MODE_Y=1,
	MD_MEASURE_MODE_V=2,
	MD_MEASURE_MODE_R=3,
	MD_MEASURE_MODE_G=4,
	MD_MEASURE_MODE_B=5
};

enum {
	MD_DO_IMMEDIATE_SHOOT=1,
	MD_MAKE_DEBUG_LOG_FILE=2,
	MD_MAKE_RAM_DUMP_FILE=4,
      MD_NO_SHUTTER_RELEASE_ON_SHOOT=8,
      MD_SHOOT_WHEN_STILL=16,
      MD_SHOW_SAMPLE_POINTS=32
};

enum {
	MD_REGION_NONE=0,
	MD_REGION_INCLUDE=1,
	MD_REGION_EXCLUDE=2
};


static struct {
	int *curr; //points to buff1 or buff2
	int *prev; //points to buff2 or buff1
	int buff1[MOTION_DETECTOR_CELLS];
	int buff2[MOTION_DETECTOR_CELLS];
	int points[MOTION_DETECTOR_CELLS];

	int columns;
	int rows;
	int threshold;
	int pixel_measure_mode;
	int timeout;
	int measure_interval;

	int last_measure_time;
	int start_time;

	int running;
	int detected_cells;

	int draw_grid;
	int clipping_region_mode;
	int clipping_region_row1;
	int clipping_region_column1;
	int clipping_region_row2;
	int clipping_region_column2;

	int previous_picture_is_ready;

	int ret_var_num;
	int parameters;
	int pixels_step;
	int msecs_before_trigger;

//debug
	int comp_calls_cnt;
	int comp_calls[MD_REC_CALLS_CNT];

} motion_detector;


//motion_detector.curr=NULL;

static int overlay=0;                        // luminance metering pattern on/off control

static int clip(int v) {
    if (v<0) v=0;
    if (v>255) v=255;
    return v;
}


int md_init_motion_detector(
 int columns,
 int rows,
 int pixel_measure_mode,
 int detection_timeout,
 int measure_interval,
 int threshold,
 int draw_grid,
 int ret_var_num,
 int clipping_region_mode,
 int clipping_region_column1,
 int clipping_region_row1,
 int clipping_region_column2,
 int clipping_region_row2,
 int parameters,
 int pixels_step,
 int msecs_before_trigger
){

	motion_detector.comp_calls_cnt=0;
	motion_detector.previous_picture_is_ready=0;
	motion_detector.curr=motion_detector.buff1;
	motion_detector.prev=motion_detector.buff2;

	if(		pixel_measure_mode != MD_MEASURE_MODE_Y 
		&&	pixel_measure_mode != MD_MEASURE_MODE_U
		&&	pixel_measure_mode != MD_MEASURE_MODE_V
		&&	pixel_measure_mode != MD_MEASURE_MODE_R
		&&	pixel_measure_mode != MD_MEASURE_MODE_G
		&&	pixel_measure_mode != MD_MEASURE_MODE_B
		){
		pixel_measure_mode = MD_MEASURE_MODE_Y;
	}


	if( columns<1 || rows<1 || columns * rows > MOTION_DETECTOR_CELLS ){
		columns=3;
		rows=3;
	}

	if(msecs_before_trigger<0){
		msecs_before_trigger=0;
	}

	if (pixels_step<1){
		pixels_step=1;
	}

	if(detection_timeout<0){
		detection_timeout=0;
	}

	if(measure_interval<0) {
		measure_interval=0;
	}

	if(threshold<0) 
      {
       if((motion_detector.parameters & MD_SHOOT_WHEN_STILL)!=0)threshold=1;
       else threshold=0;
	}


	motion_detector.msecs_before_trigger=msecs_before_trigger;
	motion_detector.parameters = parameters;
	motion_detector.pixels_step=pixels_step;
	motion_detector.columns=columns;
	motion_detector.rows=rows;
	motion_detector.ret_var_num=ret_var_num;
	

	motion_detector.pixel_measure_mode=pixel_measure_mode;
	motion_detector.timeout=detection_timeout;
	motion_detector.measure_interval=measure_interval;
	motion_detector.threshold=threshold;
	motion_detector.draw_grid=draw_grid;


	if (clipping_region_column1>clipping_region_column2){
		motion_detector.clipping_region_column2=clipping_region_column1;
		motion_detector.clipping_region_column1=clipping_region_column2;
	} else {
		motion_detector.clipping_region_column2=clipping_region_column2;
		motion_detector.clipping_region_column1=clipping_region_column1;
	}

	if (clipping_region_row1>clipping_region_row2){
		motion_detector.clipping_region_row2=clipping_region_row1;
		motion_detector.clipping_region_row1=clipping_region_row2;
	} else {
		motion_detector.clipping_region_row2=clipping_region_row2;
		motion_detector.clipping_region_row1=clipping_region_row1;
	}

	if (clipping_region_mode!=MD_REGION_NONE && clipping_region_mode!=MD_REGION_INCLUDE && clipping_region_mode!=MD_REGION_EXCLUDE){
		clipping_region_mode=MD_REGION_NONE;
	}
	motion_detector.clipping_region_mode=clipping_region_mode;

	motion_detector.detected_cells=0;
	motion_detector.previous_picture_is_ready=0;
      motion_detector.start_time=get_tick_count();

	motion_detector.last_measure_time = motion_detector.start_time - motion_detector.measure_interval;

	motion_detector.running=1;

      if((motion_detector.parameters & MD_SHOOT_WHEN_STILL)!=0)
       {
        kbd_sched_release(KEY_SHOOT_HALF);
	  kbd_sched_motion_detector();
        kbd_sched_press(KEY_SHOOT_HALF);
       }
      else kbd_sched_motion_detector();
	draw_clear();
      overlay=1;
	return 1;
}


void md_save_calls_history(){
	char buf[200], fn[30];
	char big[MD_INFO_BUF_SIZE];
	int big_ln;
	int calls,i, ln, fd;
  static struct utimbuf t;
    unsigned long t2;
    static struct tm *ttm;


	if( (motion_detector.parameters & MD_MAKE_DEBUG_LOG_FILE) == 0 ){
		return;
	}
	

	strcpy(fn,"A/MD_INFO.TXT");//,BUILD_NUMBER,motion_detector.pixels_step);
	fd = open(fn, O_WRONLY|O_CREAT, 0777);
	if( fd>=0) {
		script_console_add_line("Writing info file...");
		lseek(fd,0,SEEK_END);
    t2 = time(NULL);
    ttm = localtime(&t2);
    big_ln=sprintf(big, 
				"\r\n--- %04u-%02u-%02u  %02u:%02u:%02u\r\n"
				"CHDK Ver: %s [ #%s ]\r\nBuild Date: %s %s\r\nCamera:  %s [ %s ]\r\n"
				"[%dx%d], threshold: %d, interval: %d, pixels step: %d\r\n"
				"region: [%d,%d-%d,%d], region type: %d\r\n"
				"wait interval: %d, parameters: %d, calls: %d, detected cells: %d\r\n", 
				1900+ttm->tm_year, ttm->tm_mon+1, ttm->tm_mday, ttm->tm_hour, ttm->tm_min, ttm->tm_sec,
				SDM_VERSION, BUILD_NUMBER, __DATE__, __TIME__, PLATFORM, PLATFORMSUB,
				motion_detector.columns, motion_detector.rows, motion_detector.threshold, motion_detector.measure_interval, motion_detector.pixels_step,
				motion_detector.clipping_region_column1, motion_detector.clipping_region_row1, motion_detector.clipping_region_column2, motion_detector.clipping_region_row2, motion_detector.clipping_region_mode,
				motion_detector.msecs_before_trigger, motion_detector.parameters, motion_detector.comp_calls_cnt,
				motion_detector.detected_cells
		);

		calls = ( motion_detector.comp_calls_cnt < MD_REC_CALLS_CNT) ?motion_detector.comp_calls_cnt: MD_REC_CALLS_CNT;

		for(i=0;i<calls;i++){
			ln=sprintf(buf,"[%d] - %d\r\n",i,motion_detector.comp_calls[i]);
			if(big_ln+ln>MD_INFO_BUF_SIZE){
	      write(fd,big,big_ln);
				big_ln=0;
			} 
			memcpy(big+big_ln,buf,ln+1);
			big_ln+=ln;
		}
    write(fd,big,big_ln);
		close(fd);
	  t.actime = t.modtime = time(NULL);
    utime(fn, &t);
	}
}


static void mx_dump_memory(void *img){
	char fn[36];
	int fd, i;
	static int cnt=0;

    started();
    mkdir("A/MD");

		do {
			cnt++;
			sprintf(fn, "A/MD/%04d.FB", cnt);
			fd = open(fn, O_RDONLY, 0777);

			if(fd>=0){
				close(fd);
			}
		} while(fd>=0);


		sprintf(fn, "A/MD/%04d.FB", cnt );
		fd = open(fn, O_WRONLY|O_CREAT, 0777);
		if (fd) {
#if !defined(CAMERA_tx1)
	    write(fd, img,vid_get_viewport_width()*screen_height*3);
#else
	    write(fd, img, (screen_width-120)*screen_height*3);
#endif
	    close(fd);
		}
  vid_bitmap_refresh();
  finished();

}




int md_detect_motion(void)
{

#define DISPLAYED_POINTS 1000                 // 925 required for max sampling of every 10 pixels
	int *tmp;
	unsigned char * img;
	int viewport_size;
	int pix_N, idx,tmp1, tmp2, tick, vp_w_mul_y, in_clipping_region;
      int vp_w=0;
      int vp_h=0;
      int do_calc=0;
      int x_step=0;  // cell width
      int y_step=0;  // cell height
	int val;
	register int i, col, row, x, y;
	static char buf[128];
	double vp_hr, vp_wr;
      static int cell_prev=0;
      static int cell_prev2=0;
      static int cell_curr;
      static int cnt = 0;  
      static short spotmeter[3]={0};                 // sample-points for spotmeter array
      static unsigned short int *points = NULL;      // sample points for motion detection display
	if(!motion_detector.running) return 0;
#if defined(CAMERA_sx200is)||defined(CAMERA_g11)|| defined(CAMERA_s90) || defined(CAMERA_ixus95)|| defined(CAMERA_ixus105)
     int screen_width=320;
#elif defined (CAMERA_ixus100)
    int screen_width=360;
#endif
	tick=shooting_get_tick_count();

    if(points==NULL) 
   {
    points= (unsigned short int*)malloc(sizeof(unsigned short int)*DISPLAYED_POINTS*2);     // allocate memory for array
   }


/*
   spotmeter command sends  md_init_motion_detector(15,15,1,2000,3000,255,1,0,1,8,8,8,8,0,1,10);
   15 rows, 15 columns,luminance, two second timeout, three second compare, 255 threshold,draw grid
   return variable 'a', 10 msec delay before triggering
*/

 
img = vid_get_viewport_live_fb();
if(img==NULL){img = vid_get_viewport_fb();}
if(get_luminance)
 {
  int cy;
  vp_w=vid_get_viewport_width();
#if defined(CAMERA_ixus100)|| defined(CAMERA_ixus980)
vp_w=360;
#endif
  if((motion_detector.start_time + motion_detector.timeout <= tick))
     {
      for(cy=0;cy<3;cy++){spotmeter[cy]=0;}
           if(1)                                                                 // buffer allocated
                 {
			for(x=(vp_w>>1)-12;x<(vp_w>>1)+12;x+=2)                          // step horizontally every two pixels
                      {
				for(y=112;y<128;y+=2)                                      // step vertically every two pixels 
                            {					
				      spotmeter[0]+=img[ (y*vp_w+x)*3 + 1];                // pixel luminance
					if((x%2)==0)
                              {
					 spotmeter[1]+=(signed char)img[(y*vp_w+x)*3];       // shared blue-luminance value
					 spotmeter[2]+=(signed char)img[(y*vp_w+x)*3 + 2];   // shared red-luminance  value
					} 
                              else 
                              {
					 spotmeter[1]+=(signed char)img[(y*vp_w+x-1)*3];
					 spotmeter[2]+=(signed char)img[(y*vp_w+x-1)*3 + 2];
					}
				    } 
			    } // _next x

                        ubasic_set_variable(0,spotmeter[0]/96);
                        ubasic_set_variable(1,spotmeter[1]/96);
                        ubasic_set_variable(2,spotmeter[2]/96);
                        get_luminance=0;
                        motion_detector.curr[0]=0;
                        motion_detector.running = 0;
		            return 0;
                } // buffer not allocated
      } // still waiting for time-out to end
      return 1;
  } // not get_luminance

else
  {


// if comparison calls < 2048 save tick time in array
	if(motion_detector.comp_calls_cnt < MD_REC_CALLS_CNT) 
      {
		motion_detector.comp_calls[motion_detector.comp_calls_cnt]=tick;
	}
	motion_detector.comp_calls_cnt++;

// --------         operation has timed-out  ---------------------------------------
	if(motion_detector.start_time + motion_detector.timeout < tick ) 
      {
		md_save_calls_history();
		motion_detector.running = 0;
            if(points)                                                               // free points memory
             {
              free(points);
              points=NULL;
             }
            if((motion_detector.parameters & MD_SHOOT_WHEN_STILL)!=0)
             {
              cell_curr=cell_prev=0;
              ubasic_set_variable(motion_detector.ret_var_num,0);                     // return variable to indicate time-out
              overlay=0;                                                              // turn-off luminance overlay pattern
              md_draw_grid();
             }
            gDone_once = 0;
		return 0;
	}


    if(!gDone_once && ((motion_detector.parameters & MD_SHOW_SAMPLE_POINTS)!=0))            //first time through and show sample-points option set
   {
     	vp_h=vid_get_viewport_height();
#if defined(CAMERA_TX1)
     vp_w=screen_width-120;
#elif defined(CAMERA_sx200is)|| defined(CAMERA_g11)|| defined(CAMERA_s90)|| defined(CAMERA_ixus95)
     vp_w=320;                                                                             // because we are calculating display points for 320 screen
#else
     vp_w=360;
#endif
	x_step=vp_w/motion_detector.columns;
	y_step=vp_h/motion_detector.rows;                                             
      x = y = cnt = 0;

  if(points)                                                                                // if memory was allocated for points
    {
	for(row=0, col=0; row < motion_detector.rows ; )                                   // for each column in row and then next row
      {
            do_calc=0;
		in_clipping_region=0;
          // ----------- set status of in_clipping_region ----------
		if (
		      col+1 >= motion_detector.clipping_region_column1 
			&& col+1 <= motion_detector.clipping_region_column2
			&& row+1 >= motion_detector.clipping_region_row1
			&& row+1 <= motion_detector.clipping_region_row2
		   )
              
		    in_clipping_region=1;
		  
          // Check if outside excluded region, in clipping region or no clipping/excluded regions

		if(motion_detector.clipping_region_mode==MD_REGION_EXCLUDE && !in_clipping_region) do_calc=1;
		if(motion_detector.clipping_region_mode==MD_REGION_INCLUDE && in_clipping_region)  do_calc=1;
		if(motion_detector.clipping_region_mode==MD_REGION_NONE)                           do_calc=1;

         // ------- cell is in a valid region so count sample points
		if(do_calc==1)
                {  
                 for(x=col*x_step;x<((col+1)*x_step);x+=motion_detector.pixels_step)        // index of cell from row and column value
                      {
				for(y=row*y_step;y<((row+1)*y_step);y+=motion_detector.pixels_step) // step vertically in cell by defined number of pixels 
                            {
                             points[cnt]=(unsigned short int)x;
                             points[cnt+1]=(unsigned short int)y;
                             cnt+=2;                           
				    }                                                               // next pixel row in cell
			    }                                                                     // next pixel column in cell
		     }               
		col++;                                                                          // do next cell in the row
		if(col>=motion_detector.columns)
                 {
			col=0;
			row++;
		     }
	  } // next row
         gDone_once = 1;
      }  // _memory was allocated for points array
   }     // first time through and show points option

      if(points && gDone_once && ((motion_detector.parameters & MD_SHOW_SAMPLE_POINTS)!=0))  // show sample-points if were stored
      {
       int xx;
       for(xx=0;xx<cnt;xx+=2)
        {
         draw_filled_rect((coord)points[xx],(coord)(points[xx+1]),(coord)(points[xx]+3),(coord)(points[xx+1]+3),MAKE_COLOR(COLOR_RED,COLOR_RED));
        }
      }


// -------       not time for next measurement yet  ---------------------------------
 
	if(motion_detector.last_measure_time + motion_detector.measure_interval > tick)
      {
		//wait for the next time
		return 1;
	}
// ----------------------------------------------------------------------------------

	motion_detector.last_measure_time=tick;

//--------------swap pointers so we don't need to copy last data array into Previous one
	tmp=motion_detector.curr;
	motion_detector.curr=motion_detector.prev;
	motion_detector.prev=tmp;


//--------------WARNING. maybe not optimized -------------------------------------------
	for(i=0 ; i<motion_detector.rows*motion_detector.columns ; i++ )
      {
		motion_detector.points[i]=0;
		motion_detector.curr[i]=0;
	}

// -------- based on viewport width, calculate cell width and height in pixels----------
	vp_h=vid_get_viewport_height();

#if defined(CAMERA_TX1)
     vp_w=screen_width-120;
#else
     vp_w=360;                                                                                                  // width of live buffer, not screen
#endif
	x_step=vp_w/motion_detector.columns;
	y_step=vp_h/motion_detector.rows;


// ----------- for every row of cells  ------------------------------------

	for(row=0, col=0; row < motion_detector.rows ; )
      {
		do_calc=0;
		in_clipping_region=0;
          // ----------- set status of in_clipping_region ----------
		if (
		      col+1 >= motion_detector.clipping_region_column1 
			&& col+1 <= motion_detector.clipping_region_column2
			&& row+1 >= motion_detector.clipping_region_row1
			&& row+1 <= motion_detector.clipping_region_row2
		   )
               {
		    in_clipping_region=1;
		   }
          // -------------------------------------------------------
          // Check if outside excluded region, in clipping region or no clipping/excluded regions

		if(motion_detector.clipping_region_mode==MD_REGION_EXCLUDE && in_clipping_region==0)
            {
			do_calc=1;
		} 

		if(motion_detector.clipping_region_mode==MD_REGION_INCLUDE && in_clipping_region==1)
            {
			do_calc=1;
		}

		if(motion_detector.clipping_region_mode==MD_REGION_NONE)
            {
			do_calc=1;
		}
         // ------- cell is in a valid region so step pixels horizontally and vertically and do calculation ----
		if(do_calc==1)
           {
		  idx=MD_XY2IDX(col,row);                                                     // index of cell from row and column value
                 for(x=col*x_step;x<((col+1)*x_step);x+=motion_detector.pixels_step)        // index of cell from row and column value
                     {
				for(y=row*y_step;y<(row+1)*y_step;y+=motion_detector.pixels_step) // step vertically in cell by defined number of pixels 
                            {

                            // -- set 'val' depending on sampling mode (luminance or chroma)
                            //ARRAY of UYVYYY values 6 bytes - 4 pixels

					int cy;
                              signed char cv,cu;
					cy=img[ (y*vp_w+x)*3 + 1];              // pixel luminance
					if((x%2)==0)                            // even pixels
                              {
						cu=(signed char)img[ (y*vp_w+x)*3];            // shared blue-luminance value
						cv=(signed char)img[ (y*vp_w+x)*3 + 2];        // shared red-luminance  value
					} 
                              else                                    // odd pixels
                              {
						cu=(signed char)img[ (y*vp_w+x-1)*3];
						cv=(signed char)img[ (y*vp_w+x-1)*3 + 2];
					}
					switch(motion_detector.pixel_measure_mode)
                              {
					  case	MD_MEASURE_MODE_Y:
							val=cy;
							break;

					  case	MD_MEASURE_MODE_U:
							val=cu;
                                          break;

					  case	MD_MEASURE_MODE_V:
							val = cv;
							break;

					  case	MD_MEASURE_MODE_R:
                                          val = clip(((cy<<12)+ cv*5743 + 2048)/4096); //R
							break;

					  case	MD_MEASURE_MODE_G:
                                          val = clip(((cy<<12) - cu*1411 - cv*2925 + 2048)/4096); //G
							break;

					  case	MD_MEASURE_MODE_B:
                                          val = clip(((cy<<12) + cu*7258 + 2048)/4096); //B
							break;

						      default:
					            val=cy;
							break;
					}
                             // ------------    store 'val' in 'curr' array --------------
					motion_detector.curr[ idx ]+=val;
					motion_detector.points[ idx ]++;
				   } // _next pixel row in cell
			    }      // _next pixel column in cell
		 }               // _do calculation

            // ------- do next cell in the row
		col++;
		if(col>=motion_detector.columns)
            {
			col=0;
			row++;
		}
	  } //  -----------  do next row of cells --------------------------------------------

	//<< fill "curr" array

      // -----------------------------------------------------------------------------------
	if(motion_detector.previous_picture_is_ready==0)
      {
		motion_detector.previous_picture_is_ready=1;
		motion_detector.start_time=get_tick_count();
		motion_detector.last_measure_time=motion_detector.start_time-motion_detector.measure_interval;
		return 1;
	}
// ----------------------------------------  Shoot when still ------------------------------

          if((motion_detector.parameters & MD_SHOOT_WHEN_STILL)!=0)
           {                
             cell_curr=motion_detector.curr[0];
             tmp1=(cell_prev2<cell_prev)?cell_prev-cell_prev2:cell_prev2-cell_prev;
             tmp2=(cell_curr<cell_prev)?cell_prev-cell_curr:cell_curr-cell_prev;
             cell_prev2=cell_prev;
             cell_prev=cell_curr;
             if((motion_detector.start_time+motion_detector.msecs_before_trigger < tick))               // after initial delay ?
             {
             if((tmp2<(motion_detector.curr[0]/motion_detector.threshold))&&(tmp1<(motion_detector.curr[0]/motion_detector.threshold)))                               // yes, small enough movement ?
              {
               motion_detector.running=0;
               cell_prev2=cell_prev=cell_curr=0;
               ubasic_set_variable(motion_detector.ret_var_num,1);                                      // return variable to indicate success
               md_kbd_sched_immediate_shoot(motion_detector.parameters&MD_NO_SHUTTER_RELEASE_ON_SHOOT); // yes, so shoot
               overlay=0; 
               md_draw_grid();                                                                          // turn-off luminance overlay pattern
               gDone_once = 0;
             if(points)                                                                               // free points memory
             {
              free(points);
              points=NULL;
             }
               return 0;
              } 
             }
              return 1;                                                                                 // too soon yet
           }

     //- compare arrays here and if absolute average change greater than threshold and after specified delay, record as detected

		for ( col=0, row=0; row < motion_detector.rows; )
             {
		  idx=MD_XY2IDX(col,row);                    // index of cell from row and column value
			tmp2=0;
			if(motion_detector.points[idx]>0)      // if an included-region cell, set to average pixel change compared to previous
                  {
				motion_detector.prev[idx] = (motion_detector.curr[idx]-motion_detector.prev[idx])/motion_detector.points[idx]; // average difference per pixel
				tmp2 = ( motion_detector.prev[idx] < 0 ) ? -motion_detector.prev[idx] : motion_detector.prev[idx] ;            // as absolute value
			}
	
			if( tmp2 > motion_detector.threshold )                                          // if greater than threshold value
                  {
			  if (motion_detector.start_time+motion_detector.msecs_before_trigger < tick)   // and after specified waiting time
                       {
				motion_detector.detected_cells++;                                         // record as detected
			     }
			}

			col++;                                                                          // next cell in row
			if(col>=motion_detector.columns)
                  {
			 col=0;
			 row++;
			}
		 }
     //--------- if movement detected 
	      if( motion_detector.detected_cells > 0 )
             {
		  if (motion_detector.start_time+motion_detector.msecs_before_trigger < tick)
               {
		    motion_detector.running=0;
	          ubasic_set_variable(motion_detector.ret_var_num, motion_detector.detected_cells);
		     if(( motion_detector.parameters&MD_DO_IMMEDIATE_SHOOT ) !=0)
                  {
		       //make shoot
                   md_kbd_sched_immediate_shoot(motion_detector.parameters&MD_NO_SHUTTER_RELEASE_ON_SHOOT);
			}
                  gDone_once = 0;
                  if(points)                                                               // free points memory
                 {
                  free(points);
                  points=NULL;
                 }
			return 0;
		   }
	       }
	       return 1;
         }  // _else
}

// ---- for a valid cell row and column return average pixel delta ------------------------------------------

int md_get_cell_diff(int column, int row)
{

	if (column<1 || column > motion_detector.columns)
      {
		return 0;
	}

	if (row<1 || row > motion_detector.rows )
      {
		return 0;
	}
	
	return motion_detector.prev[ MD_XY2IDX(column-1,row-1) ];
}

// -----------------------------------------------------------------------------------------------------------
void md_init()
{
	motion_detector.curr=motion_detector.buff1;
	motion_detector.prev=motion_detector.buff2;
	motion_detector.running=0;
	motion_detector.ret_var_num=-1;
}

// -----------------------------------------------------------------------------------------------------------

int md_running()
{
	return motion_detector.running==1?1:0;//detected_cells>0?1:0;
}

// -----------------------------------------------------------------------------------------------------------

void md_draw_grid()
{
	int x_step, y_step, col, row;
	int do_draw_rect, i, tmp2, in_clipping_region, col_start, col_stop, row_start, row_stop;
      int x,y,sw;
      sw=360;                                                                              // actually the BMP buffer area is 720x240
    #if defined(CAMERA_sx200is)|| defined(CAMERA_g11)|| defined(CAMERA_s90)|| defined(CAMERA_ixus95)
      sw=320;                                                                             // 360 'x' mapped to 720 'x' 
   #endif
	if(motion_detector.running==0 || motion_detector.draw_grid==0)
      {
	 return ;
	}

      if((motion_detector.parameters & MD_SHOOT_WHEN_STILL)!=0)
      { 
       for(y=1;y<screen_height/30;y++)
       {
        for(x=1;x<(sw/30);x++)
        {
         draw_rect(x*30,y*30,x*30+4,y*30+4,(overlay)?COLOR_RED:COLOR_TRANSPARENT);
        }
       }
       return;
      }

	x_step=sw/motion_detector.columns;
	y_step=screen_height/motion_detector.rows;
     if((((motion_detector.parameters & MD_SHOW_SAMPLE_POINTS)!=0)&&gDone_once)||((motion_detector.parameters & MD_SHOW_SAMPLE_POINTS)==0))
	for ( col=0, row=0; row < motion_detector.rows; )
          {
		i=MD_XY2IDX(col,row);
		tmp2 = ( motion_detector.prev[i] < 0 ) ? -motion_detector.prev[i] : motion_detector.prev[i] ;

			in_clipping_region=0;
			if ( col+1>=motion_detector.clipping_region_column1 
				&& col+1<=motion_detector.clipping_region_column2
				&& row+1>=motion_detector.clipping_region_row1
				&& row+1<=motion_detector.clipping_region_row2
				)
                     {
			   in_clipping_region=1;
			 }

			do_draw_rect=0;

			if(motion_detector.clipping_region_mode==MD_REGION_EXCLUDE && in_clipping_region==0) do_draw_rect=1;
			if(motion_detector.clipping_region_mode==MD_REGION_INCLUDE && in_clipping_region==1) do_draw_rect=1;
			if(motion_detector.clipping_region_mode==MD_REGION_NONE)                             do_draw_rect=1;
			if(do_draw_rect==1)
                   {
                    draw_rect(x_step*col+1,y_step*row+2, x_step*(col+1)-3, y_step*(row+1)-2,(tmp2 > motion_detector.threshold)?COLOR_RED:COLOR_GREEN);
                   }
			col++;
			if( col >= motion_detector.columns )
                  {
				row++;
				col=0;
			}
	      }

}

// ----------------------------------------------------------------------------------------------------------
