#include "stdlib.h"
#include "keyboard.h"
#include "platform.h"
#include "core.h"
#include "gui.h"
#include "draw_palette.h"
#include "conf.h"
#include "script.h"

//-------------------------------------------------------------------

#define SCRIPT_BUF_SIZE             8192
#define SCRIPT_CONSOLE_NUM_LINES    3
#define SCRIPT_CONSOLE_LINE_LENGTH  25
#define SCRIPT_CONSOLE_X            0
#define SCRIPT_CONSOLE_Y            (12-SCRIPT_CONSOLE_NUM_LINES)
//-------------------------------------------------------------------
const char *state_ubasic_script=NULL; //ERR99
char cfg_name[100] = "\0";
char cfg_set_name[100] = "\0";
static char ubasic_script_buf[SCRIPT_BUF_SIZE];
static const char *ubasic_script_default =
"@title Inbuilt script\n"
"print \"Hello !\"\n"
"sleep_for_seconds 5\n"
"end\n";
char paramdesc[4][30]={{'\0'},{'\0'}};
char script_title[36];
char script_params[SCRIPT_NUM_PARAMS][28];
int script_param_order[SCRIPT_NUM_PARAMS];
static char script_params_update[SCRIPT_NUM_PARAMS];
static int script_loaded_params[SCRIPT_NUM_PARAMS];
char script_console_buf[SCRIPT_CONSOLE_NUM_LINES][SCRIPT_CONSOLE_LINE_LENGTH+1];
static int script_console_lines=0;
extern int gConsoleLine;

//-------------------------------------------------------------------
static void process_title(const char *title) 
{
    register const char *ptr = title;
    register int i=0;

    while (ptr[0]==' ' || ptr[0]=='\t') ++ptr; // whitespaces
    while (i<(sizeof(script_title)-1) && ptr[i] && ptr[i]!='\r' && ptr[i]!='\n') 
    {
      script_title[i]=ptr[i];
      ++i;
    }
    script_title[i]=0;
}

static void process_description(const char *desc, int line) 
{
    register const char *ptr = desc;
    register int i=0;

    while (ptr[0]==' ' || ptr[0]=='\t') ++ptr;                                      // skip whitespaces
    while (i<(sizeof(paramdesc[0])-1) && ptr[i] && ptr[i]!='\r' && ptr[i]!='\n')   // copy 30 characters max.
      {paramdesc[line][i]=ptr[i];++i;}
       paramdesc[line][i]=0;                                                       // string terminator
}
//-------------------------------------------------------------------
static int process_param(const char *param, int update) 
{
    register const char *ptr = param;
    register int n, i=0;

    while (ptr[0]==' ' || ptr[0]=='\t') ++ptr; //whitespaces
    if (ptr[0] && (ptr[0]>='a' && ptr[0]<='a'+SCRIPT_NUM_PARAMS) && (ptr[1]==' ' || ptr[1]=='\t')) 
     {
        n=ptr[0]-'a';
        ptr+=2;
        while (ptr[0]==' ' || ptr[0]=='\t') ++ptr; //whitespaces
		script_params_update[n] = 1;
        while (i<(sizeof(script_params[0])-1) && ptr[i] && ptr[i]!='\r' && ptr[i]!='\n') 
         {
		if (update) 
		{ 
             if (script_params[n][i]!=ptr[i]) { script_params_update[n] = 0; break; }
		} 
            else script_params[n][i]=ptr[i];
            ++i;
         }
		if (!update) script_params[n][i]=0;
        n++;
      } 
     else n=0;                                                       //??? else produce error message    
    return n;                                                       //n=1 if '@param a' was processed, n=2 for 'b' ... n=26 for 'z'. n=0 if failed.
}

//-------------------------------------------------------------------
static void process_default(const char *param, char update) 
{
    register const char *ptr = param;
    register int n;

    while (ptr[0]==' ' || ptr[0]=='\t') ++ptr; //whitespaces
    if (ptr[0] && (ptr[0]>='a' && ptr[0]<='a'+SCRIPT_NUM_PARAMS) && (ptr[1]==' ' || ptr[1]=='\t')) 
    {
        n=ptr[0]-'a';
        ptr+=2;
		if (!update || script_params_update[n])
        {
			conf.ubasic_vars[n] = strtol(ptr, NULL, 0);
            script_loaded_params[n] = conf.ubasic_vars[n];
        }
    } //??? else produce error message
}

//-------------------------------------------------------------------
static void script_scan(const char *fn, int update_vars) 
{
    register const char *ptr = state_ubasic_script;                    // loaded-script start
    register int i, j=0, n;
    char *c;

    c=strrchr(fn, '/');                                               //  find last occurrence of '/', next location is start of file base-name
    strncpy(script_title, (c)?c+1:fn, sizeof(script_title));
    script_title[sizeof(script_title)-1]=0;
    for (i=0; i<SCRIPT_NUM_PARAMS; ++i) 
    {
      script_params[i][0]=0;
      script_param_order[i]=0;
    }

    while (ptr[0]) 
      {
        while (ptr[0]==' ' || ptr[0]=='\t') ++ptr; //whitespaces
        if (ptr[0]=='@') 
        {
            if (strncmp("@title", ptr, 6)==0) 
            {
                ptr+=6;
                process_title(ptr);
            } 
           else if (strncmp("@param", ptr, 6)==0) 
            {
                ptr+=6;
                n=process_param(ptr, 0); //n=1 if '@param a' was processed, n=2 for 'b' ... n=26 for 'z'. n=0 if failed.
                if (n>0 && n<=SCRIPT_NUM_PARAMS) 
                {
                  script_param_order[j]=n;
                  j++;
                }
            } 
           else if (update_vars && strncmp("@default", ptr,8)==0) 
            {
                ptr+=8;
                process_default(ptr, 0);
            }

        }
        while (ptr[0] && ptr[0]!='\n') ++ptr; //unless end of line
        if (ptr[0]) ++ptr;
    }
}

//-------------------------------------------------------------------
void set_params_values_name(const char *fn, int param_set)
{
	int shift;
	register char *ptr = (param_set >= 0 ? cfg_set_name : cfg_name);
	const char *name;
	
	if (fn == NULL || fn[0] == 0) { ptr[0] = 0; return; }
	strncpy(ptr, SCRIPT_DATA_PATH, 100); ptr[99]=0;
	shift = strlen(SCRIPT_DATA_PATH);
	name = strrchr(fn, '/');                                        // pointer to last occurrence of '/'
	if (name) name++; 
      else name=fn;                                                  // if found, move to next character else use file-name
	strncpy(ptr+shift, name, 100-shift); ptr[99]=0;
	shift = strlen(ptr); if (shift >= 100) shift=99;

	if (param_set >= 0) sprintf(ptr+shift-4, "%d.txt\0", param_set); 
      else  strcpy(ptr+shift-3, "cfg\0");
}

//-------------------------------------------------------------------
int load_params_values(const char *fn, int update_vars, int read_param_set)
{
	int i, fd=-1, rcnt;
	register const char *ptr;	
	struct stat st;
	char *buf;
	static int descnt;                      // count for occurences of @desc
	if (fn == NULL || fn[0] == 0) return 0;
// ............................................................
	if (read_param_set)
	{
		set_params_values_name(fn, -1);
		//find param set
		fd = open(cfg_name, O_RDONLY, 0777);
		if (fd >= 0)
		{
			buf=umalloc(16);
			rcnt = read(fd, buf, 15);
			buf[rcnt] = 0;
			close(fd);
			conf.script_param_set = strtol(buf, NULL, 0);
			ufree(buf);
		} 
           else 
           conf.script_param_set = 0;
	}
// ............................................................
	set_params_values_name(fn, conf.script_param_set);
	if (!update_vars) return 0;
	//open and read file
	if (stat(cfg_set_name,&st) != 0)
		return 0;
	buf=umalloc(st.st_size+1);
	if(!buf) return 0;

	fd = open(cfg_set_name, O_RDONLY, 0777);
	if (fd < 0) 
       {
	  ufree(buf);
	  return 0;
	 }
	rcnt = read(fd, buf, st.st_size);
	buf[rcnt] = 0;
	close(fd);

	for(i = 0; i < SCRIPT_NUM_PARAMS; ++i) script_params_update[i]=0;
    ptr = buf;
// ............................................................
    descnt=0;                                // reset count of occurences of @desc
    while (ptr[0]) 
	{
 
        while (ptr[0]==' ' || ptr[0]=='\t') ++ptr; //whitespaces


        if (ptr[0]=='@')
	   {
            if (strncmp("@param", ptr, 6) == 0) 
		{
              ptr+=6;
		  process_param(ptr, 1);
            } 
           else if (strncmp("@default", ptr, 8)==0) 
            {
              ptr+=8;
		  process_default(ptr, 1);
            }
            else if (strncmp("@desc", ptr,5)==0) 
            {
              ptr+=5;
              process_description(ptr,descnt);
              descnt+=1;                          // increment occurences of @desc
            }
         }
        while (ptr[0] && ptr[0]!='\n') ++ptr; //unless end of line
        if (ptr[0]) ++ptr;
      }
// ............................................................
	 ufree(buf);

	 return 1;
}

//-------------------------------------------------------------------

void script_load(const char *fn, int saved_params) 
{
    int i, update_vars;
    FILE *fd = NULL;
    struct stat st;
                                                                                             
    for(i=0;i<4;i++)paramdesc[i][0]='\0';                                                          // clear description
    if(state_ubasic_script && state_ubasic_script != ubasic_script_default)                     // if a non-default script loaded
        free((void *)state_ubasic_script);                                                      // clear it from memory

    state_ubasic_script = ubasic_script_default;                                                // and set current script as default script
    update_vars = (strcmp(fn, conf.script_file) != 0) || !saved_params || (saved_params == 2);  //update if new file
// ............................................................
    if (!fn[0])                                                                                 // no filename given
     {                                                                                          // so load internal script
        if (!conf.script_file[0])                                                               // internal script was used last time
        { 
            fd =  fopen(SCRIPT_DEFAULT_FILENAME, "rb");                                
            if (fd) 
            {
             fn = SCRIPT_DEFAULT_FILENAME;
             update_vars = 1; 
            }
        }
     } 
    else                                                                                        // filename given so open file
     {
       fd = fopen(fn, "rb");
       if (!fd) 
        {
         conf.script_file[0]=0;
         update_vars = 1; 
        }
     }
// ............................................................
    //zero size = default script
    if(stat((char *)fn,&st) != 0 || st.st_size == 0) 
      {
        conf.script_file[0]=0;
        update_vars = 1; 
        if(fd) 
          {
            fclose(fd);
            fd=0;
          }
       }
// ............................................................
    if (fd)
     {
        int rcnt;
        char *buf;

        buf = malloc(st.st_size+1);                                                              // allocate memory
        if(!buf)                                                                                 // return if could not be allocated
          {
            fclose(fd);
            return;
          }

        rcnt = fread(buf, 1, st.st_size,fd);
        if (rcnt > 0)
         {
          buf[rcnt] = 0;
          state_ubasic_script = buf;                                                             // memory address of script start
          strcpy(conf.script_file, fn);
         }
        else 
        {
         free(buf);
        }
        fclose(fd);
     }
// ..............................................................
    if (update_vars)                                                                            // if variables are to be updated
     {
        for (i=0; i<SCRIPT_NUM_PARAMS; ++i)                                                     // zero the arrays
        {
         conf.ubasic_vars[i] = 0;
         script_loaded_params[i] = 0;
        }
     }
// ............................................................
    script_scan(conf.script_file, update_vars);                                                // get params into script_params[] and script_param_order[]
    if (saved_params)                                                                          // if params previously saved
        load_params_values(conf.script_file, update_vars, (saved_params!=2));	                 // load them
    gui_update_script_submenu();                                                               // update script submenu
}

//-------------------------------------------------------------------

void script_console_clear() 
{
    register int i;

    for (i=0; i<SCRIPT_CONSOLE_NUM_LINES; ++i) 
    {
        script_console_buf[i][0]=0;
    }
    script_console_lines=0;
    draw_restore();
}

//-------------------------------------------------------------------
void script_console_draw() 
{
    register int i, l;
    static char buf[8];

    static int num_lines;
    static int line_length;
    static int cons_x;
    static int cons_y;

    if(conf.sdm_console)
    {
     num_lines=conf.num_lines;                                           // SDM console
     line_length=conf.line_length;
     cons_x=SCRIPT_CONSOLE_X;
     cons_y=12-num_lines;
    }

    else                                                                 // CHDK console
    { 
     num_lines=(int)SCRIPT_CONSOLE_NUM_LINES;
     line_length=(int)SCRIPT_CONSOLE_LINE_LENGTH;
     cons_x=(int)SCRIPT_CONSOLE_X;
     cons_y=(int)SCRIPT_CONSOLE_Y;
    }

     for (i=0; i<script_console_lines; ++i)                             // for occupied lines
       {
        l=strlen(script_console_buf[i]);                               // length of string in the line
        if(l)                                                          // if not an empty line
          {
           if(((char)script_console_buf[i][0]=='!')||((char)script_console_buf[i][0]==' ')||((char)script_console_buf[i][0]=='#'))l-=1; // string length one character shorter as colour format character not pronted
           draw_txt_string(cons_x,cons_y+num_lines-script_console_lines+i, script_console_buf[i],((char)script_console_buf[i][0]=='!')?MAKE_COLOR(COLOR_RED,COLOR_WHITE):((char)script_console_buf[i][0]==' ')?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):((char)script_console_buf[i][0]=='#')?MAKE_COLOR(COLOR_GREEN,COLOR_WHITE):MAKE_COLOR(COLOR_BG, COLOR_FG));
           for (; l<line_length; ++l)                                                                    // pad end of line with white-space
           draw_txt_char(cons_x+l,cons_y+num_lines-script_console_lines+i, ' ',((char)script_console_buf[i][0]=='!')?MAKE_COLOR(COLOR_RED,COLOR_WHITE):((char)script_console_buf[i][0]==' ')?MAKE_COLOR(COLOR_BLUE,COLOR_WHITE):((char)script_console_buf[i][0]=='#')?MAKE_COLOR(COLOR_GREEN,COLOR_WHITE):MAKE_COLOR(COLOR_BG, COLOR_FG));
          }
        else                                                           // empty line
          {
           for (; l<line_length; ++l) 
           draw_txt_char(cons_x+1,cons_y+num_lines-script_console_lines+i, ' ',COLOR_TRANSPARENT);
          }
       }                                                               // next line
}

//-------------------------------------------------------------------
void script_console_add_line(const char *str) 
{
    register int i;

    static int num_lines;
    static int line_length;

    if(conf.sdm_console)
    {
     num_lines=conf.num_lines;
     line_length=conf.line_length;
    }

    else
    {
     num_lines=(int)SCRIPT_CONSOLE_NUM_LINES;
     line_length=(int)SCRIPT_CONSOLE_LINE_LENGTH;
    }

  if(gConsoleLine<0)                                                               // not a lineX command, so scrolling enabled
    {

     if ((script_console_lines == num_lines))                                      // if at lat line of console
      {
        for (i=1; i<num_lines; ++i)                                                // move everything up one line
        {
            strcpy(script_console_buf[i-1], script_console_buf[i]);
        }
        --script_console_lines;
      }

      strncpy(script_console_buf[script_console_lines], str,line_length);           // copy string to console lines buffer
      script_console_buf[script_console_lines][line_length]=0;                      // and add terminator
    
      ++script_console_lines;                                                       // increment to next available line
      script_console_draw();                                                        // draw the console
    }
    
   else  
    {
     console_add_line(str,gConsoleLine);
    }
}

//-------------------------------------------------------------------

void console_add_line(const char *str,int x) 
{
    register int i;
    strncpy(script_console_buf[x-1], str, SCRIPT_CONSOLE_LINE_LENGTH);             // add string to buffer
    script_console_buf[x-1][SCRIPT_CONSOLE_LINE_LENGTH]=0;                         // terming '\0'
    script_console_lines=SCRIPT_CONSOLE_NUM_LINES;                                 // draw all lines of console
    script_console_draw();
}
