/**
 * Copyright (c) 2006, Adam Dunkels
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
#if DEBUG
#define DEBUG_PRINTF(...)  printf(__VA_ARGS__)
#else
#define DEBUG_PRINTF(...)
#endif
#ifdef TEST
#include "../../include/ubasic.h"
#include "../../include/platform.h"
#include "../../include/script.h"
#include <string.h>
#include <fcntl.h>
#include <io.h>
#else
#include "ubasic.h"
#include "platform.h"
#include "script.h"
#endif
#include "../../core/gui_osd.h"
#include "tokenizer.h"
#include "../../include/keyboard.h"
#include "../../include/conf.h"
#include "../../core/edgeoverlay.h"
#include "math.h"
#include "camera_functions.h"
#include "stdlib.h"
extern  long get_batt_average();
extern float myhyp,coc,deviation,my_max_deviation,max_distance;
extern long myav;
extern int shift,afl,edge_count;
extern Conf conf;
extern long sensor_width;
extern int sertx,fpd,sd_repeat;
extern char pcode;
extern char lobyte;
extern char hibyte;
extern int rx_ok;
extern int up,msstv96,ub;
extern int gOverlayNumber;                      
extern int gLoadOverlay;                        
extern int block_script,sertx;
int get_luminance,put_off_ev96;
static const double log_2=0.6931471805599;      
char buf[32];                                   
int gSpecial_token;
int gConsoleLine= -1;
int af_all;                                      
int st_tv_exp=0;                                 
int st_tv_ev=0;                                  
extern int gDebug;
static int current_display_mode;
int bls = 1; 
int speed = 1;
int yield = 0;
int has_refocused=0;
#define MAX_SPEED 5
#include "stdlib.h" 
#ifdef DEBUG
#include <stdio.h>
#endif
static char const *program_ptr;
#define MAX_STRINGLEN 40
static char string[MAX_STRINGLEN];
#define MAX_GOSUB_STACK_DEPTH 10
static short gosub_stack[MAX_GOSUB_STACK_DEPTH];
static int gosub_stack_ptr;
#define MAX_IF_STACK_DEPTH 4
static short if_stack[MAX_IF_STACK_DEPTH];
static int if_stack_ptr;
#define MAX_WHILE_STACK_DEPTH 4
static short while_stack[MAX_WHILE_STACK_DEPTH];
static int while_stack_ptr;
#define MAX_DO_STACK_DEPTH 4
static short do_stack[MAX_DO_STACK_DEPTH];
static int do_stack_ptr;
struct for_state {
  short line_after_for;
  short for_variable;
  int to;
  int step;
};
#define MAX_FOR_STACK_DEPTH 4
static struct for_state for_stack[MAX_FOR_STACK_DEPTH];
static int for_stack_ptr;
#define MAX_VARNUM 52
static int variables[MAX_VARNUM];
static int  print_screen_p;             
static int  print_screen_d = -1;        
static const char print_screen_file[] ="A/CHDK/SCRIPTS/PR_SCREEN.TXT";
#define TL_PARAMS_NAME "A/CHDK/PARAMS/Tlapse%1d.txt"
#define TL_PARAM_NUMS 20                
static int ended;
static int expr(void);
static void line_statement(void);
static void statement(void);
static int relation(void);
#define USBD_MASK  0x4
int ubasic_error;
const char *ubasic_errstrings[UBASIC_E_ENDMARK] = 
{
    "No err",
    "Parse err",
    "Unk stmt",
    "Unk key",
    "Unk label",
    "Stack ful",
    "bad return",
    "Unk err" 
};
int
ubasic_linenumber()
{
  return tokenizer_line_number();
}
void
ubasic_init(const char *program)
{
  program_ptr = program;
  for_stack_ptr = gosub_stack_ptr = while_stack_ptr = do_stack_ptr = if_stack_ptr = 0;
  tokenizer_init(program);
  ended = 0;
  print_screen_p = 0;
  if (print_screen_d >= 0) {
    close(print_screen_d);
    print_screen_d = -1;
  }
  ubasic_error = UBASIC_E_NONE;
}
static void
accept(int token)
{
  if(token != tokenizer_token()) 
  {
    DEBUG_PRINTF("Token not what was expected (expected %d, got %d)\n",
		 token, tokenizer_token());
    tokenizer_error_print();
     tokenizer_next();
     ended = 1;
     ubasic_error = UBASIC_E_PARSE;
     return;
  }
  DEBUG_PRINTF("Expected %d, got it\n", token);
  tokenizer_next();
}
static void
accept_cr()
{
    while(tokenizer_token() != TOKENIZER_CR &&
        tokenizer_token() != TOKENIZER_ENDOFINPUT)
      tokenizer_next();
    accept(TOKENIZER_CR);
}
static int
varfactor(void)
{
  int r;
  DEBUG_PRINTF("varfactor: obtaining %d from variable %d\n", variables[tokenizer_variable_num()], tokenizer_variable_num());
  r = ubasic_get_variable(tokenizer_variable_num());      
  accept(TOKENIZER_VARIABLE);
  return r;
}
static int
factor(void)                                              
{
  int r = 0; 
  int zp; 
  DEBUG_PRINTF("factor: token %d\n", tokenizer_token());
  switch(tokenizer_token()) 
 {                              
  case TOKENIZER_NUMBER:
    r = tokenizer_num();
    DEBUG_PRINTF("factor: number %d\n", r);
    accept(TOKENIZER_NUMBER);
    break;
  case TOKENIZER_LEFTPAREN:
    accept(TOKENIZER_LEFTPAREN);
    r = relation();
    accept(TOKENIZER_RIGHTPAREN);
    break;
  case TOKENIZER_PLUS:
    accept(TOKENIZER_PLUS);
    r = factor();
    break;
  case TOKENIZER_MINUS:
    accept(TOKENIZER_MINUS);
    r = - factor();
    break;
  case TOKENIZER_LNOT:
    accept(TOKENIZER_LNOT);
    r = ! relation();
    break;
 case TOKENIZER_GET_DAY_SECONDS:
    accept(TOKENIZER_GET_DAY_SECONDS);
    r = shooting_get_day_seconds();     
    break;
 case TOKENIZER_GET_TICK_COUNT:
    accept(TOKENIZER_GET_TICK_COUNT);
    r = shooting_get_tick_count();     
    break;
 case TOKENIZER_GET_MODE:
    accept(TOKENIZER_GET_MODE);
    int m=mode_get()&MODE_SHOOTING_MASK;
		int mode_video=MODE_IS_VIDEO(m);
		if ((mode_get()&MODE_MASK) != MODE_PLAY) r = 0;
    if ((mode_get()&MODE_MASK) == MODE_PLAY) r = 1;
    if (((mode_get()&MODE_MASK) != MODE_PLAY) && mode_video) r = 2;
    break;
 case TOKENIZER_IS_KEY:
    accept(TOKENIZER_IS_KEY);
    tokenizer_string(string, sizeof(string));
    tokenizer_next();
    r = ubasic_camera_is_clicked(string);     
    break;
 case TOKENIZER_SCRIPT_AUTOSTART:
    accept(TOKENIZER_SCRIPT_AUTOSTART);
    r = ubasic_camera_script_autostart();     
    break;
 case TOKENIZER_GET_USB_POWER:
    accept(TOKENIZER_GET_USB_POWER);
    r = get_usb_power(0);              
    break;
 case TOKENIZER_DR:                           
    accept(TOKENIZER_DR);
    r = rx_ok;                               
    rx_ok=0;                                 
    break;
  case TOKENIZER_GET_FREE_DISK_SPACE:
    accept(TOKENIZER_GET_FREE_DISK_SPACE);
    r = GetFreeCardSpaceKb();
    break;
  case TOKENIZER_GET_FOCUS:
    accept(TOKENIZER_GET_FOCUS);
    r = lens_get_focus_pos()-fpd;
    break;
  case TOKENIZER_GET_VIDEO_BUTTON:
    accept(TOKENIZER_GET_VIDEO_BUTTON);
    #if CAM_HAS_VIDEO_BUTTON
    r = 1;
    #else
    r = 0;
    #endif
    break;
 case TOKENIZER_GET_FLASH_MODE:
    accept(TOKENIZER_GET_FLASH_MODE);
    r = shooting_get_prop(PROPCASE_FLASH_MODE);
    break;
 case TOKENIZER_GET_SHOOTING:
    accept(TOKENIZER_GET_SHOOTING);
    r = shooting_get_prop(PROPCASE_SHOOTING);
    break;
 case TOKENIZER_GET_RESOLUTION: 
  accept(TOKENIZER_GET_RESOLUTION); 
  r = shooting_get_prop(PROPCASE_RESOLUTION); 
  break;
 case TOKENIZER_GET_QUALITY: 
  accept(TOKENIZER_GET_QUALITY); 
  r = shooting_get_prop(PROPCASE_QUALITY); 
      break;
  case TOKENIZER_GET_ZOOM_STEPS:
    accept(TOKENIZER_GET_ZOOM_STEPS);
    r = zoom_points;
    break;
 case TOKENIZER_BRACKETING:
      accept (TOKENIZER_BRACKETING);
      r = get_inf_focus();
  break; 
 case TOKENIZER_GET_MAX_SPT:
      accept(TOKENIZER_GET_MAX_SPT);
      r = speed;     
      break;
case TOKENIZER_GET_FOV:
      accept(TOKENIZER_GET_FOV);
      zp=lens_get_zoom_point();
      r = get_fov(zp);     
      break;
  default:
    r = varfactor();                 
    break;
  }
  return r;
}
static int
term(void)                              
{
  int f1, f2;
  int op;
  f1 = factor();                         
  op = tokenizer_token();                
 if((gSpecial_token)&& ((op == TOKENIZER_SLASH)||(op == TOKENIZER_PERIOD))) op= TOKENIZER_COMMA;
  DEBUG_PRINTF("term: token %d\n", op);
  while(op == TOKENIZER_ASTR ||
 op == TOKENIZER_SLASH ||
 op == TOKENIZER_LT ||
 op == TOKENIZER_GT ||
 op == TOKENIZER_GE ||
 op == TOKENIZER_LE ||
 op == TOKENIZER_NE ||
 op == TOKENIZER_EQ ||
 op == TOKENIZER_XOR || 
        op == TOKENIZER_OR ||
 op == TOKENIZER_MOD) 
    {
    tokenizer_next();                             
    f2 = factor();                                
    DEBUG_PRINTF("term: %d %d %d\n", f1, op, f2);
    switch(op)                                    
     {
    case TOKENIZER_ASTR:
      f1 = f1 * f2;
      break;
    case TOKENIZER_SLASH:
      f1 = f1 / f2;
      break;
    case TOKENIZER_MOD:
      f1 = f1 % f2;
      break;
    case TOKENIZER_LT:
      f1 = f1 < f2;
      break;
    case TOKENIZER_GT:
      f1 = f1 > f2;
      break;
    case TOKENIZER_EQ:
      f1 = f1 == f2;
      break;
    case TOKENIZER_NE:
      f1 = f1 != f2;
      break;
    case TOKENIZER_LE:
      f1 = f1 <= f2;
      break;
    case TOKENIZER_GE:
      f1 = f1 >= f2;
      break;
    case TOKENIZER_OR:
      f1 = f1 | f2;
      break;
    case TOKENIZER_XOR:
      f1 = f1 ^ f2;
      break;
    }
    op = tokenizer_token();               
  }                                       
  DEBUG_PRINTF("term: %d\n", f1);
  return f1;                             
}
static int
expr(void)                                         
{
  int t1, t2;
  int op;
  
  t1 = term();                                     
  op = tokenizer_token();                          
  DEBUG_PRINTF("expr: token %d\n", op);
  while(op == TOKENIZER_PLUS ||                   
 op == TOKENIZER_MINUS ||
 op == TOKENIZER_AND ||
        op == TOKENIZER_LOR ||
 op == TOKENIZER_XOR) 
  {                                               
    tokenizer_next();                             
    t2 = term();                                   
    DEBUG_PRINTF("expr: %d %d %d\n", t1, op, t2);
    switch(op)                                    
   {
    case TOKENIZER_PLUS:
      t1 = t1 + t2;
      break;
    case TOKENIZER_MINUS:
      t1 = t1 - t2;
      break;
    case TOKENIZER_AND:
      t1 = t1 & t2;
      break;
    case TOKENIZER_LOR:
      t1 = t1 || t2;
      break;
    }
    op = tokenizer_token();                      
  }
  DEBUG_PRINTF("expr: %d\n", t1);
  return t1;                                     
}
static int
relation(void)
{
  int r1, r2;
  int op;
  
  r1 = expr();
  op = tokenizer_token();
  DEBUG_PRINTF("relation: token %d\n", op);
  while(op == TOKENIZER_LAND) {
    tokenizer_next();
    r2 = expr();
    DEBUG_PRINTF("relation: %d %d %d\n", r1, op, r2);
    switch(op) {
    case TOKENIZER_LAND:
      r1 = r1 && r2;
      break;
    }
    op = tokenizer_token();
  }
  return r1;
}
#if 0
static void
jump_linenum(int linenum)
{
  tokenizer_init(program_ptr);
  while(tokenizer_num() != linenum) {
    do {
      do {
 tokenizer_next();
      } while(tokenizer_token() != TOKENIZER_CR &&
       tokenizer_token() != TOKENIZER_ENDOFINPUT);
      if(tokenizer_token() == TOKENIZER_CR) {
 tokenizer_next();
      }
    } while(tokenizer_token() != TOKENIZER_NUMBER);
    DEBUG_PRINTF("jump_linenum: Found line %d\n", tokenizer_num());
  }
}
#endif
static void
jump_line(int linenum)
{
  tokenizer_init(program_ptr);
  while(tokenizer_line_number() != linenum) {
    tokenizer_next();
  }
  /*swallow the CR that would be read next */
  if (linenum >1) accept(TOKENIZER_CR);  
}
static int
jump_label(char * label)
{
  char currLabel[MAX_STRINGLEN];
  tokenizer_init(program_ptr);
  currLabel[0] = 0;
  while(tokenizer_token() != TOKENIZER_ENDOFINPUT) {
    tokenizer_next();
    if (tokenizer_token() == TOKENIZER_LABEL) {
      tokenizer_label(currLabel, sizeof(currLabel));
      tokenizer_next();
      if(strcmp(label, currLabel) == 0) {
        accept(TOKENIZER_CR);
        DEBUG_PRINTF("jump_linenum: Found line %d\n", tokenizer_line_number());
        break;
      }
    }
  }
  if (tokenizer_token() == TOKENIZER_ENDOFINPUT) {
      DEBUG_PRINTF("Label %s not found", label);
      ubasic_error = UBASIC_E_UNK_LABEL;
      return 0;
  } else {
      return 1;
  }
}
static void
goto_statement(void)
{
  accept(TOKENIZER_GOTO);
  if(tokenizer_token() == TOKENIZER_STRING) {
    tokenizer_string(string, sizeof(string));
    tokenizer_next();
    jump_label(string);
  } else {
    DEBUG_PRINTF("ubasic.c: goto_statement(): no label specified\n");
    ended = 1;
    ubasic_error = UBASIC_E_UNK_LABEL;
  }
}
static void
print_screen_statement(void)
{
  int val;
  accept(TOKENIZER_PRINT_SCREEN);
  val = expr();
  accept(TOKENIZER_CR);
  if (val && print_screen_d<0) {
      print_screen_d = open(print_screen_file, O_WRONLY|O_CREAT|O_TRUNC, 0777);
  }
  print_screen_p = val;
}
static void
print_statement(void)
{
  static char buf[128];
  buf[0]=0;
  do 
  {
    DEBUG_PRINTF("Print loop\n");
    if(tokenizer_token() == TOKENIZER_STRING) 
    {
      tokenizer_string(string, sizeof(string));
      sprintf(buf+strlen(buf), "%.25s", string);  
      tokenizer_next();
    } 
   else if(tokenizer_token() == TOKENIZER_COMMA) 
    {
      strcat(buf, " ");
      tokenizer_next();
    } 
   else if(tokenizer_token() == TOKENIZER_SEMICOLON) 
    {
      tokenizer_next();
    } 
   else 
    {
      sprintf(buf+strlen(buf), "%d", expr());
    }
  } while(tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ENDOFINPUT && tokenizer_token() != TOKENIZER_ELSE);
  script_console_add_line(buf);
  if (print_screen_p && print_screen_d>=0) 
  {
    int bl = strlen(buf);
    buf[bl]='\n';
    write(print_screen_d, buf, bl+1);
  }
  DEBUG_PRINTF("End of print\n");
  accept_cr();
}
static void
prnt_statement(void)
{
 accept(TOKENIZER_PRNT);
 gConsoleLine=-1;
 print_statement();
}
static void
ip_statement(void)
{
  static char buf[128];
  buf[0]=0;
  do 
  {
    tokenizer_string(string, sizeof(string));
    sprintf(buf+strlen(buf), "%-25s", string);
    tokenizer_next();
  } while(tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ENDOFINPUT && tokenizer_token() != TOKENIZER_ELSE);
  script_console_add_line(buf);
  if (print_screen_p && print_screen_d>=0) 
   {
    int bl = strlen(buf);
    buf[bl]='\n';
    write(print_screen_d, buf, bl+1);
   }
  accept_cr();
}
static void
if_statement(void)
{
  int r, else_cntr,endif_cntr;
  
  accept(TOKENIZER_IF);
  r = relation(); 
 
  accept(TOKENIZER_THEN);
  if (tokenizer_token() == TOKENIZER_CR) {  
 if(if_stack_ptr < MAX_IF_STACK_DEPTH) {
  if_stack[if_stack_ptr] = r;
  if_stack_ptr++;
 }
   accept(TOKENIZER_CR);
   if(r) {
    return;
   }
   else {
    else_cntr=endif_cntr=0;
    tokenizer_next();        
  while(((tokenizer_token() != TOKENIZER_ELSE &&  tokenizer_token() != TOKENIZER_ENDIF) 
         || else_cntr || endif_cntr) && tokenizer_token() != TOKENIZER_ENDOFINPUT){
    if( tokenizer_token() == TOKENIZER_IF) {else_cntr+=1;endif_cntr+=1;}      
    if( tokenizer_token() == TOKENIZER_ELSE) {
     else_cntr--;    
    }
    if( tokenizer_token() == TOKENIZER_ENDIF)  {
      endif_cntr--;
            if (endif_cntr != else_cntr) else_cntr--;
    }    
    tokenizer_next();                                                         
   }
     if(tokenizer_token() == TOKENIZER_ELSE) {
      return;
  }
   }                                                  
   accept(TOKENIZER_ENDIF);
	  accept(TOKENIZER_CR);  
	   if(if_stack_ptr > 0) 
         {
    	    if_stack_ptr--;
         }              
  }else {
	  if(r) {
     statement();
   } else {
     do {
       tokenizer_next();
     } while(tokenizer_token() != TOKENIZER_ELSE &&
      tokenizer_token() != TOKENIZER_CR &&
      tokenizer_token() != TOKENIZER_ENDOFINPUT);
     if(tokenizer_token() == TOKENIZER_ELSE) {
       tokenizer_next();
       statement();
     } else if(tokenizer_token() == TOKENIZER_CR) {
       tokenizer_next();
     }
   }
  }
}
static void
else_statement(void)
{
  int r=0, endif_cntr;
  
  accept(TOKENIZER_ELSE);
  if(if_stack_ptr > 0) {
    r = if_stack[if_stack_ptr-1];
  }
 
  if (tokenizer_token() == TOKENIZER_CR) {  
   accept(TOKENIZER_CR);
   if(!r) {
    return;
   }
   else {                                      
    endif_cntr=0;
  tokenizer_next();        
  while(((tokenizer_token() != TOKENIZER_ENDIF ) 
         || endif_cntr) && tokenizer_token() != TOKENIZER_ENDOFINPUT){
    if( tokenizer_token() == TOKENIZER_IF) {endif_cntr+=1;}      
    if( tokenizer_token() == TOKENIZER_ENDIF)  {
      endif_cntr--;
    }    
    tokenizer_next();                                                         
   }
   }                                                  
              
  }
}
static void
endif_statement(void)
{
 accept(TOKENIZER_ENDIF);
 accept(TOKENIZER_CR);                 
    if(if_stack_ptr > 0) {
     if_stack_ptr--;
    }
}
static void
let_statement(void)
{
 
  int var;
  var = tokenizer_variable_num();
  accept(TOKENIZER_VARIABLE);
  accept(TOKENIZER_EQ);
  ubasic_set_variable(var, expr());
  DEBUG_PRINTF("let_statement: assign %d to %d\n", variables[var], var);
  accept_cr();
}
static void
rem_statement(void)
{
  accept(TOKENIZER_REM);
  DEBUG_PRINTF("rem_statement\n");
  accept(TOKENIZER_CR);
}
static void
cls_statement(void)
{
  accept(TOKENIZER_CLS);
  script_console_clear();
  DEBUG_PRINTF("cls_statement\n");
  accept(TOKENIZER_CR);
}
static void
gosub_statement(void)
{
  accept(TOKENIZER_GOSUB);
  if(tokenizer_token() == TOKENIZER_STRING) {
    tokenizer_string(string, sizeof(string));
    do {
    tokenizer_next();
    } while(tokenizer_token() != TOKENIZER_CR);
    accept(TOKENIZER_CR);
    if(gosub_stack_ptr < MAX_GOSUB_STACK_DEPTH) {
      gosub_stack[gosub_stack_ptr] = tokenizer_line_number();
      gosub_stack_ptr++;
      jump_label(string);
    } else {
      DEBUG_PRINTF("gosub_statement: gosub stack exhausted\n");
      ended = 1;
      ubasic_error = UBASIC_E_GOSUB_STACK_EXHAUSTED;
    }
  } else {
    DEBUG_PRINTF("ubasic.c: goto_statement(): no label specified\n");
    ended = 1;
    ubasic_error = UBASIC_E_UNK_LABEL;
  }
}
static void
return_statement(void)
{
  accept(TOKENIZER_RETURN);
  if(gosub_stack_ptr > 0) {
    gosub_stack_ptr--;
    jump_line(gosub_stack[gosub_stack_ptr]);
  } else {
    DEBUG_PRINTF("return_statement: non-matching return\n");
    ended = 1;
    ubasic_error = UBASIC_E_UNMATCHED_RETURN;
  }
}
static void
next_statement(void)
{
  int var, value;
  
  accept(TOKENIZER_NEXT);
  var = tokenizer_variable_num();
  accept(TOKENIZER_VARIABLE);
  if(for_stack_ptr > 0 &&
     var == for_stack[for_stack_ptr - 1].for_variable) {
    value = ubasic_get_variable(var) + for_stack[for_stack_ptr - 1].step;
    ubasic_set_variable(var, value);
    if(((for_stack[for_stack_ptr - 1].step > 0) && (value <= for_stack[for_stack_ptr - 1].to)) ||
       ((for_stack[for_stack_ptr - 1].step < 0) && (value >= for_stack[for_stack_ptr - 1].to)))
        jump_line(for_stack[for_stack_ptr - 1].line_after_for); 
    else {
      for_stack_ptr--;
      accept(TOKENIZER_CR);
    }
  } else {
    DEBUG_PRINTF("next_statement: non-matching next (expected %d, found %d)\n", for_stack[for_stack_ptr - 1].for_variable, var);
    accept(TOKENIZER_CR);
  }
}
static void
for_statement(void)
{
  int for_variable, to, step;
  
  accept(TOKENIZER_FOR);
  for_variable = tokenizer_variable_num();
  accept(TOKENIZER_VARIABLE);
  accept(TOKENIZER_EQ);
  ubasic_set_variable(for_variable, expr());
  accept(TOKENIZER_TO);
  to = expr();                    
  step = 1;
  if (tokenizer_token() != TOKENIZER_CR) {
   accept(TOKENIZER_STEP);
   step = expr();         
  }
  accept(TOKENIZER_CR);
  if(for_stack_ptr < MAX_FOR_STACK_DEPTH) {
    for_stack[for_stack_ptr].line_after_for = tokenizer_line_number();
    for_stack[for_stack_ptr].for_variable = for_variable;
    for_stack[for_stack_ptr].to = to;
    for_stack[for_stack_ptr].step = step;
    DEBUG_PRINTF("for_statement: new for, var %d to %d\n",
   for_stack[for_stack_ptr].for_variable,
   for_stack[for_stack_ptr].to);
   
    for_stack_ptr++;
  } else {
    DEBUG_PRINTF("for_statement: for stack depth exceeded\n");
  }
}
static void
do_statement(void)
{
  accept(TOKENIZER_DO);
  accept(TOKENIZER_CR);
  if(do_stack_ptr < MAX_DO_STACK_DEPTH) {
     do_stack[do_stack_ptr] = tokenizer_line_number();
     do_stack_ptr++;
  }
}
static void
until_statement(void)
{
  int r;
  
  accept(TOKENIZER_UNTIL);
  r = relation();
  if(!r) {
    if(do_stack_ptr > 0) {
      jump_line(do_stack[do_stack_ptr-1]);
    }
  }
  else {
    do_stack_ptr--;
   accept_cr();
  }
}
static void
while_statement(void)
{
  int r, while_cntr;
  
  accept(TOKENIZER_WHILE);
  if(while_stack_ptr < MAX_WHILE_STACK_DEPTH) {
    if ((while_stack_ptr == 0)||((while_stack_ptr > 0) && (while_stack[while_stack_ptr-1] != tokenizer_line_number()))){
      while_stack[while_stack_ptr] = tokenizer_line_number();
      while_stack_ptr++;
    }
  }
  r = relation();
  if(!r) {
   while_cntr=0;
    while((tokenizer_token() != TOKENIZER_WEND  || while_cntr ) && 
     tokenizer_token() != TOKENIZER_ENDOFINPUT){   
     if (tokenizer_token() == TOKENIZER_WHILE) while_cntr+=1;
     if (tokenizer_token() == TOKENIZER_WEND) while_cntr-=1;           
   tokenizer_next();
 }  
    while_stack_ptr--;
    
    accept(TOKENIZER_WEND);
    accept(TOKENIZER_CR);   
  }
  else  {
   accept_cr();        
  } 
}
static void
wend_statement(void)
{
  accept(TOKENIZER_WEND);
  if(while_stack_ptr > 0) {
    jump_line(while_stack[while_stack_ptr-1]);
  }
  else {
   accept_cr();
  }
}
static void
end_statement(void)
{
  accept(TOKENIZER_END);
  ended = 1;
}
static void
click_statement(void)
{
  accept(TOKENIZER_CLICK);
  tokenizer_string(string, sizeof(string));
  ubasic_camera_click(string);
  tokenizer_next();
  DEBUG_PRINTF("End of click\n");
  accept_cr();
}
static void
press_statement(void)
{
  accept(TOKENIZER_PRESS);
  tokenizer_string(string, sizeof(string));
  ubasic_camera_press(string);
  tokenizer_next();
  DEBUG_PRINTF("End of press\n");
  accept_cr();
}
static void
release_statement(void)
{
  accept(TOKENIZER_RELEASE);
  tokenizer_string(string, sizeof(string));
  ubasic_camera_release(string);
  tokenizer_next();
  DEBUG_PRINTF("End of release\n");
  accept_cr();
}
static void
sleep_statement(void)
{
  int val;
  accept(TOKENIZER_SLEEP);
  val = expr();
  ubasic_camera_sleep(val);
  DEBUG_PRINTF("End of sleep\n");
  accept_cr();
}
static void sleep_s_statement(void)
{
  int val;
  accept(TOKENIZER_SLEEP_S);
  val = expr();
  ubasic_camera_sleep(val*1000);
  accept_cr();
}
static void sleep_m_statement(void)
{
  int val;
  accept(TOKENIZER_SLEEP_M);
  val = expr();
  ubasic_camera_sleep(val*1000*60);
  accept_cr();
}
static void
shoot_statement(void)
{
  accept(TOKENIZER_SHOOT);
  ubasic_camera_shoot();
  DEBUG_PRINTF("End of shoot\n");
  accept_cr();
}
static void set_tv96_direct_statement()
{
    int to;
    accept(TOKENIZER_SET_TV96_DIRECT);
    to = expr();
    shooting_set_tv96_direct((short int)to, SET_LATER);
    accept_cr();
}
static void set_tv_statement()
{
    int to;
    accept(TOKENIZER_SET_TV);
    to = expr();
    shooting_set_tv(to);
    accept_cr();
}
static void set_focus_statement()                                                              
{
 int to,var;
    accept(TOKENIZER_SET_FOCUS);
    to = expr();
    ubasic_shooting_set_focus(to+fpd, SET_LATER);
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE)              
     {
      tokenizer_next();                                                                        
      var = tokenizer_variable_num();
      accept(TOKENIZER_VARIABLE);    
     if((shooting_get_farpoint(to)>to)&& (shooting_get_farpoint(to)<=65535))
       ubasic_set_variable(var,shooting_get_farpoint(to));
      else
        ubasic_set_variable(var,-1);
     }
     accept_cr();
}
static void set_led_statement()
{
    int to, to1, to2;
    accept(TOKENIZER_SET_LED);
    to = expr();
    to1 = expr();
 to2 = 200;
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
  to2 = expr();
    }
 ubasic_set_led(to, to1, to2);
    accept_cr();
}
static void set_av_statement()
{
    int to;
    accept(TOKENIZER_SET_AV);
    to = expr();
    shooting_set_av(to);
    accept_cr();
}
static void set_zoom_statement()
{
    int to;
    accept(TOKENIZER_SET_ZOOM);
    to = expr();
    shooting_set_zoom(to);
    accept_cr();
}
static void get_zoom_statement()
{
    int var;
    accept(TOKENIZER_GET_ZOOM);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var, shooting_get_zoom());
    accept_cr();
}
static void get_focus_statement()
{
    int var;
    accept(TOKENIZER_GET_FOCUS);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var,lens_get_focus_pos()-fpd);
    accept_cr();
}
static void get_campos_statement()
{
    int var;
    accept(TOKENIZER_CAMPOS);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var,(conf.camera_position)?1:0);
    accept_cr();
}
static void get_free_disk_space_statement()
{
    int var;
    accept(TOKENIZER_GET_FREE_DISK_SPACE);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var, GetFreeCardSpaceKb());
    accept_cr();
}
static void set_zoom_rel_statement()
{
    int to;
    accept(TOKENIZER_SET_ZOOM_REL);
    to = expr();
    shooting_set_zoom_rel(to);
    accept_cr();
}
static void set_zoom_speed_statement()
{
    int to;
    accept(TOKENIZER_SET_ZOOM_SPEED);
    to = expr();
    shooting_set_zoom_speed(to);
    accept_cr();
}
static void get_prop_statement()
{
    int var, var1;
    accept(TOKENIZER_GET_PROP);
    var = expr();
    var1 = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var1, shooting_get_prop(var));
 
    accept_cr();
}
static void set_sync_statement()
{
    accept(TOKENIZER_SET_SYNC);
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) 
 {
  conf.synch_enable = expr();
             if(conf.synch_enable < 0) conf.synch_enable = 0;
             if(conf.synch_enable > 1) conf.synch_enable = 1;
             if(conf.synch_enable ==1) conf.remote_enable = 1;
  if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
  {
   conf.synch_delay_enable = expr();
                   if(conf.synch_delay_enable < 0) conf.synch_delay_enable = 0;
                   if(conf.synch_delay_enable > 1) conf.synch_delay_enable = 1;
   if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
   {
    conf.synch_user_delay = expr();
                        if(conf.synch_user_delay < 1) conf.synch_user_delay = 1;
                        if(conf.synch_user_delay > 100000)conf.synch_user_delay = 100000;
                      if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
       {
     conf.synch_delay_coarse_value = expr();
                         if(conf.synch_delay_coarse_value < 0) conf.synch_delay_coarse_value = 0;
                         if(conf.synch_delay_coarse_value > 90)conf.synch_delay_coarse_value = 90;
       }
   }
                              
  }
 }
 accept_cr();
}
static void get_sync_statement()
{
    int var;
    accept(TOKENIZER_GET_SYNC);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var,conf.synch_enable);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var,conf.synch_delay_enable);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var,conf.synch_delay_value);
 
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var,conf.synch_delay_coarse_value);
    accept_cr();
}
static void set_prop_statement()
{
    int to, to1;
    accept(TOKENIZER_SET_PROP);
    to = expr();
    to1 = expr();
 shooting_set_prop(to, to1);
    accept_cr();
}
static void set_iso_statement()
{
    int to;
    accept(TOKENIZER_SET_ISO);
    to = expr();
    shooting_set_iso(to);
    accept_cr();
}
static void set_raw_statement()
{
    int to;
    accept(TOKENIZER_SET_RAW);
    to = expr();
    ubasic_camera_set_raw(to);
    accept_cr();
}
static void set_nd_filter_statement() 
{ 
    int to; 
    accept(TOKENIZER_SET_ND_FILTER); 
    to = expr(); 
    shooting_set_nd_filter_state(to, SET_LATER); 
    accept_cr(); 
} 
static void ndfi_statement()                      
{ 
    accept(TOKENIZER_NDFI); 
    shooting_set_nd_filter_state(1, SET_LATER); 
    accept_cr(); 
} 
static void ndfo_statement()                     
{ 
    accept(TOKENIZER_NDFO); 
    shooting_set_nd_filter_state(2, SET_LATER); 
    accept_cr(); 
} 
static void ndff_statement()                    
{ 
    accept(TOKENIZER_NDFF); 
    shooting_set_nd_filter_state(0, SET_LATER); 
    accept_cr(); 
}
 
static void set_autostart_statement()
{
    int to;
    accept(TOKENIZER_SET_SCRIPT_AUTOSTART);
    to = expr();
    ubasic_camera_set_script_autostart(to);
    accept_cr();
}
static void exit_alt_statement()
{
    int to;
    accept(TOKENIZER_EXIT_ALT);
    exit_alt();
    accept_cr();
}
static void get_user_tv_id_statement()
{
    int var;
    accept(TOKENIZER_GET_USER_TV_ID);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var, shooting_get_user_tv_id());
    accept_cr();
}
static void wait_click_statement()
{
    int timeout=0;
    accept(TOKENIZER_WAIT_CLICK);
    if (tokenizer_token() != TOKENIZER_CR &&
        tokenizer_token() != TOKENIZER_ELSE ) {
        timeout = expr();
    }
    ubasic_camera_wait_click(timeout);
    accept_cr();
}
static void is_key_statement(void)
{
    int var;
    accept(TOKENIZER_IS_KEY);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    tokenizer_string(string, sizeof(string));
    tokenizer_next();
    ubasic_set_variable(var, ubasic_camera_is_clicked(string));
    DEBUG_PRINTF("End of is_key\n");
    accept_cr();
}
static void wheel_left_statement(void){
  accept(TOKENIZER_WHEEL_LEFT);
#if defined (CAMERA_g7)|| defined (CAMERA_sx100is)|| defined(CAMERA_g9)
  JogDial_CCW();
#endif
  accept_cr();
}
static void wheel_right_statement(void){
  accept(TOKENIZER_WHEEL_RIGHT);
#if defined (CAMERA_g7)|| defined (CAMERA_sx100is)
  JogDial_CW();
#endif
  accept_cr();
}
static void line_one_statement()
{
 accept (TOKENIZER_LINE_ONE);
 gConsoleLine=1;
 print_statement();
}
static void line_two_statement()
{
 accept (TOKENIZER_LINE_TWO);
 gConsoleLine=2;
 print_statement();
}
static void line_three_statement()
{
 accept (TOKENIZER_LINE_THREE);
 gConsoleLine=3;
 print_statement();
}
static void get_luminance_statement()
{
 int ret_var_num,var;
 accept (TOKENIZER_GET_LUMINANCE);
 get_luminance=1;
 accept_cr();
 md_init_motion_detector(15,15,1,2000,3000,255,1,0,1,8,8,8,8,0,1,10);
}
static void sws_statement()
{
 int var,ret_var_num;
 accept(TOKENIZER_SWS);
 var =expr();
 if(var<1)var=1;
 tokenizer_next();
 ret_var_num = tokenizer_variable_num();
 accept_cr();
 md_init_motion_detector(1,1,1,10000,350,var,1,ret_var_num,0,0,0,0,0,16,30,800);
}
static void swnm_statement()
{
 int comp_interval,ret_var_num,sensitivity,timeout;
 accept(TOKENIZER_SWNM);
 comp_interval =expr();
 if(comp_interval<1)comp_interval=1;
 comp_interval*= 500;                     
 tokenizer_next();                        
 ret_var_num = tokenizer_variable_num();  
 sensitivity=128;
 timeout=10000;
 
 accept(TOKENIZER_VARIABLE);              
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE) { } 
 
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE) 
  {
   tokenizer_next();
   sensitivity=expr();
   if(sensitivity<1)sensitivity=1;
  }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE) 
  {
   tokenizer_next();
   timeout=expr();
   if(timeout<comp_interval+1) timeout=comp_interval+1;
   timeout*=1000;                           
  }
  accept_cr();
  md_init_motion_detector(1,1,1,timeout,comp_interval,sensitivity,1,ret_var_num,0,0,0,0,0,16,30,800);
}
static void sspd_statement()                      
{
 int var1,var2;
 short var3;
 gSpecial_token=1;
 var2=0;
 accept(TOKENIZER_SSPD);                           
 var1=expr();                                      
 if ((tokenizer_token() == TOKENIZER_COMMA)||(tokenizer_token() == TOKENIZER_SLASH)) 
 {
    tokenizer_next();                              
    var2 = expr(); 
 }
  if(var2==0)var3=(short)(96.0*log(1.0/var1)/log_2); 
  else var3=(short)(96.0*log(var2)/log_2);           
  if(var3<-576)var3=-576;                            
  if(var3>1440) var3=1440;                           
  shooting_set_tv96_direct(var3,SET_LATER);
  gSpecial_token=0;
  accept_cr();
}
static void ap_statement()                      
{
 int var1,var2;
 double var3;
 gSpecial_token=1;
 var2=0;
 accept(TOKENIZER_AP);                              
 var1=expr();                                      
 if ((tokenizer_token() == TOKENIZER_COMMA)||(tokenizer_token() == TOKENIZER_PERIOD)) 
 {
    tokenizer_next();                              
    var2 = expr(); 
 }
#if CAM_HAS_IRIS_DIAPHRAGM 
if ((mode_get()&MODE_MASK) != MODE_PLAY)
{
 if(var2>0)var3=(double)(var1+var2/10.0);
 else var3=var1;
 var3= (short)(96.0*(2*log(var3)/log_2));
 shooting_set_av96_direct(var3,SET_LATER);
 }
#endif
 gSpecial_token=0;
 accept_cr();
}
static void change_ev_statement()
{
 accept(TOKENIZER_CEV);
 put_off_ev96=expr();
 accept_cr();
}
static void md_get_cell_diff_statement()
{
    int var, col, row;
    accept(TOKENIZER_MD_GET_CELL_DIFF);
  col=expr();tokenizer_next();
  row=expr();tokenizer_next();
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
 
    ubasic_set_variable(var, md_get_cell_diff(col,row));
    accept_cr();
}
static void md_detect_motion_statement()
{
 int columns;
 int rows;
 int pixel_measure_mode;
 int detection_timeout;
 int measure_interval;
 int threshold;
 int draw_grid=0;
 int clipping_region_mode=0;
 int clipping_region_row1=0;
 int clipping_region_column1=0;
 int clipping_region_row2=0;
 int clipping_region_column2=0;
 int ret_var_num;
 int parameters=0;
 int pixels_step=1;
 int msecs_before_trigger=0;
    accept(TOKENIZER_MD_DETECT_MOTION);
  columns=expr();tokenizer_next();
  rows=expr();tokenizer_next();
  pixel_measure_mode=expr();tokenizer_next();
  detection_timeout=expr();tokenizer_next();
  measure_interval=expr();tokenizer_next();
  threshold=expr();tokenizer_next();
  draw_grid=expr();tokenizer_next();
    ret_var_num = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE) {
   
  } 
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE) {
    tokenizer_next();
        clipping_region_mode = expr();
    }
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
    tokenizer_next();
        clipping_region_column1 = expr();
    }
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
    tokenizer_next();
        clipping_region_row1 = expr();
    }
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
    tokenizer_next();
        clipping_region_column2 = expr();
    }
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
    tokenizer_next();
        clipping_region_row2 = expr();
    }
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
    tokenizer_next();
        parameters = expr();
    }
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
    tokenizer_next();
        pixels_step = expr();
    }
    if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) {
    tokenizer_next();
        msecs_before_trigger = expr();
    }
   
    accept_cr();
 md_init_motion_detector(
   columns, rows, pixel_measure_mode, detection_timeout, 
   measure_interval, threshold, draw_grid, ret_var_num,
   clipping_region_mode,
   clipping_region_column1, clipping_region_row1,
   clipping_region_column2, clipping_region_row2,
   parameters, pixels_step, msecs_before_trigger
 );
}
static void su_statement()               
{
 int hr,min;
 accept(TOKENIZER_SLEEP_UNTIL);
 hr = expr();tokenizer_next();
 if(hr<0)hr=0;
 if(hr>23) hr=23;
 min = expr();
 if(min<0)min=0;
 if(min>59)min=59;
 accept_cr();
 ubasic_su(hr,min);
}
static void ft_statement()                 
{
 int hr,min;
 accept(TOKENIZER_FT);
 hr = expr();tokenizer_next();
 if(hr<0)hr=0;
 if(hr>23) hr=23;
 min = expr();
 if(min<0)min=0;
 if(min>59)min=59;
 ft_time.hr=hr;
 ft_time.min=min;
 accept_cr();
}
static void so_statement()               
{
 accept(TOKENIZER_SYNC_ON);
 conf.synch_enable=1;
 accept_cr();
}
static void sf_statement()               
{
 accept(TOKENIZER_SYNC_OFF);
 conf.synch_enable=0;
 accept_cr();
}
static void load_overlay_statement()               
{
 accept(TOKENIZER_LOAD_OVERLAY);
 gOverlayNumber=expr();
 gLoadOverlay=1;
 accept_cr();
 ubasic_camera_click("shoot_half"); 
}
static void sd_statement()              
{
 int var;
 sd_repeat=0;                           
 accept(TOKENIZER_SD);                 
 var=expr();                           
 if(var<-128)var=-128;
 if(var>255)var=255;
 if(var<0)var+=256;
 pcode = (signed char)var;
 tokenizer_next();                    
 var=expr();                          
 if(var<-128)var=-128;
 if(var>255)var=255;
 if(var<0)var+=256;
 lobyte = (signed char)var;
 
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE) 
  {
   tokenizer_next();                     
   var=expr();
   if(var<-128)var=-128;
   if(var>255)var=255;
   if(var<0)var+=256;
   hibyte = (signed char)var;
   sd_repeat=1;                          
  }
                                
 accept_cr();                              
   kbd_key_press(KEY_SHOOT_HALF); 
   sertx=1; 
   block_script=1;                       
}
static void ub_statement()                 
{
 accept(TOKENIZER_UB);                                   
 accept_cr();
kbd_key_press(KEY_SHOOT_HALF); 
ub=1;  
block_script=1;                          
}
static void up_statement()                 
{
 accept(TOKENIZER_UP);
 up=1;                                     
 accept_cr();
 ubasic_camera_click("shoot_half");        
}
static void mss_statement()               
{
    int tv;
    accept(TOKENIZER_MSS);
    tv = expr();
    if(tv<1)tv=1;
    if(tv>40000)tv=40000;
    msstv96= -shooting_get_tv96_from_shutter_speed((float)tv); 
    accept_cr();
}
static void get_focus_ref_statement()
{
    int var;
    accept(TOKENIZER_GET_FR);
    var = tokenizer_variable_num();
    accept(TOKENIZER_VARIABLE);
    ubasic_set_variable(var,conf.digi_ref_dist);
    accept_cr();
}
static void blue_off_statement()
{
    accept(TOKENIZER_BLUE_OFF);
    debug_led(0);
   DEBUG_PRINTF("End of Blue off\n");
    accept_cr();
}
static void blue_on_statement()
{
    accept(TOKENIZER_BLUE_ON);
    debug_led(1);
   DEBUG_PRINTF("End of Blue on\n");
    accept_cr();
}
static void afloff_statement()
{
    accept(TOKENIZER_AFLOFF);
    ubasic_set_led(9,0,0);
    accept_cr();
}
static void aflo_statement()
{
    accept(TOKENIZER_AFLO);
    ubasic_set_led(9,1,100);
    accept_cr();
}
static void aflf_statement()
{
    int duration;
    accept(TOKENIZER_AFLF);
    duration = expr();
    if(duration<1)duration=1;
    ubasic_baflf(duration);
    accept_cr();
}
static void disable_usb_download_statement()
{
    accept(TOKENIZER_DISABLE_USB_DOWNLOAD);
    conf.remote_enable = 1;  
    accept_cr();
}
static void enable_usb_download_statement()
{
    accept(TOKENIZER_ENABLE_USB_DOWNLOAD);
    conf.remote_enable = 0;  
    accept_cr();
}
 static void play_sound_statement() 
  { 
   int to; 
   accept(TOKENIZER_PLAY_SOUND); 
   to = expr(); 
   play_sound(to); 
   accept_cr(); 
  } 
 static void beep_statement() 
  { 
   accept(TOKENIZER_BEEP);  
   play_sound(4); 
   accept_cr(); 
  } 
  static void set_resolution_statement() 
  { 
   int to; 
   accept(TOKENIZER_SET_RESOLUTION); 
   to = expr(); 
   shooting_set_prop(PROPCASE_RESOLUTION, to); 
   accept_cr(); 
  } 
    
  static void set_quality_statement() 
  { 
   int to; 
   accept(TOKENIZER_SET_QUALITY); 
   to = expr(); 
   shooting_set_prop(PROPCASE_QUALITY, to); 
   accept_cr(); 
  } 
  static void sdmcn_statement() 
  { 
   accept(TOKENIZER_SDMCN);
       conf.sdm_console=1;
   accept_cr(); 
  } 
  static void sdmcf_statement() 
  { 
   accept(TOKENIZER_SDMCF);
       conf.sdm_console=0;
   accept_cr(); 
  } 
  static void sdmcl_statement() 
  { 
   int to; 
   accept(TOKENIZER_SDMCL); 
   to = expr(); 
       if(to<1)to = 1;
       if(to>10)to=10;
       conf.num_lines=to;
   accept_cr(); 
  } 
  static void sdmcll_statement() 
  { 
   int to; 
   accept(TOKENIZER_SDMCLL); 
   to = expr();
       if(to<1)to=1;
       if(to>25)to=25; 
       conf.line_length=to;
   accept_cr(); 
  } 
  static void set_backlight_statement(void)
  {
    int val;
    accept(TOKENIZER_SET_BACKLIGHT);
    val = expr();
    if (val > 0) TurnOnBackLight();
    else TurnOffBackLight();
    accept_cr();
 }
  static void set_tbn_statement(void)
  {
    accept(TOKENIZER_TBN);
        bls=1;
        TurnOnBackLight();
    accept_cr();
  }
  static void set_tbf_statement(void)
  {
    accept(TOKENIZER_TBF);
        bls=0;
        TurnOffBackLight();
    accept_cr();
  }
  static void set_aflock_statement(void)
  {
    int val;
    accept(TOKENIZER_SET_AFLOCK);
    val = expr();
    if (val>0)
        {
         afl=1;
         DoAFLock();
        }  
    else 
        {
         afl=0;
         UnlockAF();
        }        
    accept_cr();
  }
  static void la_statement(void)
  {
    accept(TOKENIZER_LA);
        afl=1;
    DoAFLock();  
    accept_cr();
  }
  static void ua_statement(void)
 	{
 	  accept(TOKENIZER_UA);
        afl=0;
        UnlockAF();
 	  accept_cr();
 	}
    static void shutdown_statement(void)
     {
     accept(TOKENIZER_SHUT_DOWN);
     camera_shutdown_in_a_second();
     accept_cr();
     }
     static void set_bracketing_exposures_statement()
     {
  int to; 
      accept(TOKENIZER_SET_BE);
      to = expr();
      if(to<3 || to>19) to = 3;
      if((to%2) != 1) to-= 1;
      to = ((to+1)/2)+1;
      conf.bracket_type = to; 
      accept_cr();
     }
     static void shoot_movie_statement()
     {
  int to; 
      accept(TOKENIZER_SHOOT_MOVIE);
      to = expr();
      if(to<5)to=5;
      accept_cr();
      ubasic_sm(to);
     }
     static void usbup_statement()
     {
  int to; 
      accept(TOKENIZER_USBUP);
      to = expr();
      if(to<15)to=15;
      accept_cr();
      ubasic_usbupload(to);
     }
     static void movie_statement()    
     {
      accept(TOKENIZER_MOVIE);
      accept_cr();
      ubasic_mm();
     }
     static void record_statement()
     {
      accept(TOKENIZER_RECORD);
      accept_cr();
      ubasic_rm();                     
     }
     static void playback_statement()
     {
      accept(TOKENIZER_PLAYBACK);
      accept_cr();
      ubasic_pm();                     
     }
     static void musbc_statement()    
     {
      accept(TOKENIZER_MUSBC);
      accept_cr();
      ubasic_musbc();                  
     }
     static void busbc_statement()    
     {
      accept(TOKENIZER_BUSBC);
      accept_cr();
      ubasic_busbc();                  
     }
     static void set_focus_step_statement()
     {
  int to; 
      accept(TOKENIZER_SET_FS);
      to = expr();
      if(to<1 || to>5000) to = 1;
      conf.subj_dist_bracket_value = to; 
      accept_cr();
     }
     static void set_digi_focus_ref_statement()
     {
  int to; 
      accept(TOKENIZER_SET_FR);
      to = expr();
      if(to<450) to = 450;
      if(to>5000) to = 5000;
      conf.distance_setting = to;  
      accept_cr();
     }
     static void save_stack_statement()
     {
  int to; 
      accept(TOKENIZER_SS);
      to = expr();
      if(to<0) to =0;
      if(to>1) to =1;
      conf.save_auto = to;  
      accept_cr();
     }
     static void mfb_statement()
     {
      accept(TOKENIZER_MFB);
      conf.tv_bracket_value = 0;
      conf.dist_mode = 1; 
      if(conf.save_auto) script_console_add_line("  Log file will be saved  ");
      if(!shooting_can_focus()) script_console_add_line("! Cannot override focus !");
      accept_cr();
     }
     static void afb_statement()
     {
      accept(TOKENIZER_AFB);
      conf.tv_bracket_value = 0;
      conf.dist_mode = 2;
      conf.bracket_type=3;
      if(conf.save_auto) script_console_add_line("  Log file will be saved  ");
      if(!shooting_can_focus()) script_console_add_line("! Cannot override focus !");
      accept_cr();
     }
     static void db_statement()
     {
      accept(TOKENIZER_DB);
      conf.tv_bracket_value = 0;
      conf.dist_mode = 3; 
      if(conf.save_auto) script_console_add_line("  Log file will be saved  ");
      if(!shooting_can_focus()) script_console_add_line("! Cannot override focus !");
      accept_cr();
     }
     static void ssb_statement()
     {
      int to;
      accept(TOKENIZER_SSB);
      to = expr();
      if(to<0) to =0;
      if(to>12) to =12;
      conf.dist_mode = 0;
      conf.tv_bracket_value = to;
      if(conf.save_auto) script_console_add_line(" Log file will be saved  ");
      accept_cr();
     }
     static void bl_statement()
     {
      accept(TOKENIZER_BL);
      conf.bracket_type = 0; 
      conf.dist_mode = 0;
      if(shooting_get_drive_mode()==1) script_console_add_line("   ! Continuous mode !   ");
      accept_cr();
     }
     static void bd_statement()
     {
      accept(TOKENIZER_BD);
      conf.bracket_type = 1; 
      conf.dist_mode = 0;
      if(shooting_get_drive_mode()==1) script_console_add_line("   ! Continuous mode !   ");
      accept_cr();
     }
     static void ba_statement()
     {
      accept(TOKENIZER_BA);
       conf.bracket_type = 2;
       conf.dist_mode = 0;
      if(shooting_get_drive_mode()==1) script_console_add_line("   ! Continuous mode !   ");
      accept_cr();
     }
     static void scseq_statement()
     {
      accept(TOKENIZER_SCSEQ);
      kbd_key_press(KEY_SHOOT_FULL);
      accept_cr();
     }
     static void scts_statement()
     {
      accept(TOKENIZER_SCTSEQ);
      ubasic_scts();
      accept_cr();
     }
     static void ecseq_statement()
     {
      accept(TOKENIZER_ECSEQ);
      kbd_key_release(KEY_SHOOT_FULL);
      accept_cr();
     }
     static void wud_statement()
     {
      accept(TOKENIZER_WUD);
      ubasic_bracketing_done();
      accept_cr();
     }
     static void wfsp_statement()
     {
      accept(TOKENIZER_WFSP);
      ubasic_wfsp();
      accept_cr();
     }
     static void blcd_statement()
     {
      accept(TOKENIZER_BLCD);
      ubasic_blcd();
      accept_cr();
     }
     static void hdr_stack_statement()
     {
      accept(TOKENIZER_HDR_STACK);
      st_tv_exp=expr();
      tokenizer_next();
      st_tv_ev=expr();
      ubasic_hdrfs();             
      accept_cr();
     }
     static void tup_statement()
     {
      int to;
      accept(TOKENIZER_TUP);
      to=expr();
      if(to<7)to=7;
      if(to>35)to=35;
      conf.md=to;
      accept_cr();
     }
     static void uafl_statement()
     {
      accept(TOKENIZER_UAFL);
      conf.use_af_led=1;
      accept_cr();
     }
     static void upl_statement()
     {
      accept(TOKENIZER_UPL);
      conf.use_af_led=0;
      accept_cr();
     }
     static void overlay_on_statement()
     {
      accept(TOKENIZER_OVERLAY_ON);
      conf.outline_mode=1;
      accept_cr();
     }
     static void overlay_off_statement()
     {
      accept(TOKENIZER_OVERLAY_OFF);
      conf.outline_mode=0;
      edge_count=0;
      free_memory_edge_overlay();
      accept_cr();
     }
     static void reset_statement()
     {
      accept(TOKENIZER_RESET);
#if defined(CAMERA_g9)|| defined(CAMERA_a620)|| defined(CAMERA_a570)
  #if defined(CAMERA_a620)
  ((void (*)()) 0xFFD107E4)();         
      reboot(0); 
 #elif defined(CAMERA_a570)
  ((void (*)()) 0xFFDDB174)();        
      reboot(0); 
  #else
   char *fn = "A/PS.FI2";
   reboot(fn);
  #endif
#endif
      accept_cr();
     }
     static void save_focus_statement()
     {
      int num;
      accept(TOKENIZER_SAVE_FOCUS);
      conf.fpd=shooting_get_lens_to_focal_plane_width();
      if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE)      
       conf.focus_pos = expr();                                                          
      else 
       conf.focus_pos=lens_get_focus_pos_from_lense();                                   
      accept_cr();
     }
     static void save_zoom_statement()
     {
      accept(TOKENIZER_SAVE_ZOOM);
      conf.zoom_point=lens_get_zoom_point();
      accept_cr();
     }
     static void restore_focus_statement()
     {
      accept(TOKENIZER_RESTORE_FOCUS);    
      shooting_set_focus(conf.focus_pos+conf.fpd);                                            
      has_refocused=1;
      if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE)      
      {
       int var = tokenizer_variable_num();
       accept(TOKENIZER_VARIABLE);    
       ubasic_set_variable(var,conf.focus_pos);
      }
      accept_cr();
     }
     static void restore_zoom_statement()
     {
      accept(TOKENIZER_RESTORE_ZOOM);
      lens_set_zoom_point(conf.zoom_point);
      accept_cr();
     }
 
     static void af_first_statement()
     {
      accept(TOKENIZER_AF_FIRST);
      af_all=0;
      accept_cr();
     }
     static void af_all_statement()
     {
      accept(TOKENIZER_AF_ALL);
      af_all=1;
      accept_cr();
     }
     static void tl_statement()
     {
      char fn[64];            
      int fd=-1,rcnt;              
 struct stat st;         
 char *buf=NULL;         
      int paramset;           
      register const char *ptr; 
      int defaults[TL_PARAM_NUMS]; 
      int first_delay_m=0;    
      int first_delay_s=10;   
      int shoot_interval_m=0; 
      int shoot_interval_s=5; 
      int num_exposures=3;    
      int endless_mode=0;     
      int shoot_mode=0;       
      int tv_exposures=3;     
      int focus_exposures=3;  
      int bracket_type=0;     
      int exposure_mode=2;    
      int ev_steps=3;         
      int focus_mode=0;       
      int focus_step=50;      
      int focus_ref=500;      
      int blank_screen=0;     
      int shutdown=0;         
      int save_stack=0;       
      int ev96steps=0;        
      int ev96repeats=0;      
      char osdbuf[32];
      accept(TOKENIZER_TL);
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE ) 
       first_delay_m=expr();                                                      
      if (tokenizer_token() == TOKENIZER_CR)                                      
      {
       sprintf(fn,TL_PARAMS_NAME,first_delay_m);                                  
       if(stat(fn,&st)==0)                                                        
         {
          int i=0;                                                                
          fd=open(fn,O_RDONLY, 0777);                                             
          if(fd>=0)                                                               
           {
            buf=umalloc(st.st_size+1);                                            
            if(buf)
            {
        rcnt=read(fd, buf, st.st_size);                                      
        buf[rcnt] = 0;                                                       
        close(fd);                                                           
            }
            ptr=buf;                                                              
            while (ptr[0])                                                        
        {
               while (ptr[0]==' ' || ptr[0]=='\t') ++ptr;                         
               if ((strncmp("@desc", ptr, 5)==0)||(strncmp("@param", ptr, 6)==0)) 
                 {
                  while (ptr[0] && ptr[0]!='\n') ++ptr;                           
                  ++ptr;
                 }
                else if(strncmp("@default",ptr,8)==0)                             
                 {
                  ptr+=8;                                                         
                  while (ptr[0]==' ' || ptr[0]=='\t') ++ptr;                      
                  ptr+=2;                                                         
                  defaults[i]=strtol(ptr, NULL, 0);                               
                  i++;                                                            
                  while (ptr[0] && ptr[0]!='\n') ++ptr;                           
                  ++ptr;
                 }                    
               } 
               init_tl(defaults[0],defaults[1],defaults[2],defaults[3],defaults[4],defaults[5],defaults[6],defaults[7],defaults[8],defaults[9],defaults[10],defaults[11],defaults[12],defaults[13],defaults[14],defaults[15],defaults[16],defaults[17],defaults[18],defaults[19]);
             } 
           }
         else                                                                     
           {
            play_sound(4);
            accept_cr();
           }
           accept_cr();
           if(buf) ufree(buf);
        } 
   else
    {
      tokenizer_next();
  first_delay_s=expr();
       tokenizer_next();
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  shoot_interval_m=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  shoot_interval_s=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  num_exposures=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  endless_mode=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  shoot_mode=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  tv_exposures=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  focus_exposures=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  bracket_type=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  exposure_mode=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  ev_steps=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  focus_mode=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  focus_step=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  focus_ref=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  blank_screen=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  shutdown=expr();
       tokenizer_next();
      }
     
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  save_stack=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  ev96steps=expr();
       tokenizer_next();
      }
 if (tokenizer_token() != TOKENIZER_CR && tokenizer_token() != TOKENIZER_ELSE )
      {
  ev96repeats=expr();
      }
    
      accept_cr();
      init_tl(first_delay_m,first_delay_s,shoot_interval_m,shoot_interval_s,num_exposures,endless_mode,shoot_mode,tv_exposures,focus_exposures,bracket_type,exposure_mode,ev_steps,focus_mode,focus_step,focus_ref,blank_screen,shutdown,save_stack,ev96steps,ev96repeats);
 
     } 
   }
     static void usbd_statement()
     {
  int to;
      long t; 
      volatile long *usbd = (volatile long *) 0xC0721000;
      accept(TOKENIZER_USBD);
      to = expr();
       t = *usbd;
      if(to)t = t | USBD_MASK;
      else t = t & ~USBD_MASK;
      *usbd = t;
      accept_cr();
     }
static void set_max_spt_statement() 
{
   int s;
   accept(TOKENIZER_SET_MAX_SPT);
   s = expr();
   if (s > 0 && s <= MAX_SPEED) speed = s;
   accept_cr();
}
static void
statement(void)
{
  ubasic_token token;
  token = tokenizer_token();
  yield = 0;
  switch(token) 
 {
  case TOKENIZER_PRINT_SCREEN:
    print_screen_statement();
    break;
  case TOKENIZER_PRNT:
    prnt_statement();
    break;
  case TOKENIZER_SLEEP:
    sleep_statement();
    yield = 1;    
    break;
  case TOKENIZER_SLEEP_S:
    sleep_s_statement();
    yield = 1;    
    break;
  case TOKENIZER_SLEEP_M:
    sleep_m_statement();
    yield = 1;    
    break;
  case TOKENIZER_CLICK:
    click_statement();
    yield = 1;    
    break;
  case TOKENIZER_PRESS:
    press_statement();
    yield = 1;    
    break;
  case TOKENIZER_RELEASE:
    release_statement();
    yield = 1;    
    break;
  case TOKENIZER_SHOOT:
    shoot_statement();
    yield = 1;    
    break;
  case TOKENIZER_GET_USER_TV_ID:
    get_user_tv_id_statement();
    break;
  case TOKENIZER_SET_TV96_DIRECT:
    set_tv96_direct_statement();
    break;  
  case TOKENIZER_SET_TV:
    set_tv_statement();
    break;
  case TOKENIZER_SET_AV:
    set_av_statement();
    break;
  case TOKENIZER_SET_ND_FILTER: 
    set_nd_filter_statement(); 
    break; 
  case TOKENIZER_NDFI: 
    ndfi_statement(); 
    break; 
  case TOKENIZER_NDFO: 
    ndfo_statement(); 
    break; 
  case TOKENIZER_NDFF: 
    ndff_statement(); 
    break; 
    case TOKENIZER_SET_ZOOM:
    set_zoom_statement();
    break;
  case TOKENIZER_SET_ZOOM_REL:
    set_zoom_rel_statement();
    break;
  case TOKENIZER_SET_ZOOM_SPEED:
    set_zoom_speed_statement();
    break;
  case TOKENIZER_SET_FOCUS:
    set_focus_statement();
    break;
  case TOKENIZER_GET_PROP:
    get_prop_statement();
    break;
  case TOKENIZER_SET_PROP:
    set_prop_statement();
    break;
  case TOKENIZER_SET_SYNC:
    set_sync_statement();
    break;
  case TOKENIZER_GET_SYNC:
    get_sync_statement();
    break;
  case TOKENIZER_SET_ISO:
    set_iso_statement();
    break;
  case TOKENIZER_WAIT_CLICK:
    wait_click_statement();
    yield = 1;    
    break;
  case TOKENIZER_IS_KEY:
    is_key_statement();
    break;
  case TOKENIZER_WHEEL_LEFT:
    wheel_left_statement();
    break;
  case TOKENIZER_WHEEL_RIGHT:
    wheel_right_statement();
     break;
  case TOKENIZER_IF:
    if_statement();
    break;
  case TOKENIZER_ELSE:
    else_statement();
    break;
  case TOKENIZER_ENDIF:
    endif_statement();
    break;
  case TOKENIZER_GOTO:
    goto_statement();
    break;
  case TOKENIZER_GOSUB:
    gosub_statement();
    break;
  case TOKENIZER_RETURN:
    return_statement();
    break;
  case TOKENIZER_FOR:
    for_statement();
    break;
  case TOKENIZER_NEXT:
    next_statement();
    break;
  case TOKENIZER_DO:
    do_statement();
    break;
  case TOKENIZER_UNTIL:
    until_statement();
    break;
  case TOKENIZER_WHILE:
    while_statement();
    break;
  case TOKENIZER_WEND:
    wend_statement();
    break;
  case TOKENIZER_END:
    end_statement();
    break;
  case TOKENIZER_LET:
    accept(TOKENIZER_LET);
  case TOKENIZER_VARIABLE:
    let_statement();
    break;
  case TOKENIZER_REM:
    rem_statement();
    break;
  case TOKENIZER_CLS:
    cls_statement();
    break;
  case TOKENIZER_SET_RAW:
    set_raw_statement();
    break;
  case TOKENIZER_SET_SCRIPT_AUTOSTART:
    set_autostart_statement();
    break;
  case TOKENIZER_EXIT_ALT:
    exit_alt_statement();
    break;
  case TOKENIZER_SHUT_DOWN:
    shutdown_statement();
    break;
  case TOKENIZER_SET_BACKLIGHT:
    set_backlight_statement();
    break;
 case   TOKENIZER_MD_DETECT_MOTION:
  md_detect_motion_statement();
    yield = 1;    
  break;
 case  TOKENIZER_MD_GET_CELL_DIFF:
  md_get_cell_diff_statement();
  break;
  case TOKENIZER_BLUE_ON:
    blue_on_statement();
    break;
  case TOKENIZER_BLUE_OFF:
    blue_off_statement();
    break;
  case TOKENIZER_AFLF:
    aflf_statement();
    break;
  case TOKENIZER_AFLO:
    aflo_statement();
    break;
  case TOKENIZER_AFLOFF:
    afloff_statement();
    break;
  case TOKENIZER_ENABLE_USB_DOWNLOAD:
    enable_usb_download_statement();
    break;
  case TOKENIZER_DISABLE_USB_DOWNLOAD:
    disable_usb_download_statement();
    break;
  case TOKENIZER_PLAY_SOUND: 
  play_sound_statement(); 
  break;   
  case TOKENIZER_BEEP: 
  beep_statement(); 
  break;
 
  case TOKENIZER_SET_RESOLUTION: 
  set_resolution_statement(); 
  break;
 
  case TOKENIZER_SET_QUALITY: 
  set_quality_statement(); 
  break;
 
  case TOKENIZER_TBN:
   set_tbn_statement();
   break;
  case TOKENIZER_TBF:
   set_tbf_statement();
   break;
 
  case TOKENIZER_SET_BE: 
  set_bracketing_exposures_statement(); 
  break; 
  case TOKENIZER_SET_FS: 
  set_focus_step_statement(); 
  break; 
  case TOKENIZER_SET_FR: 
  set_digi_focus_ref_statement(); 
  break; 
  case TOKENIZER_GET_FR: 
  get_focus_ref_statement(); 
  break; 
  case TOKENIZER_SS: 
  save_stack_statement(); 
  break; 
  case TOKENIZER_MFB: 
  mfb_statement(); 
  break; 
  case TOKENIZER_AFB: 
  afb_statement(); 
  break; 
  case TOKENIZER_DB: 
  db_statement(); 
  break; 
  case TOKENIZER_SSB: 
  ssb_statement(); 
  break; 
  case TOKENIZER_BL: 
  bl_statement(); 
 	break; 
  case TOKENIZER_BD: 
 	bd_statement(); 
 	break; 
  case TOKENIZER_BA: 
  ba_statement(); 
  break; 
  case TOKENIZER_SCSEQ: 
  scseq_statement(); 
  break; 
  case TOKENIZER_ECSEQ: 
  ecseq_statement(); 
  break; 
  case TOKENIZER_WUD: 
  wud_statement(); 
  break; 
  case TOKENIZER_WFSP: 
  wfsp_statement(); 
  break; 
  case TOKENIZER_SCTSEQ: 
  scts_statement(); 
  break; 
  case TOKENIZER_STRING:
      ip_statement();
      break;
  case TOKENIZER_BLCD:
      blcd_statement();
      break;
  case TOKENIZER_TL:
      tl_statement();
      break;
  case TOKENIZER_USBD:
      usbd_statement();
      break;
  case TOKENIZER_SET_AFLOCK:
   set_aflock_statement();
   break;
  case TOKENIZER_LA:
   la_statement();
   break;
  case TOKENIZER_UA:
   ua_statement();
   break;
  case TOKENIZER_SDMCN:
   sdmcn_statement();
   break;
  case TOKENIZER_SDMCF:
   sdmcf_statement();
   break;
  case TOKENIZER_SDMCL:
   sdmcl_statement();
   break;
  case TOKENIZER_SDMCLL:
   sdmcll_statement();
   break;
  case TOKENIZER_SHOOT_MOVIE:
   shoot_movie_statement();
   break;
  case TOKENIZER_USBUP:
   usbup_statement();
   break;
  case TOKENIZER_MOVIE:
   movie_statement();
   break;
  case TOKENIZER_RECORD:
       record_statement();
       break;
  case TOKENIZER_PLAYBACK:
       playback_statement();
       break;
  case TOKENIZER_MUSBC:
       musbc_statement();
       break;
  case TOKENIZER_BUSBC:
       busbc_statement();
       break;
  case TOKENIZER_SET_MAX_SPT:
    set_max_spt_statement();
    break;
  case TOKENIZER_SLEEP_UNTIL:
    su_statement();
    break;
  case TOKENIZER_FT:
    ft_statement();
    break;
  case TOKENIZER_SYNC_ON:
    so_statement();
    break;
  case TOKENIZER_SYNC_OFF:
    sf_statement();
    break;
  case TOKENIZER_SD:
    sd_statement();
    break;
  case TOKENIZER_UB:
    ub_statement();
    break;
  case TOKENIZER_UP:
    up_statement();
    break;
  case TOKENIZER_GET_ZOOM:
    get_zoom_statement();
    break;
  case TOKENIZER_GET_FREE_DISK_SPACE:
    get_free_disk_space_statement();
    break;
  case TOKENIZER_MSS:                   
    mss_statement();
    break;
  case TOKENIZER_GET_FOCUS:
    get_focus_statement();
    break;
  case TOKENIZER_CAMPOS:
    get_campos_statement();
    break;
  case TOKENIZER_GET_LUMINANCE:
   get_luminance_statement();
   break;
   case TOKENIZER_SWS:
   sws_statement();
   break;
   case TOKENIZER_SWNM:
   swnm_statement();
   break;
   case TOKENIZER_SSPD:
   sspd_statement();
   break;
   case TOKENIZER_AP:
   ap_statement();
   break;
   case TOKENIZER_LINE_ONE:
   line_one_statement();
   break;
   case TOKENIZER_LINE_TWO:
   line_two_statement();
   break;
   case TOKENIZER_LINE_THREE:
   line_three_statement();
   break;
   case TOKENIZER_CEV:
   change_ev_statement();
   break;
   case TOKENIZER_HDR_STACK:
   hdr_stack_statement();
   break;
   case TOKENIZER_TUP:
   tup_statement();
   break;
   case TOKENIZER_UAFL:
   uafl_statement();
   break;
   case TOKENIZER_UPL:
   upl_statement();
   break;
   case TOKENIZER_OVERLAY_ON:
   overlay_on_statement();
   break;
   case TOKENIZER_OVERLAY_OFF:
   overlay_off_statement();
   break;
   case TOKENIZER_LOAD_OVERLAY:
   load_overlay_statement();
   break;
    case TOKENIZER_SET_LED:
    set_led_statement();
    break;
    case TOKENIZER_RESET:
    reset_statement();
    break;
    case TOKENIZER_SAVE_FOCUS:
    save_focus_statement();
    break;
    case TOKENIZER_SAVE_ZOOM:
    save_zoom_statement();
    break;
    case  TOKENIZER_RESTORE_FOCUS:
    restore_focus_statement();
    break;
    case TOKENIZER_RESTORE_ZOOM:
    restore_zoom_statement();
    break;
    case TOKENIZER_AF_FIRST:
    af_first_statement();
    break;
    case TOKENIZER_AF_ALL:
    af_all_statement();
    break;
  default:
    DEBUG_PRINTF("ubasic.c: statement(): not implemented %d\n", token);
    ended = 1;
    ubasic_error = UBASIC_E_UNK_STATEMENT;
  }
}
static void
line_statement(void) 
{
 int t1, t2, count; 
  /*line numbers have been removed */
  DEBUG_PRINTF("----------- Line number %d ---------\n", tokenizer_line_number());
  
  if (tokenizer_token() == TOKENIZER_LABEL) 
 {
#ifdef DEBUG
      tokenizer_label(string, sizeof(string));
      DEBUG_PRINTF("line_statement: label: %s\n", string );
#endif
      accept(TOKENIZER_LABEL);
      accept(TOKENIZER_CR);
      return;
  }
  
  t1 = get_tick_count();
  count = 0;
  while (1) 
    {
      statement();
      count++;
      t2 = get_tick_count();
      if (yield)          
         break;
      if (t2 != t1)       
         break;        
      if (count == speed) 
         break;
    }
  return;
}
void
ubasic_run(void)
{
  if(tokenizer_finished()) {
    DEBUG_PRINTF("uBASIC program finished\n");
    return;
  }
  line_statement();
}
int
ubasic_finished(void)
{
  return ended || tokenizer_finished();
}
void
ubasic_set_variable(int varnum, int value)
{
  if(varnum >= 0 && varnum < MAX_VARNUM) {
    variables[varnum] = value;
  }
}
int
ubasic_get_variable(int varnum)
{
  if(varnum >= 0 && varnum < MAX_VARNUM) {
    return variables[varnum];
  }
  return 0;
}
void
ubasic_end() {
  if (print_screen_d >= 0) {
    close(print_screen_d);
    print_screen_d = -1;
    print_screen_p = 0;
  }
}
