#include "lolevel.h"
#include "platform.h"
#include "core.h"
#include "keyboard.h"


extern long link_bss_start;
extern long link_bss_end;
extern void boot();

long get_sensor_width()
{
    return 6160;          // 1/2.3"
}

void startup()
{
	long *bss = &link_bss_start;
	long *ptr;
	// sanity check
	if ((long)&link_bss_end > (MEMISOSTART + MEMISOSIZE)){
		started();
		shutdown();
	}

	// initialize .bss senment
	while (bss<&link_bss_end)
		*bss++ = 0;


	boot();
}

static struct {
	int hackmode;
	int canonmode;
} modemap[] = {
	{ MODE_AUTO,               32768 },
	{ MODE_P,                  32772 },
	{ MODE_EASY,		   33311 },
	{ MODE_PORTRAIT,	   	   32781 },
	{ MODE_LANDSCAPE,	   	   32780 },
	{ MODE_NIGHT_SNAPSHOT,	   32779 },
	{ MODE_KIDS_PETS,	   	   32784 },
	{ MODE_INDOOR,		   32785 },
	{ MODE_SCN_ISO_3200,	   16413 },
	{ MODE_SCN_NIGHT,		   16398 },
	{ MODE_SCN_SUNSET,	   16402 },
	{ MODE_SCN_FOLIAGE,	   16403 },
	{ MODE_SCN_SNOW,		   16404 },
	{ MODE_SCN_BEACH,		   16405 },
	{ MODE_SCN_FIREWORK,	   16406 },
	{ MODE_SCN_AQUARIUM,	   16408 },
	{ MODE_VIDEO_STD,		    2600 },
	{ MODE_VIDEO_COMPACT,	    2602 },
};

	
#define MODESCNT (sizeof(modemap)/sizeof(modemap[0]))
	

static const int fl_tbl[] = {5800, 6420, 7060, 7700, 8340, 9950, 11550, 13160, 14750, 17150, 19570, 22760, 26750, 30750, 34800};
#define NUM_FL (sizeof(fl_tbl)/sizeof(fl_tbl[0]))
#define CF_EFL 60345

const int zoom_points = NUM_FL;

static const int fov_tbl[]={53,48,44,41,38,32,28,25,22,19,17,14,12,11,9};
static const int mav_tbl[] = {307,307,325,347,367,389,413,443,470,494,518,542,566,590,614};  // min Av
static const int xav_tbl[] = {495,492,510,532,552,574,598,628,655,679,703,727,751,775,799};  // max Av

int get_mav(int zp)
{
 return mav_tbl[zp];
}

int get_xav(int zp)
{
 return xav_tbl[zp];
}

int get_fov(int zp)
{
 if (zp<0) return fov_tbl[0];
 else if (zp>NUM_FL-1) return fov_tbl[NUM_FL-1];
 else return fov_tbl[zp];
}

int get_effective_focal_length(int zp) {
    return (CF_EFL*get_focal_length(zp))/10000;
}

int get_focal_length(int zp) {
    if (zp<0) return fl_tbl[0];
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1];
    else return fl_tbl[zp];
}

int get_zoom_x(int zp) {
    if (zp<1) return 10;
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1]*10/fl_tbl[0];
    else return fl_tbl[zp]*10/fl_tbl[0];
}


int mode_get() {
    int mode, i, t=0xFF;

    mode  = (physw_status[1] & 0x4)?MODE_REC:MODE_PLAY;
    
    _GetPropertyCase(PROPCASE_SHOOTING_MODE, &t, 4);
    for (i=0; i<MODESCNT; ++i) {
	if (modemap[i].canonmode == t) {
	    return (mode | (modemap[i].hackmode & MODE_SHOOTING_MASK));
	}
    }
    return (mode);
}

long get_vbatt_min()
{
    return 2300;
}

long get_vbatt_max()
{
    return 2550;
}
