#include "lolevel.h"
#include "platform.h"
#include "core.h"
#include "keyboard.h"


extern long link_bss_start;
extern long link_bss_end;
extern void boot();

long get_sensor_width()
{
    return 6160;          // 1/2.3"
}

void startup()
{
	long *bss = &link_bss_start;
	long *ptr;
	
	// sanity check
	if ((long)&link_bss_end > (MEMISOSTART + MEMISOSIZE)){
		started();
		shutdown();
	}

	// initialize .bss senment
	while (bss<&link_bss_end)
		*bss++ = 0;

	boot();
}

static struct {
	int hackmode;
	int canonmode;
} modemap[] = 
{
	{ MODE_AUTO,               32768  },
	{ MODE_P,                  32772  },
	{ MODE_VIDEO_STD,          2598   }, 
	{ MODE_SCN_PORTRAIT,       16397  },
	{ MODE_SCN_NIGHT_SCENE,    16395  },
	{ MODE_SCN_KIDS_PETS,      16400  },
	{ MODE_SCN_INDOOR,         16401  },
	{ MODE_SCN_SUNSET,         16402  },
	{ MODE_SCN_FIREWORK,       16406  },
      { MODE_SCN_LONG_SHUTTER,   16390  },
      { MODE_SCN_BEACH,          16405  },
      { MODE_SCN_AQUARIUM,       16408  },
      { MODE_SCN_FOLIAGE,        16403  },
      { MODE_SCN_SNOW,           16404  },
      { MODE_SUPER_MACRO,        16905  },
 };

#define MODESCNT (sizeof(modemap)/sizeof(modemap[0]))

static const int fl_tbl[] = {6600, 8060, 9840, 12050, 14800, 17550, 21600};
#define NUM_FL (sizeof(fl_tbl)/sizeof(fl_tbl[0]))
#define CF_EFL 56481L

const int zoom_points = NUM_FL;

static const int fov_tbl[]={50,42,35,29,24,20,16};
static const int mav_tbl[] = {307,307,325,347,367,389,413};  // min Av
static const int xav_tbl[] = {495,492,510,532,552,574,598};  // max Av

int get_mav(int zp)
{
 return mav_tbl[zp];
}

int get_xav(int zp)
{
 return xav_tbl[zp];
}

int get_fov(int zp)
{
 if (zp<0) return fov_tbl[0];
 else if (zp>NUM_FL-1) return fov_tbl[NUM_FL-1];
 else return fov_tbl[zp];
}


int get_effective_focal_length(int zp) {
    return (CF_EFL*get_focal_length(zp))/10000;
}

int get_focal_length(int zp) {
    if (zp<0) return fl_tbl[0];
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1];
    else return fl_tbl[zp];
}

int get_zoom_x(int zp) {
    if (zp<1) return 10;
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1]*10/fl_tbl[0];
    else return fl_tbl[zp]*10/fl_tbl[0];
}

/*                  NOT IN CHDK BUILD  check if correct*/

int mode_get() 
{
    int mode, i, t=0xFF;
    mode  = (playrec_mode==2 || playrec_mode==5)?MODE_REC:MODE_PLAY;

    _GetPropertyCase(PROPCASE_SHOOTING_MODE, &t, 4);
    for (i=0; i<MODESCNT; ++i) 
    {
	if (modemap[i].canonmode == t) 
      {
	    return (mode | (modemap[i].hackmode & MODE_SHOOTING_MASK));
	}
    }
    return (mode);
}

long get_vbatt_min()
{
	return 2300;
}

long get_vbatt_max()
{
	return 2550;
}

