#include "camera.h"
#include "lolevel.h"
#include "platform.h"
#include "conf.h"
#include "math.h"

#if defined(MOVIE_SYNCH)
volatile long *printled=(void*)0xc0220084;
#endif

int  apex2us(int apex_tv){
#if CAM_EXT_TV_RANGE
/*
 Extended Tv, by barberofcivil, http://chdk.setepontos.com/index.php/topic,4392.0.html
 Explanation by reyalP:
 In every port, the original shutter overrides (as opposed to super long exposure) worked by
 setting the propcase values at some point after auto-exposure has happened (except in manual
 modes, where the manual control propcases may be used instead). The Canon code previously took
 these values unchanged for short exposures. In newer cameras, like on the SX10 / SD980, the value
 is changed, apparently some time after it has been retrieved from the propcase. We know this is
 the case, because the propcase value itself doesn't get clamped to the allowed range (if it did,
 barberofcivil's code wouldn't work).
*/
	short tv;
	tv = shooting_get_tv96();
	if (tv<-576 || tv!=apex_tv) return 1000000.0*pow(2.0, -tv/96.0);
	else return _apex2us(apex_tv);
#else
	return 0;
#endif
}

#if defined CAM_DRYOS
 		#define _U      0x01    /*upper */ 
 		#define _L      0x02    /*lower */ 
 		#define _D      0x04    /*digit */ 
 		#define _C      0x08    /*cntrl */ 
 		#define _P      0x10    /*punct */ 
 		#define _S      0x20    /*white space (space/lf/tab) */ 
 		#define _X      0x40    /*hex digit */ 
 		#define _SP     0x80    /*hard space (0x20) */ 
 		unsigned char _ctype[] = { 
 		_C,_C,_C,_C,_C,_C,_C,_C,                        /*0-7 */ 
 		_C,_C|_S,_C|_S,_C|_S,_C|_S,_C|_S,_C,_C,         /*8-15 */ 
 		_C,_C,_C,_C,_C,_C,_C,_C,                        /*16-23 */ 
 		_C,_C,_C,_C,_C,_C,_C,_C,                        /*24-31 */ 
 		_S|_SP,_P,_P,_P,_P,_P,_P,_P,                    /*32-39 */ 
 		_P,_P,_P,_P,_P,_P,_P,_P,                        /*40-47 */ 
 		_D,_D,_D,_D,_D,_D,_D,_D,                        /*48-55 */ 
 		_D,_D,_P,_P,_P,_P,_P,_P,                        /*56-63 */ 
 		_P,_U|_X,_U|_X,_U|_X,_U|_X,_U|_X,_U|_X,_U,      /*64-71 */ 
 		_U,_U,_U,_U,_U,_U,_U,_U,                        /*72-79 */ 
 		_U,_U,_U,_U,_U,_U,_U,_U,                        /*80-87 */ 
 		_U,_U,_U,_P,_P,_P,_P,_P,                        /*88-95 */ 
 		_P,_L|_X,_L|_X,_L|_X,_L|_X,_L|_X,_L|_X,_L,      /*96-103 */ 
 		_L,_L,_L,_L,_L,_L,_L,_L,                        /*104-111 */ 
 		_L,_L,_L,_L,_L,_L,_L,_L,                        /*112-119 */ 
 		_L,_L,_L,_P,_P,_P,_P,_C,                        /*120-127 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                /*128-143 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                /*144-159 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                /*160-175 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                /*176-191 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                /*192-207 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                /*208-223 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                /*224-239 */ 
 		0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};               /*240-255 */  
#endif

#if defined(CAMERA_a1000)
#define RECORD 0x1001        // "PressRecButton"
#define PLAYBACK 0x1003      // "PressPBButton"
#define MOVIE 0x105B         // "ModeDialToMovie"

#elif !CAM_HAS_PLAYBACK_PB 
#define RECORD 0x1065       // search "ModeDialToRec"
#define PLAYBACK 0x1061     // search "ModeDialToPlay"
#define MOVIE 0x105B        // search "ModeDialToMovie"
// #endif
#elif CAM_HAS_PLAYBACK_PB 
#define RECORD 0x1001    // search "PressRecButton"
#define PLAYBACK 0x1003  // search "PressPBButton"
#define MOVIE 0x9A5      // search "PressMovieButton"
#endif


#if CAM_MULTIPART
extern int cp;  // current partition '1' or '2'
#endif


void wr_kbd_read_keys_r2(void *status) 
{
  _kbd_read_keys_r2(status);
}

void msleep(long msec)
{
    _SleepTask(msec);
}

#ifndef CAM_DRYOS

void task_lock()
{
    _taskLock();
}

void task_unlock()
{
    _taskUnlock();
}

#endif

long get_property_case(long id, void *buf, long bufsize)
{
    return _GetPropertyCase(id, buf, bufsize);
}

long set_property_case(long id, void *buf, long bufsize)
{
    return _SetPropertyCase(id, buf, bufsize);
}

void remount_filesystem()
{
    _Unmount_FileSystem();
    _Mount_FileSystem();
}



// void vid_bitmap_refresh()
void __attribute__((weak)) vid_bitmap_refresh()   // CS723, EWAVR
{
    _RefreshPhysicalScreen(1);
}


long get_parameter_data(long id, void *buf, long bufsize)
{
    return _GetParameterData(id|0x4000, buf, bufsize);
}

long set_parameter_data(long id, void *buf, long bufsize)
{
    return _SetParameterData(id|0x4000, buf, bufsize);
}

long lens_get_zoom_pos()
{
    return _GetZoomLensCurrentPosition();
}

void lens_set_zoom_pos(long newpos)
{
}

long lens_get_zoom_point()
{
    return _GetZoomLensCurrentPoint();
}

int is_zoom_busy()
{
  return zoom_busy;
}



void lens_set_zoom_point(long newpt)
{
   if (newpt < 0) 
    {
     newpt = 0;
    } 
   else if (newpt >= zoom_points) 
    {
     newpt = zoom_points-1;
    }
    _MoveZoomLensWithPoint((short*)&newpt);
#if !defined(CAMERA_s95)
    while (zoom_busy) ;
#else
	int startTime = get_tick_count();
	while (get_tick_count() < (startTime + 2000)) 
       {
	  if (!zoom_busy)
	  break;
	 }
#endif

}



void lens_set_zoom_speed(long newspd)
{
    if (newspd < 5) {
        newspd = 5;
    } else if (newspd > 100) {
        newspd = 100;
    }
    _SetZoomActuatorSpeedPercent((short*)&newspd);
}


void lens_set_focus_pos(long newpos)
{  
  if(shooting_can_focus())
   {
     _MoveFocusLensToDistance((short*)&newpos);
    while ((shooting_is_flash_ready()!=1) || (focus_busy));
     newpos = _GetFocusLensSubjectDistance();
    _SetPropertyCase(PROPCASE_SUBJECT_DIST1, &newpos, sizeof(newpos));
    _SetPropertyCase(PROPCASE_SUBJECT_DIST2, &newpos, sizeof(newpos));
  }
}


long stat_get_vbatt()
{
    return _VbattGet();
}


long get_tick_count()
{
   long t;
#if !CAM_DRYOS
    _GetSystemTime(&t);
    return t;
#else 
    return (int)_GetSystemTime(&t); 
#endif 
}


int open (const char *name, int flags, int mode )
{
#ifdef CAM_DRYOS_2_3_R39 
    if(name[0]!='A')return -1; 
#endif 
    return _Open(name, flags, mode);
}
int close (int fd)
{
    return _Close(fd);
}
int write (int fd, void *buffer, long nbytes)
{
    return _Write(fd, buffer, nbytes);
}
int read (int fd, void *buffer, long nbytes)
{
    return _Read(fd, buffer, nbytes);
}
int lseek (int fd, long offset, int whence)
{
    return _lseek(fd, offset, whence); /*yes, it's lower-case lseek here since Lseek calls just lseek (A610) */
}
long mkdir(const char *dirname)
{
    return _mkdir(dirname);
}

int remove(const char *name) {
    return _Remove(name);
}

long DeleteFile_Fut(const char *name)
{
 // return _DeleteFile_Fut(name);
  return 0;
}


int isdigit(int c) {
#if !CAM_DRYOS
    return _isdigit(c);
#else 
    return _ctype[c]&_D; 
#endif 
}

int isspace(int c) {
#if !CAM_DRYOS
    return _isspace(c);
#else 
    return _ctype[c]&_S; 
#endif 
}

int isalpha(int c) {
#if !CAM_DRYOS
    return _isalpha(c);
#else 
    return _ctype[c]&(_U|_L); 
#endif 
}

int isupper(int c) {
#if !CAM_DRYOS
    return _isupper(c);
#else 
    return _ctype[c]&_U; 
#endif 
}


long strlen(const char *s) {
    return _strlen(s);
}

int strcmp(const char *s1, const char *s2) {
    return _strcmp(s1, s2);
}

int strncmp(const char *s1, const char *s2, long n) {
    return _strncmp(s1, s2, n);
}

char *strchr(const char *s, int c) {
    return _strchr(s, c);
}

char *strcpy(char *dest, const char *src) {
    return _strcpy(dest, src);
}

char *strncpy(char *dest, const char *src, long n) {
    return _strncpy(dest, src, n);
}

char *strcat(char *dest, const char *app) {
    return _strcat(dest, app);
}

char *strrchr(const char *s, int c) {
    return _strrchr(s, c);
}

long strtol(const char *nptr, char **endptr, int base) {
    return _strtol(nptr, endptr, base);
}

char *strpbrk(const char *s, const char *accept) {
#if !CAM_DRYOS
    return _strpbrk(s, accept);
#else 
 	const char *sc1,*sc2; 
 	 
 		    for( sc1 = s; *sc1 != '\0'; ++sc1) { 
 		     for( sc2 = accept; *sc2 != '\0'; ++sc2) { 
 		      if (*sc1 == *sc2) return (char *) sc1; 
 		     } 
 		    } 
 		return (void*)0; 
#endif 
}

long sprintf(char *s, const char *st, ...)
{
    long res;
    __builtin_va_list va;
    __builtin_va_start(va, st);
    res = _vsprintf(s, st, va);
    __builtin_va_end(va);
    return res;
}

unsigned long time(unsigned long *timer) {
    return _time(timer);
}

int utime(char *file, void *newTimes) {
#if !CAM_DRYOS
  return _utime(file, newTimes);
#else
 int res=0;
 int fd;
 fd = _open(file, 0, 0);

#ifdef CAM_DRYOS_2_3_R39
   if (fd>=0) {
       _close(fd);
       res=_SetFileTimeStamp(file, ((int*)newTimes)[0] , ((int*)newTimes)[1]);
   }
#else
     if (fd>=0) {
      res=_SetFileTimeStamp(fd, ((int*)newTimes)[0] , ((int*)newTimes)[1]);
      _close(fd);
     }
     // return value compatibe with utime: ok=0 fail=-1
#endif
  return (res)?0:-1;
#endif
}

void *localtime(const unsigned long *_tod) {
#if !CAM_DRYOS
    return _localtime(_tod);
#else 
 #warning  do something with this!  - sizeof(x[]) must be >= sizeof(struct tm) 
    static int x[9]; 
    return _LocalTime(_tod, &x);    
#endif 
}

double _log(double x) {
    return __log(x);
}

double _log10(double x) {
    return __log10(x);
}

double _pow(double x, double y) {
    return __pow(x, y);
}

double _sqrt(double x) {
    return __sqrt(x);
}


#if defined(CAM_USES_EXMEM)
 void *mem_exmem_heap;

 		// I set this up to 16 mb and it still booted... 
 		#define EXMEM_HEAP_SIZE (1024*1024*8) 
 		// these aren't currently needed elsewhere 
 
 		/* 
 		void * exmem_alloc(unsigned pool_id, unsigned size) 
 		{ 
 		        return _exmem_alloc(pool_id,size,0); 
 		} 
 		 
 		void exmem_free(unsigned pool_id) 
 		{ 
 		        _exmem_free(pool_id); 
 		} 
 		*/ 
		 
            static  void *exmem_heap;	 
 		void *suba_init(void *heap, unsigned size, unsigned rst, unsigned mincell); 
 		void *suba_alloc(void *heap, unsigned size, unsigned zero); 
 		int suba_free(void *heap, void *p); 
 		 
 		void exmem_malloc_init() 
            { 
 		        // pool zero is EXMEM_RAMDISK on d10 
 		        void *mem = _exmem_alloc(0,EXMEM_HEAP_SIZE,0);
 		        if(mem) 
                     { 
                      mem_exmem_heap = mem;
 		          exmem_heap = suba_init(mem,EXMEM_HEAP_SIZE,1,1024); 
 		         } 
 		} 
 		 
 		void *malloc(unsigned size) 
            { 
 		        if(exmem_heap) 
 		             return suba_alloc(exmem_heap,size,0); 
 		        else 
 		             return _malloc(size); 
 		}
 
 		void free(void *p) 
            { 
 		        if(exmem_heap) 
 		             suba_free(exmem_heap,p); 
 		        else 
                        {
 		             _free(p); 
                        }
                    
 		} 
 		// regular malloc 
#else 
     

void *malloc(long size) 
{
    return _malloc(size);
}

void free(void *p) 
{
    return _free(p);
}

#endif

void *memcpy(void *dest, const void *src, long n) {
    return _memcpy(dest, src, n);
}

void *memset(void *s, int c, int n) {
    return _memset(s, c, n);
}

int memcmp(const void *s1, const void *s2, long n) {
    return _memcmp(s1, s2, n);
}

int rand(void) {
    return _rand();
}

int rename(const char *oldname, const char *newname) 
{
    return _rename(oldname,newname);
}

void *srand(unsigned int seed) {
    return _srand(seed);
}

void qsort(void *__base, int __nelem, int __size, int (*__cmp)(const void *__e1, const void *__e2)) {
    _qsort(__base, __nelem, __size, __cmp);
}

void *opendir(const char* name) {
    return _opendir(name);
}

void* readdir(void *d) 
{
# if !CAM_DRYOS
    return _readdir(d);
#else
// for DRYOS cameras  A650, A720  do something with this!  - sizeof(de[]) must be >= sizeof(struct dirent): 'static char de[40];'
 static char de[80];
  _ReadFastDir(d, &de);
  return de[0]? &de : (void*)0;
#endif
}

int closedir(void *d) {
    return _closedir(d);
}

void rewinddir(void *d) {
    return _rewinddir(d);
}

long Fopen_Fut(const char *filename, const char *mode){
 return _Fopen_Fut(filename,mode);
}

long Fclose_Fut(long file){
 return _Fclose_Fut(file);
}

long Fread_Fut(void *buf, long elsize, long count, long f){
 return _Fread_Fut(buf,  elsize,  count, f);
}

long Fwrite_Fut(const void *buf, long elsize, long count, long f){
  return _Fwrite_Fut(buf, elsize, count, f);
}

long Fseek_Fut(long file, long offset, long whence){
 return  _Fseek_Fut(file, offset, whence);
}

long Fflush_Fut(long file) {
 return _Fflush_Fut(file);
}

long RenameFile_Fut(const char *oldname, const char *newname) 
{
 // return _RenameFile_Fut(oldname, newname);
 return 0;
}

int stat(char *name, void *pStat) {
    return _stat(name, pStat);
}

void *umalloc(long size) {
    return _AllocateUncacheableMemory(size);
}

void ufree(void *p) {
    return _FreeUncacheableMemory(p);
}

static int shutdown_disabled = 0;
void disable_shutdown() {
    if (!shutdown_disabled) {
        _LockMainPower();
        shutdown_disabled = 1;
    }
}

void enable_shutdown() {
    if (shutdown_disabled) {
        _UnlockMainPower();
        shutdown_disabled = 0;
    }
}

void camera_shutdown_in_a_second(void)
{
 int i;
 _SetAutoShutdownTime(1); //1 sec
 for (i=0;i<200;i++) _UnlockMainPower(); //set power unlock counter to 200 or more, because every keyboard function call try to lock power again ( if "Disable LCD off" menu is "alt" or "script"). 
}

int get_sd_status()
{
 return shutdown_disabled;
}

void TurnOnBackLight(void)
{
 _TurnOnBackLight();
}
	
void TurnOffBackLight(void)
{
 _TurnOffBackLight();
}

void DoAFLock(void) 
{ 
 _DoAFLock(); 
} 
	 
void UnlockAF(void) 
{ 
 _UnlockAF(); 
} 

unsigned int GetFreeCardSpaceKb(void)
{
 return (_GetDrive_FreeClusters(0)*(_GetDrive_ClusterSize(0)>>9))>>1;
}


unsigned int GetTotalCardSpaceKb(void)
{
 return (_GetDrive_TotalClusters(0)*(_GetDrive_ClusterSize(0)>>9))>>1;
}


unsigned int GetJpgCount(void)
{
 return strtol(camera_jpeg_count_str(),((void*)0),0);
}

unsigned int GetRawCount(void)
{
 return GetFreeCardSpaceKb()/((hook_raw_size() / 1024)+GetFreeCardSpaceKb()/GetJpgCount());
}

void record_mode()
{
#if defined(CAMERA_a480)|| defined(CAMERA_a2000)|| defined(CAMERA_s90)|| defined(CAMERA_s95) || defined(CAMERA_ixus90) || defined(CAMERA_ixus95)|| defined(CAMERA_ixus105)
PostLogicalEventToUI(4097,0);
 msleep(300);
PostLogicalEventToUI(4098,0);
#else
 #if !defined(CAMERA_a1000)
 PostLogicalEventForNotPowerType((int)PLAYBACK);
 #endif
 msleep(10);
 PostLogicalEventForNotPowerType((int)RECORD);
 msleep(10);
#endif
}

void playback_mode()
{
#if defined(CAMERA_a480)|| defined(CAMERA_a2000)|| defined(CAMERA_s90)|| defined(CAMERA_s95) || defined(CAMERA_ixus90)|| defined(CAMERA_ixus95)|| defined(CAMERA_ixus105)
 PostLogicalEventToUI(4099,0);
 PostLogicalEventToUI(4100,0);
#else
 PostLogicalEventForNotPowerType((int)PLAYBACK);
#endif
}

void movie_mode()
{
#if defined(CAMERA_a480)|| defined(CAMERA_a2000)|| defined(CAMERA_s90)|| defined(CAMERA_s95)|| defined(CAMERA_ixus105)
shooting_set_mode_canon(2598);
#elif defined CAM_MOVIE_PLE 
int old=mode_magic;
 PostLogicalEventForNotPowerType((int)MOVIE);
 msleep(10);
 mode_magic=old;
#endif
}


void RefreshUSBMode(void)
{
  _RefreshUSBMode();                                                     // and refresh mode
}


void PostLogicalEventForNotPowerType(int mode)
{
 _PostLogicalEventForNotPowerType(mode,0);
}

void PostLogicalEventToUI(unsigned id, unsigned x) 
{
 _PostLogicalEventToUI(id,x);
}

#if CAM_MULTIPART

#define SECTOR_SIZE 512
char *mbr_buf;
static unsigned long drive_sectors;

int mbr_read(char* mbr_sector, unsigned long drive_total_sectors, unsigned long *part_start_sector,  unsigned long *part_length){
// return value: 1 - success, 0 - fail
 
 int offset=0x10; // points to partition #2
 int valid;
  
 if ((mbr_sector[0x1FE]!=0x55) || (mbr_sector[0x1FF]!=0xAA)) return 0; //signature check 

 mbr_buf=_AllocateUncacheableMemory(SECTOR_SIZE);
 _memcpy(mbr_buf,mbr_sector,SECTOR_SIZE);
 drive_sectors=drive_total_sectors;

 while(offset>=0) {
 
  *part_start_sector=(*(unsigned short*)(mbr_sector+offset+0x1C8)<<16) | *(unsigned short*)(mbr_sector+offset+0x1C6); 
  *part_length=(*(unsigned short*)(mbr_sector+offset+0x1CC)<<16) | *(unsigned short*)(mbr_sector+offset+0x1CA); 

  valid= (*part_start_sector) && (*part_length) &&
         (*part_start_sector<=drive_total_sectors) && 
         (*part_start_sector+*part_length<=drive_total_sectors) &&
         ((mbr_sector[offset+0x1BE]==0) || (mbr_sector[offset+0x1BE]==0x80)); //status: 0x80 (active) or 0 (non-active)

  if (valid && ((mbr_sector[0x1C2+offset]==0x0B) || (mbr_sector[0x1C2+offset]==0x0C))) break;   //FAT32 secondary partition

  offset-=0x10;

 }

 return valid;
}

int mbr_read_dryos(unsigned long drive_total_sectors, char* mbr_sector ){
// Called only in DRYOS
 mbr_buf=_AllocateUncacheableMemory(SECTOR_SIZE);
 _memcpy(mbr_buf,mbr_sector,SECTOR_SIZE);
 drive_sectors=drive_total_sectors;
 return drive_total_sectors;
}


int get_part_count(void)
{
 unsigned long part_start_sector, part_length;
 char part_status, part_type;
 int i;
 int count=0;
 for (i=0; i<=1;i++){
  part_start_sector=(*(unsigned short*)(mbr_buf+i*16+0x1C8)<<16) | *(unsigned short*)(mbr_buf+i*16+0x1C6); 
  part_length=(*(unsigned short*)(mbr_buf+i*16+0x1CC)<<16) | *(unsigned short*)(mbr_buf+i*16+0x1CA); 
  part_status=mbr_buf[i*16+0x1BE];
  part_type=mbr_buf[0x1C2+i*16];
  if ( part_start_sector && part_length && part_type && ((part_status==0) || (part_status==0x80)) ) count++;
 }
 return count;
}


void swap_partitions(void)
{
 int i;
 char c;
 for(i=0;i<16;i++){
  c=mbr_buf[i+0x1BE];
  mbr_buf[i+0x1BE]=mbr_buf[i+0x1CE];
  mbr_buf[i+0x1CE]=c;
 }
 _WriteSDCard(0,0,1,mbr_buf);
 play_sound(4);
 if(cp==1)cp=2;
 else cp=1;
}

#endif

#if defined(MOVIE_SYNCH)
int movie_led(int x)
{
 printled[0]^=1<<1;                // toggles bit '1'
 return x;                                //preserve R0 if called from assembler
}
#endif

#if CAM_CAN_MUTE_MICROPHONE
int mute_on_zoom(int x)
{
 static int old_busy=0;
 int busy=zoom_busy||focus_busy;
 if (conf.mute_on_zoom && (old_busy!=busy)) 
 { 
  if (busy) _TurnOffMic(); else _TurnOnMic();
  old_busy=busy;
 }
 return x; //preserve R0 if called from assembler
}
#endif

long __attribute__((weak)) get_jogdial_direction(void)
{
 return 0;
}

	void play_sound(short sound) 
 		{ 
 		    short tmp = 0xC211; // shutup compiler
 		    switch (sound) 
 		{ 
 		  case 0: 
 		    tmp = 0x2001; //startup sound 
 		    break; 
 		  case 1: 
 		    tmp = 0x2002; //shutter sound 
 		    break; 
 		  case 2: 
 		    tmp = 0x2003; //button press sound 
 		    break; 
 		  case 3: 
 		    tmp = 0x2004; //self-timer sound 
 		    break; 
 		  case 4: 
 		   tmp = 0xC211; //short beep 
 		    break; 
 		  case 5: 
 		   tmp = 50000; //AF confirmation  
 		    break; 
 		  case 6: 
 		   tmp = 0xC507; //error beep imo 
 		    break; 
 		  case 7: 
 		   tmp = 0x400D; //LONG ERROR BEEP CONTINIUOUS- warning, cannot be stopped (yet) 
 		    break; 
 		 } 
 		    _PT_PlaySound(tmp, 0); 
 		} 
 #if defined(PTP_ENABLED)		 
 int add_ptp_handler(int opcode, ptp_handler handler, int unknown)
{
  return _add_ptp_handler(opcode,handler,unknown);
}

void ExitTask(void)
{
 _ExitTask();
}

int __attribute__((weak)) switch_mode_usb(int mode)
{
  return 0;
}
#endif

// use _GetFocusLensSubjectDistance for this on dryos, vx functions are basically equivlent
// not used in CHDK currently for either OS

#ifdef CAM_DRYOS
long __attribute__((weak)) _GetCurrentTargetDistance()
{
	return _GetFocusLensSubjectDistance();
}
#endif
