#include "lolevel.h"
#include "platform.h"
#include "core.h"
#include "keyboard.h"


extern long link_bss_start;
extern long link_bss_end;
extern void boot();

long get_sensor_width()
{
    return 6160;          // 1/2.3"
}

void startup()
{
    long *bss = &link_bss_start;
    long *ptr;

   // sanity check (pointless with automemiso)
    if ((long)&link_bss_end > (MEMISOSTART + MEMISOSIZE)){
	started();
	shutdown();
    }

    // initialize .bss senment
    while (bss<&link_bss_end)
	*bss++ = 0;

    boot();
}


// TODO setting the DP button as a shortcut to movie in canon menu
// gives a value of (current mode)+1024 while movie is recording, unless
// already in movie mode
static struct {
	int hackmode;
	int canonmode;
} modemap[] = {
    { MODE_AUTO,               32768 },
    { MODE_M,                  32769 },
    { MODE_P,                  32772 },
    { MODE_PORTRAIT,           0x800D },
    { MODE_NIGHT_SNAPSHOT,     0x800B },
    { MODE_SCN_KIDS_PETS,      0x8010 },
    { MODE_SCN_INDOOR,         0x8011 },
    { MODE_SCN_SUNSET,         0x4012 },
    { MODE_SCN_FOLIAGE,        0x4013 },
    { MODE_SCN_SNOW,           0x4014 },
    { MODE_SCN_BEACH,          0x4015 },
    { MODE_SCN_FIREWORK,       0x4016 },
    { MODE_SCN_NIGHT,          0x4006 }, //AKA Long Shutter
    { MODE_SCN_UNDERWATER,     0x4017 },
    { MODE_SCN_AQUARIUM,       0x4018 },
    { MODE_SCN_ISO_3200,       0x401D },
    { MODE_DIGITAL_MACRO,      0x4208 },
    { MODE_SCN_COLOR_ACCENT,   0x421B },
    { MODE_SCN_COLOR_SWAP,     0x421C },
    { MODE_STITCH,             0x420A },
//    { MODE_QUICK,              33312 },

    { MODE_VIDEO_STD,          0xA29  },
    { MODE_VIDEO_COLOR_ACCENT, 0xA27  },
    { MODE_VIDEO_COLOR_SWAP,   0xA28  },
};


#define MODESCNT (sizeof(modemap)/sizeof(modemap[0]))

static const int fl_tbl[] = {5900,7170,8740,10550,12490,14780,17900};
#define NUM_FL (sizeof(fl_tbl)/sizeof(fl_tbl[0]))
#define CF_EFL 55932 

const int zoom_points = NUM_FL;

static const int fov_tbl[]={55,46,39,33,28,24,20};
static const int mav_tbl[] = {329,350,376,404,431,460,490};  // min Av
static const int xav_tbl[] = {604,625,651,679,706,735,765};  // max Av

int get_mav(int zp)
{
 return mav_tbl[zp];
}

int get_xav(int zp)
{
 return xav_tbl[zp];
}

int get_fov(int zp)
{
 if (zp<0) return fov_tbl[0];
 else if (zp>NUM_FL-1) return fov_tbl[NUM_FL-1];
 else return fov_tbl[zp];
}

int get_effective_focal_length(int zp) {
    return (CF_EFL*get_focal_length(zp))/10000;
}

int get_focal_length(int zp) {
    if (zp<0) return fl_tbl[0];
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1];
    else return fl_tbl[zp];
}

int get_zoom_x(int zp) {
    if (zp<1) return 10;
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1]*10/fl_tbl[0];
    else return fl_tbl[zp]*10/fl_tbl[0];
}

/*
physw_ bit OK
*/
int mode_get() {
    int mode, i, t=0xFF;
    mode  = (physw_status[1] & 0x00000001)?MODE_REC:MODE_PLAY;

    _GetPropertyCase(PROPCASE_SHOOTING_MODE, &t, 4);
    //draw_txt_string(20, 11, osd_buf, conf.osd_color);
    for (i=0; i<MODESCNT; ++i) {
	if (modemap[i].canonmode == t) {
	    return (mode | (modemap[i].hackmode & MODE_SHOOTING_MASK));
	}
    }
    return (mode);
}

//VERIFY_SD780 min observed
long get_vbatt_min()
{
    return 3375; // hnikesch on forum
}

//VERIFY_SD780 max observed
long get_vbatt_max()
{
    return 4125; // fresh off charger slightly greater
}


