#include "lolevel.h"
#include "platform.h"
#include "core.h"
#include "conf.h"

static long *nrflag = (long*)0x5b08; // ffd20124: (00005b08)

#define RAWDATA_AVAILABLE (1)
#define RAWDATA_SAVED (2)

// some cams use different values
// note: many cams ROM also accepts a value of 3, meaning unknown
#ifndef NR_ON
#define NR_ON (2)
#endif
#ifndef NR_OFF
#define NR_OFF (1)
#endif

static long raw_save_stage;
volatile long shutter_open_time=0; // for DNG EXIF creation

void __attribute__((naked,noinline)) capt_seq_hook_raw_here()
{
 asm volatile("STMFD   SP!, {R0-R12,LR}\n");

    raw_save_stage = RAWDATA_AVAILABLE;
    core_rawdata_available();
    while (raw_save_stage != RAWDATA_SAVED){
	_SleepTask(10);
    }

 asm volatile("LDMFD   SP!, {R0-R12,PC}\n");
}


void hook_raw_save_complete()
{
    raw_save_stage = RAWDATA_SAVED;
}

void __attribute__((naked,noinline)) capt_seq_hook_set_nr()
{
 asm volatile("STMFD   SP!, {R0-R12,LR}\n");
    switch (core_get_noise_reduction_value()){
    case NOISE_REDUCTION_AUTO_CANON:
        // leave it alone
        break;
    case NOISE_REDUCTION_OFF:
        *nrflag = NR_OFF;
        break;
    case NOISE_REDUCTION_ON:
        *nrflag = NR_ON;
        break;
    };

 shutter_open_time=_time((void*)0); 

 asm volatile("LDMFD   SP!, {R0-R12,PC}\n");
}

void __attribute__((naked,noinline)) capt_seq_task() { // ffc5fa40
    asm volatile (
                 "STMFD   SP!, {R3-R9,LR}\n"
                 "LDR     R6, =0x2850\n"
                 "LDR     R4, =0x349a0\n"
                 "MOV     R9, #1\n"
                 "MOV     R7, #0\n"
"loc_ffc5fa40:\n"
                 "LDR     R0, [R6,#4]\n"
                 "MOV     R2, #0\n"
                 "MOV     R1, SP\n"
                 "BL      sub_ffc2936c\n"
                 "TST     R0, #1\n"
                 "BEQ     loc_ffc5fa6c\n"
                 "LDR     R1, =0x5b4\n"
                 "LDR     R0, =0xffc5f5e4\n" // "LDR     R0, =0x68537353\n"
                 "BL      sub_FFC0EB14\n"
                 "BL      sub_ffc0eacc\n"
                 "LDMFD   SP!, {R3-R9,PC}\n"
"loc_ffc5fa6c:\n"
                 "LDR     R0, [SP]\n"
                 "LDR     R1, [R0]\n"
                 "CMP     R1, #0x1D\n"
                 "ADDLS   PC, PC, R1,LSL#2\n"
                 "B       loc_ffc5fcf4\n"
"loc_ffc5fa80:\n"
                 "B       loc_ffc5faf8\n"
"loc_ffc5fa84:\n"
                 "B       loc_ffc5fb5c\n"
"loc_ffc5fa88:\n"
                 "B       loc_ffc5fb98\n"
"loc_ffc5fa8c:\n"
                 "B       loc_ffc5fbac\n"
"loc_ffc5fa90:\n"
                 "B       loc_ffc5fba4\n"
"loc_ffc5fa94:\n"
                 "B       loc_ffc5fbb4\n"
"loc_ffc5fa98:\n"
                 "B       loc_ffc5fbbc\n"
"loc_ffc5fa9c:\n"
                 "B       loc_ffc5fbc4\n"
"loc_ffc5faa0:\n"
                 "B       loc_ffc5fc1c\n"
"loc_ffc5faa4:\n"
                 "B       loc_ffc5fc44\n"
"loc_ffc5faa8:\n"
                 "B       loc_ffc5fc28\n"
"loc_ffc5faac:\n"
                 "B       loc_ffc5fc34\n"
"loc_ffc5fab0:\n"
                 "B       loc_ffc5fc3c\n"
"loc_ffc5fab4:\n"
                 "B       loc_ffc5fc4c\n"
"loc_ffc5fab8:\n"
                 "B       loc_ffc5fc54\n"
"loc_ffc5fabc:\n"
                 "B       loc_ffc5fc5c\n"
"loc_ffc5fac0:\n"
                 "B       loc_ffc5fc64\n"
"loc_ffc5fac4:\n"
                 "B       loc_ffc5fc6c\n"
"loc_ffc5fac8:\n"
                 "B       loc_ffc5fc78\n"
"loc_ffc5facc:\n"
                 "B       loc_ffc5fc80\n"
"loc_ffc5fad0:\n"
                 "B       loc_ffc5fc88\n"
"loc_ffc5fad4:\n"
                 "B       loc_ffc5fc90\n"
"loc_ffc5fad8:\n"
                 "B       loc_ffc5fc98\n"
"loc_ffc5fadc:\n"
                 "B       loc_ffc5fca4\n"
"loc_ffc5fae0:\n"
                 "B       loc_ffc5fcac\n"
"loc_ffc5fae4:\n"
                 "B       loc_ffc5fcb4\n"
"loc_ffc5fae8:\n"
                 "B       loc_ffc5fcbc\n"
"loc_ffc5faec:\n"
                 "B       loc_ffc5fcc4\n"
"loc_ffc5faf0:\n"
                 "B       loc_ffc5fcd0\n"
"loc_ffc5faf4:\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5faf8:\n"
                 "BL      sub_ffc6033c\n"
                 "BL      tv_override\n" // added
                 "BL      sub_ffc5db3c\n"
// copied over from SX10/SX200
// added to avoid some incorrect behavior if overrides are used.
// it can cause some unexpected side effects. In this case, remove this code!
                 "MOV     R0, #0\n"
                 "STR     R0, [R4,#0x24]\n"  // fixes overrides  behavior at short shutter press
// end of added code
                 "LDR     R0, [R4,#0x24]\n"
                 "CMP     R0, #0\n"
                 "BEQ     loc_ffc5fd00\n"
                 "BL      sub_ffc5f2c4\n"
                 "MOV     R5, R0\n"
                 "LDR     R0, [R4,#0x24]\n"
                 "CMP     R0, #0\n"
                 "BEQ     loc_ffc5fb40\n"
                 "MOV     R0, #0xC\n"
                 "BL      sub_ffc64130\n"
                 "TST     R0, #1\n"
                 "STRNE   R9, [R6,#0x10]\n"
                 "LDRNE   R0, [R5,#8]\n"
                 "ORRNE   R0, R0, #0x40000000\n"
                 "STRNE   R0, [R5,#8]\n"
                 "BNE     loc_ffc5fd00\n"
"loc_ffc5fb40:\n"
                 "MOV     R0, R5\n"
                 "BL      sub_ffc5f568\n"
                 "MOV     R0, R5\n"
                 "BL      sub_ffd203c4_my\n"       // ------------------> 
                 "BL      capt_seq_hook_raw_here\n"      // +
                 "TST     R0, #1\n"
                 "STRNE   R9, [R6,#0x10]\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fb5c:\n"
                 "LDR     R0, [R4,#0x24]\n"
                 "CMP     R0, #0\n"
                 "BNE     loc_ffc5fb88\n"
                 "MOV     R0, #0xC\n"
                 "BL      sub_ffc64130\n"
                 "TST     R0, #1\n"
                 "LDRNE   R0, [SP]\n"
                 "MOVNE   R1, #1\n"
                 "LDRNE   R2, [R0,#0xC]\n"
                 "MOVNE   R0, #1\n"
                 "BNE     loc_ffc5fc14\n"
"loc_ffc5fb88:\n"
                 "LDR     R0, [SP]\n"
                 "BL      sub_ffc5fe18_my\n"       // ----------------->
"loc_ffc5fb90:\n"
                 "STR     R7, [R4,#0x24]\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fb98:\n"
                 "MOV     R0, #1\n"
                 "BL      sub_ffc605c4\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fba4:\n"
                 "BL      sub_ffc5ff90\n"
                 "B       loc_ffc5fb90\n"
"loc_ffc5fbac:\n"
                 "BL      sub_ffc6031c\n"
                 "B       loc_ffc5fb90\n"
"loc_ffc5fbb4:\n"
                 "BL      sub_ffc60324\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fbbc:\n"
                 "BL      sub_ffc604d4\n"
                 "B       loc_ffc5fc20\n"
"loc_ffc5fbc4:\n"
                 "LDR     R5, [R0,#0xC]\n"
                 "BL      sub_ffc6032c\n"
                 "MOV     R0, R5\n"
                 "BL      sub_ffd1f3bc\n"
                 "TST     R0, #1\n"
                 "MOV     R8, R0\n"
                 "BNE     loc_ffc5fc04\n"
                 "BL      sub_ffc70d8c\n"
                 "STR     R0, [R5,#0x18]\n"
                 "MOV     R0, R5\n"
                 "BL      sub_ffd202fc\n"
                 "MOV     R0, R5\n"
                 "BL      sub_ffd206a0\n"
                 "MOV     R8, R0\n"
                 "LDR     R0, [R5,#0x18]\n"
                 "BL      sub_ffc70fc4\n"
"loc_ffc5fc04:\n"
                 "BL      sub_ffc6031c\n"
                 "MOV     R2, R5\n"
                 "MOV     R1, #9\n"
                 "MOV     R0, R8\n"
"loc_ffc5fc14:\n"
                 "BL      sub_ffc5dfb0\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc1c:\n"
                 "BL      sub_ffc6053c\n"
"loc_ffc5fc20:\n"
                 "BL      sub_ffc5db3c\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc28:\n"
                 "LDR     R0, [R4,#0x54]\n"
                 "BL      sub_ffc6092c\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc34:\n"
                 "BL      sub_ffc60be0\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc3c:\n"
                 "BL      sub_ffc60c74\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc44:\n"
                 "BL      sub_ffc6031c\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc4c:\n"
                 "BL      sub_ffd1f5e8\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc54:\n"
                 "BL      sub_ffd1f7e0\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc5c:\n"
                 "BL      sub_ffd1f874\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc64:\n"
                 "BL      sub_ffd1f934\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc6c:\n"
                 "MOV     R0, #0\n"
"loc_ffc5fc70:\n"
                 "BL      sub_ffd1fb2c\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc78:\n"
                 "BL      sub_ffd1fc7c\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc80:\n"
                 "BL      sub_ffd1fd0c\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc88:\n"
                 "BL      sub_ffd1fdcc\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc90:\n"
                 "BL      sub_ffc60718\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fc98:\n"
                 "BL      sub_ffc607ac\n"
                 "BL      sub_ffc276bc\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fca4:\n"
                 "BL      sub_ffd1fa00\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fcac:\n"
                 "BL      sub_ffd1fa44\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fcb4:\n"
                 "BL      sub_ffc62900\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fcbc:\n"
                 "BL      sub_ffc62980\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fcc4:\n"
                 "BL      sub_ffc629dc\n"
                 "BL      sub_ffc629dc\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fcd0:\n"
                 "LDRH    R0, [R4,#0x94]\n"
                 "CMP     R0, #4\n"
                 "LDRNEH  R0, [R4]\n" //"LDRHNE  R0, [R4]\n"
                 "SUBNE   R12, R0, #0x8200\n" // "SUBNE   IP, R0, 0x8200\n"
                 "SUBNES  R12, R12, #0x2A\n" // "SUBSNE  IP, IP, 0x2a\n"
                 "BNE     sub_ffc5fd00\n"
"loc_ffc5fce8:\n"
                 "BL      sub_ffc62980\n"
                 "BL      sub_ffc62da0\n"
                 "B       loc_ffc5fd00\n"
"loc_ffc5fcf4:\n"
                 "LDR     R1, =0x70b\n"
                 "LDR     R0, =0xffc5f5e4\n"
                 "BL      sub_FFC0EB14\n"
"loc_ffc5fd00:\n"
                 "LDR     R0, [SP]\n"
                 "LDR     R1, [R0,#4]\n"
                 "LDR     R0, [R6]\n"
                 "BL      sub_ffc68c70\n"
                 "LDR     R5, [SP]\n"
                 "LDR     R0, [R5,#8]\n"
                 "CMP     R0, #0\n"
                 "LDREQ   R1, =0x132\n"
                 "LDREQ   R0, =0xffc5f5e4\n"
                 "BLEQ    sub_FFC0EB14\n"
                 "STR     R7, [R5,#8]\n"
                 "B       loc_ffc5fa40\n"
    );
}


void __attribute__((naked,noinline)) sub_ffd203c4_my() {
    asm volatile (
                 "STMFD   SP!, {R0-R8,LR}\n"
                 "MOV     R4, R0\n"
                 "BL      sub_ffd210d8\n"
                 "MVN     R1, #0x0\n" // "MOVL    R1, 0xFFFFFFFF\n"
                 "BL      sub_FFC68CA4\n"
                 "LDR     R5, =0x5B08\n"
                 "LDR     R0, [R5,#0xC]\n"
                 "CMP     R0, #0\n"
                 "BNE     loc_FFD20414\n"
                 "MOV     R1, #1\n"
                 "MOV     R0, #0\n"
                 "BL      sub_FFC29BB4\n"
                 "STR     R0, [R5,#0xC]\n"
                 "MOV     R3, #0\n"
                 "STR     R3, [SP]\n"
                 "LDR     R3, =0xffd1febc\n"
                 "LDR     R0, =0xffd2062c\n" // "ADR     R0, 0xffd2062c\n"
                 "MOV     R2, #0x400\n"
                 "MOV     R1, #0x17\n"
                 "BL      sub_ffc29b80\n"
"loc_FFD20414:\n"
                 "MOV     R2, #4\n"
                 "ADD     R1, SP, #0x8\n"
                 "MOV     R0, #0x8A\n"
                 "BL      sub_ffc70bfc\n"
                 "TST     R0, #1\n"
                 "MOVNE   R1, #0x3B4\n"
                 "LDRNE   R0, =0xFFD20128\n" // "ADRNE   R0, 0xFFD20128\n"
                 "BLNE    sub_FFC0EB14\n"
                 "LDR     R6, =0x34A64\n"
                 "LDR     R7, =0x349A0\n"
                 "LDR     R3, [R6]\n"
                 "LDRSH   R2, [R6,#0xC]\n"
                 "LDRSH   R1, [R6,#0xE]\n"
                 "LDR     R0, [R7,#0x88]\n"
                 "BL      sub_FFCEF168\n"
                 "BL      sub_FFC49E68\n"
                 "LDR     R3, =0x5B10\n"
                 "STRH    R0, [R4,#0xA4]\n"
                 "SUB     R2, R3, #4\n"
                 "STRD    R2, [SP]\n"
                 "MOV     R1, R0\n"
                 "LDRH    R0, [R7,#0x5C]\n"
                 "LDRSH   R2, [R6,#0xC]\n"
                 "SUB     R3, R3, #8\n"
                 "BL      sub_FFD21744\n"
                 "BL      wait_until_remote_button_is_released\n"
                 "BL      capt_seq_hook_set_nr\n"                     // +
                 "B       sub_ffd20478\n"                             // continue function in firmware
    );
}

void __attribute__((naked,noinline)) sub_ffc5fe18_my() {
    asm volatile (
                 "STMFD   SP!, {R4-R6,LR}\n"
                 "LDR     R4, [R0,#0xC]\n"
                 "LDR     R6, =0x349A0\n"
                 "LDR     R0, [R4,#8]\n"
                 "MOV     R5, #0\n"
                 "ORR     R0, R0, #1\n"
                 "STR     R0, [R4,#8]\n"
                 "LDR     R0, [R6,#0x24]\n"
                 "CMP     R0, #0\n"
                 "MOVEQ   R0, #2\n"
                 "BLEQ    sub_FFC5C330\n"
                 "BL      sub_FFC6032C\n"
                 "LDR     R0, [R6,#0x24]\n"
                 "CMP     R0, #0\n"
                 "BNE     loc_FFC5FEB4\n"
                 "MOV     R0, R4\n"
                 "BL      sub_FFC606CC\n"
                 "MOV     R0, R4\n"
                 "BL      sub_FFD1F028\n"
                 "CMP     R0, #0\n"
                 "MOV     R0, R4\n"
                 "BEQ     loc_FFC5FE8C\n"
                 "BL      sub_FFD1F0B4\n"
                 "TST     R0, #1\n"
                 "MOVNE   R2, R4\n"
                 "LDMNEFD SP!, {R4-R6,LR}\n"
                 "MOVNE   R1, #1\n"
                 "BNE     sub_FFC5DFB0\n"
                 "B       loc_FFC5FE90\n"
"loc_FFC5FE8C:\n"
                 "BL      sub_FFD1F078\n"
"loc_FFC5FE90:\n"
                 "MOV     R0, R4\n"
                 "BL      sub_FFC5F568\n"
                 "MOV     R0, R4\n"
                 "BL      sub_FFD202FC\n"
                 "BL      sub_FFD20F44\n"
                 "MOV     R0, R4\n"
                 "BL      sub_ffd203c4_my\n" // --------------->
                 "MOV     R5, R0\n"
                 "BL      capt_seq_hook_raw_here\n"      // +
                 "B       loc_FFC5FEC4\n"
"loc_FFC5FEB4:\n"
                 "LDR     R0, =0x2850\n"
                 "LDR     R0, [R0,#0x10]\n"
                 "CMP     R0, #0\n"
                 "MOVNE   R5, #0x1D\n"
"loc_FFC5FEC4:\n"
                 "BL      sub_FFC62980\n"
                 "BL      sub_FFC629C8\n"
                 "BL      sub_FFC62A08\n"
                 "MOV     R2, R4\n"
                 "MOV     R1, #1\n"
                 "MOV     R0, R5\n"
                 "BL      sub_FFC5DFB0\n"
                 "BL      sub_FFD20654\n"
                 "CMP     R0, #0\n"
                 "LDRNE   R0, [R4,#8]\n"
                 "ORRNE   R0, R0, #0x2000\n"
                 "STRNE   R0, [R4,#8]\n"
                 "LDMFD   SP!, {R4-R6,PC}\n"
    );
}

/*************************************************************/
void __attribute__((naked,noinline)) exp_drv_task(){
    asm volatile(
                 "STMFD   SP!, {R4-R8,LR}\n"
                 "SUB     SP, SP, #0x20\n"
                 "LDR     R8, =0xBB8\n"
                 "LDR     R7, =0x3A74\n"
                 "LDR     R5, =0x3DDB4\n"
                 "MOV     R0, #0\n"
                 "ADD     R6, SP, #0x10\n"
                 "STR     R0, [SP,#0xC]\n"
"loc_FFC99688:\n"
                 "LDR     R0, [R7,#0x20]\n"
                 "MOV     R2, #0\n"
                 "ADD     R1, SP, #0x1C\n"
                 "BL      sub_FFC2936C\n"
                 "LDR     R0, [SP,#0xC]\n"
                 "CMP     R0, #1\n"
                 "BNE     loc_FFC996D4\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R0, [R0]\n"
                 "CMP     R0, #0x13\n"
                 "CMPNE   R0, #0x14\n"
                 "CMPNE   R0, #0x15\n"
                 "CMPNE   R0, #0x16\n"
                 "BEQ     loc_FFC99838\n"
                 "CMP     R0, #0x28\n"
                 "BEQ     loc_FFC997C0\n"
                 "ADD     R1, SP, #0xC\n"
                 "MOV     R0, #0\n"
                 "BL      sub_FFC99618\n"
"loc_FFC996D4:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R1, [R0]\n"
                 "CMP     R1, #0x2E\n"
                 "BNE     loc_FFC99704\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "BL      sub_FFC9A970\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R1, #1\n"
                 "BL      sub_FFC68C70\n"
                 "BL      sub_ffc0eacc\n"
                 "ADD     SP, SP, #0x20\n"
                 "LDMFD   SP!, {R4-R8,PC}\n"
"loc_FFC99704:\n"
                 "CMP     R1, #0x2D\n"
                 "BNE     loc_FFC99720\n"
                 "LDR     R2, [R0,#0x8C]!\n"
                 "LDR     R1, [R0,#4]\n"
                 "MOV     R0, R1\n"
                 "BLX     R2\n"
                 "B       loc_FFC99C60\n"
"loc_FFC99720:\n"
                 "CMP     R1, #0x26\n"
                 "BNE     loc_FFC99770\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R1, #0x80\n"
                 "BL      sub_FFC68CA4\n"
                 "LDR     R0, =0xFFC95DE4\n"
                 "MOV     R1, #0x80\n"
                 "BL      sub_FFD14BD0\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R2, R8\n"
                 "MOV     R1, #0x80\n"
                 "BL      sub_FFC68BB0\n"
                 "TST     R0, #1\n"
                 "LDRNE   R1, =0xE5F\n"
                 "BNE     loc_FFC9982C\n"
"loc_FFC9975C:\n"
                 "LDR     R1, [SP,#0x1C]\n"
                 "LDR     R0, [R1,#0x90]\n"
                 "LDR     R1, [R1,#0x8C]\n"
                 "BLX     R1\n"
                 "B       loc_FFC99C60\n"
"loc_FFC99770:\n"
                 "CMP     R1, #0x27\n"
                 "BNE     loc_FFC997B8\n"
                 "ADD     R1, SP, #0xC\n"
                 "BL      sub_FFC99618\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R1, #0x100\n"
                 "BL      sub_FFC68CA4\n"
                 "LDR     R0, =0xFFC95DF4\n"
                 "MOV     R1, #0x100\n"
                 "BL      sub_FFD14E58\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R2, R8\n"
                 "MOV     R1, #0x100\n"
                 "BL      sub_FFC68BB0\n"
                 "TST     R0, #1\n"
                 "BEQ     loc_FFC9975C\n"
                 "LDR     R1, =0xE69\n"
                 "B       loc_FFC9982C\n"
"loc_FFC997B8:\n"
                 "CMP     R1, #0x28\n"
                 "BNE     loc_FFC997D0\n"
"loc_FFC997C0:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "ADD     R1, SP, #0xC\n"
                 "BL      sub_FFC99618\n"
                 "B       loc_FFC9975C\n"
"loc_FFC997D0:\n"
                 "CMP     R1, #0x2B\n"
                 "BNE     loc_FFC997E8\n"
                 "BL      sub_FFC88E2C\n"
                 "BL      sub_FFC89A54\n"
                 "BL      sub_FFC895A4\n"
                 "B       loc_FFC9975C\n"
"loc_FFC997E8:\n"
                 "CMP     R1, #0x2C\n"
                 "BNE     loc_FFC99838\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R1, #4\n"
                 "BL      sub_FFC68CA4\n"
                 "LDR     R1, =0xFFC95E14\n"
                 "LDR     R0, =0xFFFFF400\n"
                 "MOV     R2, #4\n"
                 "BL      sub_FFC888A8\n"
                 "BL      sub_FFC88B30\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R2, R8\n"
                 "MOV     R1, #4\n"
                 "BL      sub_FFC68ACC\n"
                 "TST     R0, #1\n"
                 "BEQ     loc_FFC9975C\n"
                 "LDR     R1, =0xE91\n"
"loc_FFC9982C:\n"
                 "LDR     R0, =0xffc96454\n"
                 "BL      sub_FFC0EB14\n"
                 "B       loc_FFC9975C\n"
"loc_FFC99838:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "MOV     R4, #1\n"
                 "LDR     R1, [R0]\n"
                 "CMP     R1, #0x11\n"
                 "CMPNE   R1, #0x12\n"
                 "BNE     loc_FFC998A8\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "ADD     R1, R1, R1,LSL#1\n"
                 "ADD     R1, R0, R1,LSL#2\n"
                 "SUB     R1, R1, #8\n"
                 "LDMIA   R1, {R2-R4}\n"
                 "STMIA   R6, {R2-R4}\n"
                 "BL      sub_FFC97FA4\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "LDR     R3, [R0,#0x8C]\n"
                 "LDR     R2, [R0,#0x90]\n"
                 "ADD     R0, R0, #4\n"
                 "BLX     R3\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "BL      sub_FFC9AD38\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "LDR     R3, [R0,#0x94]\n"
                 "LDR     R2, [R0,#0x98]\n"
                 "ADD     R0, R0, #4\n"
                 "BLX     R3\n"
                 "B       loc_FFC99BA0\n"
"loc_FFC998A8:\n"
                 "CMP     R1, #0x13\n"
                 "CMPNE   R1, #0x14\n"
                 "CMPNE   R1, #0x15\n"
                 "CMPNE   R1, #0x16\n"
                 "BNE     loc_FFC99960\n"
                 "ADD     R3, SP, #0xC\n"
                 "MOV     R2, SP\n"
                 "ADD     R1, SP, #0x10\n"
                 "BL      sub_FFC98280\n"
                 "CMP     R0, #1\n"
                 "MOV     R4, R0\n"
                 "CMPNE   R4, #5\n"
                 "BNE     loc_FFC998FC\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "MOV     R2, R4\n"
                 "LDR     R1, [R0,#0x7C]!\n"
                 "LDR     R12, [R0,#0x10]!\n"
                 "LDR     R3, [R0,#4]\n"
                 "MOV     R0, SP\n"
                 "BLX     R12\n"
                 "B       loc_FFC99934\n"
"loc_FFC998FC:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "CMP     R4, #2\n"
                 "LDR     R3, [R0,#0x90]\n"
                 "CMPNE   R4, #6\n"
                 "BNE     loc_FFC99948\n"
                 "LDR     R12, [R0,#0x8C]\n"
                 "MOV     R0, SP\n"
                 "MOV     R2, R4\n"
                 "MOV     R1, #1\n"
                 "BLX     R12\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "MOV     R2, SP\n"
                 "ADD     R1, SP, #0x10\n"
                 "BL      sub_FFC99320\n"
"loc_FFC99934:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R2, [SP,#0xC]\n"
                 "MOV     R1, R4\n"
                 "BL      sub_FFC995B8\n"
                 "B       loc_FFC99BA0\n"
"loc_FFC99948:\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "LDR     R12, [R0,#0x8C]\n"
                 "ADD     R0, R0, #4\n"
                 "MOV     R2, R4\n"
                 "BLX     R12\n"
                 "B       loc_FFC99BA0\n"
"loc_FFC99960:\n"
                 "CMP     R1, #0x22\n"
                 "CMPNE   R1, #0x23\n"
                 "BNE     loc_FFC999AC\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "ADD     R1, R1, R1,LSL#1\n"
                 "ADD     R1, R0, R1,LSL#2\n"
                 "SUB     R1, R1, #8\n"
                 "LDMIA   R1, {R2-R4}\n"
                 "STMIA   R6, {R2-R4}\n"
                 "BL      sub_FFC974F4\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "LDR     R3, [R0,#0x8C]\n"
                 "LDR     R2, [R0,#0x90]\n"
                 "ADD     R0, R0, #4\n"
                 "BLX     R3\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "BL      sub_FFC977E8\n"
                 "B       loc_FFC99BA0\n"
"loc_FFC999AC:\n"
                 "ADD     R1, R0, #4\n"
                 "LDMIA   R1, {R2,R3,R12}\n"
                 "STMIA   R6, {R2,R3,R12}\n"
                 "LDR     R1, [R0]\n"
                 "CMP     R1, #0x25\n"
                 "ADDLS   PC, PC, R1,LSL#2\n"
                 "B       loc_FFC99B80\n"
"loc_FFC999C8:\n"
                 "B       loc_FFC99A60\n"
"loc_FFC999CC:\n"
                 "B       loc_FFC99A60\n"
"loc_FFC999D0:\n"
                 "B       loc_FFC99A68\n"
"loc_FFC999D4:\n"
                 "B       loc_FFC99A70\n"
"loc_FFC999D8:\n"
                 "B       loc_FFC99A70\n"
"loc_FFC999DC:\n"
                 "B       loc_FFC99A70\n"
"loc_FFC999E0:\n"
                 "B       loc_FFC99A60\n"
"loc_FFC999E4:\n"
                 "B       loc_FFC99A68\n"
"loc_FFC999E8:\n"
                 "B       loc_FFC99A70\n"
"loc_FFC999EC:\n"
                 "B       loc_FFC99A70\n"
"loc_FFC999F0:\n"
                 "B       loc_FFC99A88\n"
"loc_FFC999F4:\n"
                 "B       loc_FFC99A88\n"
"loc_FFC999F8:\n"
                 "B       loc_FFC99B74\n"
"loc_FFC999FC:\n"
                 "B       loc_FFC99B7C\n"
"loc_FFC99A00:\n"
                 "B       loc_FFC99B7C\n"
"loc_FFC99A04:\n"
                 "B       loc_FFC99B7C\n"
"loc_FFC99A08:\n"
                 "B       loc_FFC99B7C\n"
"loc_FFC99A0C:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A10:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A14:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A18:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A1C:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A20:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A24:\n"
                 "B       loc_FFC99A78\n"
"loc_FFC99A28:\n"
                 "B       loc_FFC99A80\n"
"loc_FFC99A2C:\n"
                 "B       loc_FFC99A80\n"
"loc_FFC99A30:\n"
                 "B       loc_FFC99A94\n"
"loc_FFC99A34:\n"
                 "B       loc_FFC99A94\n"
"loc_FFC99A38:\n"
                 "B       loc_FFC99A9C\n"
"loc_FFC99A3C:\n"
                 "B       loc_FFC99ACC\n"
"loc_FFC99A40:\n"
                 "B       loc_FFC99AFC\n"
"loc_FFC99A44:\n"
                 "B       loc_FFC99B2C\n"
"loc_FFC99A48:\n"
                 "B       loc_FFC99B5C\n"
"loc_FFC99A4C:\n"
                 "B       loc_FFC99B5C\n"
"loc_FFC99A50:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A54:\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A58:\n"
                 "B       loc_FFC99B64\n"
"loc_FFC99A5C:\n"
                 "B       loc_FFC99B6C\n"
"loc_FFC99A60:\n"
                 "BL      sub_FFC962FC\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A68:\n"
                 "BL      sub_FFC96580\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A70:\n"
                 "BL      sub_FFC96788\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A78:\n"
                 "BL      sub_FFC96A00\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A80:\n"
                 "BL      sub_FFC96BF8\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A88:\n"
                 "BL      sub_FFC96EB4_my\n" // ------------>
                 "MOV     R4, #0\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A94:\n"
                 "BL      sub_FFC96FF4\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99A9C:\n"
                 "LDRH    R1, [R0,#4]\n"
                 "STRH    R1, [SP,#0x10]\n"
                 "LDRH    R1, [R5,#2]\n"
                 "STRH    R1, [SP,#0x12]\n"
                 "LDRH    R1, [R5,#4]\n"
                 "STRH    R1, [SP,#0x14]\n"
                 "LDRH    R1, [R5,#6]\n"
                 "STRH    R1, [SP,#0x16]\n"
                 "LDRH    R1, [R0,#0xC]\n"
                 "STRH    R1, [SP,#0x18]\n"
                 "BL      sub_FFC9A9E4\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99ACC:\n"
                 "LDRH    R1, [R0,#4]\n"
                 "STRH    R1, [SP,#0x10]\n"
                 "LDRH    R1, [R5,#2]\n"
                 "STRH    R1, [SP,#0x12]\n"
                 "LDRH    R1, [R5,#4]\n"
                 "STRH    R1, [SP,#0x14]\n"
                 "LDRH    R1, [R5,#6]\n"
                 "STRH    R1, [SP,#0x16]\n"
                 "LDRH    R1, [R5,#8]\n"
                 "STRH    R1, [SP,#0x18]\n"
                 "BL      sub_FFC9AB50\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99AFC:\n"
                 "LDRH    R1, [R5]\n"
                 "STRH    R1, [SP,#0x10]\n"
                 "LDRH    R1, [R0,#6]\n"
                 "STRH    R1, [SP,#0x12]\n"
                 "LDRH    R1, [R5,#4]\n"
                 "STRH    R1, [SP,#0x14]\n"
                 "LDRH    R1, [R5,#6]\n"
                 "STRH    R1, [SP,#0x16]\n"
                 "LDRH    R1, [R5,#8]\n"
                 "STRH    R1, [SP,#0x18]\n"
                 "BL      sub_FFC9ABFC\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99B2C:\n"
                 "LDRH    R1, [R5]\n"
                 "STRH    R1, [SP,#0x10]\n"
                 "LDRH    R1, [R5,#2]\n"
                 "STRH    R1, [SP,#0x12]\n"
                 "LDRH    R1, [R5,#4]\n"
                 "STRH    R1, [SP,#0x14]\n"
                 "LDRH    R1, [R5,#6]\n"
                 "STRH    R1, [SP,#0x16]\n"
                 "LDRH    R1, [R0,#0xC]\n"
                 "STRH    R1, [SP,#0x18]\n"
                 "BL      sub_FFC9AC9C\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99B5C:\n"
                 "BL      sub_FFC9734C\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99B64:\n"

                 "BL      sub_FFC978EC\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99B6C:\n"
                 "BL      sub_FFC97B28\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99B74:\n"
                 "BL      sub_FFC97CA4\n"
                 "B       loc_FFC99B80\n"
"loc_FFC99B7C:\n"
                 "BL      sub_FFC97E40\n"
"loc_FFC99B80:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "LDR     R3, [R0,#0x8C]\n"
                 "LDR     R2, [R0,#0x90]\n"
                 "ADD     R0, R0, #4\n"
                 "BLX     R3\n"
                 "CMP     R4, #1\n"
                 "BNE     loc_FFC99BE8\n"
"loc_FFC99BA0:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "MOV     R2, #0xC\n"
                 "LDR     R1, [R0,#0x7C]\n"
                 "ADD     R1, R1, R1,LSL#1\n"
                 "ADD     R0, R0, R1,LSL#2\n"
                 "SUB     R4, R0, #8\n"
                 "LDR     R0, =0x3DDB4\n"
                 "ADD     R1, SP, #0x10\n"
                 "BL      sub_FFEA2D58\n"
                 "LDR     R0, =0x3DDC0\n"
                 "MOV     R2, #0xC\n"
                 "ADD     R1, SP, #0x10\n"
                 "BL      sub_FFEA2D58\n"
                 "LDR     R0, =0x3DDCC\n"
                 "MOV     R2, #0xC\n"
                 "MOV     R1, R4\n"
                 "BL      sub_FFEA2D58\n"
                 "B       loc_FFC99C60\n"
"loc_FFC99BE8:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "LDR     R0, [R0]\n"
                 "CMP     R0, #0xB\n"
                 "BNE     loc_FFC99C30\n"
                 "MOV     R3, #0\n"
                 "STR     R3, [SP,#0x0]\n"
                 "MOV     R3, #1\n"
                 "MOV     R2, #1\n"
                 "MOV     R1, #1\n"
                 "MOV     R0, #0\n"
                 "BL      sub_FFC96104\n"
                 "MOV     R3, #0\n"
                 "STR     R3, [SP,#0x0]\n"
                 "MOV     R3, #1\n"
                 "MOV     R2, #1\n"
                 "MOV     R1, #1\n"
                 "MOV     R0, #0\n"
                 "B       loc_FFC99C5C\n"
"loc_FFC99C30:\n"
                 "MOV     R3, #1\n"
                 "MOV     R2, #1\n"
                 "MOV     R1, #1\n"
                 "MOV     R0, #1\n"
                 "STR     R3, [SP,#0x0]\n"
                 "BL      sub_FFC96104\n"
                 "MOV     R3, #1\n"
                 "MOV     R2, #1\n"
                 "MOV     R1, #1\n"
                 "MOV     R0, #1\n"
                 "STR     R3, [SP,#0x0]\n"
"loc_FFC99C5C:\n"
                 "BL      sub_FFC96244\n"
"loc_FFC99C60:\n"
                 "LDR     R0, [SP,#0x1C]\n"
                 "BL      sub_FFC9A970\n"
                 "B       loc_FFC99688\n"
    );
}

void __attribute__((naked,noinline)) sub_FFC96EB4_my() { 
    asm volatile(
                 "STMFD   SP!, {R4-R8,LR}\n"
                 "LDR     R7, =0x3A74\n"
                 "MOV     R4, R0\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R1, #0x3E\n"
                 "BL      sub_FFC68CA4\n"
                 "LDRSH   R0, [R4,#4]\n"
                 "MOV     R2, #0\n"
                 "MOV     R1, #0\n"
                 "BL      sub_FFC95E68\n"
                 "MOV     R6, R0\n"
                 "LDRSH   R0, [R4,#6]\n"
                 "BL      sub_FFC95F78\n"
                 "LDRSH   R0, [R4,#8]\n"
                 "BL      sub_FFC95FD0\n"
                 "LDRSH   R0, [R4,#0xA]\n"
                 "BL      sub_FFC96028\n"
                 "LDRSH   R0, [R4,#0xC]\n"
                 "MOV     R1, #0\n"
                 "BL      sub_FFC96080\n"
                 "MOV     R5, R0\n"
                 "LDR     R0, [R4]\n"
                 "LDR     R8, =0x3DDCC\n"
                 "CMP     R0, #0xB\n"
                 "MOVEQ   R6, #0\n"
                 "MOVEQ   R5, #0\n"
                 "BEQ     loc_FFC96F48\n"
                 "CMP     R6, #1\n"
                 "BNE     loc_FFC96F48\n"
                 "LDRSH   R0, [R4,#4]\n"
                 "LDR     R1, =0xFFC95DD4\n"
                 "MOV     R2, #2\n"
                 "BL      sub_FFD14D24\n"
                 "STRH    R0, [R4,#4]\n"
                 "MOV     R0, #0\n"
                 "STR     R0, [R7,#0x28]\n"
                 "B       loc_FFC96F50\n"
"loc_FFC96F48:\n"
                 "LDRH    R0, [R8]\n"
                 "STRH    R0, [R4,#4]\n"
"loc_FFC96F50:\n"
                 "CMP     R5, #1\n"
                 "LDRNEH  R0, [R8,#8]\n"
                 "BNE     loc_FFC96F6C\n"
                 "LDRSH   R0, [R4,#0xC]\n"
                 "LDR     R1, =0xFFC95E58\n"
                 "MOV     R2, #0x20\n"
                 "BL      sub_FFC9A9A0\n"
"loc_FFC96F6C:\n"
                 "STRH    R0, [R4,#0xC]\n"
                 "LDRSH   R0, [R4,#6]\n"
                 "BL      sub_FFC88B9C_my\n" // -------------->
                 "LDRSH   R0, [R4,#8]\n"
                 "MOV     R1, #1\n"
                 "BL      sub_FFC892EC\n"
                 "MOV     R1, #0\n"
                 "ADD     R0, R4, #8\n"
                 "BL      sub_FFC89374\n"
                 "LDRSH   R0, [R4,#0xE]\n"
                 "BL      sub_FFC911C8\n"
                 "LDR     R4, =0xBB8\n"
                 "CMP     R6, #1\n"
                 "BNE     loc_FFC96FC4\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R2, R4\n"
                 "MOV     R1, #2\n"
                 "BL      sub_FFC68BB0\n"
                 "TST     R0, #1\n"
                 "LDRNE   R1, =0x5A3\n"
                 "LDRNE   R0, =0xffc96454\n" // "ADRNE   R0, aExpdrv_c\n"
                 "BLNE    sub_FFC0EB14\n"
"loc_FFC96FC4:\n"
                 "CMP     R5, #1\n"
                 "LDMNEFD SP!, {R4-R8,PC}\n"
                 "LDR     R0, [R7,#0x1C]\n"
                 "MOV     R2, R4\n"
                 "MOV     R1, #0x20\n"
                 "BL      sub_FFC68BB0\n"
                 "TST     R0, #1\n"
                 "LDRNE   R1, =0x5A8\n"
                 "LDRNE   R0, =0xffc96454\n"
                 "LDMNEFD SP!, {R4-R8,LR}\n"
                 "BNE     sub_FFC0EB14\n"
                 "LDMFD   SP!, {R4-R8,PC}\n"
    );
}

void __attribute__((naked,noinline)) sub_FFC88B9C_my(){ // 
    asm volatile(
                 "STMFD   SP!, {R4-R6,LR}\n"
                 "LDR     R5, =0x3760\n"
                 "MOV     R4, R0\n"
                 "LDR     R0, [R5,#4]\n"
                 "CMP     R0, #1\n"
                 "LDRNE   R1, =0x146\n"
                 "LDRNE   R0, =0xffc889a0\n" // "ADRNE   R0, aShutter_c\n"
                 "BLNE    sub_FFC0EB14\n"
                 "CMN     R4, #0xC00\n"
                 "LDREQSH R4, [R5,#2]\n"
                 "CMN     R4, #0xC00\n"
                 "MOVEQ   R1, #0x14C\n"
                 "LDREQ   R0, =0xffc889a0\n" // "ADREQ   R0, aShutter_c\n"
                 "STRH    R4, [R5,#2]\n"
                 "BLEQ    sub_FFC0EB14\n"
                 "MOV     R0, R4\n"
                 "BL      sub_FFD8F4B8\n"
            //   "BL      apex2us\n"  // +
                 "MOV     R4, R0\n"
                 // "BL      nullsub_65\n"
                 "MOV     R0, R4\n"
                 "BL      sub_FFCBF680\n"
                 "TST     R0, #1\n"
                 "LDRNE   R1, =0x151\n"
                 "LDMNEFD SP!, {R4-R6,LR}\n"
                 "LDREQ   R0, =0xffc889a0\n" // "ADRNE   R0, aShutter_c\n"
                 "BNE     sub_FFC0EB14\n"
                 "LDMFD   SP!, {R4-R6,PC}\n"
    );
}
