#include "camera.h"
#include "lolevel.h"
#include "platform.h"
#include "core.h"
#include "keyboard.h"
#include "stdlib.h"

extern long link_bss_start;
extern long link_bss_end;
extern void boot();

long get_sensor_width()
{
    return 6160;          // 1/2.3"
}

void startup()
{
    long *bss = &link_bss_start;

    // sanity check
    if ((long)&link_bss_end > (MEMISOSTART + MEMISOSIZE)){
		started();
		shutdown();
    }

    // initialize .bss senment
    while (bss<&link_bss_end)
	*bss++ = 0;

    boot();
}

// Ixus 90 focal length range is 6.2 - 18.6 mm, 35-105 in 35-mm equivalent.
// So, CF_EFL = 35/6.2*10000=56452 or 105/18.6*10000=56452.
static const int fl_tbl[] = {6200, 7230, 8295, 9681, 11614, 14303, 18600};
#define CF_EFL 56452
#define NUM_FL (sizeof(fl_tbl)/sizeof(fl_tbl[0]))

const int zoom_points = NUM_FL;

static const int fov_tbl[] = {53,46,41,35,30,24,19};
static const int mav_tbl[] = {307,307,325,347,367,389,413};  // min Av **** TO DO ****
static const int xav_tbl[] = {495,492,510,532,552,574,598};  // max Av **** TO DO ****

int get_mav(int zp)
{
 return mav_tbl[zp];
}

int get_xav(int zp)
{
 return xav_tbl[zp];
}

int get_fov(int zp)
{
 if (zp<0) return fov_tbl[0];
 else if (zp>NUM_FL-1) return fov_tbl[NUM_FL-1];
 else return fov_tbl[zp];
}

int get_effective_focal_length(int zp) {
    return (CF_EFL*get_focal_length(zp))/10000;
}

int get_focal_length(int zp) {
    if (zp<0) return fl_tbl[0];
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1];
    else return fl_tbl[zp];
}

int get_zoom_x(int zp) {
    if (zp<1) return 10;
    else if (zp>NUM_FL-1) return fl_tbl[NUM_FL-1]*10/fl_tbl[0];
    else return fl_tbl[zp]*10/fl_tbl[0];
}

static struct 
{
   int hackmode;
   int canonmode;
} modemap[] = {
    { MODE_AUTO,               32768 }, // PROPCASE 49
    { MODE_P,                  32772 },
    { MODE_DIGITAL_MACRO,      33288 },
    { MODE_PORTRAIT,           16397 },
    { MODE_COLOR_ACCENT,       33307 }, 
    { MODE_COLOR_SWAP,         33308 },
    { MODE_STITCH,             33290 },
//  { MODE_LONG_SHUTTER,       32774 },
    { MODE_NIGHT_SNAPSHOT,     16395 },
    { MODE_SCN_KIDS_PETS,      16400 }, 
    { MODE_SCN_INDOOR,         16401 }, 
    { MODE_SCN_SUNSET,         16402 },
    { MODE_SCN_FOLIAGE,        16403 }, 
    { MODE_SCN_SNOW,           16404 },
    { MODE_SCN_BEACH,          16405 },
    { MODE_SCN_FIREWORK,       16406 },
    { MODE_SCN_AQUARIUM,       16408 },
    { MODE_SCN_UNDERWATER,     16407 },
    { MODE_SCN_ISO_3200,       16413 },
    { MODE_VIDEO_STD,          2599  },
    { MODE_VIDEO_COMPACT,      2601  },
    { MODE_VIDEO_COLOR_ACCENT, 2597  },
    { MODE_VIDEO_COLOR_SWAP,   2598  },
    { MODE_VIDEO_TIME_LAPSE,   2603  },
}; 

#define MODESCNT (sizeof(modemap)/sizeof(modemap[0]))


/*                  NOT IN CHDK BUILD  check if correct*/

int mode_get() 
{
    int mode, i, t=0xFF;
    mode  = (playrec_mode==2 || playrec_mode==5)?MODE_REC:MODE_PLAY;

    _GetPropertyCase(PROPCASE_SHOOTING_MODE, &t, 4);
    for (i=0; i<MODESCNT; ++i) 
    {
	if (modemap[i].canonmode == t) 
      {
	    return (mode | (modemap[i].hackmode & MODE_SHOOTING_MASK));
	}
    }
    return (mode);
}

long get_vbatt_min()
{
    return 3500;
}

long get_vbatt_max()
{
    return 4100;
}

